Package: shinyWGD
Type: Package
Title: 'Shiny' Application for Whole Genome Duplication Analysis
Version: 1.0.0
Maintainer: Jia Li <li081766@gmail.com>
Authors@R: c(
    person("Jia", "Li", email = "li081766@gmail.com", role = c("aut", "cre")),
    person("Zhen", "Li", email = "zhen.li@psb.vib-ugent.be", role = "ctb"),
    person("Arthur", "Zwaenepoel", email = "arthur.zwaenepoel@gmail.com", role = "ctb"))
Description: Provides a comprehensive 'Shiny' application for analyzing 
    Whole Genome Duplication ('WGD') events.
    This package provides a user-friendly 'Shiny' web application for 
    non-experienced researchers to prepare input data and execute command lines 
    for several well-known 'WGD' analysis tools, including 'wgd', 'ksrates', 
    'i-ADHoRe', 'OrthoFinder', and 'Whale'. This package also provides the source code 
    for experienced researchers to adjust and install the package to their own server. 
    Key Features
    1) Input Data Preparation 
    This package allows users to conveniently upload and format their data, 
	making it compatible with various 'WGD' analysis tools.
    2) Command Line Generation
    This package automatically generates the necessary command lines for 
	selected 'WGD' analysis tools, reducing manual errors and saving time.
    3) Visualization
    This package offers interactive visualizations to explore and interpret 'WGD' results, 
	facilitating in-depth 'WGD' analysis.
    4) Comparative Genomics
    Users can study and compare 'WGD' events across different species, 
	aiding in evolutionary and comparative genomics studies.
    5) User-Friendly Interface 
	This 'Shiny' web application provides an intuitive and accessible interface, 
	making 'WGD' analysis accessible to researchers and 'bioinformaticians' of all levels.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.2.3
SystemRequirements: pandoc (>= 1.12.3), pandoc-citeproc
Imports: shiny, shinyalert, stringr, vroom, fs, tidyr, data.table,
        dplyr, ape, ks, mclust, htmltools, seqinr, httr, jsonlite
Suggests: tidyverse, knitr, rmarkdown, DT, argparse, bslib, bsplus,
        english, fontawesome, igraph, shinyBS, shinyFiles,
        shinyWidgets, shinyjs, stringi, tools, testthat (>= 3.0.0)
NeedsCompilation: no
Packaged: 2024-11-13 01:08:49 UTC; jiali
Author: Jia Li [aut, cre],
  Zhen Li [ctb],
  Arthur Zwaenepoel [ctb]
Repository: CRAN
Date/Publication: 2024-11-13 15:10:01 UTC
Built: R 4.6.0; ; 2026-04-26 21:36:39 UTC; unix

Package: shinyWGD
Type: Package
Title: 'Shiny' Application for Whole Genome Duplication Analysis
Version: 1.0.0
Maintainer: Jia Li <li081766@gmail.com>
Authors@R: c(
    person("Jia", "Li", email = "li081766@gmail.com", role = c("aut", "cre")),
    person("Zhen", "Li", email = "zhen.li@psb.vib-ugent.be", role = "ctb"),
    person("Arthur", "Zwaenepoel", email = "arthur.zwaenepoel@gmail.com", role = "ctb"))
Description: Provides a comprehensive 'Shiny' application for analyzing 
    Whole Genome Duplication ('WGD') events.
    This package provides a user-friendly 'Shiny' web application for 
    non-experienced researchers to prepare input data and execute command lines 
    for several well-known 'WGD' analysis tools, including 'wgd', 'ksrates', 
    'i-ADHoRe', 'OrthoFinder', and 'Whale'. This package also provides the source code 
    for experienced researchers to adjust and install the package to their own server. 
    Key Features
    1) Input Data Preparation 
    This package allows users to conveniently upload and format their data, 
	making it compatible with various 'WGD' analysis tools.
    2) Command Line Generation
    This package automatically generates the necessary command lines for 
	selected 'WGD' analysis tools, reducing manual errors and saving time.
    3) Visualization
    This package offers interactive visualizations to explore and interpret 'WGD' results, 
	facilitating in-depth 'WGD' analysis.
    4) Comparative Genomics
    Users can study and compare 'WGD' events across different species, 
	aiding in evolutionary and comparative genomics studies.
    5) User-Friendly Interface 
	This 'Shiny' web application provides an intuitive and accessible interface, 
	making 'WGD' analysis accessible to researchers and 'bioinformaticians' of all levels.
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.2.3
SystemRequirements: pandoc (>= 1.12.3), pandoc-citeproc
Imports: shiny, shinyalert, stringr, vroom, fs, tidyr, data.table,
        dplyr, ape, ks, mclust, htmltools, seqinr, httr, jsonlite
Suggests: tidyverse, knitr, rmarkdown, DT, argparse, bslib, bsplus,
        english, fontawesome, igraph, shinyBS, shinyFiles,
        shinyWidgets, shinyjs, stringi, tools, testthat (>= 3.0.0)
NeedsCompilation: no
Packaged: 2024-11-13 01:08:49 UTC; jiali
Author: Jia Li [aut, cre],
  Zhen Li [ctb],
  Arthur Zwaenepoel [ctb]
Repository: CRAN
Date/Publication: 2024-11-13 15:10:01 UTC
Built: R 4.6.0; ; 2026-04-30 03:34:46 UTC; unix

# Generated by roxygen2: do not edit by hand

export(TimeTreeFecher)
export(runshinyWGD)
importFrom(ape,as.phylo)
importFrom(ape,write.nexus)
importFrom(ape,write.tree)
importFrom(data.table,copy)
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,lag)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,row_number)
importFrom(dplyr,rowwise)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(fs,file_temp)
importFrom(graphics,hist)
importFrom(htmltools,tags)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(jsonlite,fromJSON)
importFrom(seqinr,getLength)
importFrom(seqinr,read.fasta)
importFrom(seqinr,write.fasta)
importFrom(shiny,incProgress)
importFrom(shinyalert,shinyalert)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,fft)
importFrom(stats,hclust)
importFrom(stats,kmeans)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,ppois)
importFrom(stats,predict)
importFrom(stats,quantile)
importFrom(stats,smooth.spline)
importFrom(stats,var)
importFrom(stringr,regex)
importFrom(stringr,str_detect)
importFrom(stringr,str_trim)
importFrom(tidyr,fill)
importFrom(utils,read.delim)
importFrom(utils,read.table)
importFrom(utils,write.table)
importFrom(vroom,vroom)

# shinyWGD <img src="inst/shinyWGD/www/images/sticker_github.png" align="right" width="120" />

<!-- badges: start -->
![license](https://img.shields.io/pypi/l/wgd.svg)
![Supported Platforms](https://img.shields.io/badge/Platforms-Linux%20%7C%20OSX%20%7C%20Windows-brightgreen.svg)
[![GitHub issues](https://img.shields.io/github/issues/li081766/shinyWGD)](https://github.com/li081766/shinyWGD/issues)
[![Visit Server](https://img.shields.io/badge/Visit%20Server-blue.svg)](https://bioinformatics.psb.ugent.be/shiny/ShinyWGD/)
<!-- badges: end -->

### Overview

`shinyWGD` can prepare the input and command lines for [`wgd`](https://github.com/arzwa/wgd), [`ksrates`](https://github.com/VIB-PSB/ksrates), [`i-ADHoRe`](https://github.com/VIB-PSB/i-ADHoRe), and [`OrthoFinder`](https://github.com/davidemms/OrthoFinder). 

`shinyWGD` can also assist users in using [`Whale`](https://github.com/arzwa/whaleprep/tree/master) to infer reconciled gene trees and parameters of a model of gene family evolution given a known species tree.

After directly uploading the output of `wgd`, `ksrates`, `i-ADHoRe`, or `OrthoFinder`, `shinyWGD` can study the whole genome duplication events (WGDs).

### Installation

* Install the latest development version from GitHub (requires [devtools](https://github.com/r-lib/devtools) package):

```r
if (!require("devtools")) {
  install.packages("devtools")
}
devtools::install_github("li081766/shinyWGD", dependencies = TRUE, build_vignettes = TRUE)
```

To ensure the successfully loading of `Gallery` and `Help` pages in `shinyWGD`, please set `build_vignettes` to `TRUE` when installing the package.

### Usage

Please refer to the [`shinyWGD` server](https://bioinformatics.psb.ugent.be/shiny/ShinyWGD/) or the vignettes of `shinyWGD` to see the detailed usage. 

```r
library(shinyWGD)
runshinyWGD()
help(package="shinyWGD")
```

### The structure of `shinyWGD`

- ##### <img src="inst/shinyWGD/www/images/house-solid.svg" alt="Icon" width="15" height="15"> Home
- ##### <img src="inst/shinyWGD/www/images/terminal-solid.svg" alt="Icon" width="15" height="15"> Scripts
  - <img src="inst/shinyWGD/www/images/microscope-solid.svg" alt="Icon" width="15" height="15"> Data Preparation
  - <img src="inst/shinyWGD/www/images/code-solid.svg" alt="Icon" width="15" height="15"> Codes
- ##### <img src="inst/shinyWGD/www/images/pencil-solid.svg" alt="Icon" width="15" height="15"> Analysis
  - <img src="inst/shinyWGD/www/images/ksIcon.svg" alt="Icon" width="20" height="20"> Age Distribution Analysis
  - <img src="inst/shinyWGD/www/images/syntenyIcon.svg" alt="Icon" width="20" height="20"> Synteny Analysis
  - <img src="inst/shinyWGD/www/images/ksTreeIcon.svg" alt="Icon" width="20" height="20"> Tree Building
  - <img src="inst/shinyWGD/www/images/treeReconciliationIcon.svg" alt="Icon" width="20" height="20"> Gene Tree – Species Tree Reconciliation Analysis
- ##### <img src="inst/shinyWGD/www/images/question-solid.svg" alt="Icon" width="15" height="15"> Help

[Usage](https://li081766.github.io/shinyWGD_Demo_Data/intro_to_shinywgd.html)

[Gallery](https://li081766.github.io/shinyWGD_Demo_Data/gallery.html)



### Dependencies

- ##### External software
  - [`wgd`](https://github.com/arzwa/wgd)
  - [`ksrates`](https://github.com/VIB-PSB/ksrates)
  - [`i-ADHoRe`](https://github.com/VIB-PSB/i-ADHoRe)
  - [`Whale`](https://github.com/arzwa/Whale.jl/tree/master)
  - [`OrthoFinder`](https://github.com/davidemms/OrthoFinder)
  - [`diamond`](https://github.com/bbuchfink/diamond)
  - [`MAFFT`](https://mafft.cbrc.jp/alignment/software/)
  - [`trimAl`](https://trimal.cgenomics.org/)
  - [`PAML`](http://abacus.gene.ucl.ac.uk/software/paml.html)
  - [`MrBayes`](https://nbisweden.github.io/MrBayes/)
  - [`ALE`](https://github.com/ssolo/ALE)

- ##### R packages
  - `{shiny}`
  - `{shinyjs}`
  - `{shinyFiles}`
  - `{shinyBS}`
  - `{shinyWidgets}`
  - `{shinyalert}`
  - `{bslib}`
  - `{bsplus}`
  - `{htmltools}`
  - `{tidyverse}`
  - `{vroom}`
  - `{english}`
  - `{data.table}`
  - `{argparse}`
  - `{dplyr}`
  - `{tools}`
  - `{seqinr}`
  - `{DT}`
  - `{stringr}`
  - `{fs}`
  - `{tidyr}`
  - `{ape}`
  - `{ks}`
  - `{mclust}`

#' Find Peaks in a Numeric Vector
#'
#' This function identifies peaks in a numeric vector by analyzing the shape of the curve.
#'
#' @param x A numeric vector in which peaks will be identified.
#' @param m An integer indicating the half-width of the neighborhood to consider when identifying peaks. A larger value of `m` makes peak detection less sensitive.
#'
#' @return A numeric vector containing the indices of the identified peaks in the input vector `x`.
#'
#'
#' @examples
#' # Generate some example data
#' x <- c(1, 3, 7, 2, 6, 8, 5, 4, 9, 3, 2, 1)
#'
#' # Find peaks in the data with a half-width of 2
#' peaks <- find_peaks(x, m=2)
#'
#' # Print the indices of the identified peaks
#' print(peaks)
find_peaks <- function (x, m=3){
    shape <- diff(sign(diff(x, na.pad=FALSE)))
    pks <- sapply(which(shape < 0), FUN=function(i){
        z <- i - m + 1
        z <- ifelse(z > 0, z, 1)
        w <- i + m + 1
        w <- ifelse(w < length(x), w, length(x))
        if(all(x[c(z : i, (i + 2) : w)] <= x[i + 1])) return(i + 1) else return(numeric(0))
    })
    pks <- unlist(pks)

    return(pks)
}

#' Find Peaks in the Ks Distribution
#'
#' This function identifies peaks in a distribution of Ks (synonymous substitution rates) values.
# It performs a histogram analysis on the Ks values and detects peaks based on the smoothed curve.
# The function is useful for identifying modes or clusters in Ks distributions.
#
#' @param ks A numeric vector containing Ks values for which peaks will be identified.
#' @param binWidth A numeric value specifying the bin width for creating the histogram.
#' @param maxK A numeric value indicating the maximum Ks value to consider.
#' @param m An integer indicating the half-width of the neighborhood to consider when identifying peaks. A larger value of `m` makes peak detection less sensitive.
#' @param peak.maxK A numeric value specifying the maximum Ks value to consider when identifying peaks.
#' @param spar A numeric value controlling the smoothness of the spline fit. Higher values make the fit smoother.
#'
#' @importFrom graphics hist
#' @importFrom stats smooth.spline
#' @importFrom stats predict
#'
#' @return A numeric vector containing the identified peaks in the Ks distribution.
#'
#' @examples
#' # Generate a vector of Ks values (replace with your data)
#' ks_values <- c(0.1, 0.2, 0.3, 0.4, 0.5, 1.0, 1.2, 1.5, 2.0, 2.5)
#'
#' # Find peaks in the Ks distribution
#' peaks <- PeaksInKsDistributionValues(
#'   ks=ks_values,
#'   binWidth=0.1,
#'   maxK=2.5,
#'   m=3,
#'   peak.maxK=2.0,
#'   spar=0.25
#' )
#'
#' # Print the identified peaks
#' print(peaks)
PeaksInKsDistributionValues <- function(ks, binWidth=0.1, maxK=5,
                                        m=3, peak.maxK=2, spar=0.25) {

    ksDist <- hist(ks, seq(0, maxK, 0.01), plot=F)$counts

    ksDistBinned <- NULL
    valuesPerBin <- binWidth / 0.01
    for (i in seq(1, 500, valuesPerBin)) {
        ks <- seq(0.01 * valuesPerBin / 2, 5, 0.01 * valuesPerBin)
        ksDistBinned=c(ksDistBinned, sum(ksDist[i:(i + valuesPerBin - 1)]))
    }
    maxBin=length(ks[ks <= maxK])
    s02 <- smooth.spline(ks[1:maxBin], ksDistBinned[1:maxBin], spar=spar)
    xx <- seq(0, maxK, 0.01)
    peaks <- xx[find_peaks(predict(s02, xx)$y, m=m)]
    peaks.max <- peaks[peaks <= peak.maxK]

    return(peaks.max)
}

#' Generate the Ks Distribution
#'
#' This function generates a Ks (synonymous substitution rates) distribution from raw Ks values.
# It calculates the histogram of Ks values and returns the binned Ks distribution.
#
#' @param ksraw A numeric vector containing raw Ks values.
#' @param speciesName (Optional) A character string specifying the species name associated with the Ks values.
#' @param maxK A numeric value indicating the maximum Ks value to consider in the distribution.
#'
#' @return A numeric vector containing the binned Ks distribution.
#'
#' @examples
#' # Load or obtain raw Ks values (replace with your data)
#' raw_ks_values <- c(0.1, 0.2, 0.3, 0.4, 0.5, 1.0, 1.2, 1.5, 2.0, 2.5)
#'
#' # Generate the Ks distribution
#' ks_distribution <- generateKsDistribution(ksraw=raw_ks_values, maxK=2.5)
#'
#' # Print the binned Ks distribution
#' print(ks_distribution)
generateKsDistribution <- function(ksraw, speciesName=NULL, maxK=5) {
    ksDist <- hist(ksraw, seq(0, maxK, 0.01), plot=F)$counts
    ksDistBinned <- NULL
    valuesPerBin <- 1
    maxCount <- maxK * 100
    for (i in seq(1, maxCount, valuesPerBin)) {
        ks <- seq(0.01, maxK, 0.01 * valuesPerBin)
        if( !is.na(ksDist[i:(i + valuesPerBin - 1)]) ){
            ksDistBinned=c(ksDistBinned, rep(ks[i], sum(ksDist[i:(i + valuesPerBin - 1)])))
        }
    }
    return(ksDistBinned)
}

#' Resample a Ks Distribution
#'
#' This function resamples a given Ks (synonymous substitution rates) distribution.
# It generates a new sample of the Ks values based on the provided distribution.
#
#' @param ks A numeric vector representing the Ks distribution to be resampled.
#' @param maxK A numeric value indicating the maximum Ks value to consider in the distribution.
#'
#' @return A numeric vector containing a resampled Ks distribution.
#'
#' @examples
#' # Load or obtain a Ks distribution (replace with your data)
#' ks_distribution <- c(0.1, 0.2, 0.3, 0.4, 0.5, 1.0, 1.2, 1.5, 2.0, 2.5)
#'
#' # Resample the Ks distribution
#' resampled_ks <- resampleKsDistribution(ks=ks_distribution, maxK=2.5)
#'
#' # Print the resampled Ks distribution
#' print(resampled_ks)
resampleKsDistribution <- function(ks, maxK=5) {
    ksDist <- generateKsDistribution(ks, maxK=maxK)
    return(sample(x=ksDist, size=length(ksDist), replace=T))
}

#' Bootstrap Peaks in the Ks Distribution
#'
#' This function performs bootstrapping on a given Ks (synonymous substitution rates) distribution
#' to estimate peaks within the distribution.
#
#' @param ksRaw A numeric vector representing the raw Ks distribution to be bootstrapped.
#' @param binWidth A numeric value indicating the bin width for histogram calculation.
#' @param maxK A numeric value indicating the maximum Ks value to consider in the distribution.
#' @param m An integer specifying the parameter for peak detection.
#' @param peak.index An integer indicating the index of the peak to be estimated.
#' @param peak.maxK A numeric value indicating the maximum Ks value for peak estimation.
#' @param spar A numeric value controlling the smoothness of spline fitting.
#' @param rep An integer specifying the number of bootstrap repetitions.
#' @param from A numeric value indicating the lower bound for peak estimation.
#' @param to A numeric value indicating the upper bound for peak estimation.
#'
#' @return A numeric vector containing bootstrapped peak estimates.
#'
#' @examples
#' # Load or obtain a Ks distribution (replace with your data)
#' ks_distribution <- c(0.1, 0.2, 0.3, 0.4, 0.5, 1.0, 1.2, 1.5, 2.0, 2.5)
#'
#' # Bootstrap peak estimation
#' bootstrap_peaks <- bootStrapPeaks(
#'     ksRaw=ks_distribution,
#'     peak.index=1,
#'     rep=10,
#'     peak.maxK=2.5
#' )
#'
#' # Print the bootstrapped peak estimates
#' print(bootstrap_peaks)
bootStrapPeaks <- function(ksRaw, binWidth=0.1, maxK=5, m=3, peak.index=1,
                           peak.maxK=2, spar=0.25, rep=1000, from=0, to=maxK) {
    peaks <- c()
    r=0
    while (r < rep) {
        s <- resampleKsDistribution(ksRaw, maxK=maxK)
        p <- PeaksInKsDistributionValues(ks=s, binWidth=binWidth, maxK=maxK, m=m,
                                         peak.maxK=peak.maxK, spar=spar)
        if (is.na(p[peak.index])) {
            next
        } else if (p[peak.index] < from || p[peak.index] > to) {
            next
        } else {
            peaks <- c(peaks, p[peak.index])
            r <- r + 1
        }
    }

    return(peaks)
}

suppressPackageStartupMessages({
    library(shiny)
    library(shinyjs)
    library(shinyFiles)
    library(bslib)
    library(shinyBS)
    library(bsplus)
    library(htmltools)
    library(igraph)
    library(shinyWidgets)
    library(shinyalert)
    library(stringi)
    library(tidyverse)
    library(vroom)
    library(fs)
    library(english)
    library(data.table)
    library(argparse)
    library(dplyr)
    library(tools)
    library(seqinr)
    library(DT)
    library(ape)
})

## clean r session and temporary folder first
rm(list=ls())
## change upload file size limit to 5GB
options(shiny.maxRequestSize=5000*1024^2)

## source the necessary R script before deploying shinyWGD to the PSB server
# R_files <- list.files(path="R", pattern="\\.R$", full.names=TRUE)
# for( file in R_files ){
#     source(file)
# }

### load ui pages
source(file="ui/introduction.ui.R", local=T, encoding="UTF-8")
source(file="ui/data_preparation.ui.R", local=T, encoding="UTF-8")
source(file="ui/whale_preparation.ui.R", local=T, encoding="UTF-8")
source(file="ui/extracting_tree.ui.R", local=T, encoding="UTF-8")
source(file="ui/ks_age_distribution.ui.R", local=T, encoding="UTF-8")
source(file="ui/joint_tree.ui.R", local=T, encoding="UTF-8")
source(file="ui/synteny_analysis.ui.R", local=T, encoding="UTF-8")
source(file="ui/tree_reconciliation.ui.R", local=T, encoding="UTF-8")
source(file="ui/gallery.ui.R", local=T, encoding="UTF-8")
source(file="ui/help.ui.R", local=T, encoding="UTF-8")

ui <- tagList(
    includeScript("www/js/d3.min.js"),
    includeScript("www/js/iadhore.synteny.js"),
    includeScript("www/js/clustering.js"),
    includeScript("www/js/Ksdistribution.js"),
    includeScript("www/js/figtree.js"),
    includeScript("www/js/findOutgroup.js"),
    includeScript("https://unpkg.com/@popperjs/core@2"),
    includeScript("www/js/tippy-bundle.umd.v6.3.7.min.js"),
    includeScript("www/js/progress_bar.js"),
    includeScript("https://cdnjs.cloudflare.com/ajax/libs/seedrandom/3.0.5/seedrandom.min.js"),
    includeScript("https://cdn.jsdelivr.net/npm/sweetalert2@11"),
    includeScript("https://cdn.jsdelivr.net/npm/progressbar.js@1.1.0/dist/progressbar.min.js"),
    shinyjs::useShinyjs(),
    tags$head(
        tags$link(
            rel="stylesheet",
            href="custom.css"
        )
    ),
    navbarPage(
        id="shinywgd",
        theme=bs_theme(
            version=4,
            bootswatch="flatly",
            bg="#fffff4",
            fg="#336666",
            "input-border-color"="#feb24c"
        ),
        title=tags$img(
            src="images/sticker_server.png",
            width="80",
        ),
        windowTitle="shinyWGD",
        Introduction_ui,
        navbarMenu(
            "Setup",
            icon=icon("terminal"),
            Data_preparation_ui,
            Whale_Preparation_ui,
            # Codes_checking_ui,
            Extracting_Tree_ui,
        ),
        navbarMenu(
            "Analysis",
            icon=icon("pencil-alt"),
            Ks_Age_Distribution_ui,
            Synteny_Analysis_ui,
            Tree_Reconciliation_ui,
            Joint_Tree_ui
        ),
        Gallery_ui,
        Help_ui,
        #tabPanel(
        #    "Downlaod",
        #    icon=icon("download"),
        #    fluidRow(
        #        column(
        #            6,
        #            uiOutput("search_download_page")
        #        )
        #    )
        #)
    ),
    tags$head(
        tags$style(HTML("
            .my-start-button-class:hover {
                background-color: #2E8B57 !important;
            }
            .my-confirm-button-class:hover {
                background-color: #808080 !important;
            }
            .my-download-button-class:hover {
                background-color: #556B2F !important;
            }
        "))
    ),
    # add website for lab and UGent
    tags$a(href="https://www.vandepeerlab.org/", id="mylink1", target="_blank", rel="noopener",
           tags$img(
               src="images/logo_lab.png",
               width="222.50",
               height="41.95"
               )
    ),
    tags$head(
        tags$style(HTML("
            #mylink1 {
                position: absolute;
                top: 30px;
                right: 110px;
                z-index: 1;
            }
            .modal {
                z-index: 1050 !important;
            }
            .modal-backdrop {
                z-index: 1040 !important;
            }
        "))
    ),
    tags$a(href="https://www.ugent.be/en", id="mylink2", target="_blank", rel="noopener",
           tags$img(
               src="images/logo_ugent.png",
               width="100.65",
               height="81.64"
           )
    ),
    tags$head(
        tags$style(HTML("
            #mylink2 {
                position: absolute;
                top: 10px;
                right: 10px;
                z-index: 1;
            }
            .modal {
                z-index: 1050 !important;
            }
            .modal-backdrop {
                z-index: 1040 !important;
            }
        "))
    ),
    tags$script(HTML(
            'function openTab(tabClass) {
        Shiny.setInputValue("openTab", tabClass);
      }'
    ))
)

server <- function(input, output, session){
    observe({
        jsCode <- '
                  $(window).bind("beforeunload", function() {
                    Shiny.onInputChange("refreshClicked", new Date().getTime());
                  });
                '
        session$sendCustomMessage("jsCode", jsCode)
    })

    # Reload the app when the refresh button is clicked
    observeEvent(input$refreshClicked, {
        session$reload()
    })

    source(file="server/introduction.server.R", local=T, encoding="UTF-8")
    source(file="server/data_preparation.server.R", local=T, encoding="UTF-8")
    source(file="server/whale_preparation.server.R", local=T, encoding="UTF-8")
    source(file="server/extracting_tree.server.R", local=T, encoding="UTF-8")
    source(file="server/ks_age_distribution.server.R", local=T, encoding="UTF-8")
    source(file="server/joint_tree.server.R", local=T, encoding="UTF-8")
    source(file="server/synteny_analysis.server.R", local=T, encoding="UTF-8")
    source(file="server/tree_reconciliation.server.R", local=T, encoding="UTF-8")
    source(file="server/gallery.server.R", local=T, encoding="UTF-8")
    source(file="server/help.server.R", local=T, encoding="UTF-8")
}

shinyApp(ui, server, options=list(markdownTemplate="html_document"))

output$ObtainTreeFromTimeTreeSettingDisplay <- renderUI({
    div(class="boxLike",
        style="background-color: #F5FFE8;
               padding-left: 20px;
               padding-right: 20px;
               padding-top: 10px;",
        fluidRow(
            column(
                12,
                h4(
                    icon("tree", style="color: #64A600;"),
                    HTML("Extracting Tree from <font color='#64A600'><a href=\"http://www.timetree.org/\">TimeTree.org</a></font>")
                )
            )
        ),
        hr(class="setting"),
        fluidRow(
            column(
                12,
                HTML(
                    "If you don’t ensure the evolutionary relationships among the studied species, click the button below to obtain a tree from <a href=\"http://www.timetree.org/\">TimeTree.org</a>."
                )
            ),
            column(
                12,
                HTML("<br>Each row of this file contains a <font color='orange'>Species <b><i>Latin Name</i></b></font>.</br><br>"),
            ),
            fluidRow(
                column(
                    12,
                    div(
                        style="padding-left: 20px;
                               position: relative;",
                        fileInput(
                            'upload_species_name_list_file',
                            "Upload Species List File:",
                            multiple=FALSE,
                            width="80%",
                            accept=c(
                                ".txt",
                                ".xls"
                            )
                        ),
                        actionButton(
                            inputId="species_name_example",
                            "",
                            icon=icon("question"),
                            title="Click to see the example of species name file",
                            status="secondary",
                            class="my-start-button-class",
                            style="color: #fff;
                                   background-color: #87CEEB;
                                   border-color: #fff;
                                   position: absolute;
                                   top: 53%;
                                   left: 90%;
                                   margin-top: -15px;
                                   margin-left: -15px;
                                   padding: 5px 14px 5px 10px;
                                   width: 30px; height: 30px; border-radius: 50%;"
                        )
                    )
                )
            ),
            column(
                12,
                hr(class="setting"),
                actionButton(
                    inputId="extract_tree_go",
                    "Extract Tree",
                    width="100%",
                    icon=icon("tree"),
                    title="Click to start the process",
                    status="secondary",
                    class="my-start-button-class",
                    style="color: #fff;
                          background-color: #27ae60;
                          border-color: #fff;
                          padding: 5px 14px 5px 14px;
                          margin-bottom: 20px;"
                ),
                div(
                    id="extract_progress_container_js"
                )
            )
        )
    )
})

observeEvent(input$species_name_example, {
    species_name_file <- "www/content/species_name_example.xls"
    showModal(
        modalDialog(
            title="The example file",
            size="xl",
            uiOutput("speceisNameExamplePanel")
        )
    )

    output$speceisNameExampleTxt <- renderText({
        command_info <- readChar(
            species_name_file,
            file.info(species_name_file)$size
        )
    })

    output$speceisNameExamplePanel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    h5(
                        HTML(
                            paste0(
                                "Each row of this file contains a <font color='orange'>Species <b><i>Latin Name</i></b></font>"
                            )
                        )
                    ),
                    verbatimTextOutput(
                        "speceisNameExampleTxt",
                        placeholder=TRUE
                    )
                )
            )
        )
    })
})


output$timeTreeOrgPlot <- renderUI({
    div(class="boxLike",
        style="padding-top: 10px;
               padding-right: 50px;
               padding-left: 50px;
               background-color: white;",
        fluidRow(
            column(
                12,
                h4(HTML("The Tree Extracted from <font color='#64A600'> <a href=\"http://www.timetree.org/\">TimeTree.org</a>")),
            )
        ),
        hr(class="setting"),
        fluidRow(
            column(
                4,
                div(
                    style="margin: 10px 10px 0px 10px;
                           border-radius: 10px;
                           padding: 10px 10px 0px 10px;
                           background-color: #FFF5EE;",
                    prettyRadioButtons(
                        inputId="timetree_method",
                        label=HTML("<font color='orange'>Strategy used</font>:"),
                        choices=c("As timetree.org", "Median time"),
                        selected="As timetree.org",
                        icon=icon("check"),
                        inline=TRUE,
                        status="info",
                        animation="jelly"
                    )
                ),
            ),
            column(
                4,
                div(
                    style="text-align: center;
                           margin: 10px 10px 10px 10px;
                           border-radius: 10px;
                           padding: 10px 10px 10px 10px;
                           background-color: #FFF5EE;",
                    downloadButton_custom(
                        "download_nwk_tree",
                        title="Download the Tree in Newick",
                        status="secondary",
                        icon=icon("download"),
                        label=".nwk",
                        class="my-download-button-class",
                        style="color: #fff;
                               background-color: #6B8E23;
                               border-color: #fff;
                               padding: 5px 14px 5px 14px;
                               margin: 5px 5px 5px 5px;"
                    ),
                    downloadButton_custom(
                        "download_nexus_tree",
                        title="Download the Tree in Nexus",
                        status="secondary",
                        icon=icon("download"),
                        label=".nexus",
                        class="my-download-button-class",
                        style="color: #fff;
                               background-color: #6B8E23;
                               border-color: #fff;
                               padding: 5px 14px 5px 14px;
                               margin: 5px 5px 5px 5px;"
                    )
                )
            ),
            column(
                12,
                div(
                    style="margin: 10px 10px 10px 10px;
                           padding: 10px 10px 0px 10px;",
                    id="timetreeOrg_plot"
                )
            )
        ),
        column(
            12,
            uiOutput("species_info_timetreeOrg")
        ),
        h6(
            HTML("<font color='red'><b>Note</b></font> TimeTree database does not include all the species.
                 <br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;If the species are not included in the TimeTree database,
                 <br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;you should ensure the relationship in other ways."
            )
        )
    )
})

observeEvent(input$extract_tree_go, {
    if( !is.null(input$upload_species_name_list_file) ){
        shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")
        shinyjs::runjs('$("#timetreeOrg_plot").html("");')
        shinyjs::runjs('$("#species_info_timetreeOrg").html("");')

        progress_data <- list("actionbutton"="extract_tree_go",
                              "container"="extract_progress_container_js")
        session$sendCustomMessage(
            "Progress_Bar_Complete",
            progress_data
        )

        species_name_file <- input$upload_species_name_list_file$datapath
        prefix <- paste0(dirname(species_name_file), "/obtained")
        withProgress(message='Extracting in progress', value=0, {
            incProgress(amount=.1, message="Checking the upload file")
            updateProgress(
                container="extract_progress_container_js",
                width=10,
                type="Extracting tree"
            )

            species_data <- readLines(species_name_file)
            species_data <- str_trim(species_data, side="right")
            latin_name_pattern <- "^[A-Z][a-z]+ [a-z]+$"
            invalid_rows <- which(!grepl(latin_name_pattern, species_data))
            if( length(invalid_rows) == 0 ){
                incProgress(amount=.25, message="Contacting with timetree.org ...")
                updateProgress(
                    container="extract_progress_container_js",
                    width=25,
                    type="Extracting tree"
                )
                Sys.sleep(1)

                tryCatch(
                    withCallingHandlers(
                        species_info_df <- TimeTreeFecher(species_name_file, prefix)
                    ),
                    error=function(e) {
                        shinyalert(
                            "Oops!",
                            paste0(e$message, ". Fail to extract tree from Timetree.org. Please try other ways!"),
                            type="error"
                        )
                    }
                )

                incProgress(amount=.7, message="Drawing tree...")
                updateProgress(
                    container="extract_progress_container_js",
                    width=70,
                    type="Extracting tree"
                )
                Sys.sleep(1)

                median_time_newick_tree_file <- paste0(prefix, ".sum_median_time.nwk")
                if( file.exists(median_time_newick_tree_file) ){
                    widthSpacing <- reactiveValues(value=500)
                    heightSpacing <- reactiveValues(value=NULL)

                    speciesTree <- readLines(textConnection(readChar(median_time_newick_tree_file, file.info(median_time_newick_tree_file)$size)))
                    sp_count <- str_count(speciesTree, ":")
                    trunc_val <- as.numeric(sp_count) * 20
                    heightSpacing$value <- trunc_val[1]

                    species_tree_data <- list(
                        "width"=widthSpacing$value
                    )

                    observe({
                        if( isTruthy(input$timetree_method) ){
                            if( input$timetree_method == "As timetree.org"){
                                tree_nexus_file <- paste0(prefix, ".as_timetree.nexus")
                                tree_nwk_file <- paste0(prefix, ".as_timetree.nwk")
                                speciesTree <- readLines(textConnection(readChar(tree_nexus_file, file.info(tree_nexus_file)$size)))
                                closeAllConnections()

                                output$download_nexus_tree <- downloadHandler(
                                    filename=function(){
                                        "obtained.as_timetree.nexus"
                                    },
                                    content=function(file){
                                        file.copy(tree_nexus_file, file)
                                    }
                                )

                                output$download_nwk_tree <- downloadHandler(
                                    filename=function(){
                                        "obtained.as_timetree.nwk"
                                    },
                                    content=function(file){
                                        file.copy(tree_nwk_file, file)
                                    }
                                )
                            }
                            else{
                                tree_nexus_file <- paste0(prefix, ".sum_median_time.nexus")
                                tree_nwk_file <- paste0(prefix, ".sum_median_time.nwk")
                                speciesTree <- readLines(textConnection(readChar(tree_nexus_file, file.info(tree_nexus_file)$size)))
                                closeAllConnections()

                                output$download_nexus_tree <- downloadHandler(
                                    filename=function(){
                                        "obtained.sum_median_time.nexus"
                                    },
                                    content=function(file){
                                        file.copy(tree_nexus_file, file)
                                    }
                                )

                                output$download_nwk_tree <- downloadHandler(
                                    filename=function(){
                                        "obtained.sum_median_time.nwk"
                                    },
                                    content=function(file){
                                        file.copy(tree_nwk_file, file)
                                    }
                                )
                            }

                            species_tree_data[["timeTreeOrgTree"]] <- speciesTree
                            species_tree_data[["height"]] <- heightSpacing$value
                            session$sendCustomMessage("timeTreeOrgPlot", species_tree_data)
                        }
                    })
                }

                if( nrow(species_info_df) > 0 ){
                    output$species_info_timetreeOrg <- renderUI({
                        div(
                            style="padding-top: 10px;
                                   padding-right: 50px;
                                   padding-left: 50px;
                                   background-color: white;",
                            fluidRow(
                                column(
                                    12,
                                    hr(class="splitting")
                                ),
                                column(
                                    8,
                                    fluidRow(
                                        column(
                                            6,
                                            h4(HTML("<b><font color='#9B3A4D'>Species info</font></b>"))
                                        ),
                                        column(
                                            12,
                                            species_info_df %>%
                                                setNames(., colnames(.) %>% gsub("group_id", "Group id", .)) %>%
                                                setNames(., colnames(.) %>% gsub("species_list", "Included species", .)) %>%
                                                datatable(
                                                    rownames=FALSE
                                                ) %>%
                                                formatStyle(
                                                    "Included species",
                                                    fontStyle="italic"
                                                )
                                        ),
                                        "The species in the same group share the identical time recordes in TimeTree.org"
                                    )
                                )
                            )
                        )
                    })
                }

                updateProgress("extract_progress_container_js", 100, "Extracting tree")
                incProgress(amount=1)
                Sys.sleep(1)
                incProgress(amount=1)
            }
            else{
                shinyalert(
                    "Oops",
                    paste("Invalid Latin names in rows:", invalid_rows, collapse = ", "),
                    type="error"
                )
            }
        })
    }
    else{
        shinyalert(
            "Oops",
            "Please upload species name list first, then switch on this",
            type="error"
        )
    }
})

observeEvent(input$MCMC_tree_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b>Phylogenetic Tree</b></font> file"),
            size="xl",
            uiOutput("MCMC_tree_example_panel")
        )
    )

    output$MCMC_tree_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    h5(HTML("<br></br><font color='green'><b>Phylogenetic Tree</b></font> Example:"))
                ),
                column(
                    12,
                    verbatimTextOutput("UltrametricTreeExample"),
                    HTML(
                        paste0(
                            "The format of the tree is <b>Newick</b>. <br>",
                            "Users can click the text of the species name to change the color or add a symbol to the species in the <b>right Tree panel</b>. <br>",
                            "Users can click the branch to add the WGD events into the branch.<br></br>"
                        )
                    )
                ),
                hr(class="setting"),
                column(
                    12,
                    h5(HTML("<font color='green'><b>Time Tree</b></font> Example:"))
                ),
                column(
                    12,
                    verbatimTextOutput("TimeTreeExample"),
                    HTML(
                        paste0(
                            "The format of the time tree is in <b>Newick</b>, same as the output of <b>TimeTreeFetcher</b> of <b>shinyWGD</b>. <br>",
                            "Be careful with the time scale, please select the proper time scale in the <b>Time Tree Setting</b> panel.<br>",
                            "Users can click the text of the species name to change the color or add a symbol to the species in the <b>right Tree panel</b>. <br>",
                            "Users can click the branch to add the WGD events into the branch.<br></br>"
                        )
                    )
                ),
                hr(class="setting"),
                column(
                    12,
                    h5(HTML("<font color='green'><b>MCMCTree Tree</b></font> Example:"))
                ),
                column(
                    12,
                    verbatimTextOutput("MCMCTreeExample"),
                    HTML(
                        paste0(
                            "The format of the tree is the output file of <b>MCMCTree</b>, named <b>FigTree.tre</b>. <br>",
                            "Users can also upload a nexus tree with the divergence time information by revising the part after <b>\"=\"</b>. <br>",
                            "Be careful with the time time scale, please select the proper time scale in the <b>Time Tree Setting</b> panel.<br>",
                            "Users can click the text of the species name to change the color or add a symbol to the species in the <b>right Tree panel</b>. <br>",
                            "Users can click the branch to add the WGD events into the branch.<br></br>"
                        )
                    )
                )
            )
        )
    })

    output$MCMCTreeExample <- renderText({
        "#NEXUS
BEGIN TREES;

        UTREE 1=(((((Oryza_sativa: 0.976947, Ananas_comosus: 0.976947) [&95%HPD={0.926361, 1.05635}]: 0.182469, Elaeis_guineensis: 1.159417) [&95%HPD={1.05857, 1.26857}]: 0.118543, (Asparagus_officinalis: 1.122381, Phalaenopsis_equestris: 1.122381) [&95%HPD={1.00604, 1.20401}]: 0.155580) [&95%HPD={1.16136, 1.39443}]: 0.205808, (Spirodela_polyrhiza: 1.267766, Zostera_marina: 1.267766) [&95%HPD={1.06518, 1.44988}]: 0.216003) [&95%HPD={1.34808, 1.6164}]: 0.061015, Vitis_vinifera: 1.544783) [&95%HPD={1.429, 1.64351}];

END;"
    })

    output$UltrametricTreeExample <- renderText({
        "(((((Oryza_sativa, Ananas_comosus), Elaeis_guineensis), (Asparagus_officinalis, Phalaenopsis_equestris)), (Spirodela_polyrhiza, Zostera_marina)), Vitis_vinifera);"
    })

    output$TimeTreeExample <- renderText({
        "(Asparagus_officinalis:117.61,(Apostasia_shenzhenica:84.55,(Dendrobium_catenatum:52.11,Phalaenopsis_equestris:52.11):32.44):33.06);"
    })
})

observeEvent(input$wgd_time_table_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'>WGD Events Table</b></font> file"),
            size="xl",
            uiOutput("wgd_time_example_panel")
        )
    )

    wgd_time_data_file <- "www/content/wgd_time_table_example.xls"
    output$wgdTimeExampleTable <- renderTable({
        wgd_info_example <- read.table(
            wgd_time_data_file,
            header=FALSE,
            col.names=c("Species", "WGD events", "color"),
            sep="\t",
            quote="",
            comment.char=""
        )
        colnames(wgd_info_example) <- gsub("\\.", " ", colnames(wgd_info_example))

        wgd_info_example
    })

    output$wgd_time_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    tableOutput("wgdTimeExampleTable"),
                    HTML(
                        paste0(
                            "After uploading the file, the corresponding <b>WGD</b> events of each studied species will be placed into the tree.<br>"
                        )
                    )
                )
            )
        )
    })
})

ksTreeRv <- reactiveValues(data=NULL, clear=FALSE)
timeTreeRv <- reactiveValues(data=NULL, clear=FALSE)

observeEvent(input$uploadKsTree, {
    ksTreeRv$clear <- FALSE
}, priority=1000)

observeEvent(input$uploadTimeTree, {
    timeTreeRv$clear <- FALSE
}, priority=1000)

widthSpacing <- reactiveValues(value=500)
heightSpacing <- reactiveValues(value=NULL)

preDateWGDsRef <- reactiveValues(value=NULL)
observe({
    if( isTruthy(input$uploadTimeTree) ){
        timeTreeFile <- input$uploadTimeTree$datapath
        timeTreeInfo <- readLines(textConnection(readChar(timeTreeFile, file.info(timeTreeFile)$size)))
        closeAllConnections()
        if( any(grep("\\d", timeTreeInfo)) ){
            output$timeTreeSettingPanel <- renderUI({
                div(class="boxLike",
                    style="background-color: #F5FFFA;",
                    h5(icon("cog"), HTML("Time Tree Setting")),
                    hr(class="setting"),
                    fluidRow(
                        column(
                            12,
                            div(
                                style="padding-left: 10px;
                                       position: relative;",
                                awesomeRadio(
                                    inputId="vizTree_time_scale",
                                    label=HTML("<b>Time scale</b>:"),
                                    choices=c("100 MYA", "1 MYA"),
                                    selected="100 MYA",
                                    inline=TRUE
                                )
                            )
                        )
                    ),
                    fluidRow(
                        column(
                            12,
                            div(
                                style="padding-left: 10px;
                                   position: relative;",
                                fileInput(
                                    inputId="uploadTimeTable",
                                    label=HTML("<b>WGDs Time </b> File:"),
                                    width="80%",
                                    accept=c(".csv", ".txt", ".xls")
                                ),
                                actionButton(
                                    inputId="wgd_time_table_example",
                                    "",
                                    icon=icon("question"),
                                    status="secondary",
                                    title="Click to see the example of WGD time file",
                                    class="my-start-button-class",
                                    style="color: #fff;
                                           background-color: #87CEEB;
                                           border-color: #fff;
                                           position: absolute;
                                           top: 53%;
                                           left: 90%;
                                           margin-top: -15px;
                                           margin-left: -15px;
                                           padding: 5px 14px 5px 10px;
                                           width: 30px; height: 30px; border-radius: 50%;"
                                )
                            )
                        )
                    )
                )
            })
        }else{
            output$timeTreeSettingPanel <- renderUI({""})
        }

        if( any(grep("=", timeTreeInfo)) ){
            timeTree <- timeTreeInfo[grep("=", timeTreeInfo)]
            timeTree <- sub("^[^=]*=", "", timeTree)
            tree <- read.tree(text=timeTree)
            species_in_tree <- tree$tip.label
        }else{
            tree <- read.tree(timeTreeFile)
            species_in_tree <- tree$tip.label
        }

        if( length(species_in_tree) > 0 ){
            pre_dated_wgds <- suppressMessages(
                vroom(
                    file="www/content/validated_WGD_dates.info.xls",
                    delim="\t",
                    col_names=TRUE
                )
            )
            pre_dated_wgds$Species <- gsub(" ", "_", pre_dated_wgds$Species)
            filtered_pre_dated_wgds <- subset(pre_dated_wgds, Species %in% species_in_tree)
            filtered_pre_dated_wgds <- pre_dated_wgds[grepl(paste(species_in_tree, collapse="|"), pre_dated_wgds$Species), ]

            unique_dbs <- unique(filtered_pre_dated_wgds$Author)
            preDateWGDsRef$value <- length(unique_dbs)
            if( nrow(filtered_pre_dated_wgds) > 0 ){
                dated_wgd_ui_parts <- c()
                for( i in 1:length(unique_dbs) ){
                    tmp_dated_wgds <- filtered_pre_dated_wgds[filtered_pre_dated_wgds$Author == unique_dbs[i], ]
                    tmp_dated_wgds <- tmp_dated_wgds[order(tmp_dated_wgds$Species), ]
                    tmp_author <- unique(tmp_dated_wgds$Author)
                    tmp_DOI <- unique(tmp_dated_wgds$DOI)
                    tmp_dated_wgds_list <- sapply(1:nrow(tmp_dated_wgds), function(x) {
                        paste0(
                            "<i><b><font color='#DAA520'>",
                            gsub("_", " ", tmp_dated_wgds[x, "Species"]),
                            "</i></b></font><br>90% CI: <b><font color='#6B8E23'>",
                            tmp_dated_wgds[x, "90% CI"],
                            "</font></b>",
                            " MYA"
                        )
                    }, simplify="list")

                    dated_wgd_ui_parts[[i]] <- fluidRow(
                        column(
                            12,
                            HTML(
                                paste0(
                                    "Dated WGDs from <a href='", tmp_DOI, "' target='_blank'><b>",
                                    tmp_author, "</b></a>"
                                )
                            )
                        ),
                        column(
                            12,
                            # pickerInput(
                            #     inputId=paste0("dated_wgds_", i),
                            #     label=HTML("<font color='orange'>Choose WGDs</font>:&nbsp;"),
                            #     options=list(
                            #         title='Please select dated WGDs below'
                            #     ),
                            #     choices=tmp_dated_wgds_list,
                            #     choicesOpt=list(
                            #         content=lapply(tmp_dated_wgds_list, function(choice) {
                            #             HTML(choice)
                            #         })
                            #     ),
                            #     selected=NULL,
                            #     multiple=TRUE
                            # ),
                            div(
                                style="padding-top: 10px;
                                       padding-bottom: 10px;",
                                bsButton(
                                    inputId=paste0("dated_wgds_button_", i),
                                    label=HTML("<font color='white'>&nbsp;Dated WGDs&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&#x25BC;</font>"),
                                    icon=icon("list"),
                                    style="success"
                                ) %>%
                                    bs_embed_tooltip(
                                        title="Click to choose wgds",
                                        placement="right",
                                        trigger="hover",
                                        options=list(container="body")
                                    ) %>%
                                    bs_attach_collapse(paste0("dated_wgds_button_collapse_", i)),
                                bs_collapse(
                                    id=paste0("dated_wgds_button_collapse_", i),
                                    content=tags$div(
                                        class="well",
                                        checkboxGroupInput(
                                            inputId=paste0("pre_wgds_select_", i),
                                            label="",
                                            choices=tmp_dated_wgds_list,
                                            selected=FALSE
                                        ),
                                        tags$script(
                                            HTML(paste0("
                                                $(document).ready(function() {
                                                  $('#pre_wgds_select_", i, " .checkbox label span').map(function(){
                                                    this.innerHTML=$(this).text();
                                                  });
                                                });
                                              ")
                                            )
                                        ),
                                        colorPickr(
                                            inputId=paste0("select_pre_wgds_color_", i),
                                            label="Pick a color (swatches + opacity):",
                                            swatches=scales::viridis_pal()(10),
                                            opacity=TRUE
                                        ),
                                    )
                                )
                            )
                        )
                    )
                }

                output$preDatedWGDsSettingPanel <- renderUI({
                    div(class="boxLike",
                        style="background-color: #F5F5F5;",
                        h5(icon("book"), HTML("Dated WGDs from Literatures")),
                        hr(class="setting"),
                        dated_wgd_ui_parts
                    )
                })
            }else{
                output$preDatedWGDsSettingPanel <- renderUI({""})
            }
        }
    }
})

observe({
    if( isTruthy(input$uploadKsTree) || isTruthy(input$uploadTimeTree) ){
        if( isTruthy(input$uploadKsTree) ){
            ksTreeFile <- input$uploadKsTree$datapath
            ksTree <- readLines(textConnection(readChar(ksTreeFile, file.info(ksTreeFile)$size)))
            closeAllConnections()
            sp_count <- str_count(ksTree[1], ":")
        }
        if( isTruthy(input$uploadTimeTree) ){
            timeTreeFile <- input$uploadTimeTree$datapath
            timeTreeInfo <- readLines(textConnection(readChar(timeTreeFile, file.info(timeTreeFile)$size)))
            closeAllConnections()
            if( any(grep("=", timeTreeInfo)) ){
                timeTree <- timeTreeInfo[grep("=", timeTreeInfo)]
                sp_count <- str_count(timeTree, ":")
            }else{
                sp_count <- as.numeric(str_count(timeTreeInfo[1], ",")) * 2
            }
        }
        trunc_val <- as.numeric(sp_count) * 20
        heightSpacing$value <- trunc_val
    }
})

observeEvent(input$svg_vertical_spacing_add, {
    heightSpacing$value <- heightSpacing$value + 50
})
observeEvent(input$svg_vertical_spacing_sub, {
    heightSpacing$value <- heightSpacing$value - 50
})
observeEvent(input$svg_horizontal_spacing_add, {
    widthSpacing$value <- widthSpacing$value + 50
})
observeEvent(input$svg_horizontal_spacing_sub, {
    widthSpacing$value <- widthSpacing$value - 50
})

observe({
    if( isTruthy(input$uploadKsTree) || isTruthy(input$uploadTimeTree) ){
        joint_tree_data <- list(
            "width"=widthSpacing$value
        )
        if( isTruthy(input$uploadKsTree) ){
            ksTreeFile <- input$uploadKsTree$datapath
            ksTree <- readLines(textConnection(readChar(ksTreeFile, file.info(ksTreeFile)$size)))
            closeAllConnections()
            joint_tree_data[["ksTree"]] <- ksTree[1]
        }
        if( isTruthy(input$uploadTimeTree) ){
            timeTreeFile <- input$uploadTimeTree$datapath
            timeTreeInfo <- readLines(textConnection(readChar(timeTreeFile, file.info(timeTreeFile)$size)))
            closeAllConnections()
            if( any(grep("\\d", timeTreeInfo)) ){
                vizTreeTimeScale <- gsub(" MYA", "", input$vizTree_time_scale)
                if( any(grep("=", timeTreeInfo)) ){
                    timeTree <- timeTreeInfo[grep("=", timeTreeInfo)]
                    joint_tree_data[["timeScale"]] <- vizTreeTimeScale
                    joint_tree_data[["timeTree"]] <- timeTree
                }else{
                    joint_tree_data[["timeScale"]] <- vizTreeTimeScale
                    joint_tree_data[["timeTree"]] <- timeTreeInfo[1]
                }
                selected_pre_dated_wgd_df <- data.frame()
                if( preDateWGDsRef$value >0 ){
                    extract_info <- function(element) {
                        split_string <- unlist(strsplit(element, ">"))
                        species <- gsub("<.*", "", split_string[4])
                        if( vizTreeTimeScale == "1" ){
                            wgds_range <- gsub("</.*", "", split_string[10])
                            wgds <- as.numeric(unlist(strsplit(wgds_range, "-"))) / 100
                            wgds <- paste(wgds, collapse="-")
                        }else{
                            wgds <- gsub("</.*", "", split_string[10])
                        }
                        return(data.frame(species=species, wgds=wgds, stringsAsFactors=FALSE))
                    }
                    for( i in 1:preDateWGDsRef$value ){
                        tmp_pre_id <- paste0("pre_wgds_select_", i)
                        tmp_pre_color <- paste0("select_pre_wgds_color_", i)
                        each_wgd_df <- do.call(rbind, lapply(input[[tmp_pre_id]], extract_info))
                        if( !is.null(each_wgd_df) && ncol(each_wgd_df) > 1 ){
                            each_wgd_df$color <- input[[tmp_pre_color]]
                            selected_pre_dated_wgd_df <- rbind(
                                selected_pre_dated_wgd_df,
                                each_wgd_df
                            )
                        }
                    }
                }
                if( isTruthy(input$uploadTimeTable) ){
                    timeTableFile <- input$uploadTimeTable$datapath
                    timeTable <- suppressMessages(
                        vroom(
                            timeTableFile,
                            col_names=c("species", "wgds", "color"),
                            delim="\t"
                        )
                    )
                    if( ncol(selected_pre_dated_wgd_df) > 1 ){
                        selected_pre_dated_wgd_df <- rbind(
                            selected_pre_dated_wgd_df,
                            timeTable
                        )
                    }else{
                        selected_pre_dated_wgd_df <- timeTable
                    }
                }
                if( ncol(selected_pre_dated_wgd_df) > 1 ){
                    joint_tree_data[["wgdtable"]] <- selected_pre_dated_wgd_df
                }
            }else{
                joint_tree_data[["ultrametricTree"]] <- timeTreeInfo[1]
            }
        }
        if( isTruthy(input$uploadKsPeakTable) ){
            ksPeakTableFile <- input$uploadKsPeakTable$datapath
            ksPeak <- suppressMessages(
                vroom(
                    ksPeakTableFile,
                    col_names=c("species", "type", "peak", "confidence_interval"),
                    delim="\t",
                    skip=1
                )
            )
            if( ncol(ksPeak) == 4 ){
                color_list <- c(
                    "#ff7f00", "#FFA750", "#0064A7", "#008DEC",
                    "#088A00", "#0CD300", "#e31a1c", "#fb9a99", "#cab2d6"
                )
                ksPeak$color <- color_list[match(ksPeak$species, unique(ksPeak$species))]
            }
            joint_tree_data[["ksPeak"]] <- ksPeak
        }

        joint_tree_data[["height"]] <- heightSpacing$value
        joint_tree_data[["plot_id"]] <- "jointtree_plot"
        joint_tree_data[["download_id"]] <- "jointTreePlotDownload"
        session$sendCustomMessage("jointTreePlot", joint_tree_data)
    }
})

Gallery_ui <- tabPanel(
    HTML("Gallery"),
    value='gallery_page',
    fluidRow(
        column(
            12,
            #uiOutput("galleryPanel")
			HTML('<iframe src="https://li081766.github.io/shinyWGD_Demo_Data/gallery.html" style="width:100%; height:800px; border:none;"></iframe>')
        )
    ),
    icon=icon("image")
)

#' Obtain Coordinates and Ks Values for Anchorpoints
#'
#' This function extracts coordinates and Ks (synonymous substitution rate) values for anchorpoints
#' from input data and merges them into a single output file.
#
#' @param anchorpoints A character string specifying the file path for anchorpoints data.
#' @param anchorpoints_ks A character string specifying the file path for anchorpoints Ks values data.
#' @param genes_file A character string specifying the file path for genes information.
#' @param out_file A character string specifying the output file path for coordinates.
#' @param out_ks_file A character string specifying the output file path for Ks values.
#' @param species A character string specifying the species name.
#'
#' @importFrom dplyr select
#' @importFrom dplyr rowwise
#' @importFrom dplyr distinct
#' @importFrom vroom vroom
#'
#' @return NULL (output files are generated with the specified information).
#'
obtain_coordiantes_for_anchorpoints_ks <- function(
        anchorpoints,
        anchorpoints_ks,
        genes_file,
        out_file,
        out_ks_file,
        species){
    # library(vroom)
    # library(dplyr)

    id <- genome <- list <- orientation <- remapped_coordinate <- NULL
    multiplicon <- NULL

    position_df <- suppressMessages(
        vroom(
            genes_file,
            delim="\t",
            comment="#",
            col_names=TRUE
        )
    ) %>%
        select(id, genome, list, orientation, remapped_coordinate)

    colnames(position_df) <- c("gene", "genome", "list", "strand", "coord")

    anchors_df <- suppressMessages(
        vroom(
            anchorpoints,
            delim="\t",
            col_names=TRUE
        )
    )

    gene_x <- genome.x <- list.x <- strand.x <- coord.x <- NULL
    gene_y <- genome.y <- list.y <- strand.y <- coord.y <- NULL

    merged_x <- left_join(anchors_df, position_df, by=c("gene_x"="gene"))
    final_df <- left_join(merged_x, position_df, by=c("gene_y"="gene")) %>%
        select(multiplicon,
               gene_x, genome.x, list.x, strand.x, coord.x,
               gene_y, genome.y, list.y, strand.y, coord.y)

    colnames(final_df) <- c("multiplicon",
                            "geneX", "speciesX", "listX", "strandX", "coordX",
                            "geneY", "speciesY", "listY", "strandY", "coordY")

    genome_list <- unique(position_df$genome)
    write.table(
        final_df,
        file=out_file,
        sep="\t",
        row.names=FALSE,
        quote=FALSE
    )

    speciesX <- speciesY <- NULL

    if( length(genome_list) > 1 ){
        final_df <- final_df %>%
            filter(speciesX != speciesY)

        final_df_out <- data.frame(
            matrix(
                ncol=ncol(final_df),
                nrow=nrow(final_df)
            )
        )

        for( i in 1:nrow(final_df) ){
            each_row <- final_df[i, ]
            if( each_row$speciesX != species ){
                tmp1 <- each_row[2:6]
                each_row[2:6] <- each_row[7:11]
                each_row[7:11] <- tmp1
            }
            final_df_out[i, ] <- each_row
        }
        colnames(final_df_out) <- colnames(final_df)
        final_df <- final_df_out
    }

    geneX <- geneY <- gene_pair <- NULL

    anchors_ks_df <- suppressMessages(
        vroom(
            anchorpoints_ks,
            delim="\t",
            col_names=TRUE
        )
    ) %>%
        rowwise() %>%
        mutate(gene_pair=paste(sort(c(geneX, geneY)), collapse="_")) %>%
        distinct(gene_pair, .keep_all=TRUE) %>%
        select(-gene_pair)

    Omega <- Ka <- Ks <- NULL

    merged_x <- left_join(anchors_ks_df, position_df, by=c("geneX"="gene"))
    anchors_ks_df <- left_join(merged_x, position_df, by=c("geneY"="gene")) %>%
        select(geneX, genome.x, list.x, strand.x, coord.x,
               geneY, genome.y, list.y, strand.y, coord.y,
               Omega, Ka, Ks)

    if( length(genome_list) > 1 ){
        anchors_ks_df <- anchors_ks_df %>%
            filter(genome.x != genome.y)

        final_df_out <- data.frame(
            matrix(
                ncol=ncol(anchors_ks_df),
                nrow=nrow(anchors_ks_df)
            )
        )

        for( i in 1:nrow(anchors_ks_df) ){
            each_row <- anchors_ks_df[i, ]
            if( each_row$genome.x != species ){
                tmp1 <- each_row[1:5]
                each_row[1:5] <- each_row[6:10]
                each_row[6:10] <- tmp1
            }
            final_df_out[i, ] <- each_row
        }
        colnames(final_df_out) <- colnames(anchors_ks_df)
        final_anchors_ks_df <- final_df_out
    }else{
        final_anchors_ks_df <- anchors_ks_df
    }

    final_anchors_ks_df <- final_anchors_ks_df %>%
        select(geneX, geneY, Omega, Ka, Ks)

    listX <- strandX <- coordX <- NULL
    listY <- strandY <- coordY <- NULL

    final_table_ks_df <- merge(
        final_df,
        final_anchors_ks_df,
        by.x=c("geneX", "geneY"),
        by.y=c("geneX", "geneY"),
        all.x=TRUE
    ) %>%
        select(multiplicon,
               geneX, speciesX, listX, strandX, coordX,
               geneY, speciesY, listY, strandY, coordY,
               Omega, Ka, Ks) %>%
        arrange(multiplicon)

    final_table_ks_df[is.na(final_table_ks_df)] <- -1

    write.table(final_table_ks_df,
                file=out_ks_file,
                sep="\t",
                row.names=FALSE,
                quote=FALSE)
}

#' Obtain Coordinates for Segments in Multiple Synteny Blocks
#'
#' This function extracts coordinates for segments within multiple synteny blocks based on input dataframes.
# It merges information from GFF and segment dataframes to generate a final output.
#
#' @param seg_df A dataframe containing information about synteny segments.
#' @param gff_df A dataframe containing GFF (General Feature Format) information.
#' @param input A list containing input data, typically multiple synteny query chromosomes.
#' @param out_file A character string specifying the output file path.
#'
#' @importFrom vroom vroom
#' @importFrom dplyr filter
#' @importFrom dplyr mutate
#' @importFrom dplyr select
#'
#' @return A dataframe with coordinates for segments within multiple synteny blocks.
#'
obtain_coordinates_for_segments_multiple <- function(
        seg_df,
        gff_df,
        input,
        out_file
    ){
    # library(vroom)
    # library(dplyr)

    X1 <- X4 <- X5 <- X7 <- X9 <- NULL

    position_df <- data.frame()
    for( x in 1:nrow(gff_df) ){
        each_row <- gff_df[x, ]
        selected_chrs <- input[[paste0("multiple_synteny_query_chr_", gsub(" ", "_", each_row$species))]]
        gff_df_tmp <- suppressMessages(
            vroom(
                each_row$gffPath,
                delim="\t",
                comment="#",
                col_names=FALSE
            )
        ) %>%
            filter(X1 %in% selected_chrs)
        gff_df_tmp <- gff_df_tmp %>%
            filter(gff_df_tmp$X3 == "mRNA") %>%
            select(X1, X9, X4, X5, X7) %>%
            mutate(X9=gsub("ID=([^;]+).*", "\\1", X9))
        colnames(gff_df_tmp) <- c("list", "gene", "start", "end", "strand")
        gff_df_tmp$sp <- each_row$species
        position_df <- rbind(position_df, gff_df_tmp)
    }

    gene <- start <- end <- multiplicon <- genome <- list <- first <- last <- NULL
    genome.1 <- list.1 <- first.1 <- last.1 <- start.1 <- end.1 <- order.1 <- NULL
    genomeX <- genomeY <- NULL

    segs <- seg_df
    start_subset <- select(position_df, gene, start)
    merged_data <- left_join(segs, start_subset, by=c("first"="gene"))
    end_subset <- select(position_df, gene, end)
    merged_data <- left_join(merged_data, end_subset, by=c("last"="gene"))

    multiplicon_list <- unique(merged_data$multiplicon)
    final_df <- data.frame()
    for( i in multiplicon_list ){
        df_subset <- merged_data[merged_data$multiplicon==i, ]
        for( x in 1:nrow(df_subset) ){
            df_row1 <- df_subset[x, ]
            for( y in 1:nrow(df_subset) ){
                if( y > x ){
                    df_row2 <- df_subset[y, ]
                    new_row <- c(df_row1, df_row2)
                    final_df <- rbind(final_df, new_row)
                }
            }
        }
    }
    final_output <- select(final_df, multiplicon,
                           genome, list, first, last, start, end, min, max, order,
                           genome.1, list.1, first.1, last.1, start.1, end.1, min.1, max.1, order.1)
    colnames(final_output) <- c("multiplicon", "genomeX",
                                "listX", "firstX", "lastX", "startX", "endX", "coordStartX", "coordEndX", "orderX",
                                "genomeY", "listY", "firstY", "lastY", "startY", "endY", "coordStartY", "coordEndY", "orderY")
    rownames(final_output) <- NULL
    final_output1 <- subset(final_output, genomeX != genomeY)
    write.table(
        final_output1,
        file=out_file,
        sep="\t",
        row.names=FALSE,
        quote=FALSE
    )
}

Data_preparation_ui <- tabPanel(
    "Data Preparation",
    value="data_preparation",
    fluidRow(
        tags$head(
            tags$link(rel="stylesheet", type="text/css", href="sand_clock_loader.css"),
        ),
        tags$head(
            tags$style(
                HTML(
                    "
                    #progress_modal {
                        display: none;
                        position: fixed;
                        width: 100%;
                        height: 100%;
                        background-color: none;
                        left: 50%;
                        top: 50%;
                        transform: translate(-50%, -50%);
                        align-items: center;
                        justify-content: center;
                        z-index: 2002;
                    }

                    #progress_container_parent {
                        background-color: rgba(128, 128, 128, 0.1);
                        width: 100%;
                        height: 100%;
                        border: none;
                        align-items: center;
                        justify-content: center;
                        z-index: 2008;
                    }
                    "
                )
            )
        ),
        fluidRow(
            div(
                id="progress_modal",
                div(
                    id="progress_container_parent",
                    class="modal-content",
                    includeHTML("www/sand_clock_loader.html"),
                )
            )
        )
    ),
    fluidRow(
        column(
            id="SettingPanel",
            width=6,
            div(class="boxLike",
                style="background-color: #FAF9F6;",
                h4(icon("upload"), "Data Uploading"),
                hr(class="setting"),
                fluidRow(
                    column(
                        12,
                        numericInput(
                            inputId="number_of_study_species",
                            label="Select Number of Species to Analyze:",
                            min=1,
                            value=1
                        )
                    )
                ),
                hr(class="setting"),
                fluidRow(
                    column(
                        12,
                        h5(
                            tags$i(
                                class="fa fa-star",
                                style="color: #e17f00"
                            ),
                            HTML(paste0("If the analysis contains more than <font color='#e17f00'><b>three species</font></b>, ",
                                        "we recommend using <font color='green'><b>a tab-separated file</b></font> to manage data. ")
                            )
                        ),
                        h5(
                            tags$i(
                                class="fa fa-star",
                                style="color: #e17f00"
                            ),
                            HTML("Please switch to the <font color='green'><b><a href='#' onclick='switchToHelpTab()'>?Help</a></font> </b> page to see detail..."),

                            tags$script("
                              function switchToHelpTab() {
                                Shiny.setInputValue('switchTab', 'help', {priority: 'event'});
                              }
                            ")
                        )
                    )
                ),
                hr(class="setting"),
                fluidRow(
                    column(
                        5,
                        fileInput(
                            'upload_data_file',
                            HTML("Upload <font color='green'><b>a Tab-Separated</b></font> File:"),
                            multiple=FALSE,
                            width="100%",
                            accept=c(
                                ".txt",
                                ".xls"
                            )
                        )
                    ),
                    column(
                        1,
                        actionButton(
                            inputId="upload_data_file_example",
                            "",
                            icon=icon("question"),
                            status="secondary",
                            class="my-start-button-class",
                            title="Click to see the example of the Tab-Separated File",
                            style="text-align: left;
                                   color: #fff;
                                   background-color: #87CEEB;
                                   border-color: #fff;
                                   padding: 5px 14px 5px 10px;
                                   margin: 33px 5px 5px -15px;
                                   width: 30px; height: 30px; border-radius: 50%;"
                        )
                    ),
                    column(
                        6,
                        fileInput(
                            'selected_data_files',
                            HTML("Upload <font color='green'><b>CDS / Annotation </b></font> Files:"),
                            multiple=TRUE,
                            accept=c(
                                ".gff",
                                ".gff3",
                                ".gff.gz",
                                ".gff3.gz",
                                ".fa",
                                ".fas",
                                ".fasta",
                                ".fa.gz",
                                ".fas.gz",
                                ".fasta.gz",
                                ".gz"
                            )
                        )
                    )
                ),
                hr(class="setting"),
                column(
                    12,
                    h4(HTML("<font color='#e17f00'><b><i>OR</i></b></font>"))
                ),
                column(
                    12,
                    uiOutput("UploadDisplay")
                )
            )
        ),
        column(
            id="WgdksratesSettingPanel",
            width=6,
            div(class="boxLike",
                style="background-color: #FFFFF4;",
                h4(icon("cog"), "Setting"),
                hr(class="setting"),
                fluidRow(
                    column(
                        12,
                        uiOutput("WgdksratesSettingDisplay"),
                        hr(class="setting"),
                        uiOutput("WgdKsratesIadhoreDataDownload")
                    )
                )
            ),
            # This div container only works for a remote server mode
#             div(class="boxLike",
#                 style="background-color: #FFFFF4;",
#                 h4(icon("play"), "Script Executing"),
#                 hr(class="setting"),
#                 fluidRow(
#                     column(
#                         12,
#                         div(
#                             style="padding-left: 5px;",
# 							  HTML("The scripts will be submited in the <b><font color='#9F5000'>PSB computing cluster</b></font> if you click the sumbit button below. A <b>unique identifier</b> will be generated."),
#                     		  column(
#                         	  	  12,
#                         		  uiOutput("WgdKsratesIadhoreScriptRun")
#                     		  )
#                         )
#                     ),
#                     column(
#                         12,
# 						hr(class="setting"),
# 						h5(HTML("<b>OR</b>")),
# 						HTML("You can put you email address below. A confirmation email will be sent to the provided email address when the job is done"),
#                         div(
#                             style="padding-top: 10px;
#                                    padding-bottom: 10px;",
#                             fluidRow(
#                                 column(
#                                     5,
#                                     textInput(
#                                         inputId="users_email_address",
#                                         label=HTML("<b>Your email</b>:&nbsp;&nbsp;"),
#                                         value="",
#                                         width="100%",
#                                         placeholder="your_name@mail.com"
#                                     )
#                                 ),
#                                 column(
#                                     1,
#                                     actionButton(
#                                         inputId="comfirm_email",
#                                         "",
#                                         width="40px",
#                                         icon=icon("check"),
#                                         status="secondary",
#                                         class="my-start-button-class",
#                                         style="color: #fff;
#                                                background-color: #8080C0;
#                                                border-color: #fff;
#                                                margin: 30px 0px 0px 0px;"
#                                     )
#                                 )
#                             )
#                         )
#                     )
#                 )
#             ),
            # div(class="boxLike",
            #     style="background-color: #FFFFF4;",
            #     h4(icon("download"), "Data Downloading"),
            #     hr(class="setting"),
            #     fluidRow(
            #         column(
            #             12,
            #             uiOutput("WgdKsratesIadhoreDataDownload")
            #         )
            #     )
            # )
        )
    ),
    icon=icon("microscope")
)

#' Compute the Mean of Ks values for Each Multiplicon
#'
#' This function takes as input a multiplicon file, an anchorpoint file, Ks values, and other relevant information.
#' It calculates the mean of Ks values for each multiplicon and associates them with the corresponding data.
#'
#' @param multiplicon_file A file containing multiplicon information.
#' @param anchorpoint_file A file containing anchorpoints information with columns like geneX, geneY, and other relevant data.
#' @param species1 The name of the first species.
#' @param ks_file A file containing Ks values.
#' @param outfile The output file where the results will be saved.
#' @param anchorpointout_file The output file for anchorpoint data with Ks values.
#' @param species2 (Optional) The name of the second species. Specify this parameter and ks_file if working with two species.
#'
#' @importFrom vroom vroom
#' @importFrom stats median
#' @importFrom tidyr fill
#'
#' @return None. The function saves the results to the specified outfile and anchorpointout_file.
#'
obtain_mean_ks_for_each_multiplicon <- function(multiplicon_file, anchorpoint_file, species1, ks_file, outfile, anchorpointout_file, species2=NULL){
    multiplicons <- suppressMessages(
        vroom(
            multiplicon_file,
            col_names=TRUE,
            delim="\t"
        )
    )
    anchorpoints <- suppressMessages(
        vroom(
            anchorpoint_file,
            col_names=TRUE,
            delim="\t"
        )
    )

    Ks <- genome_x <- list_x <- genome_y <- NULL

    if( !is.null(species2) ){
        ks_value_df <- suppressMessages(
            vroom(
                ks_file,
                col_names=TRUE,
                delim="\t"
            ) %>%
                mutate(
                    pair1=paste(geneX, "VS", geneY, sep="_"),
                    pair2=paste(geneY, "VS", geneX, sep="_")
                )
        )

        colnames(ks_value_df) <- c("geneA", "geneB", "Omega", "Ka", "Ks", "pair1", "pair2")

        # ks_updateX_df <- left_join(
        #     ks_value_df,
        #     genes_df,
        #     by=c("geneX"="id")
        # ) %>%
        #     select(geneX, geneY, Omega, Ka, Ks, genome)
        # colnames(ks_updateX_df) <- c("geneX", "geneY", "Omega", "Ka", "Ks", "genomeX")
        #
        # ks_updateXY_df <- left_join(
        #     ks_updateX_df,
        #     genes_df,
        #     by=c("geneY"="id")
        # ) %>%
        #     select(geneX, geneY, Omega, Ka, Ks, genomeX, genome)
        # colnames(ks_updateXY_df) <- c("geneX", "geneY", "Omega", "Ka", "Ks", "genomeX", "genomeY")
        #
        # print(ks_updateXY_df)
        #
        # ks_updateXY_df <- ks_updateXY_df %>%
        #     rowwise() %>%
        #     filter(genomeX != genomeY)
        #
        # t1_df <- ks_updateXY_df$genomeX != species1
        # ks_updateXY_df[t1_df, c("geneX", "geneY")] <- ks_updateXY_df[t1_df, c("geneY", "geneX")]
        # ks_updateXY_df[t1_df, c("genomeX", "genomeY")] <- ks_updateXY_df[t1_df, c("genomeY", "genomeX")]
        #
        # ks_updateXY_df <- select(ks_updateXY_df, geneX, geneY, Omega, Ka, Ks)
        #
        # merged_ks <- left_join(
        #     anchorpoints,
        #     ks_updateXY_df,
        #     by=c("geneX"="geneX", "geneY"="geneY"),
        #     multiple="all") %>%
        #     filter(!is.na(Ks))
        #
        # if( nrow(merged_ks) == 0 ){
        #     merged_ks <- left_join(
        #         anchorpoints,
        #         ks_updateXY_df,
        #         by=c("geneY"="geneX",
        #              "geneX"="geneY"),
        #         multiple="all") %>%
        #         filter(!is.na(Ks))
        # }

        # colnames(ks_value_df) <- c("geneA", "geneB", "Omega", "Ka", "Ks")

        # merged_ks <- merge(
        #     anchorpoints,
        #     ks_value_df,
        #     by.x=c("geneX", "geneY"),
        #     by.y=c("geneA", "geneB")
        # )

        # merged_ks <- data.frame()
        # for( i in 1:nrow(anchorpoints) ){
        #     each_row <- anchorpoints[i, ]
        #     tmp_geneX <- each_row$geneX
        #     tmp_geneY <- each_row$geneY
        #     tmp_pair1 <- paste0(tmp_geneX, "_VS_", tmp_geneY)
        #     tmp_pair2 <- paste0(tmp_geneY, "_VS_", tmp_geneX)
        #     pair_list <- c(tmp_pair1, tmp_pair2)
        #
        #     tmp_ks_df <- ks_value_df %>%
        #         filter(pair1 %in% pair_list | pair2 %in% pair_list)
        #     if( nrow(tmp_ks_df) > 0 ){
        #         tmp_df <- bind_rows(each_row, tmp_ks_df) %>%
        #             select(-pair1, -pair2)
        #         merged_ks <- bind_rows(merged_ks, tmp_df)
        #     }
        # }
        anchorpoints <- anchorpoints %>%
            mutate(
                tmp_pair1=paste0(geneX, "_VS_", geneY),
                tmp_pair2=paste0(geneY, "_VS_", geneX)
            )

        # Merge data frames
        merged_ks <- bind_rows(
            anchorpoints %>%
                left_join(
                    ks_value_df,
                    by=c("tmp_pair1"="pair1"),
                    multiple="all"
                ),

            anchorpoints %>%
                left_join(
                    ks_value_df,
                    by=c("tmp_pair2"="pair1"),
                    multiple="all"
                ),

            anchorpoints %>%
                left_join(
                    ks_value_df,
                    by=c("tmp_pair1"="pair2"),
                    multiple="all"
                ),

            anchorpoints %>%
                left_join(
                    ks_value_df,
                    by=c("tmp_pair2"="pair2"),
                    multiple="all"
                )
        ) %>%
            filter(!is.na(Ks)) %>%
            select(-tmp_pair1, -tmp_pair2, -geneA, -geneB, -pair1, -pair2) %>%
            distinct()

        merged_ks <- subset(merged_ks, Ks<=5)
        merged_ks <- merged_ks[merged_ks$speciesX != merged_ks$speciesY, ]

        final_merged_ks <- data.frame(matrix(ncol=ncol(merged_ks), nrow=nrow(merged_ks)))
        for( i in 1:nrow(merged_ks) ){
            each_row <- merged_ks[i, ]
            if( each_row$speciesX != species1 ){
                x_columns <- grep("X", names(each_row))
                y_columns <- gsub("X", "Y", names(each_row[x_columns]))

                temp_values <- each_row[x_columns]
                each_row[x_columns] <- each_row[y_columns]
                each_row[y_columns] <- temp_values
            }
            final_merged_ks[i,] <- each_row
        }
        names(final_merged_ks) <- names(merged_ks)

        mean_ks_for_each_multiplicon <- aggregate(
            Ks ~ multiplicon,
            data=merged_ks,
            FUN=function(x) median(x, na.rm=TRUE)
        )

        multiplicons_filled <- fill(multiplicons, genome_x, list_x, .direction="down") # %>%
            # filter(genome_x != genome_y)

        # final_multiplicons <- data.frame(matrix(ncol=ncol(multiplicons_filled), nrow=nrow(multiplicons_filled)))
        # for( i in 1:nrow(multiplicons_filled) ){
        #     each_row <- multiplicons_filled[i, ]
        #     each_row$genome_x <- gsub("_", " ", each_row$genome_x)
        #     if( each_row$genome_x != species1 ){
        #         tmp1 <- each_row[2:3]
        #         each_row[2:3] <- each_row[5:6]
        #         each_row[5:6] <- tmp1
        #
        #         tmp2 <- each_row[10:11]
        #         each_row[10:11] <- each_row[12:13]
        #         each_row[12:13] <- tmp2
        #     }
        #
        #     final_multiplicons[i,] <- each_row
        # }
        colnames(multiplicons_filled) <- c("multiplicon", "genomeX", "listX", "parent", "genomeY", "listY", "level",
                                          "num_anchorpoints", "profile_len", "startX", "endX", "startY", "endY", "is_redundant")

        merged_ks_multiplicons <- left_join(
            multiplicons_filled,
            mean_ks_for_each_multiplicon,
            by="multiplicon"
        )
        merged_ks_multiplicons$Ks[is.na(merged_ks_multiplicons$Ks)] <- 0
    }
    else{
        ks_value_df <- suppressMessages(
            vroom(ks_file,
                  col_names=TRUE,
                  delim="\t")
        )

        # merge ks value for each anchor pair
        merged_ks <- left_join(
            anchorpoints,
            ks_value_df,
            by=c("geneX"="geneX",
                 "geneY"="geneY"),
            multiple="all") %>%
            filter(!is.na(Ks))
        if( nrow(merged_ks) == 0 ){
            merged_ks <- left_join(
                anchorpoints,
                ks_value_df,
                by=c("geneY"="geneX",
                     "geneX"="geneY"),
                multiple="all") %>%
                filter(!is.na(Ks))
        }
        merged_ks <- subset(merged_ks, Ks<=5)

        mean_ks_for_each_multiplicon <- aggregate(
            Ks ~ multiplicon,
            data=merged_ks,
            FUN=function(x) median(x, na.rm=TRUE)
        )

        multiplicons_filled <- fill(multiplicons, genome_x, list_x, .direction="down")

        final_multiplicons <- multiplicons_filled
        # for (i in 1:nrow(multiplicons_filled)) {
        #     each_row <- multiplicons_filled[i, ]
        #     tmp1 <- each_row[2:3]
        #     each_row[2:3] <- each_row[5:6]
        #     each_row[5:6] <- tmp1
        #
        #     tmp2 <- each_row[10:11]
        #     each_row[10:11] <- each_row[12:13]
        #     each_row[12:13] <- tmp2
        #     final_multiplicons <- rbind(final_multiplicons, each_row)
        # }
        colnames(final_multiplicons) <- c("multiplicon", "genomeX", "listX", "parent", "genomeY", "listY", "level",
                                          "num_anchorpoints", "profile_len", "startX", "endX", "startY", "endY", "is_redundant")
        merged_ks_multiplicons <- left_join(
            final_multiplicons,
            mean_ks_for_each_multiplicon,
            by="multiplicon"
        )
        merged_ks_multiplicons$Ks[is.na(merged_ks_multiplicons$Ks)] <- 0
        final_merged_ks <- copy(merged_ks)
    }
    final_merged_ks <- final_merged_ks %>%
        distinct()
    write.table(final_merged_ks,
                file=anchorpointout_file,
                sep="\t",
                row.names=FALSE,
                quote=FALSE)
    write.table(merged_ks_multiplicons,
                file=outfile,
                sep="\t",
                row.names=FALSE,
                quote=FALSE)
}

library(seqinr)
library(argparse)

parser <- argparse::ArgumentParser()
parser$add_argument("-i", "--input_file", help="Path to the input file", required=TRUE)
parser$add_argument("-o", "--output_file", help="Path to the output file", required=TRUE)

args <- parser$parse_args()

cds_sequences <- read.fasta(args$input_file)
proteins <- sapply(cds_sequences, function(seq) {
    translated_seq <- translate(seq)
    if (tail(translated_seq, 1) == "*") {
        translated_seq <- head(translated_seq, -1)
    }
    return(translated_seq)
})

write.fasta(
    sequences=proteins,
    names=names(proteins),
    file.out=args$output_file
)

#' A wrapper to run EM analysis of \(ln\) Ks values with k-means
#'
#' @param v A list include a vector of Ks values namely `ks_value`,
#'   and a bolean variable namely `log`
#' @param k.centers Number of k-means centers, default 2.
#' @param k.nstart Number of random start of k-means clustering, default 10.
#'   For a formal analysis, it is recommended to use 500.
#'
#' @importFrom stats kmeans
#' @importFrom stats var
#'
#' @return A list, i.e., the original output of mclust::emV
#'
run_emmix_kmeas <- function(v, k.centers=2, k.nstart=500) {
  if (v$log == FALSE) {
    warning("Better to use the log-normal distribution")
  }
  n <- length(v$ks_value)
  suppressMessages(
      k_fit <- kmeans(v$ks_value, centers=k.centers, nstart=k.nstart,
                      algorithm="Lloyd", iter.max=1000)
  )
  # prepare data as a data frame for mclust::em
  k_data <- data.frame(v=v$ks_value, cluster=k_fit$cluster)

  cluster <- NULL
  # prepare parameters as a list for mclust::em
  df <- k_data %>%
    dplyr::group_by(cluster) %>%
    dplyr::summarise(sigmasq=var(v))

    #dplyr::summarise_at(dplyr::vars(v), list(sigmasq=var))

  var.list <- list(modelName="V",
                   d=1, # dimension
                   G=k.centers,
                   sigmasq=df$sigmasq)

  par.list <- list(pro=k_fit$size / n,
                   mean=t(k_fit$centers),
                   variance=var.list)


  kk <- mclust::emV(data=v$ks_value, parameters=par.list)

  kk$BIC <- mclust::bic("V", kk$loglik, n=n, d=1, G=k.centers)

  kk
}


#' Log-Normal mixturing analyses of a Ks distributions for the whole paranome
#'
#' @param G An integer vector specifying the range of the mixtured components.
#'   A BIC is calculated for each component. The default is G=1:5.
#'   For a formal analysis, it is recommended to use 1:10.
#'
#' @param k.nstart How many random sets should be chosen in the k-means
#'   clustering.
#'   For a formal analysis, it is recommended to use 500.
#'
#' @param maxK Maximum Ks values used in the mixture modeling analysis.
#'
#' @param ksv A `ksv` object.
#'
#' @return A data frame with seven variables.
#'
#' @examples
#' ksv <- c(0.1, 0.3, 0.12, 0.34, 0.45, 0.32, 0.33)
#' mix_logNormal_Ks <- function(ksv)
#'
mix_logNormal_Ks <- function(ksv, G=1:5, k.nstart=500, maxK=5) {

  if (maxK <= ksv$ks_dist$maxK) {
    ksv$ks_dist$Ks <- ksv$ks_dist$Ks[ksv$ks_dist$Ks <= maxK]
    ksv$ks_dist$maxK <- maxK
  } else {
    warning("The max Ks is smaller than maxK, please read the Ks file again.")
  }

  Ks <- ksv$ks_dist$Ks
  v <- list(ks_value=log(Ks, exp(1)),
            log=TRUE)

  ks.mix <- data.frame()
  for (i in G) {
    ks_g <- run_emmix_kmeas(v, k.centers=i, k.nstart=k.nstart)
    df <- data.frame(comp=i,
                     mean=ks_g$parameters$mean,
                     sigmasq=ks_g$parameters$variance$sigmasq,
                     prop=ks_g$parameters$pro,
                     logLik=ks_g$loglik,
                     BIC=ks_g$BIC,
                     entities=length(v$ks_value))

    df <- df %>% dplyr::arrange(mean)
    ks.mix <- rbind(ks.mix, df)
  }

  ks.mix$mode <- exp(ks.mix$mean - ks.mix$sigmasq)

  ks.mix
}

#' Read the EMMIX output for a range of components
#'
#' @param emmix.out The output file from EMMIX software.
#'
#' @param G An integer vector specifying the range of the mixture components.
#'   The default is G=1:3.
#'
#' @return A data frame with seven variables.
#'
parse_EMMIX <- function(emmix.out, G=1:3) {
  final_df <- parse_one_EMMIX(emmix.out, ncomponent=G[1])
  for (i in G[2:length(G)]) {
    df <- parse_one_EMMIX(emmix.out, ncomponent=i)
    final_df <- rbind(final_df, df)
  }

  final_df
}

#' Read the EMMIX output for a specify number of components
#'
#' @param emmix.out The output file from EMMIX software.
#' @param ncomponent Number of components to read from the file.
#'
#' @return A data frame with seven variables.
#'
parse_one_EMMIX <- function(emmix.out, ncomponent=3) {
  fh <- file(emmix.out, "r")
  comp <- 0
  m <- c()
  sigmasq <- c()
  tag <- ""
  while (TRUE) {
    line <- readLines(fh, n=1)
    if (length(line) == 0) {
      break
    }

    if (grepl("entities", line)) {
      entities <- as.numeric(regmatches(line, regexpr("\\d+", line)))
    }

    # parse the output of EMMIX
    if (grepl("Results for", line)) {
      comp <- as.numeric(regmatches(line, regexpr("\\d+", line)))
      if (comp == ncomponent) {
        next
      }
      else {
        comp <- 0
        next
      }
    } else if (comp == 0) {
      next
    } else if (tag == ""){
      if (grepl("Final Log", line)) {
        logLik <- as.numeric(regmatches(line, regexpr("-\\d+.\\d+", line)))
      } else if (grepl("proportion", line)) {
        tag <- "proportion"
      } else if (grepl("Estimated mean", line)) {
        tag <- "mean"
      } else if (grepl("Estimated (common )?covariance", line)) {
        tag <- "covariance"
      } else if (grepl("BIC", line)) {
        tag <- "BIC"
      }
    } else if (tag == "proportion") {
      #print(tag)
      prop <- as.numeric(unlist(regmatches(line, gregexpr("\\d+.\\d+", line))))
      tag <- ""
    } else if (tag == "mean") {
      #print(tag)
      m <- c(m, as.numeric(regmatches(line, regexpr("-?\\d+.\\d+", line))))
      if (length(m) == comp) {
        tag <- ""
      }
    } else if (tag == "covariance") {
      #print(tag)
      sigmasq <- c(sigmasq, as.numeric(regmatches(line, regexpr("\\d+.\\d+(E-\\d+)?", line))))
      if (length(sigmasq) == comp) {
        tag <- ""
      }
    } else if (tag == "BIC") {
      #print(tag)
      BIC <- as.numeric(unlist(regmatches(line, gregexpr("\\d+.\\d+", line))[[1]][2]))
      break
    }
  }
  close(fh)

  if (length(m) == 0) {
    stop("The specified number of components does not exist.")

    return(NULL)
  }

  if (ncomponent == 1) {
    prop <- 1
  }
  df <- data.frame(comp=rep(comp, comp), mean=m,
                   sigmasq=sigmasq, prop=prop,
                   logLik=logLik, BIC=BIC)
  df$entities <- entities

  df
}

output$myMarkdown <- renderUI({
    div(
        # style="padding-left: 100px;
        #        padding-bottom: 10px;
        #        padding-right: 200px;
        #        padding-top: 10px;",
        column(
            12,
            #HTML(includeMarkdown("www/content/help.md"))
            #HTML(includeMarkdown("../../../vignettes/intro_to_shinywgd.Rmd"))
            htmltools::tags$iframe(
                src="intro_to_shinywgd.html",
                width='100%',
                height=1000,
                style="border:none;"
            )
        )
    )
})

#' Check if an Object is Not NULL
#'
#' This function checks if an object is not NULL.
#'
#' @param x An R object to check.
#'
#' @return A logical value indicating whether the object is not NULL.
#'
#' @examples
#' is.not.null(5)
#' is.not.null(NULL)
is.not.null <- function(x) {
    !is.null(x)
}

#' Creating a Custom Download Button
#'
#' Use this function to create a custom download button or link. When clicked, it will initiate a browser download. The filename and contents are specified by the corresponding downloadHandler() defined in the server function.
#'
#' @param outputId The name of the output slot that the downloadHandler is assigned to.
#' @param label The label that should appear on the button.
#' @param class Additional CSS classes to apply to the tag, if any. Default NULL.
#' @param status The status of the button; default is "primary."
#' @param ... Other arguments to pass to the container tag function.
#' @param icon An icon() to appear on the button; default is icon("download").
#'
#' @importFrom htmltools tags
#'
#' @export
#'
#' @return An HTML tag to allow users to download the object.
#'
downloadButton_custom <- function(
        outputId,
        label="Download",
        class=NULL,
        status="primary",
        ...,
        icon=shiny::icon("download")
        ){
    aTag <- tags$a(
        id=outputId,
        class=paste(paste0("btn btn-", status, " shiny-download-link"), class),
        href="",
        target="_blank",
        download=NA,
        icon,
        label,
        ...
    )
}

#' Read Data from Uploaded File
#'
#' This function reads data from an uploaded file in a Shiny application and returns it as a data frame.
#'
#' @param uploadfile The object representing the uploaded file obtained through the Shiny upload function.
#'
#' @return A data frame containing the data from the uploaded file.
#'
read_data_file <- function(uploadfile) {
    dataframe <- read.delim(
        uploadfile$datapath,
        sep="\t",
        header=FALSE,
        fill=TRUE,
        na.strings=""
    )
    return(dataframe)
}

#' Check and Prepare GFF/GTF Input File
#'
#' This function checks the file format of a GFF/GTF input file and prepares it for analysis. It can handle both uncompressed and compressed formats.
#'
#' @param gff_input_name A descriptive name for the GFF/GTF file.
#' @param gff_input_path The file path to the GFF/GTF file.
#'
#' @importFrom stringr str_detect
#' @importFrom stringr regex
#' @importFrom fs file_temp
#' @importFrom shinyalert shinyalert
#'
#' @return The path to the prepared GFF file for analysis.
#'
check_gff_input <- function(gff_input_name, gff_input_path, working_wd){
    checked_gff <- paste0(working_wd, "/", gff_input_name, ".gff")
    if( str_detect(gff_input_path$name, ".(gff|gff3)$") ){
        link_gff_cmd <- paste0("cat ", gff_input_path$datapath, " | grep -v '#' > ", checked_gff)
        system(link_gff_cmd)
    }
    else if( str_detect(gff_input_path$name, regex(".(gff.gz|gff3.gz|gz)$")) ){
        gzip_gff_cmd <- paste0("gunzip -c ", gff_input_path$datapath, " | grep -v '#' > ", checked_gff)
        system(gzip_gff_cmd)
    }
    else{
        shinyalert(
            "Oops!",
            paste0("Please upload the correct annotatoin file for ", gff_input_name, " then switch this on"),
            type="error"
        )
    }
    return(checked_gff)
}

#' Check and Process GFF Input File from a Specific Path
#'
#' This function checks the type of GFF input file specified by its path and processes it accordingly.
#'
#' @param gff_input_name The informal name of the GFF input file.
#' @param gff_input_path The path to the GFF input file.
#'
#' @importFrom stringr str_detect
#' @importFrom stringr regex
#' @importFrom fs file_temp
#' @importFrom shinyalert shinyalert
#'
#' @return A string containing the processed GFF file's path.
#'
check_gff_from_file <- function(gff_input_name, gff_input_path, working_wd){
    checked_gff <- paste0(working_wd, "/", gff_input_name, ".gff")
    if( str_detect(gff_input_path, ".(gff|gff3)$") ){
        link_gff_cmd <- paste0("cat ", gff_input_path, " | grep -v '#' > ", checked_gff)
        system(link_gff_cmd)
    }
    else if( str_detect(gff_input_path, regex(".(gff.gz|gff3.gz|gz)$")) ){
        gzip_gff_cmd <- paste0("gunzip -c ", gff_input_path, " | grep -v '#' > ", checked_gff)
        system(gzip_gff_cmd)
    }
    else{
        shinyalert(
            "Oops!",
            paste0("Please upload the correct annotatoin file for ", gff_input_name, " then switch this on"),
            type="error"
        )
    }
    return(checked_gff)
}


#' Check if a file is in FASTA format with cds sequences.
#'
#' This function checks whether a given file is in FASTA format with cds sequences.
#'
#' @param file_path The path to the input file.
#'
#' @return TRUE if the file is in FASTA format with cds sequences, FALSE otherwise.
#'
is_fasta_cds <- function(file_path) {
    if( endsWith(file_path, ".gz") ){
        con <- gzfile(file_path, "rt")
    } else {
        con <- file(file_path, "rt")
    }

    lines <- readLines(con, n=2, warn=FALSE)
    close(con)

    if( length(lines) < 2 ){
        return(FALSE)
    }

    if( !grepl("^>", lines[1]) ){
        return(FALSE)
    }

    sequence_lines <- lines[-1]
    nucleotide_chars <- c("A", "T", "C", "G", "N", "a", "t", "c", "g", "n")

    if( all(unlist(strsplit(sequence_lines, "")) %in% nucleotide_chars) ){
        return(TRUE)
    }else{
        return(FALSE)
    }
}

#' Extract the first part of a string by splitting it at tab characters.
#'
#' This function takes a string and splits it at tab characters. It then
#' returns the first part of the resulting character vector.
#'
#' @param name The input string to be split.
#'
#' @return Returns the first part of the input string.
#'
extract_first_part <- function(name) {
    parts <- unlist(strsplit(name, "\t"))
    return(parts[1])
}

#' Remove Genes Contain Stop Codons within the Sequence
#'
#' This function removes the gene contains stop codons (TAA, TAG, TGA, taa, tag, tga)
#' within its sequence.
#'
#' @param sequence A nucleotide sequence as a character string.
#'
#' @return A character string or NULL.
#'
#' @examples
#' sequence <- "ATGTAACTAGTGAAGTAGCTAACATAG"
#' modified_sequence <- remove_inner_stop_codon_sequence(sequence)
#' modified_sequence
#' # [1] NULL
#'
remove_inner_stop_codon_sequence <- function(sequence) {
    is_stop_codon <- function(codon) {
        stop_codons <- c("TAA", "TAG", "TGA", "taa", "tag", "tga")
        return(codon %in% stop_codons)
    }

    codons <- strsplit(sequence, "")
    codons <- split(codons, rep(1:(length(codons) / 3), each=3))
    codons <- sapply(codons, paste, collapse="")
    codons <- codons[-length(codons)]

    if( any(is_stop_codon(codons)) ){
        return(NULL)
    }else{
        return(sequence)
    }
}

#' Check and Process Proteome Input File
#'
#' This function checks the type of proteome input file and processes it accordingly.
#'
#' @param proteome_name The informal name of the proteome input file.
#' @param proteome_input The proteome input data.
#'
#' @importFrom stringr str_detect
#' @importFrom stringr regex
#' @importFrom shinyalert shinyalert
#' @importFrom seqinr read.fasta
#' @importFrom seqinr getLength
#' @importFrom seqinr write.fasta
#'
#' @return A string containing the processed proteome file's path.
#'
check_proteome_input <- function(proteome_name, proteome_input, working_wd){
    proteome_file <- paste0(working_wd, "/", proteome_name, ext=".fa")
    if( !is_fasta_cds(proteome_input$datapath) ){
        proteome_name <- gsub("[0-9]", "", proteome_name)
        proteome_name <- gsub("_", " ", proteome_name)
        shinyalert(
            "Oops!",
            paste0("Please upload the correct proteome file for ", proteome_name, " then switch this on"),
            type="error"
        )
        return(NULL)
    }

    sequences <- read.fasta(proteome_input$datapath)

    if( any(grepl("\\|", names(sequences))) ){
        proteome_name <- gsub("[0-9]", "", proteome_name)
        proteome_name <- gsub("_", " ", proteome_name)
        shinyalert(
            "Oops!",
            paste0(
                "Please upload the correct proteome file for ",
                proteome_name,
                ". Do not keep \"|\" in the identifier of each sequences.",
                "Then switch this on"
            ),
            type="error"
        )
        return(NULL)
    }

    lengths <- getLength(sequences)
    filtered_sequences <- sequences[lengths %% 3 == 0]

    filtered_sequences_2 <- sapply(filtered_sequences, remove_inner_stop_codon_sequence, simplify=FALSE)
    filtered_sequences_2 <- filtered_sequences_2[sapply(filtered_sequences_2, function(x) !is.null(x))]

    write.fasta(
        sequences=filtered_sequences_2,
        names=sapply(names(filtered_sequences_2), extract_first_part),
        file.out=proteome_file,
        nbchar=100000000
    )

    # Get the Log Info
    log_file <- paste0(working_wd, "/Sequence_processing.log")
    if( file.exists(log_file) ){
        logInfo <- file(log_file, open="a")
    }else{
        logInfo <- file(log_file, open="w")
        cat(paste("Species", "Tot_gene", "genes_not_triple_codon", "genes_within_stop_codons", "refined_gene", collapse="\t"), file=logInfo, sep="\n")
    }
    original_num <- length(sequences)
    filter_num1 <- length(filtered_sequences)
    filter_num2 <- length(filtered_sequences_2)

    if( original_num == filter_num1 && filter_num1 == filter_num2 ){
        cat(paste(proteome_name, original_num, 0, 0, original_num, collapse="\t"), file=logInfo, append=TRUE, sep="\n")
    }else{
        tmp1_num <- original_num - filter_num1
        tmp2_num <- filter_num1 - filter_num2
        cat(paste(proteome_name, original_num, tmp1_num, tmp2_num, filter_num2, collapse="\t"), file=logInfo, append=TRUE, sep="\n")
    }
    close(logInfo)

    return(proteome_file)
}

#' Check and Process Proteome Input File From a Special Path
#'
#' This function checks the type of proteome input file and processes it accordingly.
#'
#' @param proteome_name The informal name of the proteome input file.
#' @param proteome_input The proteome input data.
#'
#' @importFrom stringr str_detect
#' @importFrom stringr regex
#' @importFrom shinyalert shinyalert
#' @importFrom seqinr read.fasta
#' @importFrom seqinr getLength
#' @importFrom seqinr write.fasta
#'
#' @return A string containing the processed proteome file's path.
#'
check_proteome_from_file <- function(proteome_name, proteome_input, working_wd){
    tmp_file <- paste0(working_wd, "/", proteome_name, ext=".tmp.fa")
    proteome_file <- paste0(working_wd, "/", proteome_name, ext=".fa")

    if( !is_fasta_cds(proteome_input) ){
        proteome_name <- gsub("[0-9]", "", proteome_name)
        proteome_name <- gsub("_", " ", proteome_name)
        shinyalert(
            "Oops!",
            paste0("Please upload the correct proteome file for ", proteome_name, " then switch this on"),
            type="error"
        )
        return(NULL)
    }

    sequences <- read.fasta(proteome_input)

    if( any(grepl("\\|", names(sequences))) ){
        proteome_name <- gsub("[0-9_]", " ", proteome_name)
        shinyalert(
            "Oops!",
            paste0(
                "Please upload the correct proteome file for ",
                proteome_name,
                ". Do not keep \"|\" in the identifier of each sequences.",
                "Then switch this on"
            ),
            type="error"
        )
        return(NULL)
    }

    lengths <- getLength(sequences)
    filtered_sequences <- sequences[lengths %% 3 == 0]
    filtered_sequences_2 <- sapply(filtered_sequences, remove_inner_stop_codon_sequence, simplify=FALSE)
    filtered_sequences_2 <- filtered_sequences_2[sapply(filtered_sequences_2, function(x) !is.null(x))]
    write.fasta(
        sequences=filtered_sequences_2,
        names=sapply(names(filtered_sequences_2), extract_first_part),
        file.out=proteome_file,
        nbchar=100000000
    )

    # Get the Log Info
    log_file <- paste0(working_wd, "/Sequence_processing.log")
    if( file.exists(log_file) ){
        logInfo <- file(log_file, open="a")
    }else{
        logInfo <- file(log_file, open="w")
        cat(paste("Species", "Tot_gene", "genes_not_triple_codon", "genes_within_stop_codons", "refined_gene", collapse="\t"), file=logInfo, sep="\n")
    }
    original_num <- length(sequences)
    filter_num1 <- length(filtered_sequences)
    filter_num2 <- length(filtered_sequences_2)

    if( original_num == filter_num1 && filter_num1 == filter_num2 ){
        cat(paste(proteome_name, original_num, 0, 0, original_num, collapse="\t"), file=logInfo, append=TRUE, sep="\n")
    }else{
        tmp1_num <- original_num - filter_num1
        tmp2_num <- filter_num1 - filter_num2
        cat(paste(proteome_name, original_num, tmp1_num, tmp2_num, filter_num2, collapse="\t"), file=logInfo, append=TRUE, sep="\n")
    }
    close(logInfo)

    return(proteome_file)
}

#' Check File Existence in a Data Table
#'
#' This function checks the existence of files specified in a data table.
#'
#' @param data_table A data table with file paths in columns V2 and V3.
#' @param working_wd A path of the working directory
#'
#' @return This function has no return value. It prints messages to the console.
#'
#' @examples
#' data_table <- read.table(text="V1 V2 V3
#'         'Apostasia fujianica'    '../Apostasia1.fa'    '../Apostasia1.gff'
#'         'Apostasia shenzhenica'    '../Apostasia2.fa'    '../Apostasia2.gff'
#'         'Asparagus officinalis'    '../Asparagus3.fa'    '../Asparagus3.gff'", header=TRUE)
#'
#' checkFileExistence(data_table)
checkFileExistence <- function(data_table, working_wd){
    for( i in 1:nrow(data_table) ){
        file1 <- as.character(data_table[i, "V2"])

        if( !file.exists(paste0(working_wd, "/", file1)) ){
            shinyalert(
                "Oops!",
                paste0(
                    "Fail to open ",
                    file1, ".",
                    " Please set the correct proteome file for ",
                    as.character(data_table[i, "V1"]), ". ",
                    "Then continue ..."
                ),
                type="error"
            )
        }

        if( !is.na(data_table[i, "V3"]) ){
            file2 <- as.character(data_table[i, "V3"])
            if( !file.exists(paste0(working_wd, "/", file2)) ){
                shinyalert(
                    "Oops!",
                    paste0(
                        "Fail to open ",
                        file2, ".",
                        " Please set the correct annotation file for ",
                        as.character(data_table[i, "V1"]),". ",
                        "Then continue ..."
                    ),
                    type="error"
                )
            }
        }
    }
}

#' Create Ksrates Configuration File
#'
#' This function generates a configuration file for the Ksrates pipeline based on Shiny input.
#'
#' @param input The Input object of Shiny.
#' @param ksrates_conf_file The path to the Ksrates configuration file.
#' @param species_info_file The path to the species information file.
#'
create_ksrates_configure_file_v2 <- function(input, ksrates_conf_file, species_info_file){
    latin_names_temp <- c()
    fasta_filenames_temp <- c()

    collinearity <- "yes"
    workdirname <- dirname(dirname(ksrates_conf_file))
    newick_tree <- readLines(input$newick_tree$datapath)
    newick_tree <- gsub("_", " ", newick_tree)
    system(paste("cp", input$newick_tree$datapath, paste0(workdirname, "/tree.newick")))

    SpeciesInfoConf <- file(species_info_file, open="w")
    shiny::withProgress(message='Processing fasta files in progress', value=0, {
        for( i in 1:input$number_of_study_species ){
            latin_name <- paste0("latin_name_", i)
            shiny::incProgress(
                amount=0.8/input$number_of_study_species,
                message=paste0(i, "/", input$number_of_study_species, ". Dealing with ", input[[latin_name]], " ...")
            )
            Sys.sleep(.1)

            latin_name_temp <- trimws(input[[latin_name]])
            latin_name_list <- strsplit(latin_name_temp, split=' ')[[1]]
            informal_name_temp <- paste0(latin_name_list[1], i)

            newick_tree <- gsub(latin_name_temp, informal_name_temp, newick_tree)

            proteome_temp <- check_proteome_input(
                informal_name_temp,
                input[[proteome]],
                workdirname
            )
            if( is.null(proteome_temp) ){
                close(SpeciesInfoConf)
                return(NULL)
            }
            latin_names_temp <- c(latin_names_temp, paste0(informal_name_temp, ": ", latin_name_temp))
            fasta_filenames_temp <- c(fasta_filenames_temp, paste0(informal_name_temp, ": ../", informal_name_temp, ".fa"))

            gff <- paste0("gff_", i)
            if( input$select_focal_species == latin_name_temp ){
                if( is.null(input[[gff]]) ){
                    collinearity <- "no"
                    shinyalert(
                        "Note!",
                        "No Annotation file for the focal species is detected. Do not run the collinearity analysis for the focal species",
                        type="info"
                    )
                }else{
                    focal_species_gff_filenames_temp <- paste0("../", informal_name_temp, ".gff")
                }
            }
            if( input$select_focal_species == latin_name_temp ){
                focal_species_informal <- informal_name_temp
            }
            if( is.not.null(input[[gff]]) ){
                gff_temp <- check_gff_input(
                    informal_name_temp,
                    input[[gff]],
                    workdirname
                )
                cat(paste0(latin_name_temp, "\t", proteome_temp, "\t", gff_temp), file=SpeciesInfoConf, append=TRUE, sep="\n")
            }
            else{
                cat(paste0(latin_name_temp, "\t", proteome_temp), file=SpeciesInfoConf, append=TRUE, sep="\n")
            }
        }
        shiny::incProgress(amount=1)
        Sys.sleep(.1)
    })

    close(SpeciesInfoConf)

    ksratesconf <- file(ksrates_conf_file, open="w")
    cat(paste0("[SPECIES]"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("focal_species=", focal_species_informal), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("newick_tree=", newick_tree), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("latin_names=", paste(latin_names_temp, collapse=", ")), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("fasta_filenames=", paste(fasta_filenames_temp, collapse=", ")), file=ksratesconf, append=TRUE, sep="\n")
    if( collinearity == "yes" ){
        cat(paste0("gff_filename=", focal_species_gff_filenames_temp), file=ksratesconf, append=TRUE, sep="\n")
    }
    cat(paste0("peak_database_path=ortholog_peak_db.tsv"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("ks_list_database_path=ortholog_ks_list_db.tsv"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("[ANALYSIS SETTING]"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("paranome=yes"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("collinearity=", collinearity), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("gff_feature=mrna"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("gff_attribute=id"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("max_number_outgroups=4"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("consensus_mode_for_multiple_outgroups=mean among outgroups"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("[PARAMETERS]"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("x_axis_max_limit_paralogs_plot=5"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("bin_width_paralogs=0.1"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("y_axis_max_limit_paralogs_plot=None"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("num_bootstrap_iterations=200"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("divergence_colors= Red, MediumBlue, Goldenrod, Crimson, ForestGreen, Gray, SaddleBrown, Black"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("x_axis_max_limit_orthologs_plots=5"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("bin_width_orthologs=0.1"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("max_ks_paralogs=5"), append=TRUE, file=ksratesconf, sep="\n")
    cat(paste0("max_ks_orthologs=10"), append=TRUE, file=ksratesconf)
    close(ksratesconf)
}

#' Create Ksrates Configuration File Based on Data Table
#'
#' This function generates a Ksrates configuration file based on a data table and other parameters.
#'
#' @param data_table The data table containing information about species, proteomes, and GFF files.
#' @param focal_species The name of the focal species.
#' @param newick_tree_file The path to the Newick tree file.
#' @param ksrates_conf_file The path to the Ksrates configuration file to be generated.
#' @param species_info_file The path to the species information file.
#'
create_ksrates_configure_file_based_on_table <- function(
        data_table,
        focal_species,
        newick_tree_file,
        ksrates_conf_file,
        species_info_file,
        working_wd
    ){
    if( ncol(data_table) < 2 ){
        shinyalert(
            "Oops!",
            "You trigger Ksrates pipeline. Please upload the Annotation file for at lease species",
            type="error"
        )
    }

    workdirname <- dirname(dirname(ksrates_conf_file))
    system(paste("cp", newick_tree_file$datapath, paste0(workdirname, "/tree.newick")))

    newick_tree <- readLines(newick_tree_file$datapath)
    newick_tree <- gsub("_", " ", newick_tree)
    latin_names_temp <- c()
    fasta_filenames_temp <- c()
    gff_species <- c()

    collinearity <- "yes"

    SpeciesInfoConf <- file(species_info_file, open="w")

    shiny::withProgress(message='Processing fasta files in progress', value=0, {
        for( i in 1:nrow(data_table) ){
            latin_name <- data_table[i, 1]
            latin_name <- gsub("_", " ", latin_name)

            shiny::incProgress(
                amount=0.8/nrow(data_table),
                message=paste0(i, "/", nrow(data_table), ". Dealing with ", data_table[i, 1], " ...")
            )
            Sys.sleep(.1)

            proteome <- paste0(working_wd, "/original_data/", data_table[i, 2])
            latin_name_temp <- trimws(latin_name)
            latin_name_list <- strsplit(latin_name_temp, split=' ')[[1]]
            informal_name_temp <- paste0(latin_name_list[1], i)
            if( focal_species == latin_name ){
                focal_species_informal=informal_name_temp
            }

            newick_tree <- gsub(latin_name, informal_name_temp, newick_tree)
            proteome_temp <- check_proteome_from_file(
                informal_name_temp,
                proteome,
                workdirname
            )
            if( is.null(proteome_temp) ){
                close(SpeciesInfoConf)
                return(NULL)
            }
            latin_names_temp <- c(latin_names_temp, paste0(informal_name_temp, ": ", latin_name_temp))
            fasta_filenames_temp <- c(fasta_filenames_temp, paste0(informal_name_temp, ": ../", informal_name_temp, ".fa"))
            if( !is.na(data_table[i, 3]) ){
                gff_temp <- check_gff_from_file(
                    informal_name_temp,
                    paste0(working_wd, "/original_data/", data_table[i, 3]),
                    workdirname
                )
                gff_species <- c(gff_species, informal_name_temp)
                cat(paste0(latin_name_temp, "\t", proteome_temp, "\t", gff_temp), file=SpeciesInfoConf, append=TRUE, sep="\n")
            }else{
                cat(paste0(latin_name_temp, "\t", proteome_temp), file=SpeciesInfoConf, append=TRUE, sep="\n")
            }
        }
        shiny::incProgress(amount=1)
        Sys.sleep(.1)
    })

    if( !focal_species_informal %in% gff_species ){
        collinearity <- "no"
        shinyalert(
            "Note!",
            "No Annotation file for the focal species is detected. Do not run the collinearity analysis for the focal species",
            type="info"
        )
    }else{
        focal_species_gff_filenames_temp <- paste0("../", focal_species_informal, ".gff")
    }
    close(SpeciesInfoConf)
    ksratesconf <- file(ksrates_conf_file, open="w")
    cat(paste0("[SPECIES]"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("focal_species=", focal_species_informal), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("newick_tree=", newick_tree), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("latin_names=", paste(latin_names_temp, collapse=", ")), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("fasta_filenames=", paste(fasta_filenames_temp, collapse=", ")), file=ksratesconf, append=TRUE, sep="\n")
    if( collinearity == "yes" ){
        cat(paste0("gff_filename=", focal_species_gff_filenames_temp), file=ksratesconf, append=TRUE, sep="\n")
    }
    cat(paste0("peak_database_path=ortholog_peak_db.tsv"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("ks_list_database_path=ortholog_ks_list_db.tsv"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("[ANALYSIS SETTING]"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("paranome=yes"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("collinearity=", collinearity), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("gff_feature=mrna"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("gff_attribute=id"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("max_number_outgroups=4"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("consensus_mode_for_multiple_outgroups=mean among outgroups"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("[PARAMETERS]"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("x_axis_max_limit_paralogs_plot=5"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("bin_width_paralogs=0.1"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("y_axis_max_limit_paralogs_plot=None"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("num_bootstrap_iterations=200"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("divergence_colors= Red, MediumBlue, Goldenrod, Crimson, ForestGreen, Gray, SaddleBrown, Black"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("x_axis_max_limit_orthologs_plots=5"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("bin_width_orthologs=0.1"), file=ksratesconf, append=TRUE, sep="\n")
    cat(paste0("max_ks_paralogs=5"), append=TRUE, file=ksratesconf, sep="\n")
    cat(paste0("max_ks_orthologs=10"), append=TRUE, file=ksratesconf)
    close(ksratesconf)
}

#' Create ksrates Expert Parameter File
#'
#' @param ksrates_expert_parameter_file The file is used to store the ksrates expert parameter
#'
create_ksrates_expert_parameter_file <- function(ksrates_expert_parameter_file){
    expert_parameter <- file(ksrates_expert_parameter_file, open="w")
    cat(paste0("logging_level=info"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("max_gene_family_size=200"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("distribution_peak_estimate=mode"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("kde_bandwidth_modifier=0.4"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("plot_adjustment_arrows=no"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("num_mixture_model_initializations=10"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("max_mixture_model_iterations=300"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("max_mixture_model_components=5"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("max_mixture_model_ks=5"), append=TRUE, file=expert_parameter, sep="\n")
    cat(paste0("extra_paralogs_analyses_methods=no"), append=TRUE, file=expert_parameter, sep="\n")
    close(expert_parameter)
}

#' Create Ksrates Command Files from Data Table
#'
#' This function generates command files for running Ksrates and related analyses based on a data table and configuration file.
#'
#' @param data_table The data table containing information about species.
#' @param ksratesconf The path to the Ksrates configuration file.
#' @param cmd_file The path to the main Ksrates command file to be generated.
#' @param focal_species The name of the focal species.
#'
create_ksrates_cmd_from_table <- function(data_table, ksratesconf, cmd_file, focal_species){
    cmd <- file(cmd_file, open="w")
    # wgd_cmd <- file(wgd_cmd_file, open="w")
    # cat("module load wgd blast mcl paml fasttree mafft i-adhore diamond; export OMP_NUM_THREADS=1", file=wgd_cmd, append=TRUE, sep="\n")
    # Add Sbatch commond line
    cat("#!/bin/bash", file=cmd, append=TRUE, sep="\n")
    cat("", file=cmd, append=TRUE, sep="\n")
    cat("#SBATCH -p all", file=cmd, append=TRUE, sep="\n")
    cat("#SBATCH -c 1", file=cmd, append=TRUE, sep="\n")
    cat("#SBATCH --mem 4G", file=cmd, append=TRUE, sep="\n")
    cat(paste0("#SBATCH -o ", basename(cmd_file), ".o%j"), file=cmd, append=TRUE, sep="\n")
    cat(paste0("#SBATCH -e ", basename(cmd_file), ".e%j"), file=cmd, append=TRUE, sep="\n")
    cat("", file=cmd, append=TRUE, sep="\n")

    cat("module load ksrates/x86_64/1.1.3", file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates init ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates paralogs-ks ", ksratesconf, " --n-threads 1"), file=cmd, append=TRUE, sep="\n")
    for( i in 1:nrow(data_table) ){
        latin_name <- data_table[i, 1]
        latin_name <- gsub("_", " ", latin_name)
        latin_name_temp <- trimws(latin_name)
        latin_name_list <- strsplit(latin_name_temp, split=' ')[[1]]
        informal_name_i <- paste0(latin_name_list[1], i)
        for( j in 1:nrow(data_table) ){
            latin_name <- data_table[j, 1]
            latin_name <- gsub("_", " ", latin_name)
            latin_name_temp <- trimws(latin_name)
            latin_name_list <- strsplit(latin_name_temp, split=' ')[[1]]
            informal_name_j <- paste0(latin_name_list[1], j)
            if( j > i ){
                cat(
                    paste0(
                        "ksrates orthologs-ks ",
                        ksratesconf, " ",
                        informal_name_i, " ",
                        informal_name_j, " ",
                        "--n-threads 1; ",
                        "gzip ",
                        "ortholog_distributions/wgd_",
                        informal_name_i, "_", informal_name_j, "/",
                        informal_name_i, "_", informal_name_j, ".blast.tsv"
                    ),
                    file=cmd, append=TRUE, sep="\n"
                )
            }
        }
    }
    cat(paste0("ksrates orthologs-analysis ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates plot-orthologs ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates orthologs-adjustment ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates plot-paralogs ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates plot-tree ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates paralogs-analyses ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    close(cmd)
}

#' Create Ksrates Command Files from Shiny Input
#'
#' @param input The Input object of Shiny.
#' @param ksratesconf The path to the Ksrates configuration file.
#' @param cmd_file The path to the main Ksrates command file to be generated.
#'
create_ksrates_cmd <- function(input, ksratesconf, cmd_file){
    cmd <- file(cmd_file, open="w")
    # Add Sbatch commond line
    cat("#!/bin/bash", file=cmd, append=TRUE, sep="\n\n")
    cat("#SBATCH -p all", file=cmd, append=TRUE, sep="\n")
    cat("#SBATCH -c 1", file=cmd, append=TRUE, sep="\n")
    cat("#SBATCH --mem 4G", file=cmd, append=TRUE, sep="\n")
    cat(paste0("#SBATCH -o ", basename(cmd_file), ".o%j"), file=cmd, append=TRUE, sep="\n")
    cat(paste0("#SBATCH -e ", basename(cmd_file), ".e%j"), file=cmd, append=TRUE, sep="\n\n")
    cat("", file=cmd, append=TRUE, sep="\n")

    cat("module load ksrates/x86_64/1.1.3", file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates init ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates paralogs-ks ", ksratesconf, " --n-threads 1"), file=cmd, append=TRUE, sep="\n")
    informal_name_list <- paste0("seq_", 1:input$number_of_study_species)
    for( i in 1:length(informal_name_list) ){
        latin_name <- paste0("latin_name_", i)
        latin_name_temp <- trimws(input[[latin_name]])
        latin_name_list <- strsplit(latin_name_temp, split=' ')[[1]]
        informal_name_i <- paste0(latin_name_list[1], i)
        for( j in 1:length(informal_name_list) ){
            latin_name <- paste0("latin_name_", j)
            latin_name_temp <- trimws(input[[latin_name]])
            latin_name_list <- strsplit(latin_name_temp, split=' ')
            informal_name_j <- paste0(latin_name_list[[1]], j)
            if( j > i ){
                cat(
                    paste0(
                        "ksrates orthologs-ks ",
                        ksratesconf, " ",
                        informal_name_i, " ",
                        informal_name_j, " ",
                        "--n-threads 1; ",
                        "gzip ",
                        "ortholog_distributions/wgd_",
                        informal_name_i, "_", informal_name_j, "/",
                        informal_name_i, "_", informal_name_j, ".blast.tsv"
                    ),
                    file=cmd, append=TRUE, sep="\n"
                )
            }
        }
    }
    cat(paste0("ksrates orthologs-analysis ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates plot-orthologs ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates orthologs-adjustment ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates plot-paralogs ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates plot-tree ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    cat(paste0("ksrates paralogs-analyses ", ksratesconf), file=cmd, append=TRUE, sep="\n")
    close(cmd)
}

#' Map Informal Names to Latin Names
#'
#' This function reads information from an Excel file (XLS) containing columns "latin_name," "informal_name," and "gff." It extracts the "latin_name" and "informal_name" columns, performs some data manipulation, and returns a data frame with these two columns.
#'
#' @param sp_gff_info_xls The path to the Excel file containing species information.
#'
#' @return A data frame with "latin_name" and "informal_name" columns.
#'
map_informal_name_to_latin_name <- function(sp_gff_info_xls){
    df <- read.table(
        sp_gff_info_xls,
        sep="\t",
        header=FALSE,
        fill=TRUE,
        na.strings="",
        col.names=c("latin_name", "informal_name", "gff")
    )

    informal_files <- gsub(".*/", "", df$informal_name)
    informal_files <- gsub(".fa", "", informal_files)
    gff_files <- gsub(".*/", "", df$gff)
    df$informal_name <- informal_files
    df$gff <- gff_files
    return(df[, 1:2])
}

#' Replace Informal Names with Latin Names
#'
#' This function takes a data frame `names_df` containing "latin_name" and "informal_name" columns and an `input` string as input. It replaces informal species names in the `input` string with their corresponding Latin names based on the information in `names_df`. If the `input` string contains underscores ("_"), it assumes a comparison between two species and replaces both informal names. Otherwise, it replaces the informal name in the `input` string.
#'
#' @param names_df A data frame with "latin_name" and "informal_name" columns.
#' @param input The input string that may contain informal species names.
#'
#' @return A modified input string with informal names replaced by Latin names.
#'
replace_informal_name_to_latin_name <- function(names_df, input){
    if( grepl("_", input) ){
        species_list <- strsplit(input, "_")
        latin_name1 <- species_list[[1]][1]
        for( i in 1:nrow(names_df) ){
            latin_name1 <- gsub(names_df$informal_name[i],
                                names_df$latin_name[i],
                                latin_name1)
        }
        latin_name2 <- species_list[[1]][2]
        for( i in 1:nrow(names_df) ){
            latin_name2 <- gsub(names_df$informal_name[i],
                                names_df$latin_name[i],
                                latin_name2)
        }
        return(paste(latin_name1, "vs", latin_name2))
    }
    else{
        for( i in 1:nrow(names_df) ){
            input <- gsub(names_df$informal_name[i],
                               names_df$latin_name[i],
                               input)
        }
        return(input)
    }
}

Synteny_Analysis_ui <- tabPanel(
    "Collinearity",
    value="synteny_analysis",
    fluidRow(
        column(
            id="syntenyPageTitle",
            width=12,
            h3(icon(
                name=NULL,
                style="
                    background: url('images/syntenyIcon.svg');
                    background-size: contain;
                    background-position: center;
                    background-repeat: no-repeat;
                    height: 50px;
                    width: 50px;
                    display: inline-block;
                    vertical-align: middle;"
                ),
                "Collinearity Analysis"
            ),
            style="padding-bottom: 5px;
                   padding-top: 5px;
                   padding-left: 30px;",
        ),
        column(
            id="iadhoreOutputsSettingPanel",
            width=3,
            div(class="boxLike",
                style="background-color: #FAF9F6;",
                h4(icon("upload"), "Uploading"),
                hr(class="setting"),
                h5(HTML("<font color='green'><i>shinyWGD</i> <b>Collinearity Analysis</b></font> Data")),
                fluidRow(
                    class="justify-content-left",
                    style="padding-bottom: 15px;
                           padding-top: 5px",
                    column(
                        12,
                        div(
                            style="padding-left: 10px;
                                   position: relative;",
                            fileInput(
                                'collinear_data_zip_file',
                                label=h6(icon("file-zipper"), HTML("Upload the <b>zipped</b> File")),
                                multiple=FALSE,
                                accept=c(
                                    ".zip",
                                    ".gz"
                                ),
                                width="80%"
                            ),
                            actionButton(
                                inputId="collinear_data_example",
                                "",
                                icon=icon("question"),
                                status="secondary",
                                class="my-start-button-class",
                                title="Click to use the example data to demo run the Collinearity Analysis",
                                style="color: #fff;
                                       background-color: #87CEEB;
                                       border-color: #fff;
                                       position: absolute;
                                       top: 63%;
                                       left: 90%;
                                       margin-top: -15px;
                                       margin-left: -15px;
                                       padding: 5px 14px 5px 10px;
                                       width: 30px; height: 30px; border-radius: 50%;"
                            )
                        )
                    ),
                    column(
                        12,
                        uiOutput("selectedSyntenyDirName")
                    )
                )
            ),
            fluidRow(
                column(
                    12,
                    uiOutput("iadhoreAnalysisPanel")
                )
            )
        ),
        column(
            id="iadhoreOutputPanel",
            width=9,
            fluidRow(
                column(
                    12,
                    uiOutput("iadhore_output"),
                )
            )
        )
    ),
    icon=icon(
        name=NULL,
        style="
            background: url('images/syntenyIcon.svg');
            background-size: contain;
            background-position: center;
            background-repeat: no-repeat;
            height: 21px;
            width: 21px;
            display: inline-block;
            vertical-align: middle;
        "
    )
)

#' ks_mclust_v2
#'
#' A wrapper to run emmix modeling using the mclust package.
#'
#' @param input_data The input data for clustering and modeling.
#'
#' @return A data frame containing clustering and modeling results.
#'
ks_mclust_v2 <- function(input_data) {
    # suppressMessages(
    #     library(mclust)
    # )
    ks.mclust <- data.frame()

    for (i in 1:10) {
        ks_dist <- list(Ks=input_data, maxK=max(input_data))
        ksv <- list(ks_dist=ks_dist)

        ks_g <- mix_logNormal_Ks(ksv, G=i, k.nstart=10)

        if (nrow(ks_g) > 0) {
            df <- data.frame(comp=ks_g$comp,
                             mean=ks_g$mean,
                             sigmasq=ks_g$sigmasq,
                             prop=ks_g$prop,
                             logLik=ks_g$logLik,
                             BIC=ks_g$BIC,
                             entities=ks_g$entities,
                             mode=ks_g$mode)
        } else {
            df <- data.frame(comp=NA,
                             mean=NA,
                             sigmasq=NA,
                             prop=NA,
                             logLik=NA,
                             BIC=NA,
                             entities=NA,
                             mode=NA)
        }

        ks.mclust <- rbind(ks.mclust, df)
    }

    return(ks.mclust)
}

#' SiZer (Significant Zero Crossings)
#'
#' The SiZer (Significant Zero Crossings) method is a technique used for assessing the statistical significance of zero crossings in data density estimation.
#'
#' @param x A numeric vector containing the data for which you want to calculate SiZer.
#' @param bw Bandwidth parameter for kernel density estimation. If not provided, default values are used.
#' @param gridsize A vector specifying the grid size for SiZer. Default is c(401, 151).
#' @param signifLevel The significance level for SiZer. Default is 0.05.
#'
#' @importFrom stats na.omit
#'
#' @return A list containing SiZer results, including the SiZer curve, the SiZer map, and the bandwidth.
#'
#' @examples
#' ks_value_tmp <- c(81.3724, 77.5539, 76.6121, 3.5945, 3.3308, -1, 15.4158, 2.7786, 1.8054)
#' df_sizer <- SiZer(
#'     ks_value_tmp,
#'     gridsize=c(500, 50),
#'     bw=c(0.01, 5)
#' )
SiZer <- function(x, bw, gridsize, signifLevel=0.05) {

	if (!is.vector(x)){
		stop("SiZer is currently only available for 1-dimensional data")
	}

	x <- na.omit(x)

	x <- as.matrix(x)
	tau <- 5

	d <- 1
	n <- length(x)

	if (missing(gridsize)) gridsize <- c(401,151)
	if (length(gridsize)==1) gridsize <- rep(gridsize, 2)

	gs <- gridsize[1]

	## Set some defaults
	if (missing(bw)){
		bw.range  <- dfltBWrange(x,tau)
		bw <- matrix(unlist(bw.range), nrow=2, byrow=FALSE)
	}
	else{
		bw <- matrix(bw, ncol=1, nrow=2)
	}

	range.x <- list(c(min(x), c(max(x))))

	dfltCounts.out  <- dfltCounts(x, gridsize=gs, apply(bw, 2, max), range.x=range.x)
	range.x <-dfltCounts.out$range.x
	gcounts <- dfltCounts.out$counts

	x.SiZer <- seq(range.x[[1]][1], range.x[[1]][2], length=gs)
	bw <- seq(log10(bw[1,1]), log10(bw[2,1]), length=gridsize[2])
	SiZer.map <- matrix(0, ncol=length(bw), nrow=length(x.SiZer))

	i <- 0
	for (logh in bw){
		h <- 10^logh
		i <- i + 1
		est.dens <- drvkde(gcounts,drv=0,bandwidth=h, binned=TRUE, range.x=range.x, se=FALSE)
		est.dens$est[est.dens$est<0] <- 0
		ESS <- n*est.dens$est*prod(h)*(sqrt(2*pi)^d)
		sig.ESS <- ESS >= 5

	  	SiZer.col <- rep(0, length(ESS))
	  	SiZer.col[sig.ESS] <- 1

	  	sig.deriv <- SignifFeatureRegion(n, d, gcounts, gridsize = gs,
										   est.dens, h, signifLevel,
			   							   range.x, grad=TRUE, curv=FALSE)
	  	sig.grad <- sig.deriv$grad
	  	est.grad <- drvkde(gcounts, drv = 1,
	  	                   bandwidth = h, binned = TRUE,
						   range.x = range.x, se = FALSE)$est
	  	SiZer.col[sig.ESS & sig.grad & est.grad >0] <- 2
	 	SiZer.col[sig.ESS & sig.grad & est.grad <0] <- 3
	  	SiZer.map[,i] <- SiZer.col
	}
	return(
	    list(
	        sizer=x.SiZer,
	        map=SiZer.map,
	        bw=bw
	    )
	)
}

Joint_Tree_ui <- tabPanel(
    "VizWGD",
    value="jointTree",
    fluidRow(
        column(
            id="jointTreeTitle",
            width=12,
            h3(icon(
                name=NULL,
                style="
                    background: url('images/ksTreeIcon.svg');
                    background-size: contain;
                    background-position: center;
                    background-repeat: no-repeat;
                    height: 50px;
                    width: 50px;
                    display: inline-block;
                    vertical-align: middle;
                "
                ),
                "VizWGD: WGD Events Visualization in Tree"
            ),
            style="padding-bottom: 5px;
                   padding-top: 5px;
                   padding-left: 30px;",
        ),
        column(
            id="jointTreeSettingPanel",
            width=3,
            # div(class="boxLike",
            #     style="background-color: #FAF9F6;",
            #     h5(icon("upload"), HTML("Uploading <font color='green'><i>K</i><sub>s</sub> Tree</font>")),
            #     hr(class="setting"),
            #     fluidRow(
            #         column(
            #             12,
            #             div(
            #                 style="padding-left: 10px;
            #                        position: relative;",
            #                 fileInput(
            #                     inputId="uploadKsTree",
            #                     label=HTML("<font color='green'><b><i>K</i><sub>s</sub> Tree</b></font> File:"),
            #                     width="80%",
            #                     accept=c(".newick", ".tre", ".tree")
            #                 ),
            #                 actionButton(
            #                     inputId="ks_tree_example",
            #                     "",
            #                     icon=icon("question"),
            #                     status="secondary",
            #                     title="Click to see the example of Ks Unit Tree file",
            #                     class="my-start-button-class",
            #                     style="color: #fff;
            #                            background-color: #87CEEB;
            #                            border-color: #fff;
            #                            position: absolute;
            #                            top: 53%;
            #                            left: 90%;
            #                            margin-top: -15px;
            #                            margin-left: -15px;
            #                            padding: 5px 14px 5px 10px;
            #                            width: 30px; height: 30px; border-radius: 50%;"
            #                 )
            #             )
            #         )
            #     ),
            #     fluidRow(
            #         column(
            #             12,
            #             div(
            #                 style="padding-left: 10px;
            #                        position: relative;",
            #                 fileInput(
            #                     inputId="uploadKsPeakTable",
            #                     label=HTML("<font color='green'><b><i>K</i><sub>s</sub> Peak</b></font> File (Optional):"),
            #                     width="80%",
            #                     accept=c(".csv", ".txt", ".xls")
            #                 ),
            #                 actionButton(
            #                     inputId="ks_peaks_example",
            #                     "",
            #                     icon=icon("question"),
            #                     title="Click to see the example of Ks Peak file",
            #                     status="secondary",
            #                     class="my-start-button-class",
            #                     style="color: #fff;
            #                            background-color: #87CEEB;
            #                            border-color: #fff;
            #                            position: absolute;
            #                            top: 53%;
            #                            left: 90%;
            #                            margin-top: -15px;
            #                            margin-left: -15px;
            #                            padding: 5px 14px 5px 10px;
            #                            width: 30px; height: 30px; border-radius: 50%;"
            #                 )
            #             )
            #         )
            #     )
            # ),
            div(class="boxLike",
                style="background-color: #FBFEEC;",
                h5(icon("upload"), HTML("Uploading Phylogenetic Tree")),
                hr(class="setting"),
                fluidRow(
                    column(
                        12,
                        div(
                            style="padding-left: 10px;
                                   position: relative;",
                            fileInput(
                                inputId="uploadTimeTree",
                                label=HTML("<b>Phylogenetic Tree</b> file:"),
                                width="80%",
                                accept=c(".nexus", ".tre", ".tree", ".newick", ".nwk")
                            ),
                            actionButton(
                                inputId="MCMC_tree_example",
                                "",
                                icon=icon("question"),
                                title="Click to see the example of the Phylogenetic Tree file",
                                class="my-start-button-class",
                                status="secondary",
                                style="color: #fff;
                                       background-color: #87CEEB;
                                       border-color: #fff;
                                       position: absolute;
                                       top: 53%;
                                       left: 90%;
                                       margin-top: -15px;
                                       margin-left: -15px;
                                       padding: 5px 14px 5px 10px;
                                       width: 30px; height: 30px; border-radius: 50%;"
                            )
                        )
                    )
                ),
                # fluidRow(
                #     column(
                #         12,
                #         div(
                #             style="padding-left: 10px;
                #                    position: relative;",
                #             fileInput(
                #                 inputId="uploadTimeTable",
                #                 label=HTML("<b>WGDs Time </b> File (Optional but only for <i><b>Time Tree</i></b>):"),
                #                 width="80%",
                #                 accept=c(".csv", ".txt", ".xls")
                #             ),
                #             actionButton(
                #                 inputId="wgd_time_table_example",
                #                 "",
                #                 icon=icon("question"),
                #                 status="secondary",
                #                 title="Click to see the example of WGD time file",
                #                 class="my-start-button-class",
                #                 style="color: #fff;
                #                        background-color: #87CEEB;
                #                        border-color: #fff;
                #                        position: absolute;
                #                        top: 53%;
                #                        left: 90%;
                #                        margin-top: -15px;
                #                        margin-left: -15px;
                #                        padding: 5px 14px 5px 10px;
                #                        width: 30px; height: 30px; border-radius: 50%;"
                #             )
                #         )
                #     )
                # )
            ),
            fluidRow(
                column(
                    12,
                    uiOutput("timeTreeSettingPanel")
                )
            ),
            fluidRow(
                column(
                    12,
                    uiOutput("preDatedWGDsSettingPanel")
                )
            )
        ),
        column(
            id="TreePanel",
            width=9,
            div(class="boxLike",
                style="background-color: white;
                       padding-bottom: 10px;
                       padding-top: 10px",
                fluidRow(
                    column(
                        9,
                        tags$style(
                            HTML(".rotate-135 {
                                    transform: rotate(135deg);
                                }"),
                            HTML(".rotate-45{
                                    transform: rotate(45deg);
                                }")
                        ),
                        actionButton(
                            "svg_vertical_spacing_add",
                            "",
                            icon("arrows-alt-v"),
                            title="Expand vertical spacing"
                        ),
                        actionButton(
                            "svg_vertical_spacing_sub",
                            "",
                            icon(
                                "down-left-and-up-right-to-center",
                                verify_fa=FALSE,
                                class="rotate-135"
                            ),
                            title="Compress vertical spacing"
                        ),
                        actionButton(
                            "svg_horizontal_spacing_add",
                            "",
                            icon("arrows-alt-h"),
                            title="Expand horizontal spacing"
                        ),
                        actionButton(
                            "svg_horizontal_spacing_sub",
                            "",
                            icon(
                                "down-left-and-up-right-to-center",
                                verify_fa=FALSE,
                                class="rotate-45"
                            ),
                            title="Compress horizontal spacing"
                        ),
                        downloadButton_custom(
                            "jointTreePlotDownload",
                            status="secondary",
                            icon=icon("download"),
                            title="Download the plot",
                            class="my-donwload-button-class",
                            label=HTML(""),
                            style="color: #fff;
                                  background-color: #6B8E23;
                                  border-color: #fff;
                                  padding: 5px 14px 5px 14px;
                                  margin: 5px 5px 5px 5px;"
                        )
                    ),
                    column(
                        12,
                        div(
                            id="jointtree_plot",
                        )
                    )
                )
            )
        )
    ),
    icon=icon(
        name=NULL,
        style="
            background: url('images/ksTreeIcon.svg');
            background-size: contain;
            background-position: center;
            background-repeat: no-repeat;
            height: 21px;
            width: 21px;
            display: inline-block;
            vertical-align: middle;
        "
    )
)

observeEvent(input$ks_data_example, {
    showModal(
        modalDialog(
            title=HTML("The description of the demo data used in the <b><i>K</i><sub>s</sub> Age Distribution Analysis</b>"),
            size="xl",
            uiOutput("ks_data_example_panel")
        )
    )

    output$ks_data_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    HTML(
                        paste0(
                            "<p>In the demo data, we selected four species: <i>Elaeis guineensis</i>, <i>Oryza sativa</i>, <i>Asparagus officinalis</i>, and <i>Vitis vinifera</i>, to generate the data.</p>",
                            "<p>First, we followed the preparation steps in the Data Preparation Page of the <b>shinyWGD</b> server to create the script for the corresponding package, <b>ksrates</b>. ",
                            "We then submitted the job to the PSB computing server to obtain the output.</p>",
                            "<p>After obtaining the output, the <b><i>K</i><sub>s</sub>Dist</b> module reads the data and continues the analysis. ",
                            "Users can choose the type and combinations of the data to study the <b>intra-</b> and <b>inter-species</b> <i>K</i><sub>s</sub> age distribution. ",
                            "Additionally, users have the option to use the <b>rate correction</b> module to adjust the substitution rate among species.</p>",
                            "<p>To download the demo data, <a href='https://github.com/li081766/shinyWGD_Demo_Data/blob/main/4sp_Ks_Data_for_Visualization.tar.gz' target='_blank'>click here</a>.</p>",
                            "<p><br></br></p>"
                        )
                    ),
                    h5(
                        HTML(
                            "<hr><p><b><font color='#BDB76B'>For true data</font></b>"
                        )
                    ),
                    HTML(
                        "<p>Users should upload the zipped-file, named as <b><i>Ks_Data_for_Visualization.tar.gz</i></b> in the <b>Analysis-*</b> folder created by <b>shinyWGD</b>, to start the <b><i>K</i><sub>s</sub>Dist Analysis</b>.</p>"
                    )
                )
            )
        )
    })
})

example_data_dir <- file.path(getwd(), "demo_data")
ks_example_dir <- file.path(example_data_dir, "Example_Ks_Visualization")
ks_check_file <- paste0(ks_example_dir, "/ksrates_wd/ortholog_distributions/wgd_Oryza2_Vitis4/Oryza2_Vitis4.ks.tsv")

if( !dir.exists(ks_example_dir) & !file.exists(ks_check_file) ){
    withProgress(message='Downloading Ks demo data...', value=0, {
        if( !dir.exists(example_data_dir) ){
            dir.create(example_data_dir)
        }
        dir.create(ks_example_dir)

        Sys.sleep(.2)
        incProgress(amount=.3, message="Downloading in progress. Please wait...")

        downloadAndExtractData <- function() {
            download.file(
                "https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_Ks_Data_for_Visualization.tar.gz",
                destfile=file.path(getwd(), "data.zip"),
                mode="wb"
            )

            system(
                paste(
                    "tar xzf",
                    shQuote(file.path(getwd(), "data.zip")),
                    "-C",
                    shQuote(ks_example_dir)
                )
            )

            file.remove(file.path(getwd(), "data.zip"))
        }

        downloadAndExtractData()

        Sys.sleep(.2)
        incProgress(amount=1, message="Done")
    })
}else if( dir.exists(ks_example_dir) & !file.exists(ks_check_file) ){
    withProgress(message='Downloading Ks demo data...', value=0, {
        system(
            paste("rm -rf ", ks_example_dir)
        )
        dir.create(ks_example_dir)

        Sys.sleep(.2)
        incProgress(amount=.3, message="Downloading in progress. Please wait...")

        downloadAndExtractData <- function() {
            download.file(
                "https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_Ks_Data_for_Visualization.tar.gz",
                destfile=file.path(getwd(), "data.zip"),
                mode="wb"
            )

            system(
                paste(
                    "tar xzf",
                    shQuote(file.path(getwd(), "data.zip")),
                    "-C",
                    shQuote(ks_example_dir)
                )
            )

            file.remove(file.path(getwd(), "data.zip"))
        }

        downloadAndExtractData()
        Sys.sleep(.2)
        incProgress(amount=1, message="Done")
    })
}

buttonClicked <- reactiveVal(NULL)
ks_analysis_dir_Val <- reactiveVal(ks_example_dir)

observeEvent(input$Ks_data_zip_file, {
    buttonClicked("fileInput")

    base_dir <- tempdir()
    timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
    ksAnalysisDir <- file.path(base_dir, paste0("Ks_data_", gsub("[ :\\-]", "_", timestamp)))
    dir.create(ksAnalysisDir)
    system(
        paste(
            "tar xzf",
            input$Ks_data_zip_file$datapath,
            "-C",
            ksAnalysisDir
        )
    )
    ks_analysis_dir_Val(ksAnalysisDir)
})

observeEvent(input$ks_data_example, {
    buttonClicked("actionButton")
    ks_analysis_dir_Val(ks_example_dir)
})

observe({
    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
        if( length(ksAnalysisDir) > 0 ){
            dirName <- basename(ksAnalysisDir)
            output$selectedKsDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("<b>Example:<br><font color='#EE82EE'><i>K</i><sub>s</sub> Age Distribution Analysis</font></b>"))
                    )
                )
            })
        }
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
        if( length(ksAnalysisDir) > 0 ){
            dirName <- basename(ksAnalysisDir)
            output$selectedKsDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("Selected Directory:<br><b><font color='#EE82EE'>", dirName, "</font></b>"))
                    )
                )
            })
        }
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
        if( length(ksAnalysisDir) > 0 ){
            dirName <- basename(ksAnalysisDir)
            output$selectedKsDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("Selected Directory:<br><b><font color='#EE82EE'><i>K</i><sub>s</sub> Age Distribution Analysis</font></b>"))
                    )
                )
            })
        }
    }
})

output$ksanalysisPanel <- renderUI({
    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }
    ksfiles <- list.files(path=ksAnalysisDir, pattern="\\.ks.tsv$", full.names=TRUE, recursive=TRUE)
    species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
    if( file.exists(species_info_file[1]) ){
        names_df <- map_informal_name_to_latin_name(species_info_file[1])
        newick_tree_file <- paste0(dirname(species_info_file), "/tree.newick")
        newick_tree <- readLines(newick_tree_file)
        session$sendCustomMessage("findOutgroup", newick_tree)
        if( !is.null(input$treeOrderList) ){
            num_rows <- length(input$treeOrderList) / 3
            num_cols <- 3
            species_tree_df <- matrix(
                input$treeOrderList,
                nrow=num_rows,
                ncol=num_cols,
                byrow=TRUE
            )
            species_tree_df <- as.data.frame(species_tree_df)
            colnames(species_tree_df) <- c("Species", "id", "pId")
            species_tree_df <- species_tree_df[-nrow(species_tree_df), ]
        }else{
            species_tree_df <- data.frame(
                Species=character(),
                id=integer(),
                pId=integer(),
                stringsAsFactors=FALSE
            )
        }
    }
    if( any(grepl("ortholog_distributions", ksfiles)) | any(grepl("paralog_distributions", ksfiles)) ){
        ortholog_ksfiles <- ksfiles[grepl("ortholog_distributions", ksfiles)]
        paralog_ksfiles <- ksfiles[grepl("paralog_distributions", ksfiles)]

        species_list <- lapply(gsub(".ks.tsv", "", basename(paralog_ksfiles)), function(x) {
            replace_informal_name_to_latin_name(names_df, x)
        })

        ref_species_remaining <- sort(species_tree_df[, 1])

        div(class="boxLike",
            style="background-color: #FBFEEC;",
            fluidRow(
                div(
                    style="padding-bottom: 5px;
                           padding-top: 5px;
                           padding-left: 10px;",
                    h5(icon("cog"), HTML("Select a <font color='#bb5e00'><b>submodule</b></font> to start")),
                    # column(
                    #     12,
                    #     hr(class="setting")
                    # ),
                    column(
                        width=12,
                        div(
                            style="padding-top: 10px;
                                   padding-bottom: 10px;",
                            bsButton(
                                inputId="paralogous_ks_button",
                                label=HTML("<font color='white'><b>&nbsp;Paralogous <i>K</i><sub>s</sub> distribution&nbsp;&nbsp;&nbsp;&#x25BC;</b></font>"),
                                icon=icon("list"),
                                style="success"
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to choose species",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                ) %>%
                                bs_attach_collapse("paralog_ks_files_collapse"),
                            bs_collapse(
                                id="paralog_ks_files_collapse",
                                content=tags$div(
                                    class="well",
                                    pickerInput(
                                        inputId="paralog_ks_files_list",
                                        label=HTML("<b><font color='#38B0E4'>Species</font></b>"),
                                        options=list(
                                            title='Please select species below'
                                        ),
                                        choices=unlist(species_list),
                                        choicesOpt=list(
                                            content=lapply(unlist(species_list), function(choice) {
                                                paste0("<div style='color: steelblue; font-style: italic;'>", choice, "</div>")
                                            })
                                        ),
                                        multiple=FALSE
                                    ),
                                    div(
                                        class="d-flex justify-content-end",
                                        actionButton(
                                            inputId="confirm_paralog_ks_go",
                                            "Confirm analysis",
                                            title="Confirm the selection",
                                            class="my-confirm-button-class",
                                            status="secondary",
                                            style="color: #fff;
                                                   background-color: #C0C0C0;
                                                   border-color: #fff;
                                                   margin: 22px 0px 0px 0px; ",
                                        )
                                    )
                                )
                            )
                        )
                    ),
                    column(
                        width=12,
                        div(
                            style="padding-bottom: 10px;",
                            bsButton(
                                inputId="orthologous_ks_button",
                                label=HTML("<font color='white'><b>&nbsp;Orthologous <i>K</i><sub>s</sub> distribution&nbsp;&#x25BC;</b></font>"),
                                icon=icon("list"),
                                style="success"
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to choose species",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                ) %>%
                                bs_attach_collapse("ortholog_ks_files_collapse"),
                            bs_collapse(
                                id="ortholog_ks_files_collapse",
                                content=tags$div(
                                    class="well",
                                    pickerInput(
                                        inputId="ortholog_ks_files_list_A",
                                        label=HTML("Select <b><font color='#38B0E4'>Species</font></b> to compare"),
                                        options=list(
                                            title='Please select species below',
                                            `selected-text-format`="count > 1",
                                            "max-options"=5
                                        ),
                                        choices=unlist(species_list),
                                        choicesOpt=list(
                                            content=lapply(unlist(species_list), function(choice) {
                                                paste0("<div style='color: steelblue; font-style: italic;'>", choice, "</div>")
                                            })
                                        ),
                                        multiple=TRUE
                                    ),
                                    pickerInput(
                                        inputId="ortholog_paranome_species",
                                        label=HTML("Select <b><font color='#fc8d59'>Species</font></b> to draw <font color='#fc8d59'><b>paralogous <i>K</i><sub>s</sub> distribution</b></font> (optional):"),
                                        options=list(
                                            title='Please select species below'
                                        ),
                                        choices=unlist(species_list),
                                        choicesOpt=list(
                                            content=lapply(unlist(species_list), function(choice) {
                                                paste0("<div style='color: #fc8d59; font-style: italic;'>", choice, "</div>")
                                            })
                                        ),
                                        multiple=FALSE
                                    ),
                                    div(
                                        class="d-flex justify-content-end",
                                        actionButton(
                                            inputId="confirm_ortholog_ks_go",
                                            "Confirm analysis",
                                            title="Confirm the selection",
                                            class="my-confirm-button-class",
                                            status="secondary",
                                            style="color: #fff;
                                                   background-color: #C0C0C0;
                                                   border-color: #fff;
                                                   margin: 22px 0px 0px 0px; ",
                                        )
                                    )
                                )
                            )
                        )
                    ),
                    column(
                        width=12,
                        div(
                            style="padding-bottom: 10px;",
                            bsButton(
                                inputId="rate_correct_button",
                                label=HTML("<font color='white'><b>&nbsp;Relative rate test analysis&nbsp;&nbsp;&nbsp;&nbsp;&#x25BC;</b></font>"),
                                icon=icon("list"),
                                style="success"
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to set",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                ) %>%
                                bs_attach_collapse("rate_correction_collapse"),
                            bs_collapse(
                                id="rate_correction_collapse",
                                content=tags$div(
                                    class="well",
                                    pickerInput(
                                        inputId="select_ref_species",
                                        label=HTML("Choose <b><font color='#54B4D3'>Reference</font></b> species:"),
                                        options=list(
                                            title='Please select species below'
                                        ),
                                        choices=ref_species_remaining,
                                        choicesOpt=list(
                                            content=lapply(ref_species_remaining, function(choice) {
                                                choice <- gsub("_", " ", choice)
                                                paste0("<div style='color: #54B4D3; font-style: italic;'>", choice, "</div>")
                                            })
                                        )
                                    ),
                                    pickerInput(
                                        inputId="select_outgroup_species",
                                        label=HTML("Choose <b><font color='#fc8d59'>Outgroup</font></b> species:"),
                                        options=list(
                                            title='Please select species below'
                                        ),
                                        choices=species_tree_df$Species,
                                        choicesOpt=list(
                                            content=lapply(species_tree_df$Species, function(choice) {
                                                choice <- gsub("_", " ", choice)
                                                paste0("<div style='color: #fc8d59; font-style: italic;'>", choice, "</div>")
                                            })
                                        )
                                    ),
                                    pickerInput(
                                        inputId="select_study_species",
                                        label=HTML("Choose <b><font color='#C699FF'>Other</font></b> species to analyze:"),
                                        options=list(
                                            title='Please select species below',
                                            `selected-text-format`="count > 1",
                                            `actions-box`=TRUE
                                        ),
                                        choices=unlist(species_list),
                                        choicesOpt=list(
                                            content=lapply(unlist(species_list), function(choice) {
                                                paste0("<div style='color: #C699FF; font-style: italic;'>", choice, "</div>")
                                            })
                                        ),
                                        multiple=TRUE
                                    ),
                                    div(
                                        class="d-flex justify-content-end",
                                        actionButton(
                                            inputId="confirm_rate_correction_go",
                                            "Confirm analysis",
                                            title="Confirm the selection",
                                            class="my-confirm-button-class",
                                            status="secondary",
                                            style="color: #fff;
                                                   background-color: #C0C0C0;
                                                   border-color: #fff;
                                                   margin: 22px 0px 0px 0px; ",
                                        )
                                    )
                                )
                            )
                        )
                    ),
                    column(
                        width=12,
                        div(
                            style="padding-bottom: 10px;",
                            bsButton(
                                inputId="phylo_ks_button",
                                label=HTML("<font color='white'><b>&nbsp;Phylo-<i>K</i><sub>s</sub> analysis&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&#x25BC;</b></font>"),
                                icon=icon("list"),
                                style="success"
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to set",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                ) %>%
                                bs_attach_collapse("phylo_ks_collapse"),
                            bs_collapse(
                                id="phylo_ks_collapse",
                                content=tags$div(
                                    class="well",
                                    fluidRow(
                                        column(
                                            12,
                                            div(
                                                style="padding-left: 10px;
                                                       position: relative;",
                                                fileInput(
                                                    inputId="uploadPhyloKsTree",
                                                    label=HTML("<font color='green'><b><i>K</i><sub>s</sub> Tree</b></font> File:"),
                                                    width="80%",
                                                    accept=c(".newick", ".tre", ".tree")
                                                ),
                                                actionButton(
                                                    inputId="phylo_ks_tree_example",
                                                    "",
                                                    icon=icon("question"),
                                                    status="secondary",
                                                    title="Click to see the example of Ks Unit Tree file",
                                                    class="my-start-button-class",
                                                    style="color: #fff;
                                                           background-color: #87CEEB;
                                                           border-color: #fff;
                                                           position: absolute;
                                                           top: 53%;
                                                           left: 90%;
                                                           margin-top: -15px;
                                                           margin-left: -15px;
                                                           padding: 5px 14px 5px 10px;
                                                           width: 30px; height: 30px; border-radius: 50%;"
                                                )
                                            )
                                        )
                                    ),
                                    fluidRow(
                                        column(
                                            12,
                                            div(
                                                style="padding-left: 10px;
                                                       position: relative;",
                                                fileInput(
                                                    inputId="uploadPhyloKsPeakTable",
                                                    label=HTML("<font color='green'><b><i>K</i><sub>s</sub> Peak</b></font> File (Optional):"),
                                                    width="80%",
                                                    accept=c(".csv", ".txt", ".xls")
                                                ),
                                                actionButton(
                                                    inputId="phylo_ks_peaks_example",
                                                    "",
                                                    icon=icon("question"),
                                                    title="Click to see the example of Ks Peak file",
                                                    status="secondary",
                                                    class="my-start-button-class",
                                                    style="color: #fff;
                                                           background-color: #87CEEB;
                                                           border-color: #fff;
                                                           position: absolute;
                                                           top: 53%;
                                                           left: 90%;
                                                           margin-top: -15px;
                                                           margin-left: -15px;
                                                           padding: 5px 14px 5px 10px;
                                                           width: 30px; height: 30px; border-radius: 50%;"
                                                )
                                            )
                                        )
                                    ),
                                    div(
                                        class="d-flex justify-content-end",
                                        actionButton(
                                            inputId="confirm_phylo_ks_go",
                                            "Confirm analysis",
                                            title="Confirm the selection",
                                            class="my-confirm-button-class",
                                            status="secondary",
                                            style="color: #fff;
                                                   background-color: #C0C0C0;
                                                   border-color: #fff;
                                                   margin: 22px 0px 0px 0px; ",
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            )
        )
    }
})

observeEvent(input$phylo_ks_tree_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b><i>K</i><sub>s</sub> Unit Tree</b></font> file"),
            size="xl",
            uiOutput("phylo_ks_tree_example_panel")
        )
    )

    output$phylo_ks_tree_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    verbatimTextOutput("phyloKsUnitTreeExample"),
                    HTML(
                        paste0(
                            "<b>shinyWGD</b> uses the single-copy gene families built by <b>OrthoFinder</b> to calculate the <i>K</i><sub>s</sub> unit tree. <br>",
                            "Users can check the tree in the <b>Orthofinder_wd</b> folder, named <b>singleCopyGene.ds_tree.newick</b>.<br>",
                            "Once uploading the tree file in the <font color='green'><i>K</i><sub>s</sub> Tree</font> File panel, the <i>K</i><sub>s</sub> unit tree will displayed in the right side of the page. <br>"
                        )
                    )
                )
            )
        )
    })

    output$phyloKsUnitTreeExample <- renderText({
        "(((Phalaenopsis_equestris: 0.236436, Dendrobium_catenatum: 0.167833): 0.243397, Apostasia_shenzhenica: 0.443454): 0.597705, Asparagus_officinalis: 0.485116);"
    })
})

observeEvent(input$phylo_ks_peaks_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b><i>K</i><sub>s</sub> peaks table</b></font> file"),
            size="xl",
            uiOutput("phylo_ks_peaks_example_panel")
        )
    )

    ks_peak_data_file <- "www/content/ks_peaks_example.xls"
    output$phyloKsPeakExampleTable <- renderTable({
        species_info_example <- read.table(
            ks_peak_data_file,
            header=TRUE,
            sep="\t",
            quote=""
        )
        colnames(species_info_example) <- gsub("\\.", " ", colnames(species_info_example))
        colnames(species_info_example)[4] <- "95% Confidence Interval"

        species_info_example
    })

    output$phylo_ks_peaks_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    tableOutput("phyloKsPeakExampleTable"),
                    HTML(
                        paste0(
                            "After uploading the <i>K</i><sub>s</sub> peak file, the corresponding <i>K</i><sub>s</sub> peaks of each studied species will be placed into the <i>K</i><sub>s</sub> unit tree.<br>"
                        )
                    )
                )
            )
        )
    })
})

observeEvent(input$rate_correct_button, {
    shinyjs::runjs('document.getElementById("rate_correction_collapse").style.display="block";')
    shinyjs::runjs('document.getElementById("paralog_ks_files_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("ortholog_ks_files_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("phylo_ks_collapse").style.display="none";')
})

observeEvent(input$paralogous_ks_button, {
    shinyjs::runjs('document.getElementById("paralog_ks_files_collapse").style.display="block";')
    shinyjs::runjs('document.getElementById("rate_correction_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("ortholog_ks_files_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("phylo_ks_collapse").style.display="none";')
})

observeEvent(input$orthologous_ks_button, {
    shinyjs::runjs('document.getElementById("ortholog_ks_files_collapse").style.display="block";')
    shinyjs::runjs('document.getElementById("rate_correction_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("paralog_ks_files_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("phylo_ks_collapse").style.display="none";')
})

observeEvent(input$phylo_ks_button, {
    shinyjs::runjs('document.getElementById("ortholog_ks_files_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("rate_correction_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("paralog_ks_files_collapse").style.display="none";')
    shinyjs::runjs('document.getElementById("phylo_ks_collapse").style.display="block";')
})

observeEvent(input$select_ref_species, {
    if( isTruthy(input$select_ref_species) && input$select_ref_species != "" ){
        num_rows <- length(input$treeOrderList) / 3
        num_cols <- 3
        species_tree_df <- matrix(
            input$treeOrderList,
            nrow=num_rows,
            ncol=num_cols,
            byrow=TRUE
        )

        species_tree_df <- as.data.frame(species_tree_df)
        colnames(species_tree_df) <- c("Species", "id", "pId")
        species_tree_df$id <- as.numeric(species_tree_df$id)
        species_tree_df$pId <- as.numeric(species_tree_df$pId)

        under_score <- grepl("_", species_tree_df[1, 1])

        if( under_score ){
            bait_id <- species_tree_df[species_tree_df$Species == gsub(" ", "_", input$select_ref_species), "id"]
            bait_pId <- species_tree_df[species_tree_df$Species == gsub(" ", "_", input$select_ref_species), "pId"]
        }else{
            bait_id <- species_tree_df[species_tree_df$Species == gsub("_", " ", input$select_ref_species), "id"]
            bait_pId <- species_tree_df[species_tree_df$Species == gsub("_", " ", input$select_ref_species), "pId"]
        }

        filtered_df <- species_tree_df[species_tree_df$id > bait_id, ]
        if( filtered_df[1, "pId"] == bait_pId ){
            filtered_df <- filtered_df[-1, ]
        }

        updatePickerInput(
            session,
            "select_outgroup_species",
            choices=filtered_df$Species,
            choicesOpt=list(
                content=lapply(filtered_df$Species, function(choice) {
                    choice <- gsub("_", " ", choice)
                    paste0("<div style='color: #fc8d59; font-style: italic;'>", choice, "</div>")
                })
            )
        )

        observeEvent(input$select_outgroup_species, {
            outgroup_id <- species_tree_df[species_tree_df$Species == input$select_outgroup_species, "id"]
            outgroup_pId <- species_tree_df[species_tree_df$Species == input$select_outgroup_species, "pId"]
            filtered_study_df <- species_tree_df[(species_tree_df$id < outgroup_id) & (species_tree_df$Species != input$select_ref_species), ]
            if( nrow(filtered_study_df) > 0 && filtered_study_df[nrow(filtered_study_df), "pId"] == outgroup_pId ){
                filtered_study_df <- filtered_study_df[-nrow(filtered_study_df), ]
            }
            updatePickerInput(
                session,
                "select_study_species",
                choices=sort(filtered_study_df$Species),
                choicesOpt=list(
                    content=lapply(sort(filtered_study_df$Species), function(choice) {
                        choice <- gsub("_", " ", choice)
                        paste0("<div style='color: #998ec3; font-style: italic;'>", choice, "</div>")
                    })
                )
            )
        })
    }
})

observeEvent(input$confirm_paralog_ks_go, {
    shinyjs::runjs("$('#confirm_paralog_ks_go').css('background-color', 'green');")
    updateActionButton(
        session,
        "confirm_paralog_ks_go",
        icon=icon("check")
    )

    setTimeoutFunction <- "setTimeout(function() {
              $('#confirm_paralog_ks_go').css('background-color', '#C0C0C0');
              //$('#confirm_paralog_ks_go').empty();
        }, 12000);"

    shinyjs::runjs(setTimeoutFunction)

    # shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")

    shinyjs::addClass(selector = "#Wgd_plot_paralog", class = "my-svg-container")
    shinyjs::runjs('$(".my-svg-container").empty();')

    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }

    species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
    if( file.exists(species_info_file[1]) ){
        paralog_species <- input$paralog_ks_files_list
        ksfiles <- list.files(path=ksAnalysisDir, pattern="\\.ks.tsv$", full.names=TRUE, recursive=TRUE)
        species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
        ortholog_ksfiles <- ksfiles[grepl("ortholog_distributions", ksfiles)]
        paralog_ksfiles <- ksfiles[grepl("paralog_distributions", ksfiles)]

        # infer the peaks of paralog ks
        names_df <- map_informal_name_to_latin_name(species_info_file[1])

        species_list <- lapply(gsub(".ks.tsv", "", basename(paralog_ksfiles)), function(x) {
            replace_informal_name_to_latin_name(names_df, x)
        })

        paralog_ksfile_df <- data.frame(
            species=unlist(species_list),
            path=paralog_ksfiles
        )

        selected_paralog_ksfile_df <- paralog_ksfile_df[paralog_ksfile_df$species %in% input$paralog_ks_files_list, ]
        ks_file <- selected_paralog_ksfile_df$path
        ks_anchor_file <- gsub(".ks.tsv$", ".ks_anchors.tsv", ks_file)

        if( file.exists(ks_anchor_file) ){
            output$data_choosing <- renderUI({
                fluidRow(
                    column(
                        4,
                        div(
                            style="background-color: #F8F8FF;
                                   padding: 10px 10px 10px -1px;
                                   border-radius: 10px;",
                            prettyRadioButtons(
                                inputId="distribution_choice",
                                label=HTML("<font color='orange'><i>K</i><sub>s</sub> Distribution</font>:"),
                                choices=c("All", "Paranome", "Anchor pairs"),
                                selected="All",
                                icon=icon("check"),
                                status="info",
                                animation="jelly"
                            )
                        )
                    ),
                    column(
                        4,
                        div(
                            style="background-color: #F8F8FF;
                                   padding: 10px 10px 1px 10px;
                                   border-radius: 10px;",
                            prettyRadioButtons(
                                inputId="gmm_choice",
                                label=HTML("<font color='orange'>GMM modelling</font>:"),
                                choices=c("Paranome", "Anchor pairs"),
                                icon=icon("check"),
                                status="info",
                                animation="jelly"
                            )
                        )
                    ),
                    column(
                        4,
                        div(
                            style="background-color: #F8F8FF;
                                   padding: 10px 10px 1px 10px;
                                   border-radius: 10px;",
                            prettyRadioButtons(
                                inputId="sizer_choice",
                                label=HTML("<font color='orange'>Sizer analysis</font>:"),
                                choices=c("Paranome", "Anchor pairs"),
                                icon=icon("check"),
                                status="info",
                                animation="jelly"
                            )
                        )
                    )
                )
            })
        }else{
            output$data_choosing <- renderUI({
                fluidRow(
                    column(
                        4,
                        div(
                            style="background-color: #F8F8FF;
                                   padding: 10px 10px 1px 10px;
                                   border-radius: 10px;",
                            prettyRadioButtons(
                                inputId="distribution_choice",
                                label=HTML("<font color='orange'><i>K</i><sub>s</sub> Distribution</font>:"),
                                choices=c("Paranome"),
                                selected="Paranome",
                                icon=icon("check"),
                                status="info",
                                animation="jelly"
                            )
                        )
                    ),
                    column(
                        4,
                        div(
                            style="background-color: #F8F8FF;
                                   padding: 10px 10px 1px 10px;
                                   border-radius: 10px;",
                            prettyRadioButtons(
                                inputId="gmm_choice",
                                label=HTML("<font color='orange'>GMM modelling</font>:"),
                                choices=c("Paranome"),
                                selected="Paranome",
                                icon=icon("check"),
                                status="info",
                                animation="jelly"
                            )
                        ),
                    ),
                    column(
                        4,
                        div(
                            style="background-color: #F8F8FF;
                                   padding: 10px 10px 1px 10px;
                                   border-radius: 10px;",
                            prettyRadioButtons(
                                inputId="sizer_choice",
                                label=HTML("<font color='orange'>Sizer modelling</font>:"),
                                choices=c("Paranome"),
                                selected="Paranome",
                                icon=icon("check"),
                                status="info",
                                animation="jelly"
                            )
                        )
                    )
                )
            })
        }

        withProgress(message='Configure in progress', value=0, {
            output$ks_analysis_output <- renderUI({
                div(
                    class="boxLike",
                    style="background-color: #FDFFFF;
                           padding-bottom: 10px;
                           padding-top: 10px;",
                    column(
                        12,
                        h4(HTML("<b><font color='#9B3A4D'>Paralogous <i>K</i><sub>s</sub></font> Age Distribution</b>"))
                    ),
                    div(
                        style="padding: 10px 10px 10px 10px;",
                        hr(class="setting"),
                        fluidRow(
                            column(
                                2,
                                h5(HTML("Select <b><font color='orange'>Data</b></font> for:"))
                            ),
                            column(
                                8,
                                uiOutput("data_choosing")
                            ),
                            column(
                                2,
                                actionButton(
                                    inputId="paralog_ks_plot_go",
                                    "Start",
                                    icon=icon("play"),
                                    status="secondary",
                                    title="Click to start",
                                    class="my-start-button-class",
                                    style="color: #fff;
                                           background-color: #27ae60;
                                           border-color: #fff;
                                           padding: 5px 14px 5px 14px;
                                           margin: 25px 5px 5px 5px;"
                                )
                            )
                        ),
                        hr(class="setting"),
                        fluidRow(
                            column(
                                2,
                                h5(HTML("<b><font color='orange'><i>K</i><sub>s</sub> </font></b> setting:")),
                            ),
                            column(
                                10,
                                fluidRow(
                                    column(
                                        2,
                                        div(
                                            style="padding: 12px 10px 5px 10px;
                                                   border-radius: 10px;
                                                   background-color: #FFF5EE;",
                                            pickerInput(
                                                inputId="plot_mode_option_paralog",
                                                label=HTML("<font color='orange'><i>K</i><sub>s</sub> Mode</font>:"),
                                                choices=c("weighted", "average", "min", "pairwise"),
                                                multiple=FALSE,
                                                selected="weighted",
                                                inline=TRUE
                                            )
                                        )
                                    ),
                                    column(
                                        4,
                                        div(
                                            style="/*display: flex; align-items: center;*/
                                                   margin-bottom: -10px;
                                                   border-radius: 10px;
                                                   padding: 10px 10px 0px 10px;
                                                   background-color: #FFF5EE;",
                                            sliderInput(
                                                inputId="ks_binWidth_paralog",
                                                label=HTML("<font color='orange'>BinWidth</font>:&nbsp;"),
                                                min=0,
                                                max=0.2,
                                                step=0.01,
                                                value=0.1
                                            )
                                        )
                                    ),
                                    column(
                                        4,
                                        div(
                                            style="/*display: flex; align-items: center; */
                                                   margin-bottom: -10px;                                                   border-radius: 10px;
                                                   border-radius: 10px;
                                                   padding: 10px 10px 0px 10px;
                                                   background-color: #FFF5EE;",
                                            sliderInput(
                                                inputId="ks_maxK_paralog",
                                                label=HTML("<font color='orange'><i>K</i><sub>s</sub> limit</font>:&nbsp;"),
                                                min=0,
                                                step=1,
                                                max=10,
                                                value=5
                                            )
                                        )
                                    )
                                )
                            )
                        ),
                        hr(class="setting"),
                        fluidRow(
                            column(
                                12,
                                fluidRow(
                                    column(
                                        6,
                                        tags$style(
                                            HTML(".rotate-135 {
                                                transform: rotate(135deg);
                                            }"),
                                            HTML(".rotate-45{
                                                transform: rotate(45deg);
                                            }")
                                        ),
                                        actionButton(
                                            "ks_svg_vertical_spacing_add",
                                            "",
                                            icon("arrows-alt-v"),
                                            title="Expand vertical spacing"
                                        ),
                                        actionButton(
                                            "ks_svg_vertical_spacing_sub",
                                            "",
                                            icon(
                                                "down-left-and-up-right-to-center",
                                                verify_fa=FALSE,
                                                class="rotate-135"
                                            ),
                                            title="Compress vertical spacing"
                                        ),
                                        actionButton(
                                            "ks_svg_horizontal_spacing_add",
                                            "",
                                            icon("arrows-alt-h"),
                                            title="Expand horizontal spacing"
                                        ),
                                        actionButton(
                                            "ks_svg_horizontal_spacing_sub",
                                            "",
                                            icon(
                                                "down-left-and-up-right-to-center",
                                                verify_fa=FALSE,
                                                class="rotate-45"
                                            ),
                                            title="Compress horizontal spacing"
                                        ),
                                        downloadButton_custom(
                                            "ksPlotParalogousDownload",
                                            title="Download the Plot",
                                            status="secondary",
                                            icon=icon("download"),
                                            label=".svg",
                                            class="my-download-button-class",
                                            style="color: #fff;
                                                  background-color: #6B8E23;
                                                  border-color: #fff;
                                                  padding: 5px 5px 5px 5px;"
                                        )
                                    )
                                )
                            )
                        )
                    ),
                    fluidRow(
                        column(
                            6,
                            div(
                                id="Wgd_plot_paralog"
                            )
                        ),
                        column(
                            6,
                            uiOutput("ks_peak_table_output")
                        )
                    ),
                    hr(class="setting"),
                    fluidRow(
                        column(
                            2,
                            h5(HTML("<b><font color='orange'>Figure</b></font> setting:")),
                        ),
                        column(
                            10,
                            fluidRow(
                                column(
                                    4,
                                    div(
                                        style="padding: 12px 10px 5px 10px;
                                               border-radius: 10px;
                                               background-color: #F0FFFF",
                                        sliderInput(
                                            inputId="y_limit_paralog",
                                            label=HTML("<font color='orange'>Y axis limit</font>:"),
                                            min=0,
                                            max=6000,
                                            step=200,
                                            value=2000
                                        )
                                    )
                                ),
                                column(
                                    4,
                                    div(
                                        style="padding: 12px 10px 5px 10px;
                                               border-radius: 10px;
                                               background-color: #F0FFFF",
                                        pickerInput(
                                            inputId="gmm_comp_paralog",
                                            label=HTML("<font color='orange'>Choose GMM component</font>:&nbsp;"),
                                            options=list(
                                                title='Please select component below'
                                            ),
                                            choices=list(),
                                            selected=NULL,
                                            multiple=FALSE
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            })

            Sys.sleep(.2)
            incProgress(amount=.5, message="Configure done ...")
            incProgress(amount=1)
            Sys.sleep(.1)
        })
    }
    else{
        shinyalert(
            "Oops!",
            "Fail to access the output of shinyWGD. Please ensure that all the results of shinyWGD were generated successfully!",
            type="error"
        )
    }
})

observeEvent(input$distribution_choice, {
    if( isTruthy(input$distribution_choice) && input$distribution_choice == "Paranome" ){
        updatePrettyRadioButtons(
            session,
            "gmm_choice",
            choices=c("Paranome"),
            selected="Paranome"
        )
        updatePrettyRadioButtons(
            session,
            "sizer_choice",
            choices=c("Paranome"),
            selected="Paranome"
        )
    }else if( isTruthy(input$distribution_choice) && input$distribution_choice == "Anchor pairs" ){
        updatePrettyRadioButtons(
            session,
            "gmm_choice",
            choices=c("Anchor pairs"),
            selected="Anchor pairs"
        )
        updatePrettyRadioButtons(
            session,
            "sizer_choice",
            choices=c("Anchor pairs"),
            selected="Anchor pairs"
        )
    }else if( isTruthy(input$distribution_choice) && input$distribution_choice == "All" ){
        updatePrettyRadioButtons(
            session,
            "gmm_choice",
            choices=c("Paranome", "Anchor pairs"),
            selected="Paranome"
        )
        updatePrettyRadioButtons(
            session,
            "sizer_choice",
            choices=c("Paranome", "Anchor pairs"),
            selected="Paranome"
        )
    }
})

output$ks_peak_table_output <- renderUI({
    if( isTruthy(input$gmm_comp_paralog) && input$gmm_comp_paralog != "" ){
        fluidRow(
            column(
                10,
                fluidRow(
                    column(
                        12,
                        h5(HTML("<b><font color='#9B3A4D'><i>K</i><sub>s</sub></font></b> Peak Info")),
                        DTOutput("selected_gmm_table")
                    ),
                    column(
                        12,
                        div(class="float-right",
                            downloadButton_custom(
                                "ksPeakCsvDownload",
                                title="Download the Peaks info Table",
                                status="secondary",
                                icon=icon("download"),
                                label=".csv",
                                class="my-download-button-class",
                                style="color: #fff;
                                       background-color: #6B8E23;
                                       border-color: #fff;
                                       padding: 5px 14px 5px 14px;
                                       margin: 25px 5px 5px 5px;"
                            )
                        )
                    )
                )
            )
        )
    }
})

observeEvent(input$paralog_ks_plot_go, {
    shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")

    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }

    species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
    if( file.exists(species_info_file[1]) ){
        paralog_species <- input$paralog_ks_files_list
        ksfiles <- list.files(path=ksAnalysisDir, pattern="\\.ks.tsv$", full.names=TRUE, recursive=TRUE)
        species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
        ortholog_ksfiles <- ksfiles[grepl("ortholog_distributions", ksfiles)]
        paralog_ksfiles <- ksfiles[grepl("paralog_distributions", ksfiles)]

        names_df <- map_informal_name_to_latin_name(species_info_file[1])

        species_list <- lapply(gsub(".ks.tsv", "", basename(paralog_ksfiles)), function(x) {
            replace_informal_name_to_latin_name(names_df, x)
        })

        paralog_ksfile_df <- data.frame(
            species=unlist(species_list),
            path=paralog_ksfiles
        )

        selected_paralog_ksfile_df <- paralog_ksfile_df[paralog_ksfile_df$species %in% input$paralog_ks_files_list, ]
        files_list <- selected_paralog_ksfile_df$path
        ks_file <- selected_paralog_ksfile_df$path
        ks_anchor_file <- gsub(".ks.tsv$", ".ks_anchors.tsv", ks_file)

        if( isTruthy(input$distribution_choice) && input$distribution_choice == "Paranome" ){
            files_list_new <- c(ks_file)
        }else if( isTruthy(input$distribution_choice) && input$distribution_choice == "Anchor pairs" ){
            files_list_new <- c(ks_anchor_file)
        }else if( isTruthy(input$distribution_choice) && input$distribution_choice == "All" ){
            files_list_new <- c(ks_file, ks_anchor_file)
        }

        if( input$gmm_choice == "Paranome" ){
            ks_title <- gsub(".tsv$", "", basename(ks_file))
        }else{
            ks_title <- gsub(".tsv$", "", basename(ks_anchor_file))
        }

        gmm_pre_outfile <- paste0(dirname(ks_file), "/", ks_title, ".gmm.Rdata")

        withProgress(message='Analyzing in progress', value=0, {
            full_data <- calculateKsDistribution4wgd_multiple(
                files_list_new,
                plot.mode=input[["plot_mode_option_paralog"]],
                maxK=input[["ks_maxK_paralog"]],
                binWidth=input[["ks_binWidth_paralog"]],
            )
            barData <- full_data$bar
            ksDist <- full_data$density

            paralogSpecies <- files_list[grep("paralog_distributions", files_list)]
            paralog_id <- gsub(".ks.tsv$", "", basename(paralogSpecies))

            Sys.sleep(.2)
            incProgress(amount=.2, message="GMM modelling ...")

            ks.mclust <- data.frame()
            ks.sizer <- list()
            ks_data <- ksDist[ksDist$title == ks_title, ]
            ks_value <- ks_data$ks

            Sys.sleep(.2)
            incProgress(
                amount=0.2,
                message=paste0("GMM modelling for ", ks_title, " ...")
            )

            # GMM modelling
            if( file.exists(gmm_pre_outfile) ){
                load(gmm_pre_outfile)
            }else{
                df <- ks_mclust_v2(ks_value)
                save(df, file=gmm_pre_outfile)
            }
            df$title <- ks_title
            ks.mclust <- rbind(ks.mclust, df)

            ks.mclust$lower_bound <- qlnorm(0.025, meanlog=ks.mclust$mean, sdlog=ks.mclust$sigmasq)
            ks.mclust$upper_bound <- qlnorm(0.975, meanlog=ks.mclust$mean, sdlog=ks.mclust$sigmasq)

            ks.bic <- unique(ks.mclust[, c("title", "comp", "BIC")])
            emmix_outfile <- paste0(dirname(ks_file), "/", ks_title, ".emmix.output.xls")
            if( !file.exists(emmix_outfile) ){
                write.table(
                    ks.mclust,
                    file=emmix_outfile,
                    row.names=FALSE,
                    sep="\t",
                    quote=FALSE
                )
            }

            gmm_BIC_df <- unique(ks.mclust[, c("comp", "BIC")])
            best_comp <- gmm_BIC_df[gmm_BIC_df$BIC == max(gmm_BIC_df$BIC), ]
            gmm_BIC_list <- sapply(1:nrow(gmm_BIC_df), function(i) {
                paste0(
                    "comp: <b>",
                    gmm_BIC_df[i, "comp"],
                    "</b>, BIC: <b>",
                    round(gmm_BIC_df[i, "BIC"], 2),
                    "</b>"
                )
            }, simplify="list")

            updatePickerInput(
                session,
                "gmm_comp_paralog",
                choices=gmm_BIC_list,
                choicesOpt=list(
                    content=lapply(gmm_BIC_list, function(choice) {
                        HTML(choice)
                    })
                ),
                selected=gmm_BIC_list[best_comp$comp]
            )

            incProgress(
                amount=0.3,
                message=paste0("SiZer analyzing for ", ks_title, " ...")
            )

            #Sizer modelling
            sizer_pre_paranome_outfile <- paste0(dirname(ks_file), "/", ks_title, ".sizer.paranome.Rdata")
            sizer_pre_anchor_point_outfile <- paste0(dirname(ks_file), "/", ks_title, ".sizer.anchorpoint.Rdata")
            if( input$gmm_choice == "Paranome" ){
                if( file.exists(sizer_pre_paranome_outfile) ){
                    load(sizer_pre_paranome_outfile)
                }else{
                    ks_file_tmp <- ks_file
                    ksd_tmp <- read.wgd_ksd(ks_file_tmp)
                    ks_value_tmp <- ksd_tmp$ks_dist$Ks[ksd_tmp$ks_dist$Ks <= input[["ks_maxK_paralog"]]]
                    df_sizer <- SiZer(
                        ks_value_tmp,
                        gridsize=c(500, 50),
                        bw=c(0.01, 5)
                    )
                    save(df_sizer, file=sizer_pre_paranome_outfile)
                }
            }
            else{
                if( file.exists(sizer_pre_anchor_point_outfile) ){
                    load(sizer_pre_anchor_point_outfile)
                }else{
                    ks_file_tmp <- ks_anchor_file
                    ksd_tmp <- read.wgd_ksd(ks_file_tmp)
                    ks_value_tmp <- ksd_tmp$ks_dist$Ks[ksd_tmp$ks_dist$Ks <= input[["ks_maxK_paralog"]]]
                    df_sizer <- SiZer(
                        ks_value_tmp,
                        gridsize=c(500, 50),
                        bw=c(0.01, 5)
                    )
                    save(df_sizer, file=sizer_pre_anchor_point_outfile)
                }
            }

            ks.sizer[[ks_title]] <- list(
                species=ks_title,
                sizer=df_sizer$sizer,
                map=df_sizer$map,
                bw=df_sizer$bw
            )

            Sys.sleep(.2)
            incProgress(amount=.5, message="Calculating done...")

            widthPhyloKsSpacing <- reactiveValues(
                value=500
            )
            heightPhyloKsSpacing <- reactiveValues(
                value=500
            )

            observeEvent(input$ks_svg_vertical_spacing_add, {
                heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value + 50
            })
            observeEvent(input$ks_svg_vertical_spacing_sub, {
                heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value - 50
            })
            observeEvent(input$ks_svg_horizontal_spacing_add, {
                widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value + 50
            })
            observeEvent(input$ks_svg_horizontal_spacing_sub, {
                widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value - 50
            })

            observe({
                selectedBarData <- barData[barData$ks >= 0 & barData$ks <= input[["ks_maxK_paralog"]], ]
                names_df <- map_informal_name_to_latin_name(species_info_file[1])

                if( isTruthy(input$gmm_comp_paralog) && input$gmm_comp_paralog != "" ){
                    selected_comp <- as.numeric(
                        regmatches(input$gmm_comp_paralog, regexpr("\\d+", input$gmm_comp_paralog))
                    )

                    ksMclust <- ks.mclust %>%
                        filter(comp == selected_comp) %>%
                        ungroup()

                    plot_wgd_data <- list(
                        "plot_id"="Wgd_plot_paralog",
                        "species_list"=names_df,
                        "ks_title"=ks_title,
                        "ks_bar_df"=selectedBarData,
                        "paralog_id"=paralog_id,
                        "mclust_df"=ksMclust,
                        "sizer_list"=ks.sizer,
                        "xlim"=input[["ks_maxK_paralog"]],
                        "ylim"=input[["y_limit_paralog"]],
                        "color"="",
                        "opacity"=input[["ks_transparency_paralog"]],
                        "width"=widthPhyloKsSpacing$value,
                        "height"=heightPhyloKsSpacing$value,
                        "dataType"=input$gmm_choice
                    )
                }else{
                    plot_wgd_data <- list(
                        "plot_id"="Wgd_plot_paralog",
                        "species_list"=names_df,
                        "ks_title"=ks_title,
                        "ks_bar_df"=selectedBarData,
                        "paralog_id"=paralog_id,
                        #"mclust_df"=ksMclust,
                        "sizer_list"=ks.sizer,
                        "xlim"=input[["ks_maxK_paralog"]],
                        "ylim"=input[["y_limit_paralog"]],
                        "color"="",
                        "opacity"=input[["ks_transparency_paralog"]],
                        "width"=widthPhyloKsSpacing$value,
                        "height"=heightPhyloKsSpacing$value,
                        "dataType"=input$gmm_choice
                    )
                }
                session$sendCustomMessage("Paralog_Bar_Plotting", plot_wgd_data)
            })
            Sys.sleep(.2)
            incProgress(amount=.4, message="Ploting done...")

            observeEvent(input$gmm_comp_paralog, {
                if( file.exists(emmix_outfile) && input$gmm_comp_paralog != "" ){
                    ks.mclust <- read.table(
                        emmix_outfile,
                        header=TRUE,
                        sep="\t"
                    )

                    selected_comp <- as.numeric(
                        regmatches(input$gmm_comp_paralog, regexpr("\\d+", input$gmm_comp_paralog))
                    )

                    selected_gmm_info <- ks.mclust[ks.mclust$comp == selected_comp, ]
                    selected_gmm_info$CI <- paste0(
                        round(selected_gmm_info$lower_bound, 2),
                        "-",
                        round(selected_gmm_info$upper_bound, 2)
                    )
                    selected_gmm_info <- selected_gmm_info[c("comp", "BIC", "mode", "CI")]
                    selected_gmm_info$mode <- round(selected_gmm_info$mode, 2)
                    selected_gmm_info$BIC <- round(selected_gmm_info$BIC, 2)
                    selected_gmm_info$Species <- input$paralog_ks_files_list
                    selected_gmm_info[["Peak in"]] <- input$gmm_choice
                    selected_gmm_info <- selected_gmm_info[c("mode", "CI", "comp")]

                    shinyInputTest <- function(FUN, len, id, ...) {
                        inputs <- character(len)
                        for( i in seq_len(len) ){
                            inputs[i] <- as.character(
                                tagList(
                                    div(
                                        style="padding-bottom: -20px; margin-bottom: -25px; ",
                                        FUN(paste0(id, i), label=NULL, width="10px", ...)
                                    )
                                )
                            )
                        }
                        inputs
                    }

                    selected_gmm_info_with_checkb <- cbind(
                        Select=shinyInputTest(checkboxInput, nrow(selected_gmm_info), "checkb"),
                        selected_gmm_info
                    )

                    select_row_js <- c(
                        "$('[id^=checkb]').on('click', function(){",
                        "  var id = this.getAttribute('id');",
                        "  var i = parseInt(/checkb(\\d+)/.exec(id)[1]);",
                        "  var checkbox = $('[id^=checkb]:eq(' + (i - 1) + ')');",
                        "  var value = checkbox.prop('checked');",
                        "  var secondColValue = $('[id^=checkb]:eq(' + (i - 1) + ')').closest('tr').find('td:eq(1)').text();",
                        "  var info = [{row: i, peak: secondColValue, value: value}];",
                        "  Shiny.setInputValue('gmm_true_peaks:DT.cellInfo', info);",
                        "})"
                    )

                    output$selected_gmm_table <- renderDT({
                        selected_gmm_info_with_checkb %>%
                            setNames(., colnames(.) %>% gsub("CI", "95% Confidence Interval", .)) %>%
                            setNames(., colnames(.) %>% gsub("mode", "Peak", .)) %>%
                            setNames(., colnames(.) %>% gsub("comp", "Component", .)) %>%
                            setNames(., colnames(.) %>% gsub("Select", "Select to plot", .)) %>%
                            datatable(
                                options=list(
                                    searching=FALSE,
                                    lengthMenu=list(c(-1), c('All')),
                                    paging=FALSE,
                                    info=FALSE,
                                    dom='t'
                                ),
                                rownames=FALSE,
                                escape=FALSE,
                                editable=list(target="cell", disable=list(columns=7)),
                                selection="none",
                                callback=JS(select_row_js)
                            )

                    }, server=FALSE)

                    selected_comp <- as.numeric(
                        regmatches(input$gmm_comp_paralog, regexpr("\\d+", input$gmm_comp_paralog))
                    )

                    ksMclust <- ks.mclust %>%
                        filter(comp == selected_comp) %>%
                        ungroup()

                    selected_best_gmm_info <- reactiveVal(data.frame())
                    observeEvent(input$gmm_true_peaks, {
                        selected_gmm_info_tmp <- ksMclust[input$gmm_true_peaks$row, ]
                        if( input$gmm_true_peaks$value ){
                            selected_best_gmm_info(rbind(selected_best_gmm_info(), selected_gmm_info_tmp))
                        }else{
                            row_to_remove <- input$gmm_true_peaks$row
                            updated_data <- selected_best_gmm_info()[rownames(selected_best_gmm_info()) != row_to_remove, ]
                            selected_best_gmm_info(updated_data)
                        }
                    })

                    observe({
                        if( input$gmm_comp_paralog != "" ){
                            selectedBarData <- barData[barData$ks >= 0 & barData$ks <= input[["ks_maxK_paralog"]], ]
                            names_df <- map_informal_name_to_latin_name(species_info_file[1])

                            ploted_gmm_lines_info <- selected_best_gmm_info()
                            if( nrow(ploted_gmm_lines_info) > 0 ){
                                ploted_gmm_lines_info <- ploted_gmm_lines_info[order(ploted_gmm_lines_info$mode), ]

                                plot_wgd_data <- list(
                                    "plot_id"="Wgd_plot_paralog",
                                    "species_list"=names_df,
                                    "ks_title"=ks_title,
                                    "ks_bar_df"=selectedBarData,
                                    "paralog_id"=paralog_id,
                                    "mclust_df"=ploted_gmm_lines_info,
                                    "sizer_list"=ks.sizer,
                                    "xlim"=input[["ks_maxK_paralog"]],
                                    "ylim"=input[["y_limit_paralog"]],
                                    "color"="",
                                    "opacity"=input[["ks_transparency_paralog"]],
                                    "width"=widthPhyloKsSpacing$value,
                                    "height"=heightPhyloKsSpacing$value,
                                    "dataType"=input$gmm_choice
                                )
                            }else{
                                selected_comp <- as.numeric(
                                    regmatches(input$gmm_comp_paralog, regexpr("\\d+", input$gmm_comp_paralog))
                                )

                                ksMclust <- ks.mclust %>%
                                    filter(comp == selected_comp) %>%
                                    ungroup()

                                plot_wgd_data <- list(
                                    "plot_id"="Wgd_plot_paralog",
                                    "species_list"=names_df,
                                    "ks_title"=ks_title,
                                    "ks_bar_df"=selectedBarData,
                                    "paralog_id"=paralog_id,
                                    "mclust_df"=ksMclust,
                                    "sizer_list"=ks.sizer,
                                    "xlim"=input[["ks_maxK_paralog"]],
                                    "ylim"=input[["y_limit_paralog"]],
                                    "color"="",
                                    "opacity"=input[["ks_transparency_paralog"]],
                                    "width"=widthPhyloKsSpacing$value,
                                    "height"=heightPhyloKsSpacing$value,
                                    "dataType"=input$gmm_choice
                                )
                            }
                            session$sendCustomMessage("Paralog_Bar_Plotting", plot_wgd_data)
                        }
                    })

                    output$ksPeakCsvDownload <- downloadHandler(
                        filename=function() {
                            paste0(
                                gsub(" ", "_", paralog_species),
                                ".paralogous_Ks_peaksInfo.csv"
                            )
                        },
                        content=function(file) {
                            download_df <- selected_best_gmm_info()[c("mode", "lower_bound", "upper_bound")]
                            download_df["95% Confidence Interval"] <- paste0(
                                round(download_df$lower_bound, 2),
                                "-",
                                round(download_df$upper_bound, 2)
                            )
                            download_df$Species <- paralog_species
                            download_df["Peak in"] <- input$gmm_choice
                            download_df$mode <- round(download_df$mode, 2)
                            download_df <- download_df[, c("Species", "Peak in", "mode", "95% Confidence Interval")]
                            names(download_df) <- c("Species", "Peak in", "Peak", "95% Confidence Interval")
                            download_df <- download_df[order(download_df$Peak), ]

                            write.table(
                                download_df,
                                file=file,
                                sep="\t",
                                quote=FALSE,
                                row.names=FALSE
                            )
                        }
                    )
                }
            })
        })
    }
})

observeEvent(input$confirm_ortholog_ks_go, {
    shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")
    shinyjs::runjs("$('#confirm_ortholog_ks_go').css('background-color', 'green');")
    updateActionButton(
        session,
        "confirm_ortholog_ks_go",
        icon=icon("check")
    )

    setTimeoutFunction <- "setTimeout(function() {
              $('#confirm_ortholog_ks_go').css('background-color', '#C0C0C0');
              //$('#confirm_ortholog_ks_go').empty();
        }, 6000);"

    shinyjs::runjs(setTimeoutFunction)

    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }

    species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
    if( file.exists(species_info_file[1]) ){
        if( isTruthy(input$ortholog_ks_files_list_A) && length(input$ortholog_ks_files_list_A) > 1 ){
            withProgress(message='Configure in progress', value=0, {
                output$ks_analysis_output <- renderUI({
                    div(
                        class="boxLike",
                        style="background-color: #FDFFFF;
                               padding-bottom: 10px;
                               padding-top: 10px;",
                        column(
                            12,
                            h4(HTML("<b><font color='#9B3A4D'>Orthologous <i>K</i><sub>s</sub></font> Age Distribution</b>"))
                        ),
                        div(
                            style="padding: 10px 10px 10px 10px;",
                            hr(class="setting"),
                            fluidRow(
                                column(
                                    2,
                                    h5(HTML("<b><font color='orange'><i>K</i><sub>s</sub> </font></b> setting:")),
                                ),
                                if( !isTruthy(input$ortholog_paranome_species) ){
                                    column(
                                        10,
                                        fluidRow(
                                            column(
                                                4,
                                                div(
                                                    style="/*display: flex; align-items: center; */
                                                           margin-bottom: -10px;
                                                           border-radius: 10px;
                                                           padding: 10px 10px 0px 10px;
                                                           background-color: #FFF5EE;",
                                                    sliderInput(
                                                        inputId="ks_maxK_ortholog",
                                                        label=HTML("<font color='orange'><i>K</i><sub>s</sub> limit</font>:&nbsp;"),
                                                        min=0,
                                                        step=1,
                                                        max=5,
                                                        value=5
                                                    )
                                                )
                                            ),
                                            column(
                                                3,
                                                actionButton(
                                                    inputId="ortholog_ks_plot_go",
                                                    # HTML("Start<br><b>ortholog <i>K</i><sub>s</sub></b></br>analysis"),
                                                    "Start",
                                                    icon=icon("play"),
                                                    status="secondary",
                                                    class="my-start-button-class",
                                                    title="Click to start",
                                                    style="color: #fff;
                                                           background-color: #27ae60;
                                                           border-color: #fff;
                                                           padding: 5px 14px 5px 14px;
                                                           margin: 25px 5px 5px 5px;"
                                                )
                                            )
                                        )
                                    )
                                }
                                else{
                                    column(
                                        10,
                                        fluidRow(
                                            column(
                                                4,
                                                div(
                                                    style="/*display: flex; align-items: center; */
                                                           margin-bottom: -10px;
                                                           border-radius: 10px;
                                                           padding: 10px 10px 0px 10px;
                                                           background-color: #FFF5EE;",
                                                    sliderInput(
                                                        inputId="ks_maxK_ortholog",
                                                        label=HTML("<font color='orange'><i>K</i><sub>s</sub> limit</font>:&nbsp;"),
                                                        min=0,
                                                        step=1,
                                                        max=5,
                                                        value=5
                                                    )
                                                )
                                            ),
                                            column(
                                                4,
                                                div(
                                                    style="/*display: flex; align-items: center; */
                                                           margin-bottom: -10px;
                                                           border-radius: 10px;
                                                           padding: 10px 10px 0px 10px;
                                                           background-color: #FFF5EE;",
                                                    pickerInput(
                                                        inputId="plot_ortholog_paralog_species",
                                                        label=HTML("<font color='orange'>Data</font> used for <font color='orange'>Paralogous <i>K</i><sub>s</sub> Distribution</font>:"),
                                                        choices=c("All", "Paranome", "Anchor pairs"),
                                                        multiple=FALSE,
                                                        selected="All",
                                                        inline=TRUE
                                                    )
                                                )
                                            ),
                                            column(
                                                2,
                                                actionButton(
                                                    inputId="ortholog_ks_plot_go",
                                                    # HTML("Start<br><b>ortholog <i>K</i><sub>s</sub></b></br>analysis"),
                                                    "Start",
                                                    icon=icon("play"),
                                                    status="secondary",
                                                    class="my-start-button-class",
                                                    title="Click to start",
                                                    style="color: #fff;
                                                           background-color: #27ae60;
                                                           border-color: #fff;
                                                           padding: 5px 14px 5px 14px;
                                                           margin: 25px 5px 5px 5px;"
                                                )
                                            )
                                        )
                                    )
                                }
                            ),
                            hr(class="setting"),
                            fluidRow(
                                column(
                                    9,
                                    fluidRow(
                                        column(
                                            6,
                                            tags$style(
                                                HTML(".rotate-135 {
                                                transform: rotate(135deg);
                                            }"),
                                                HTML(".rotate-45{
                                                transform: rotate(45deg);
                                            }")
                                            ),
                                            actionButton(
                                                "ks_svg_vertical_spacing_add",
                                                "",
                                                icon("arrows-alt-v"),
                                                title="Expand vertical spacing"
                                            ),
                                            actionButton(
                                                "ks_svg_vertical_spacing_sub",
                                                "",
                                                icon(
                                                    "down-left-and-up-right-to-center",
                                                    verify_fa=FALSE,
                                                    class="rotate-135"
                                                ),
                                                title="Compress vertical spacing"
                                            ),
                                            actionButton(
                                                "ks_svg_horizontal_spacing_add",
                                                "",
                                                icon("arrows-alt-h"),
                                                title="Expand horizontal spacing"
                                            ),
                                            actionButton(
                                                "ks_svg_horizontal_spacing_sub",
                                                "",
                                                icon(
                                                    "down-left-and-up-right-to-center",
                                                    verify_fa=FALSE,
                                                    class="rotate-45"
                                                ),
                                                title="Compress horizontal spacing"
                                            ),
                                            downloadButton_custom(
                                                "ksPlotOrthologousDownload",
                                                title="Download the Plot",
                                                status="secondary",
                                                icon=icon("download"),
                                                class="my-download-button-class",
                                                label=".svg",
                                                style="color: #fff;
                                                      background-color: #6B8E23;
                                                      border-color: #fff;
                                                      padding: 5px 5px 5px 5px;"
                                            )
                                        )
                                    )
                                )
                            )
                        ),
                        fluidRow(
                            column(
                                12,
                                div(
                                    id="Wgd_plot_ortholog"
                                )
                            )
                        ),
                        hr(class="setting"),
                        fluidRow(
                            column(
                                2,
                                h5(HTML("<b><font color='orange'>Figure</b></font> setting:")),
                            ),
                            if( isTruthy(input$ortholog_paranome_species) ){
                                column(
                                    10,
                                    fluidRow(
                                        column(
                                            4,
                                            div(
                                                style="padding: 12px 10px 5px 10px;
                                                       border-radius: 10px;
                                                       background-color: #F0FFFF",
                                                sliderInput(
                                                    inputId="y2_limit_ortholog_paralog",
                                                    label=HTML("<font color='orange'>Retained Duplicates Y axis limit</font>:"),
                                                    min=0,
                                                    step=200,
                                                    max=6000,
                                                    value=2000
                                                )
                                            )
                                        ),
                                        column(
                                            4,
                                            div(
                                                style="padding: 12px 10px 5px 10px;
                                                       border-radius: 10px;
                                                       background-color: #F0FFFF",
                                                sliderInput(
                                                    inputId="y_limit_ortholog",
                                                    label=HTML("<font color='orange'>Ortholog Density Y axis limit</font>:"),
                                                    min=0,
                                                    step=0.2,
                                                    max=5,
                                                    value=2
                                                ),
                                            )
                                        ),
                                        column(
                                            4,
                                            div(
                                                style="padding: 12px 10px 5px 10px;
                                                       border-radius: 10px;
                                                       background-color: #F0FFFF",
                                                sliderInput(
                                                    inputId="opacity_paralog",
                                                    label=HTML("<font color='orange'>Transparency</font>:"),
                                                    min=0,
                                                    max=1,
                                                    step=0.1,
                                                    value=0.5
                                                )
                                            )
                                        )
                                    )
                                )
                            }
                            else{
                                column(
                                    10,
                                    fluidRow(
                                        column(
                                            4,
                                            div(
                                                style="padding: 12px 10px 5px 10px;
                                                       border-radius: 10px;
                                                       background-color: #F0FFFF",
                                                sliderInput(
                                                    inputId="y_limit_ortholog",
                                                    label=HTML("<font color='orange'>Y axis limit</font>:"),
                                                    min=0,
                                                    step=0.2,
                                                    max=5,
                                                    value=2
                                                ),
                                            )
                                        ),
                                        column(
                                            4,
                                            div(
                                                style="padding: 12px 10px 5px 10px;
                                                       border-radius: 10px;
                                                       background-color: #F0FFFF",
                                                sliderInput(
                                                    inputId="opacity_paralog",
                                                    label=HTML("<font color='orange'>Transparency</font>:"),
                                                    min=0,
                                                    max=1,
                                                    step=0.1,
                                                    value=0.5
                                                )
                                            )
                                        )
                                    )
                                )
                            }
                        )
                    )
                })

                Sys.sleep(.2)
                incProgress(amount=.5, message="Configure done ...")
                incProgress(amount=1)
                Sys.sleep(.1)
            })
        }else{
            shinyalert(
                "Oops!",
                "Please choose at lease two species for orthologous study!",
                type="error"
            )
        }
    }
    else{
        shinyalert(
            "Oops!",
            "Fail to access the output of shinyWGD. Please ensure that all the results of shinyWGD were generated successfully!",
            type="error"
        )
    }
})

observeEvent(input$ortholog_ks_plot_go, {
    shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")
    withProgress(message='Analyzing in progress', value=0, {
        if( is.null(buttonClicked()) ){
            ksAnalysisDir <- ks_example_dir
        }
        else if( buttonClicked() == "fileInput" ){
            ksAnalysisDir <- ks_analysis_dir_Val()
        }
        else if( buttonClicked() == "actionButton" ){
            ksAnalysisDir <- ks_example_dir
        }

        species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
        if( file.exists(species_info_file[1]) ){
            ksfiles <- list.files(path=ksAnalysisDir, pattern="\\.ks.tsv$", full.names=TRUE, recursive=TRUE)
            species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
            ortholog_ksfiles <- ksfiles[grepl("ortholog_distributions", ksfiles)]
            paralog_ksfiles <- ksfiles[grepl("paralog_distributions", ksfiles)]

            names_df <- map_informal_name_to_latin_name(species_info_file[1])

            widthPhyloKsSpacing <- reactiveValues(
                value=1000
            )
            heightPhyloKsSpacing <- reactiveValues(
                value=350
            )
            observeEvent(input$ks_svg_vertical_spacing_add, {
                heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value + 50
            })
            observeEvent(input$ks_svg_vertical_spacing_sub, {
                heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value - 50
            })
            observeEvent(input$ks_svg_horizontal_spacing_add, {
                widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value + 50
            })
            observeEvent(input$ks_svg_horizontal_spacing_sub, {
                widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value - 50
            })

            speciesA <- input$ortholog_ks_files_list_A

            files_list <- c()

            for( i in 1:length(speciesA) ){
                pattenEach <- which(names_df$latin_name == gsub("_", " ", speciesA[[i]]))
                each_informal_name <- names_df$informal_name[pattenEach]
                species_A_file <- ortholog_ksfiles[grepl(each_informal_name, ortholog_ksfiles)]
                for( j in 1:length(speciesA) ){
                    if( j > i ){
                        pattenEach_B <- which(names_df$latin_name == gsub("_", " ", speciesA[[j]]))
                        species_A_B_file <- species_A_file[grepl(pattenEach_B, species_A_file)]
                        if( !is.null(species_A_B_file) ){
                            files_list <- c(files_list, species_A_B_file)
                        }
                    }
                }
            }
            files_list <- unique(files_list)

            full_data <- calculateKsDistribution4wgd_multiple(
                files_list,
                maxK=input[["ks_maxK_ortholog"]],
            )
            denData <- full_data$density

            newick_tree_file <- paste0(dirname(species_info_file), "/tree.newick")
            treeTopology <- readLines(textConnection(readChar(newick_tree_file, file.info(newick_tree_file)$size)))
            closeAllConnections()

            ks_bar_data <- NULL

            if( isTruthy(input$ortholog_paranome_species) ){
                ortholog_paranome_informal_name <- names_df[names_df$latin_name == input$ortholog_paranome_species, ]$informal_name
                selected_ks_files <- c()
                selected_paralog_ks_file <- paralog_ksfiles[grepl(ortholog_paranome_informal_name, paralog_ksfiles)]
                selected_paralog_anchor_ks_file <- gsub("ks.tsv$", "ks_anchors.tsv", selected_paralog_ks_file)
                if( input$plot_ortholog_paralog_species == "All" ){
                    selected_ks_files <- c(selected_ks_files, selected_paralog_ks_file)
                    selected_ks_files <- c(selected_ks_files, selected_paralog_anchor_ks_file)
                }
                else if( input$plot_ortholog_paralog_species == "Anchor pairs" ){
                    selected_ks_files <- c(selected_ks_files, selected_paralog_anchor_ks_file)
                }
                else{
                    selected_ks_files <- c(selected_ks_files, selected_paralog_ks_file)
                }

                selected_ks_data <- calculateKsDistribution4wgd_multiple(
                    selected_ks_files
                )

                ks_bar_data <- selected_ks_data$bar
            }

            Sys.sleep(.2)
            incProgress(amount=.4, message="Calculating done...")

            observe({
                selectedDenData <- denData[denData$ks >= 0 & denData$ks <= input[["ks_maxK_ortholog"]], ]
                plot_wgd_data <- list(
                    "plot_id"="Wgd_plot_ortholog",
                    "ks_density_df"=selectedDenData,
                    "xlim"=input[["ks_maxK_ortholog"]],
                    "ylim"=input[["y_limit_ortholog"]],
                    "names_df"=names_df,
                    "color"="",
                    "opacity"=input[["opacity_paralog"]],
                    "width"=widthPhyloKsSpacing$value,
                    "height"=heightPhyloKsSpacing$value,
                    "tree_topology"=treeTopology[1]
                )

                if( !is.null(ks_bar_data) ){
                    selected_paralog_ks_bar_data <- ks_bar_data[ks_bar_data$ks <= input[["ks_maxK_ortholog"]], ]
                    plot_wgd_data[["ks_bar_df"]] <- selected_paralog_ks_bar_data
                    plot_wgd_data[["y2lim"]] <- input$y2_limit_ortholog_paralog
                    plot_wgd_data[["ortholog_paralog_species"]] <- input$ortholog_paranome_species
                }

                session$sendCustomMessage("Otholog_Density_Tree_Plotting", plot_wgd_data)
            })
            Sys.sleep(.2)
            incProgress(amount=.4, message="Ploting done...")
        }
    })
})

observeEvent(input$confirm_rate_correction_go, {
    shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")
    shinyjs::runjs("$('#confirm_rate_correction_go').css('background-color', 'green');")
    updateActionButton(
        session,
        "confirm_rate_correction_go",
        icon=icon("check")
    )

    setTimeoutFunction <- "setTimeout(function() {
              $('#confirm_rate_correction_go').css('background-color', '#C0C0C0');
              //$('#confirm_rate_correction_go').empty();
        }, 6000);"

    shinyjs::runjs(setTimeoutFunction)

    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }

    species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
    if( file.exists(species_info_file[1]) ){
        paralog_species <- input$paralog_ks_files_list
        ksfiles <- list.files(path=ksAnalysisDir, pattern="\\.ks.tsv$", full.names=TRUE, recursive=TRUE)
        species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
        ortholog_ksfiles <- ksfiles[grepl("ortholog_distributions", ksfiles)]
        paralog_ksfiles <- ksfiles[grepl("paralog_distributions", ksfiles)]

        names_df <- map_informal_name_to_latin_name(species_info_file[1])

        species_list <- lapply(gsub(".ks.tsv", "", basename(paralog_ksfiles)), function(x) {
            replace_informal_name_to_latin_name(names_df, x)
        })

        paralog_ksfile_df <- data.frame(
            species=unlist(species_list),
            path=paralog_ksfiles
        )
        selected_paralog_ksfile_df <- paralog_ksfile_df[paralog_ksfile_df$species %in% input$paralog_ks_files_list, ]
        ks_file <- selected_paralog_ksfile_df$path
        ks_anchor_file <- gsub(".ks.tsv$", ".ks_anchors.tsv", ks_file)

        withProgress(message='Configure in progress', value=0, {
            output$ks_analysis_output <- renderUI({
                div(
                    class="boxLike",
                    style="background-color: #FDFFFF;
                           padding-bottom: 10px;
                           padding-top: 10px;",
                    fluidRow(
                        column(
                            5,
                            h4(HTML("<b><font color='#91cf60'>Substitution Rate Correction</font></b>"))
                        )
                    ),
                    div(
                        style="padding: 10px 10px 10px 10px;",
                        hr(class="setting"),
                        fluidRow(
                            column(
                                2,
                                h5(HTML("<b><font color='orange'><i>K</i><sub>s</sub> </font></b> setting:")),
                            ),
                            column(
                                10,
                                uiOutput("rate_ks_setting")
                            )
                        ),
                        hr(class="setting"),
                        fluidRow(
                            column(
                                9,
                                fluidRow(
                                    column(
                                        6,
                                        tags$style(
                                            HTML(".rotate-135 {
                                                transform: rotate(135deg);
                                            }"),
                                            HTML(".rotate-45{
                                                transform: rotate(45deg);
                                            }")
                                        ),
                                        actionButton(
                                            "ks_svg_vertical_spacing_add",
                                            "",
                                            icon("arrows-alt-v"),
                                            title="Expand vertical spacing"
                                        ),
                                        actionButton(
                                            "ks_svg_vertical_spacing_sub",
                                            "",
                                            icon(
                                                "down-left-and-up-right-to-center",
                                                verify_fa=FALSE,
                                                class="rotate-135"
                                            ),
                                            title="Compress vertical spacing"
                                        ),
                                        actionButton(
                                            "ks_svg_horizontal_spacing_add",
                                            "",
                                            icon("arrows-alt-h"),
                                            title="Expand horizontal spacing"
                                        ),
                                        actionButton(
                                            "ks_svg_horizontal_spacing_sub",
                                            "",
                                            icon(
                                                "down-left-and-up-right-to-center",
                                                verify_fa=FALSE,
                                                class="rotate-45"
                                            ),
                                            title="Compress horizontal spacing"
                                        ),
                                        downloadButton_custom(
                                            "ksPlotRateDownload",
                                            title="Download the Plot",
                                            status="secondary",
                                            icon=icon("download"),
                                            class="my-download-button-class",
                                            label=".svg",
                                            style="color: #fff;
                                                   background-color: #6B8E23;
                                                   border-color: #fff;
                                                   padding: 5px 5px 5px 5px;"
                                        )
                                    )
                                )
                            )
                        )
                    ),
                    fluidRow(
                        column(
                            12,
                            div(
                                id="Wgd_plot_rate"
                            )
                        )
                    ),
                    hr(class="setting"),
                    fluidRow(
                        column(
                            2,
                            h5(HTML("<b><font color='orange'>Figure</b></font> setting:")),
                        ),
                        column(
                            10,
                            uiOutput("rate_figure_setting")
                        )
                    )
                )
            })

            # if( isTruthy(input$select_focal_species) & input$select_focal_species != ""  ){
                output$rate_ks_setting <- renderUI({
                    fluidRow(
                        column(
                            12,
                            fluidRow(
                                column(
                                    4,
                                    div(
                                        style="/*display: flex; align-items: center; */
                                               margin-bottom: -10px;
                                               border-radius: 10px;
                                               padding: 10px 10px 0px 10px;
                                               background-color: #FFF5EE;",
                                        pickerInput(
                                            inputId="plot_mode_option_rate",
                                            label=HTML("<font color='orange'><i>K</i><sub>s</sub> Mode</font>:"),
                                            choices=c("weighted", "average", "min", "pairwise"),
                                            multiple=FALSE,
                                            selected="weighted",
                                            inline=TRUE
                                        )
                                    )
                                ),
                                column(
                                    4,
                                    div(
                                        style="/*display: flex; align-items: center; */
                                               margin-bottom: -10px;
                                               border-radius: 10px;
                                               padding: 10px 10px 0px 10px;
                                               background-color: #FFF5EE;",
                                        pickerInput(
                                            inputId="plot_reference_species_data",
                                            label=HTML("<font color='orange'>Data</font> used for <font color='orange'>Paralogous <i>K</i><sub>s</sub> Distribution</font>:"),
                                            choices=c("All", "Paranome", "Anchor pairs"),
                                            multiple=FALSE,
                                            selected="All",
                                            inline=TRUE
                                        )
                                    )
                                )
                            )
                        ),
                        column(
                            12,
                            hr(class="setting"),
                            fluidRow(
                                column(
                                    4,
                                    div(
                                        style="/*display: flex; align-items: center;*/
                                               margin-bottom: -10px;
                                               border-radius: 10px;
                                               padding: 10px 10px 0px 10px;
                                               background-color: #FFF5EE;",
                                        sliderInput(
                                            inputId="ks_binWidth_rate",
                                            label=HTML("<font color='orange'>BinWidth</font>:&nbsp;"),
                                            min=0,
                                            max=0.2,
                                            step=0.01,
                                            value=0.1
                                        )
                                    )
                                ),
                                column(
                                    4,
                                    div(
                                        style="/*display: flex; align-items: center; */
                                               margin-bottom: -10px;
                                               border-radius: 10px;
                                               padding: 10px 10px 0px 10px;
                                               background-color: #FFF5EE;",
                                        sliderInput(
                                            inputId="ks_maxK_rate",
                                            label=HTML("<font color='orange'><i>K</i><sub>s</sub> limit</font>:&nbsp;"),
                                            min=0,
                                            step=1,
                                            max=5,
                                            value=5
                                        )
                                    )
                                ),
                                column(
                                    3,
                                    actionButton(
                                        inputId="rate_plot_go",
                                        "Start",
                                        icon=icon("play"),
                                        status="secondary",
                                        title="Click to start",
                                        class="my-start-button-class",
                                        style="color: #fff;
                                               background-color: #27ae60;
                                               border-color: #fff;
                                               padding: 5px 14px 5px 14px;
                                               margin: 13px 5px 15px 5px;"
                                    )
                                )
                            )
                        )
                    )
                })

                output$rate_figure_setting <- renderUI({
                    fluidRow(
                        column(
                            4,
                            div(
                                style="padding: 12px 10px 5px 10px;
                                       border-radius: 10px;
                                       background-color: #F0FFFF",
                                sliderInput(
                                    inputId="y2_limit_rate",
                                    label=HTML("<font color='orange'>Ortholog Density Y axis limit</font>:"),
                                    min=0,
                                    step=0.2,
                                    max=5,
                                    value=2
                                )
                            )
                        ),
                        column(
                            4,
                            div(
                                style="padding: 12px 10px 5px 10px;
                                       border-radius: 10px;
                                       background-color: #F0FFFF",
                                sliderInput(
                                    inputId="y1_limit_rate",
                                    label=HTML("<font color='orange'>Retained Duplicates Y axis limit</font>:"),
                                    min=0,
                                    step=200,
                                    max=6000,
                                    value=2500
                                )
                            )
                        ),
                        column(
                            4,
                            div(
                                style="padding: 12px 10px 5px 10px;
                                       border-radius: 10px;
                                       background-color: #F0FFFF",
                                sliderInput(
                                    inputId="opacity_rate",
                                    label=HTML("<font color='orange'>Transparency</font>:"),
                                    min=0,
                                    max=1,
                                    step=0.1,
                                    value=0.5
                                )
                            )
                        )
                    )
                })
            # }
            # else{
            #     output$rate_ks_setting <- renderUI({
            #         fluidRow(
            #             column(
            #                 4,
            #                 div(
            #                     style="/*display: flex; align-items: center; */
            #                                        margin-bottom: -10px;
            #                                        border-radius: 10px;
            #                                        padding: 10px 10px 0px 10px;
            #                                        background-color: #FFF5EE;",
            #                     sliderInput(
            #                         inputId="ks_maxK_rate",
            #                         label=HTML("<font color='orange'><i>K</i><sub>s</sub> limit</font>:&nbsp;"),
            #                         min=0,
            #                         step=1,
            #                         max=10,
            #                         value=5
            #                     )
            #                 )
            #             )
            #         )
            #     })
            #
            #     output$rate_figure_setting <- renderUI({
            #         fluidRow(
            #             column(
            #                 4,
            #                 div(
            #                     style="padding: 12px 10px 5px 10px;
            #                                    border-radius: 10px;
            #                                    background-color: #F0FFFF",
            #                     sliderInput(
            #                         inputId="y1_limit_rate",
            #                         label=HTML("Set the <font color='orange'>Y axis limit</font>:"),
            #                         min=0,
            #                         step=0.2,
            #                         max=5,
            #                         value=2
            #                     ),
            #                 )
            #             ),
            #             column(
            #                 4,
            #                 div(
            #                     style="padding: 12px 10px 5px 10px;
            #                                    border-radius: 10px;
            #                                    background-color: #F0FFFF",
            #                     sliderInput(
            #                         inputId="opacity_rate",
            #                         label=HTML("Set the <font color='orange'>Transparency</font>:"),
            #                         min=0,
            #                         max=1,
            #                         step=0.1,
            #                         value=0.5
            #                     )
            #                 )
            #             )
            #         )
            #     })
            # }


            Sys.sleep(.2)
            incProgress(amount=.5, message="Configure done ...")
            incProgress(amount=1)
            Sys.sleep(.1)
        })
    }
    else{
        shinyalert(
            "Oops!",
            "Fail to access the output of shinyWGD. Please ensure that all the results of shinyWGD were generated successfully!",
            type="error"
        )
    }
})

observeEvent(input$rate_plot_go, {
    shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")
    shinyjs::runjs("$('#confirm_rate_correction_go').css('background-color', 'green');")
    updateActionButton(
        session,
        "confirm_rate_correction_go",
        icon=icon("check")
    )

    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }

    species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
    if( file.exists(species_info_file[1]) ){
        paralog_species <- input$paralog_ks_files_list
        ksfiles <- list.files(path=ksAnalysisDir, pattern="\\.ks.tsv$", full.names=TRUE, recursive=TRUE)
        species_info_file <- list.files(path=ksAnalysisDir, pattern="Species.info.xls", full.names=TRUE, recursive=TRUE)
        ortholog_ksfiles <- ksfiles[grepl("ortholog_distributions", ksfiles)]
        paralog_ksfiles <- ksfiles[grepl("paralog_distributions", ksfiles)]

        names_df <- map_informal_name_to_latin_name(species_info_file[1])

        withProgress(message='Analyzing in progress', value=0, {
            widthPhyloKsSpacing <- reactiveValues(
                value=1100
            )
            heightPhyloKsSpacing <- reactiveValues(
                value=350
            )
            observeEvent(input$ks_svg_vertical_spacing_add, {
                heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value + 50
            })
            observeEvent(input$ks_svg_vertical_spacing_sub, {
                heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value - 50
            })
            observeEvent(input$ks_svg_horizontal_spacing_add, {
                widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value + 50
            })
            observeEvent(input$ks_svg_horizontal_spacing_sub, {
                widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value - 50
            })

            ks_selected_files <- c()

            refSpecies <- input$select_ref_species
            outgroupSpecies <- input$select_outgroup_species
            studySpecies <- input$select_study_species

            matching_ref <- which(names_df$latin_name == gsub("_", " ", refSpecies))
            ref_informal_name <- names_df$informal_name[matching_ref]

            matching_outgroup <- which(names_df$latin_name == gsub("_", " ", outgroupSpecies))
            outgroup_informal_name <- names_df$informal_name[matching_outgroup]

            ref2outgroupFile <- ortholog_ksfiles[grepl(ref_informal_name, ortholog_ksfiles) & grepl(outgroup_informal_name, ortholog_ksfiles)]
            ks_selected_files <- c(ks_selected_files, ref2outgroupFile)

            ref2outgroup_id <- gsub(".tsv$", "", basename(ref2outgroupFile))

            ks_files_for_correction <- c()

            mode_df <- data.frame()
            for( i in 1:length(studySpecies) ){
                pattenEach <- which(names_df$latin_name == gsub("_", " ", studySpecies[[i]]))
                each_informal_name <- names_df$informal_name[pattenEach]
                ref2studyFile <- ortholog_ksfiles[grepl(ref_informal_name, ortholog_ksfiles) & grepl(each_informal_name, ortholog_ksfiles)]
                study2outgroupFile <- ortholog_ksfiles[grepl(each_informal_name, ortholog_ksfiles) & grepl(outgroup_informal_name, ortholog_ksfiles)]
                ks_selected_files <- c(ks_selected_files, study2outgroupFile)

                ks_files_for_correction <- c(ks_files_for_correction, ref2studyFile)

                # relative rate test
                # source("tools/substitution_rate_correction.R", local=T, encoding="UTF-8")
                study.mode <- relativeRate(
                    ref2outgroupFile,
                    study2outgroupFile,
                    ref2studyFile,
                    KsMax=input[["ks_maxK_rate"]]
                )
                study.mode$ref <- refSpecies
                study.mode$outgroup <- outgroupSpecies
                study.mode$study <- studySpecies[[i]]
                df_each <- as.data.frame(t(unlist(study.mode)))
                mode_df <- rbind(mode_df, df_each)

                Sys.sleep(.2)
                incProgress(
                    amount=0.4/length(studySpecies),
                    message=paste0("Relative rate correction for ", studySpecies[[i]], " ...")
                )
            }

            Sys.sleep(.2)
            incProgress(amount=.4, message="Calculating done...")

            # if( isTruthy(input$select_focal_species) & input$select_focal_species != "" ){

            req(input[["ks_binWidth_rate"]])
            req(input[["plot_mode_option_rate"]])
            req(input[["plot_reference_species_data"]])

                pattenFocal <- which(names_df$latin_name == gsub("_", " ", input$select_ref_species))
                each_focal_name <- names_df$informal_name[pattenFocal]

                ks_file <- paralog_ksfiles[grepl(each_focal_name, paralog_ksfiles)]
                paralog_id <- gsub(".ks.tsv$", "", basename(ks_file))

                ks_anchor_file <- gsub(".ks.tsv$", ".ks_anchors.tsv", ks_file)

                files_list_new <- c()
                if( input$plot_reference_species_data == "All" ){
                    files_list_new <- c(files_list_new, ks_file)
                    files_list_new <- c(files_list_new, ks_anchor_file)
                }
                else if( input$plot_reference_species_data == "Anchor pairs" ){
                    files_list_new <- c(files_list_new, ks_anchor_file)
                }
                else{
                    files_list_new <- c(files_list_new, ks_file)
                }

                files_list_new <- c(files_list_new, ks_selected_files)

                full_data <- calculateKsDistribution4wgd_multiple(
                    files_list_new,
                    plot.mode=input[["plot_mode_option_rate"]],
                    maxK=input[["ks_maxK_rate"]],
                    binWidth=input[["ks_binWidth_rate"]]
                )
                barData <- full_data$bar
                denData <- full_data$density

                full_data_correction <- calculateKsDistribution4wgd_multiple(
                    ks_files_for_correction,
                    plot.mode=input[["plot_mode_option_rate"]],
                    maxK=input[["ks_maxK_rate"]],
                    binWidth=input[["ks_binWidth_rate"]]
                )
                denData_for_correction <- full_data_correction$density

                Sys.sleep(.2)
                incProgress(amount=.4, message="Calculating done...")

                observe({
                    selectedBarData <- barData[barData$ks >= 0 & barData$ks <= input[["ks_maxK_rate"]], ]
                    selectedDenData <- denData[denData$ks >= 0 & denData$ks <= input[["ks_maxK_rate"]], ]
                    plot_wgd_data <- list(
                        "plot_id"="Wgd_plot_rate",
                        "ks_density_df"=selectedDenData,
                        "ks_density_for_correct_df"=denData_for_correction,
                        "ref2outgroup_id"=ref2outgroup_id,
                        "species_list"=names_df,
                        "ks_bar_df"=selectedBarData,
                        "rate_correction_df"=mode_df,
                        "paralog_id"=paralog_id,
                        "paralogSpecies"=gsub("_", " ", input$select_ref_species),
                        "xlim"=input[["ks_maxK_rate"]],
                        "ylim"=input[["y1_limit_rate"]],
                        "y2lim"=input[["y2_limit_rate"]],
                        "color"="",
                        "opacity"=input[["opacity_rate"]],
                        "width"=widthPhyloKsSpacing$value,
                        "height"=heightPhyloKsSpacing$value
                    )
                    session$sendCustomMessage("Bar_Density_Plotting", plot_wgd_data)
                })
            # }
            # else{
            #     plot_wgd_data <- list(
            #         "paralog_id"=""
            #     )
            #     files_list <- ks_selected_files
            #
            #     full_data <- calculateKsDistribution4wgd_multiple(
            #         files_list
            #     )
            #     denData <- full_data$density
            #     Sys.sleep(.2)
            #     incProgress(amount=.4, message="Calculating done...")
            #
            #     observe({
            #         selectedDenData <- denData[denData$ks >= 0 & denData$ks <= input[["ks_maxK_rate"]], ]
            #         plot_wgd_data <- list(
            #             "plot_id"="Wgd_plot_rate",
            #             "ks_density_df"=selectedDenData,
            #             "rate_correction_df"=mode_df,
            #             "species_list"=names_df,
            #             "xlim"=input[["ks_maxK_rate"]],
            #             "y2lim"=input[["y1_limit_rate"]],
            #             "color"="",
            #             "opacity"=input[["opacity_rate"]],
            #             "width"=widthPhyloKsSpacing$value,
            #             "height"=heightPhyloKsSpacing$value
            #         )
            #         session$sendCustomMessage("Bar_Density_Plotting", plot_wgd_data)
            #     })
            # }
            Sys.sleep(.2)
            incProgress(amount=.4, message="Ploting done...")
        })
    }
})

observeEvent(input$confirm_phylo_ks_go, {
    shinyjs::runjs("document.querySelectorAll('svg').forEach(function(svg) { svg.remove() })")
    shinyjs::runjs("$('#confirm_phylo_ks_go').css('background-color', 'green');")
    updateActionButton(
        session,
        "confirm_phylo_ks_go",
        icon=icon("check")
    )

    setTimeoutFunction <- "setTimeout(function() {
              $('#confirm_phylo_ks_go').css('background-color', '#C0C0C0');
              //$('#confirm_phylo_ks_go').empty();
        }, 6000);"

    shinyjs::runjs(setTimeoutFunction)

    if( is.null(buttonClicked()) ){
        ksAnalysisDir <- ks_example_dir
    }
    else if( buttonClicked() == "fileInput" ){
        ksAnalysisDir <- ks_analysis_dir_Val()
    }
    else if( buttonClicked() == "actionButton" ){
        ksAnalysisDir <- ks_example_dir
    }

    output$ks_analysis_output <- renderUI({
        div(
            class="boxLike",
            style="background-color: white;
                   padding-bottom: 10px;
                   padding-top: 10px",
            fluidRow(
                column(
                    9,
                    tags$style(
                        HTML(".rotate-135 {
                                transform: rotate(135deg);
                            }"),
                        HTML(".rotate-45{
                                transform: rotate(45deg);
                            }")
                    ),
                    actionButton(
                        "phylo_ks_svg_vertical_spacing_add",
                        "",
                        icon("arrows-alt-v"),
                        title="Expand vertical spacing"
                    ),
                    actionButton(
                        "phylo_ks_svg_vertical_spacing_sub",
                        "",
                        icon(
                            "down-left-and-up-right-to-center",
                            verify_fa=FALSE,
                            class="rotate-135"
                        ),
                        title="Compress vertical spacing"
                    ),
                    actionButton(
                        "phylo_ks_svg_horizontal_spacing_add",
                        "",
                        icon("arrows-alt-h"),
                        title="Expand horizontal spacing"
                    ),
                    actionButton(
                        "phylo_ks_svg_horizontal_spacing_sub",
                        "",
                        icon(
                            "down-left-and-up-right-to-center",
                            verify_fa=FALSE,
                            class="rotate-45"
                        ),
                        title="Compress horizontal spacing"
                    ),
                    downloadButton_custom(
                        "phyloKsTreePlotDownload",
                        status="secondary",
                        icon=icon("download"),
                        title="Download the plot",
                        class="my-donwload-button-class",
                        label=HTML(""),
                        style="color: #fff;
                              background-color: #6B8E23;
                              border-color: #fff;
                              padding: 5px 14px 5px 14px;
                              margin: 5px 5px 5px 5px;"
                    )
                ),
                column(
                    12,
                    div(
                        id="phylo_ks_tree_plot",
                    )
                )
            )
        )
    })


    widthPhyloKsSpacing <- reactiveValues(value=600)
    heightPhyloKsSpacing <- reactiveValues(value=NULL)

    observe({
        if( isTruthy(input$uploadPhyloKsTree) ){
            phyloKsTreeFile <- input$uploadPhyloKsTree$datapath
            phyloKsTree <- readLines(textConnection(readChar(phyloKsTreeFile, file.info(phyloKsTreeFile)$size)))
            closeAllConnections()
            sp_count <- str_count(phyloKsTree[1], ":")
            trunc_val <- as.numeric(sp_count) * 20
            heightPhyloKsSpacing$value <- trunc_val
        }
    })

    observeEvent(input$phylo_ks_svg_vertical_spacing_add, {
        heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value + 50
    })
    observeEvent(input$phylo_ks_svg_vertical_spacing_sub, {
        heightPhyloKsSpacing$value <- heightPhyloKsSpacing$value - 50
    })
    observeEvent(input$phylo_ks_svg_horizontal_spacing_add, {
        widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value + 50
    })
    observeEvent(input$phylo_ks_svg_horizontal_spacing_sub, {
        widthPhyloKsSpacing$value <- widthPhyloKsSpacing$value - 50
    })

    observe({
        if( isTruthy(input$uploadPhyloKsTree) ){
            phyloKsTreeFile <- input$uploadPhyloKsTree$datapath
            phyloKsTree <- readLines(textConnection(readChar(phyloKsTreeFile, file.info(phyloKsTreeFile)$size)))
            closeAllConnections()

            phylo_ks_tree_data <- list(
                "width"=widthPhyloKsSpacing$value
            )

            phylo_ks_tree_data[["ksTree"]] <- phyloKsTree[1]

            if( isTruthy(input$uploadPhyloKsPeakTable) ){
                ksPeakTableFile <- input$uploadPhyloKsPeakTable$datapath
                ksPeak <- suppressMessages(
                    vroom(
                        ksPeakTableFile,
                        col_names=c("species", "type", "peak", "confidence_interval"),
                        delim="\t",
                        skip=1
                    )
                )
                if( ncol(ksPeak) == 4 ){
                    color_list <- c(
                        "#ff7f00", "#FFA750", "#0064A7", "#008DEC",
                        "#088A00", "#0CD300", "#e31a1c", "#fb9a99", "#cab2d6"
                    )
                    ksPeak$color <- color_list[match(ksPeak$species, unique(ksPeak$species))]
                }
                phylo_ks_tree_data[["ksPeak"]] <- ksPeak
            }

            phylo_ks_tree_data[["height"]] <- heightPhyloKsSpacing$value
            phylo_ks_tree_data[["plot_id"]] <- "phylo_ks_tree_plot"
            phylo_ks_tree_data[["download_id"]] <- "phyloKsTreePlotDownload"
            session$sendCustomMessage("jointTreePlot", phylo_ks_tree_data)
        }
    })
})

observeEvent(input$upload_data_file_example, {
    species_data_file <- "www/content/4sp_data_file_example.xls"
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b>Tab-Separated</b></font> File"),
            size="xl",
            uiOutput("upload_data_file_example_panel")
        )
    )

    output$uploadSpeciesDataExampleTable <- renderTable({
        species_info_example <- read.delim(
            species_data_file,
            header=FALSE,
            col.names=c("species name", "cds file", "gff file"),
            sep="\t",
            quote=""
        )
    })

    output$upload_data_file_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    h5(
                        HTML(
                            paste0(
                                "Each row of this file contains the data information of a species"
                            )
                        )
                    ),
                    tableOutput("uploadSpeciesDataExampleTable"),
                    HTML(
                        paste0(
                            "<sup>a</sup> <b>cds</b> is the coding region of a gene or the portion of a gene's DNA or RNA that codes for protein. This file can be <b>fasta</b> or <b>gzipped-fasta</b>.<br>",
                            "This column is <strong>mandatory</strong>.<br>",
                            "<font color='green'><i aria-label='warning icon' class='fa fa-warning fa-fw' role='presentation'></i></font> Do not contain the <a href='https://en.wikipedia.org/wiki/Alternative_splicing' target='_blank'>alternative isoforms</a> of each gene.<br>",
                            "<sup>b</sup> <b>gff</b> is the format consists of one line per feature, each containing nine columns of data, plus optional track definition lines, see more <a href='https://www.ensembl.org/info/website/upload/gff.html' target='_blank'>click here</a>.<br>",
                            "The file can be <b>gff</b>, <b>gff3</b>, <b>gzipped-gff</b>, or <b>gzipped-gff3</b>.<br>",
                            "This column is <strong>mandatory</strong> for the <strong>focal species</strong> and is <strong>optional</strong> for the <strong>other species</strong>.</br>",
                            "<b>Make sure you upload fasta and gff files with the same name in this table field.</b>",
                            "<p><br></br></p>",
                            "<p>To download the demo data, <a href='https://github.com/li081766/shinyWGD_Demo_Data/blob/main/4sp_orchids.CDS_GFF_Example_Data.tar.gz' target='_blank'>click here</a>.</p>"
                        )
                    )
                )
            )
        )
    })
})

output$UploadDisplay <- renderUI({
    sp_range <- 1:input$number_of_study_species
    ui_parts <- c()
    scientific_names <- readLines("www/content/scientific_names.xls")
    scientific_names_selected <- rep(scientific_names, length.out=input$number_of_study_species)
    for( i in sp_range ){
        if( i == 1 ){
            ui_parts[[i]] <- fluidRow(
                tags$style(
                    HTML(
                        "input::placeholder {
                      font-style: italic;
                    }"
                    )
                ),
                column(
                    4,
                    textInput(
                        paste0("latin_name_", i),
                        paste("Species ", i, " Latin Name:"),
                        value="",
                        width="100%",
                        placeholder=scientific_names_selected[[i]]
                    )
                ),
                column(
                    3,
                    fileInput(
                        paste0("proteome_", i),
                        HTML("Upload <font color='green'><b>Fasta</b></font> File:"),
                        multiple=FALSE,
                        width="100%",
                        accept=c(
                            ".fasta",
                            ".fas",
                            ".fa",
                            ".fasta.gz",
                            ".fa.gz",
                            ".fas.gz",
                            ".gz"
                        )
                    )
                ),
                column(
                    1,
                    actionButton(
                        inputId="fasta_file_example",
                        "",
                        icon=icon("question"),
                        title="Click to see the example of the CDS Fasta File",
                        status="secondary",
                        class="my-start-button-class",
                        style="text-align: left;
                               color: #fff;
                               background-color: #87CEEB;
                               border-color: #fff;
                               padding: 5px 14px 5px 10px;
                               margin: 33px 5px 5px -15px;
                               width: 30px; height: 30px; border-radius: 50%;"
                    )
                ),
                column(
                    3,
                    fileInput(
                        paste0("gff_", i),
                        HTML("Upload <font color='green'><b>GFF</b></font> File:"),
                        multiple=FALSE,
                        width="100%",
                        accept=c(
                            ".gff",
                            ".gff.gz",
                            ".gff3",
                            ".gff3.gz",
                            ".gtf",
                            ".gtf.gz",
                            ".gz"
                        )
                    )
                ),
                column(
                    1,
                    actionButton(
                        inputId="gff_file_example",
                        "",
                        icon=icon("question"),
                        title="Click to see the example of the Annotation GFF File",
                        status="secondary",
                        class="my-start-button-class",
                        style="text-align: left;
                               color: #fff;
                               background-color: #87CEEB;
                               border-color: #fff;
                               padding: 5px 14px 5px 10px;
                               margin: 33px 5px 5px -15px;
                               width: 30px; height: 30px; border-radius: 50%;"
                    )
                )
            )
        }else{
            ui_parts[[i]] <- fluidRow(
                tags$style(
                    HTML(
                        "input::placeholder {
                      font-style: italic;
                    }"
                    )
                ),
                column(
                    4,
                    textInput(
                        paste0("latin_name_", i),
                        paste("Species ", i, " Latin Name:"),
                        value="",
                        width="100%",
                        placeholder=scientific_names_selected[[i]]
                    )
                ),
                column(
                    4,
                    fileInput(
                        paste0("proteome_", i),
                        HTML("Upload <font color='green'><b>CDS Fasta</b></font> File:"),
                        multiple=FALSE,
                        width="100%",
                        accept=c(
                            ".fasta",
                            ".fas",
                            ".fa",
                            ".fasta.gz",
                            ".fa.gz",
                            ".fas.gz",
                            ".gz"
                        )
                    )
                ),
                column(
                    4,
                    fileInput(
                        paste0("gff_", i),
                        HTML("Upload <font color='green'><b>GFF</b></font> File:"),
                        multiple=FALSE,
                        width="100%",
                        accept=c(
                            ".gff",
                            ".gff.gz",
                            ".gff3",
                            ".gff3.gz",
                            ".gtf",
                            ".gtf.gz",
                            ".gz"
                        )
                    )
                )
            )
        }
    }
    ui_parts
})

observeEvent(input$fasta_file_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b>CDS Fasta</b></font> file"),
            size="xl",
            uiOutput("fasta_file_example_panel")
        )
    )

    output$fasta_file_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    verbatimTextOutput("cdsFastaExample"),
                    HTML(
                        paste0(
                            "<b>cds</b> is the coding region of a gene or the portion of a gene's DNA or RNA that codes for protein. This file can be <b>fasta</b> or <b>gzipped-fasta</b>.<br>",
                            "<font color='green'><i aria-label='warning icon' class='fa fa-warning fa-fw' role='presentation'></i></font> Do not contain the <a href='https://en.wikipedia.org/wiki/Alternative_splicing' target='_blank'>alternative isoforms</a> of each gene.<br>",
                            "The two sequences are from <i>Oryza sativa</i>."
                        )
                    )
                )
            )
        )
    })

    output$cdsFastaExample <- renderText({
    ">Os01t0100466-00 | Os01g0100466
ATGCCGCAGTTTGTGCCGCCCACGCCGTCCTGCCAGGGGCTCTTGCGCTGCTGCACCCCGTGCCACGTCAGCAG
CAGCGGCTCGTCCAGGCCGTTCCTCACGTTCACCACCAGGTTCCAGTTGGTCGTCACGTTCAGCGCCGGCCCCG
GCAGCTGCCCGTTGATGCCAATCGCCTCCTGCTTCTTCACTCCGCCCAGCGGCGCACCCCACACGTACGATACC
TCCCACTCGTAG
>Os01t0100200-01 | Os01g0100200
ATGGAGGAGGCTGGCGAGCGGGACGCTGACGAGACGCACGCGTGGAGCGGAACAGCATCGCCTGCAGCTTTGTG
GAAGACCGTGGCGTCGTCGGCGGCGATGCTGAAGCTGGCCTTGGCGATGATCTCGGCGGCGTTCCGGACAACGC
CCTTCTCGATGTCGATGCAGCTGTGTCCCAACGCCACTATGTCGCTCCACTCGCCGAGCATCTTCGACGTCGTC
TCCTCCATCACGCCGATCATGTCCTGCATCATCAACAACAGGTTGGTGGCGGAGAAGGCAGGGGCGACGATGCA
GCGGTGGCGAGCCCACTCGTCGCCCTCGGCCATGACGCGGCCTCTCCCGAACATGGGCATGCGGTTGAGCAGTT
ACGATATAGTGTGCCAATTGGCACACCTACATTTTAGTCATGTATGTTGTTTAGTTTAA
"
    })
})

observeEvent(input$gff_file_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b>Anntation GFF</b></font> file"),
            size="xl",
            uiOutput("gff_file_example_panel")
        )
    )

    output$gff_file_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    verbatimTextOutput("gffExample"),
                    HTML(
                        paste0(
                            "<b>gff</b> is the format consists of one line per feature, each containing nine columns of data, plus optional track definition lines, see more <a href='https://www.ensembl.org/info/website/upload/gff.html' target='_blank'>click here</a>.<br>",
                            "The file can be <b>gff</b>, <b>gff3</b>, <b>gzipped-gff</b>, or <b>gzipped-gff3</b>.<br>",
                            "The example items are from <i>Oryza sativa</i>."
                        )
                    )
                )
            )
        )
    })

    output$gffExample <- renderText({
        "chr01	IRGSP-1.0-2021-05-10	gene	12808	13978	.	-	.	ID=Os01g0100466;tid=Os01t0100466-00;uniprot=A0A0P0UXH5;Name=Os01g0100466;gene_id=Os01g0100466
chr01	IRGSP-1.0-2021-05-10	mRNA	12808	13978	.	-	.	ID=Os01t0100466-00;Parent=Os01g0100466;Name=Os01g0100466;gene_id=Os01g0100466
chr01	IRGSP-1.0-2021-05-10	exon	12808	13782	.	-	.	ID=Os01t0100466-00:exon:1;Parent=Os01t0100466-00;Name=Os01g0100466;gene_id=Os01g0100466
chr01	IRGSP-1.0-2021-05-10	exon	13880	13978	.	-	.	ID=Os01t0100466-00:exon:2;Parent=Os01t0100466-00;Name=Os01g0100466;gene_id=Os01g0100466
chr01	IRGSP-1.0-2021-05-10	gene	11218	12435	.	+	.	ID=Os01g0100200;uniprot=B9EYQ4;MSU-ID=LOC_Os01g01019.1;tid=Os01t0100200-01;Name=Os01g0100200;gene_id=Os01g0100200
chr01	IRGSP-1.0-2021-05-10	mRNA	11218	12435	.	+	.	ID=Os01t0100200-01;Parent=Os01g0100200;Name=Os01g0100200;gene_id=Os01g0100200
chr01	IRGSP-1.0-2021-05-10	exon	11218	12060	.	+	.	ID=Os01t0100200-01:exon:1;Parent=Os01t0100200-01;Name=Os01g0100200;gene_id=Os01g0100200
chr01	IRGSP-1.0-2021-05-10	exon	12152	12435	.	+	.	ID=Os01t0100200-01:exon:2;Parent=Os01t0100200-01;Name=Os01g0100200;gene_id=Os01g0100200
"
    })
})

output$WgdksratesSettingDisplay <- renderUI({
    num <- toupper(as.english(input$number_of_study_species))
    if( input$number_of_study_species < 2 ){
        mode <- "Whole-Paranome"
        fluidRow(
            class="justify-content-end",
            style="padding-bottom: 5px;",
            column(
                12,
                div(HTML("Species number is set to <b><font color='#9F5000'>",
                        num,
                        "</b></font>, less than <b><font color='red'>TWO</font></b>. ",
                        "Following <b>WGD ",
                        "<font color='green'>",
                        mode,
                        "</font>",
                        "</b> pipeline ...<br></br>"),
                    div(class="d-flex justify-content-between",
                        div(class="float-left",
                            actionButton(
                                inputId="wgd_go",
                                HTML("Create <b><i>wgd</i></b> Codes"),
                                icon=icon("screwdriver-wrench"),
                                title="Click to create wgd codes",
                                status="secondary",
                                class="my-start-button-class",
                                style="color: #fff;
                                       background-color: #27ae60;
                                       border-color: #fff;
                                       padding: 5px 14px 5px 14px;
                                       margin: 5px 5px 5px 5px; "
                            ),
                            div(
                                id="wgd_progress_container_js"
                            )
                        ),
                        div(class="float-right",
                            style="padding-top: 15px; ",
                            actionLink(
                                "go_codes_wgd",
                                HTML(
                                    paste0(
                                        "<font color='#5151A2'>",
                                        icon("share"),
                                        " Go to <i><b>wgd</b></i> Scripts</font>"
                                    )
                                )
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to see detail wgd codes",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                )
                        )
                    )
                )
            )
        )
    }
    else{
        fluidRow(
            div(
                style="padding-left: 20px;
                       padding-right: 20px;",
                fluidRow(
                    column(
                        12,
                        h6(
                            HTML(
                                "Species number is set to <b><font color='#9F5000'>",
                                num,
                                "</b></font>, larger than <b><font color='red'>ONE</font></b>.",
                                " Following <b><font color='green'>ksrates</font></b> pipeline...<br></br>"
                            )
                        )
                    ),
                    column(
                        6,
                        pickerInput(
                            inputId="select_focal_species",
                            label=HTML("Set <b>Focal Species</b> for <b><font color='green'>ksrates</font></b>:"),
                            options=list(
                                title='Please select focal species below'
                            ),
                            choices=NULL,
                            multiple=FALSE
                        )
                    ),
                    column(
                        5,
                        fileInput(
                            "newick_tree",
                            HTML("Upload <b>a Newick Tree</b> for <b><font color='green'>ksrates</font></b>:"),
                            multiple=FALSE,
                            width="100%"
                        )
                    ),
                    column(
                        1,
                        actionButton(
                            inputId="newick_file_example",
                            "",
                            icon=icon("question"),
                            title="Click to see the example of the Newick Tree File",
                            status="secondary",
                            class="my-start-button-class",
                            style="text-align: left;
                                   color: #fff;
                                   background-color: #87CEEB;
                                   border-color: #fff;
                                   padding: 5px 14px 5px 10px;
                                   margin: 33px 5px 5px -15px;
                                   width: 30px; height: 30px; border-radius: 50%;"
                        )
                    ),
                    column(
                        12,
                        uiOutput("multipleSpeciesPanel")
                    ),
                ),
                hr(class="setting"),
                fluidRow(
                    column(
                        6,
                        actionButton(
                            inputId="ksrates_go",
                            HTML("Create <b><i>ksrates</b></i> Codes"),
                            width="230px",
                            icon=icon("screwdriver-wrench"),
                            title="Click to create ksrates codes",
                            status="secondary",
                            class="my-start-button-class",
                            style="color: #fff;
                                   background-color: #27ae60;
                                   border-color: #fff;
                                   padding: 5px 14px 5px 14px;
                                   margin: 5px 5px 5px 5px;"
                        ),
                        div(
                            id="ksrates_progress_container_js"
                        )
                    ),
                    column(
                        6,
                        div(class="float-right",
                            style="padding-top: 15px; ",
                            actionLink(
                                "go_codes_ksrates",
                                HTML(
                                    paste0(
                                        "<font color='#5151A2'>",
                                        icon("share"),
                                        " Go to <i><b>ksrates</b></i> Scripts</font>"
                                    )
                                )
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to review ksrates codes",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                )
                        )
                    )
                ),
                fluidRow(
                    column(
                        6,
                        div(class="float-left",
                            actionButton(
                                inputId="iadhore_go",
                                HTML("Create <b><i>i-ADHoRe</b></i> Codes"),
                                width="245px",
                                icon=icon("screwdriver-wrench"),
                                title="Click to create i-ADHoRe codes",
                                status="secondary",
                                class="my-start-button-class",
                                style="color: #fff;
                                       background-color: #27ae60;
                                       border-color: #fff;
                                       padding: 5px 14px 5px 14px;
                                       margin: 5px 5px 5px 5px;"
                            ),
                            div(
                                id="iadhore_progress_container_js"
                            )
                        )
                    ),
                    column(
                        6,
                        div(class="float-right",
                            style="padding-top: 15px;",
                            actionLink(
                                "go_codes_iadhore",
                                HTML(
                                    paste0(
                                        "<font color='#5151A2'>",
                                        icon("share"),
                                        " Go to <i><b>i-ADHoRe</b></i> Scripts</font>"
                                    )
                                )
                            ) %>%
                                bs_embed_tooltip(
                                    title="Click to review i-ADHoRe codes",
                                    placement="right",
                                    trigger="hover",
                                    options=list(container="body")
                                )
                        )
                    )
                ),
                if( input$number_of_study_species > 2 ){
                    fluidRow(
                        column(
                            6,
                            div(
                                style="padding-bottom: 15px;",
                                actionButton(
                                    inputId="orthofinder_go",
                                    HTML("Create <i><b>OrthoFinder</b></i> Codes"),
                                    width="265px",
                                    icon=icon("screwdriver-wrench"),
                                    title="Click to create OrthoFinder codes",
                                    status="secondary",
                                    class="my-start-button-class",
                                    style="color: #fff;
                                           background-color: #27ae60;
                                           border-color: #fff;
                                           padding: 5px 14px 5px 14px;
                                           margin: 5px 5px 5px 5px;"
                                ),
                                div(
                                    id="orthofinder_progress_container_js"
                                )
                            )
                        ),
                        column(
                            6,
                            div(class="float-right",
                                style="padding-top: 15px;",
                                actionLink(
                                    "go_codes_orthofinder",
                                    HTML(
                                        paste0(
                                            "<font color='#5151A2'>",
                                            icon("share"),
                                            " Go to <i><b>OrthoFinder</b></i> Scripts</font>"
                                        )
                                    )
                                ) %>%
                                    bs_embed_tooltip(
                                        title="Click to review OrthoFinder codes",
                                        placement="right",
                                        trigger="hover",
                                        options=list(container="body")
                                    )
                            )
                        )
                    )
                }
            )
        )
    }
})

observeEvent(input$newick_file_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b>Newick Tree</b></font> file"),
            size="xl",
            uiOutput("newick_file_example_panel")
        )
    )

    output$newick_file_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    verbatimTextOutput("newickExample"),
                    HTML(
                        paste0(
                            "see more <a href='https://en.wikipedia.org/wiki/Newick_format#:~:text=In%20mathematics%2C%20Newick%20tree%20format,Maddison%2C%20Christopher%20Meacham%2C%20F.' target='_blank'>click here</a>."
                        )
                    )
                )
            )
        )
    })

    output$newickExample <- renderText({
        "(Vitis_vinifera,(Asparagus_officinalis,(Elaeis_guineensis,Oryza_sativa)));"
    })
})

output$multipleSpeciesPanel <- renderUI({
    if( input$number_of_study_species > 2 ){
        div(
            column(
                12,
                HTML(
                    paste(
                        "If the below mode is enabled, <font color='green'><b>i-ADHoRe</b></font> will analyze all the species within a single run.",
                        "The corresponding code will be added to the main script of <font color='green'><i><b>run_diamond_iadhore.sh</b></i></font>.",
                        "For more details, refer to the <font color='green'><b>i-ADHoRe</b></font> manual."
                    )
                )
            ),
            column(
                12,
                style="margin-top: 5px;",
                prettyToggle(
                    inputId="multiple_iadhore",
                    label_on="Yes!",
                    icon_on=icon("check"),
                    status_on="info",
                    status_off="warning",
                    label_off="No..",
                    bigger=TRUE,
                    icon_off=icon("remove"),
                    animation="rotate"
                ) %>%
                    bs_embed_tooltip(
                        title="Switch On to activate the multiple speices model",
                        placement="right",
                        trigger="hover",
                        options=list(container="body")
                    )
            )
        )
    }
})

output$WgdKsratesIadhoreScriptRun <- renderUI({
    fluidRow(
        class="justify-content-end",
        style="padding-bottom: 5px;",
        column(
            12,
            actionButton(
                inputId="job_run_server",
                "Submit Jobs",
                icon=icon("paper-plane"),
                title="Click to submit the job to the PSB computing server",
                status="secondary",
                class="my-start-button-class",
                style="text-align: left;
                       color: #fff;
                       background-color: #27ae60;
                       border-color: #fff;
                       padding: 5px 14px 5px 14px;
                       margin: 5px 5px 5px 5px; "
            )
        ),
        column(
            12,
            uiOutput("job_unique_id")
        )
    )
})

output$WgdKsratesIadhoreDataDownload <- renderUI({
    fluidRow(
        column(
            12,
            div(class="float-left",
                downloadButton_custom(
                    outputId="wgd_ksrates_data_download",
                    label="Download Data and Scripts",
                    icon=icon("download"),
                    title="Click to download the analysis data and the scripts",
                    status="secondary",
                    class="my-download-button-class",
                    style="color: #fff;
                           background-color: #6B8E23;
                           border-color: #fff;
                           padding: 5px 14px 5px 14px;
                           margin: 5px 5px 5px 5px;"
                )
            ),
            # div(class="float-right",
            #     downloadButton(
            #         outputId="wgd_ksrates_ouput_download",
            #         label="Download Output Data",
            #         icon=icon("download"),
            #         status="secondary",
            #         style="background-color: #5151A2;
            #                padding: 5px 10px 5px 10px;
            #                margin: 5px 5px 5px 5px;
            #                animation: glowingD 5000ms infinite; "
            #     )
            # )
        )
    )
})

updateProgress <- function(container, width, type) {
    session$sendCustomMessage(
        "UpdateProgressBar",
        list(container=container, width=width, type=type)
    )
}

observeEvent(input$switchTab, {
    if( input$switchTab=="help" ){
        updateTabsetPanel(session, "shinywgd", selected="help")
    }
})

data_preparation_dir_Val <- reactiveVal(NULL)
original_data_wd_Val <- reactiveVal(NULL)

observe({
    if(
        isTruthy(input$upload_data_file) ||
        isTruthy(input$selected_data_files) ||
        isTruthy(input$proteome_1)
    ){
        observeEvent(input$upload_data_file, {
            if( !is.null(input$upload_data_file) ){
                shinyalert(
                    "Success",
                    "You use a file file to upload the data. See more details in Help page",
                    type="success"
                )
                data_table <- read_data_file(input$upload_data_file)
                ncols <- ncol(data_table)
                nrows <- nrow(data_table)

                if( input$number_of_study_species != nrows ){
                    shinyalert(
                        "Oops!",
                        paste0("The species number in the file (", nrows, " species) is not equal to the number you chose (", input$number_of_study_species, " species). Please set the right species number to analyze!"),
                        type="error",
                    )
                }else{
                    # for server
                    # base_dir <- "/www/bioinformatics01_rw/ShinyWGD"
                    # timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
                    # working_wd <- file.path(base_dir, paste0("Analysis_", gsub("[ :\\-]", "_", timestamp)))

                    base_dir <- tempdir()
                    timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
                    working_wd <- file.path(base_dir, paste0("Analysis_", gsub("[ :\\-]", "_", timestamp)))

                    while( dir.exists(working_wd) ){
                        Sys.sleep(1)
                        timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
                        working_wd <- file.path(base_dir, paste0("Analysis_", gsub("[ :\\-]", "_", timestamp)))
                    }

                    dir.create(working_wd)
                    original_data_wd <- file.path(working_wd, "original_data")
                    dir.create(original_data_wd)

                    data_preparation_dir_Val(working_wd)
                    # system(paste("chmod -R 777", working_wd))
                    original_data_wd_Val(original_data_wd)
                    # system(paste("chmod -R 777", original_data_wd))

                    system(
                        paste0(
                            "cp ",
                            input$upload_data_file$datapath,
                            " ",
                            original_data_wd,
                            "/data.original.xls"
                        )
                    )
                }
            }
        })

        observeEvent(input$selected_data_files, {
            working_wd <- data_preparation_dir_Val()
            original_data_wd <- original_data_wd_Val()
            if( isTruthy(input$selected_data_files) ){
                for( i in 1:nrow(input$selected_data_files) ){
                    system(
                        paste0(
                            "cp ",
                            input$selected_data_files[i, "datapath"],
                            " ",
                            original_data_wd,
                            "/",
                            input$selected_data_files[i, "name"]
                        )
                    )
                }

                # check file existence
                withProgress(message='Checking the path of input files', value=0, {
                    original_data_df <- read.delim(
                        paste0(original_data_wd, "/data.original.xls"),
                        sep="\t",
                        header=FALSE,
                        fill=T,
                        na.strings=""
                    )
                    checkFileExistence(original_data_df, original_data_wd)
                    incProgress(amount=1)
                })
            }
        })

        if( !is.null(input$proteome_1) ){
            # for server
            # base_dir <- "/www/bioinformatics01_rw/ShinyWGD"
            # timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
            # working_wd <- file.path(base_dir, paste0("Analysis_", gsub("[ :\\-]", "_", timestamp)))

            base_dir <- tempdir()
            timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
            working_wd <- file.path(base_dir, paste0("Analysis_", gsub("[ :\\-]", "_", timestamp)))

            while( dir.exists(working_wd) ){
                Sys.sleep(1)
                timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
                working_wd <- file.path(base_dir, paste0("Analysis_", gsub("[ :\\-]", "_", timestamp)))
            }

            dir.create(working_wd)

            data_preparation_dir_Val(working_wd)
            # system(paste("chmod -R 777", working_wd))
        }
    }
})

# update the focal species panel
get_species_from_input <- reactive({
    sp_range <- 1:input$number_of_study_species
    latin_names_list <- c()
    for( i in sp_range ){
        latin_name <- paste0("latin_name_", i)
        if( !is.null(input[[latin_name]]) ){
            latin_names_list <- c(latin_names_list, trimws(input[[latin_name]]))
        }
    }
    return(latin_names_list)
})
observeEvent(get_species_from_input(), {
    updatePickerInput(
        session,
        "select_focal_species",
        choices=get_species_from_input(),
        choicesOpt=list(
            content=lapply(get_species_from_input(), function(choice) {
                choice <- gsub("_", " ", choice)
                HTML(paste0("<div style='color: #5667E5; font-style: italic;'>", choice, "</div>"))
            })
        )
    )
})

get_species_from_file <- reactive({
    if( !is.null(input$upload_data_file)) {
        read_data_file(input$upload_data_file)[["V1"]]
    }
})
observeEvent(get_species_from_file(), {
    updatePickerInput(
        session,
        "select_focal_species",
        choices=get_species_from_file(),
        choicesOpt=list(
            content=lapply(get_species_from_file(), function(choice) {
                choice <- gsub("_", " ", choice)
                HTML(paste0("<div style='color: #5667E5; font-style: italic;'>", choice, "</div>"))
            })
        )
    )
})

observeEvent(input$wgd_go, {
    working_wd <- data_preparation_dir_Val()
    original_data_wd <- original_data_wd_Val()
    if( is.null(input$upload_data_file) ){
        if( is.null(input[[paste0("proteome_", 1)]]) ){
            shinyalert(
                "Oops!",
                "Please upload the data from at least one species, then switch this on ...",
                type="error"
            )
        }
        else{
            shinyjs::runjs('$("#progress_modal").modal("show");')
            progress_data <- list(
                "actionbutton"="wgd_go",
                "container"="wgd_progress_container_js"
            )
            session$sendCustomMessage(
                "Progress_Bar_Complete",
                progress_data
            )
            withProgress(message='Creating in progress', value=0, {
                incProgress(amount=.2, message="Processing files ...")
                updateProgress("wgd_progress_container_js", 20, "Create wgd code")
                Sys.sleep(.5)

                species_info_list <- c()

                latin_name <- gsub(" $", "", input[[paste0("latin_name_", 1)]])
                informal_name <- gsub(" ", "_", latin_name)

                species_info_list <- c(species_info_list, informal_name)

                query_proteome_t <- check_proteome_input(
                    informal_name,
                    input[[paste0("proteome_", 1)]],
                    working_wd
                )
                species_info_list <- c(species_info_list, paste0(informal_name, ".fa"))

                incProgress(amount=.6, message="Creating WGD Runing Script ...")
                updateProgress("wgd_progress_container_js", 60, "Create wgd code")
                Sys.sleep(.5)

                wgd_working_dir <- paste0(working_wd, "/wgd_wd")
                if( !dir.exists(wgd_working_dir) ){
                    dir.create(wgd_working_dir)
                    # system(paste("chmod -R 777", wgd_working_dir))
                }
                wgd_cmd_sh_file <- paste0(wgd_working_dir, "/run_wgd.sh")
                wgd_cmd_con <- file(wgd_cmd_sh_file, open="w")
                # create wgd runing script
                writeLines(
                    c(
                        "#!/bin/bash",
                        "",
                        "#SBATCH -p all",
                        "#SBATCH -c 4",
                        "#SBATCH --mem 8G",
                        paste0("#SBATCH -o ", basename(wgd_cmd_sh_file), ".o%j"),
                        paste0("#SBATCH -e ", basename(wgd_cmd_sh_file), ".e%j"),
                        ""
                    ),
                    wgd_cmd_con
                )

                writeLines(
                    "module load wgd mcl diamond mafft fasttree",
                    wgd_cmd_con
                )

                writeLines(
                    paste0("wgd dmd -I 3 ../", informal_name, ".fa -o 01.wgd_dmd --nostrictcds"),
                    wgd_cmd_con
                )

                writeLines(
                    paste0(
                        "wgd ksd 01.wgd_dmd/", informal_name, ".fa.mcl ../",
                        informal_name, ".fa -o 02.wgd_ksd"
                    ),
                    wgd_cmd_con
                )

                if( is.null(input[[paste0("gff_", 1)]]) ){
                    shinyalert(
                        "Warning",
                        "No annotation file uploaded. Skip the synteny analysis in WGD pipeline",
                        type="warning",
                    )
                }
                else{
                    query_gff_t <- check_gff_input(
                        informal_name,
                        input[[paste0("gff_", 1)]],
                        working_wd
                    )

                    species_info_list <- c(species_info_list, paste0(informal_name, ".gff"))

                    writeLines(
                        paste0(
                            "wgd syn -f mRNA -a ID -ks 02.wgd_ksd/", informal_name, ".fa.ks.tsv ../",
                            informal_name, ".gff 01.wgd_dmd/", informal_name, ".fa.mcl ",
                            "-o 03.wgd_syn"
                        ),
                        wgd_cmd_con
                    )
                }

                writeLines(
                    paste0(
                        "wgd mix -ni 100 --method bgmm 02.wgd_ksd/", informal_name, ".fa.ks.tsv",
                        " -o 04.wgd_mix"
                    ),
                    wgd_cmd_con
                )
                close(wgd_cmd_con)

                species_info_file <- paste0(working_wd, "/Species.info.xls")
                write.table(
                    paste(species_info_list, collapse="\t"),
                    file=species_info_file,
                    sep="\t",
                    col.names=FALSE,
                    row.names=FALSE,
                    quote=FALSE
                )

                incProgress(amount=1)
                updateProgress("wgd_progress_container_js", 100, "Create wgd code")
                Sys.sleep(2)
            })
            shinyjs::runjs('$("#progress_modal").modal("hide");');
        }
    }
    else{
        if( !is.null(input$selected_data_files) ){
            data_table <- read_data_file(input$upload_data_file)
            data_table <- read.delim(
                paste0(original_data_wd, "/data.original.xls"),
                sep="\t",
                header=FALSE,
                fill=T,
                na.strings=""
            )
            ncols <- ncol(data_table)
            nrows <- nrow(data_table)
            if( nrows == 1 ){
                shinyjs::runjs('$("#progress_modal").modal("show");')
                progress_data <- list(
                    "actionbutton"="wgd_go",
                    "container"="wgd_progress_container_js"
                )
                session$sendCustomMessage(
                    "Progress_Bar_Complete",
                    progress_data
                )
                withProgress(message='Creating in progress', value=0, {
                    incProgress(amount=.2, message="Processing files ...")
                    updateProgress("wgd_progress_container_js", 20, "Create wgd code")
                    Sys.sleep(.5)

                    species_info_list <- c()

                    latin_name <- gsub(" ", "_", data_table[1, "V1"])
                    proteome <- paste0(original_data_wd, "/", data_table[1, "V2"])
                    proteome_checked <- check_proteome_from_file(
                        latin_name,
                        proteome,
                        working_wd
                    )

                    species_info_list <- c(species_info_list, latin_name)
                    species_info_list <- c(species_info_list, paste0(latin_name, ".fa"))

                    incProgress(amount=.6, message="Creating WGD Runing Script ...")
                    updateProgress("wgd_progress_container_js", 60, "Create wgd code")
                    Sys.sleep(.5)

                    wgd_working_dir <- paste0(working_wd, "/wgd_wd")
                    if( !dir.exists(wgd_working_dir) ){
                        dir.create(wgd_working_dir)
                        # system(paste("chmod -R 777", wgd_working_dir))
                    }
                    wgd_cmd_sh_file <- paste0(wgd_working_dir, "/run_wgd.sh")
                    wgd_cmd_con <- file(wgd_cmd_sh_file, open="w")
                    writeLines(
                        c(
                            "#!/bin/bash",
                            "",
                            "#SBATCH -p all",
                            "#SBATCH -c 4",
                            "#SBATCH --mem 8G",
                            paste0("#SBATCH -o ", basename(wgd_cmd_sh_file), ".o%j"),
                            paste0("#SBATCH -o ", basename(wgd_cmd_sh_file), ".e%j"),
                            ""
                        ),
                        wgd_cmd_con
                    )

                    writeLines(
                        "module load wgd mcl diamond mafft fasttree",
                        wgd_cmd_con
                    )

                    writeLines(
                        paste0("wgd dmd -I 3 ../", latin_name, ".fa -o 01.wgd_dmd"),
                        wgd_cmd_con
                    )

                    writeLines(
                        paste0(
                            "wgd ksd 01.wgd_dmd/",
                            latin_name, ".fa.mcl ../",
                            latin_name, ".fa ",
                            "-o 02.wgd_ksd"
                        ),
                        wgd_cmd_con
                    )
                    if( ncols > 2 ){
                        gff <- paste0(original_data_wd, "/", data_table[1, "V3"])
                        gff_checked <- check_gff_from_file(
                            latin_name,
                            gff,
                            working_wd
                        )
                        writeLines(
                            paste0(
                                "wgd syn -f mRNA -a ID -ks ",
                                "02.wgd_ksd/", latin_name, ".fa.ks.tsv ../",
                                latin_name, ".gff 01.wgd_dmd/", latin_name, ".fa.mcl ",
                                "-o 03.wgd_syn"
                            ),
                            wgd_cmd_con
                        )

                        species_info_list <- c(species_info_list, paste0(latin_name, ".gff"))
                    }
                    else if( ncols == 2 ){
                        shinyalert(
                            "Warning",
                            "No annotation file found. Skip the synteny analysis in WGD pipeline",
                            type="warning",
                        )
                    }
                    writeLines(
                        paste0(
                            "wgd mix -ni 100 --method bgmm 02.wgd_ksd/", latin_name, ".fa.ks.tsv",
                            " -o 04.wgd_mix"
                        ),
                        wgd_cmd_con
                    )
                    close(wgd_cmd_con)

                    species_info_file <- paste0(working_wd, "/Species.info.xls")
                    write.table(
                        paste(species_info_list, collapse="\t"),
                        file=species_info_file,
                        sep="\t",
                        col.names=FALSE,
                        row.names=FALSE,
                        quote=FALSE
                    )

                    incProgress(amount=1)
                    updateProgress("wgd_progress_container_js", 100, "Create wgd code")
                    Sys.sleep(2)
                })
                shinyjs::runjs('$("#progress_modal").modal("hide");');
            }
        }else{
            shinyalert(
                "Oops",
                "No cds or annotation file found. Please upload the data first",
                type="error",
            )
        }
    }

    wgdcommmandFile <- paste0(working_wd, "/wgd_wd/run_wgd.sh")
    if( file.exists(wgdcommmandFile) ){
        output$WgdCommandTxt <- renderText({
            command_info <- readChar(
                wgdcommmandFile,
                file.info(wgdcommmandFile)$size
            )
        })
        output$wgdParameterPanel <- renderUI({
            fluidRow(
                div(
                    style="padding-bottom: 10px;
                           padding-left: 20px;
                           padding-right: 20px;
                           max-width: 100%;
                           overflow-x: auto;",
                    column(
                        12,
                        h5(HTML(paste0("The command line for <font color='green'><b><i>wgd</i></b></font>:"))),
                        verbatimTextOutput(
                            "WgdCommandTxt",
                            placeholder=TRUE
                        )
                    )
                )
            )
        })
    }
})

observeEvent(input$ksrates_go, {
    working_wd <- data_preparation_dir_Val()
    original_data_wd <- original_data_wd_Val()
    if( is.null(input$select_focal_species) ){
        shinyalert(
            "Oops!",
            "Please define focal species first, then switch this on ...",
            type="error"
        )
    }
    else if( is.null(input$newick_tree) ){
        shinyalert(
            "Oops!",
            "Please upload the newick tree first, then switch this on ...",
            type="error"
        )
    }else{
        if( is.null(input$upload_data_file) ){
            if( is.null(input[[paste0("proteome_", 2)]]) ){
                shinyalert(
                    "Oops!",
                    "Please upload the data from at least two species to trigger the ksrates pipeline, then switch this on ...",
                    type="error"
                )
            }
            else{
                shinyjs::runjs('$("#progress_modal").modal("show");')
                progress_data <- list(
                    "actionbutton"="ksrates_go",
                    "container"="ksrates_progress_container_js"
                )
                session$sendCustomMessage(
                    "Progress_Bar_Complete",
                    progress_data
                )
                withProgress(message='Creating in progress', value=0, {
                    incProgress(amount=.15, message="Preparing ksrates configure file...")
                    updateProgress(
                        container="ksrates_progress_container_js",
                        width=15,
                        type="Create ksrates code"
                    )
                    Sys.sleep(1)

                    ksratesDir <- paste0(working_wd, "/ksrates_wd")
                    if( !file.exists(ksratesDir) ){
                        dir.create(ksratesDir)
                        # system(paste("chmod -R 777", ksratesDir))
                    }
                    ksratesconf <- paste0(ksratesDir, "/ksrates_conf.txt")
                    speciesinfoconf <- paste0(working_wd, "/Species.info.xls")

                    create_ksrates_configure_file_v2(input, ksratesconf, speciesinfoconf)

                    incProgress(amount=.6, message="Preparing ksrates expert parameters...")
                    updateProgress(
                        container="ksrates_progress_container_js",
                        width=60,
                        type="Create ksrates code"
                    )
                    Sys.sleep(1)

                    # create Ksrate expert parameters file
                    ksratesexpert <- paste0(ksratesDir, "/ksrates_expert_parameter.txt")
                    create_ksrates_expert_parameter_file(ksratesexpert)

                    updateProgress("ksrates_progress_container_js", 80, "Create ksrates code")
                    incProgress(amount=.8, message="Creating ksrates Runing Script ...")
                    Sys.sleep(1)

                    ksrates_cmd_sh_file <- paste0(ksratesDir, "/run_ksrates.sh")
                    ksrates_cmd <- create_ksrates_cmd(input, "ksrates_conf.txt", ksrates_cmd_sh_file)

                    system(
                        paste(
                            "cp",
                            paste0(getwd()[1], "/tools/run_paralog_ks_rest_species.sh"),
                            ksratesDir
                        )
                    )

                    updateProgress("ksrates_progress_container_js", 100, "Create ksrates code")
                    incProgress(amount=1)
                    Sys.sleep(1)
                })
                shinyjs::runjs('$("#progress_modal").modal("hide");');
            }
        }
        else{
            if( !is.null(input$selected_data_files) ){
                shinyjs::runjs('$("#progress_modal").modal("show");')
                progress_data <- list("actionbutton"="ksrates_go",
                                      "container"="ksrates_progress_container_js")
                session$sendCustomMessage(
                    "Progress_Bar_Complete",
                    progress_data
                )

                data_table <- read_data_file(input$upload_data_file)
                ncols <- ncol(data_table)
                nrows <- nrow(data_table)
                if( nrows > 1 ){
                    if( is.null(input$select_focal_species) || input$select_focal_species == ""){
                        shinyalert(
                            "Oops!",
                            "Please define focal species first, then switch this on ...",
                            type="error"
                        )
                    }
                    else if( is.null(input$newick_tree) ){
                        shinyalert(
                            "Oops!",
                            "Please upload the newick tree first, then switch this on ...",
                            type="error"
                        )
                    }
                    else{
                        withProgress(message='Creating in progress', value=0, {
                            incProgress(amount=.35, message="Processing CDS / Annotation Files ...")
                            updateProgress("ksrates_progress_container_js", 35, "Create ksrates code")
                            Sys.sleep(1)

                            ksratesDir <- paste0(working_wd, "/ksrates_wd")
                            if( !file.exists(ksratesDir) ){
                                dir.create(ksratesDir)
                                # system(paste("chmod -R 777", ksratesDir))
                            }
                            ksratesconf <- paste0(ksratesDir, "/ksrates_conf.txt")
                            speciesinfoconf <- paste0(working_wd, "/Species.info.xls")
                            create_ksrates_configure_file_based_on_table(
                                data_table,
                                input$select_focal_species,
                                input$newick_tree,
                                ksratesconf,
                                speciesinfoconf,
                                working_wd
                            )

                            incProgress(amount=.6, message="Preparing ksrates Expert Parameters ...")
                            updateProgress("ksrates_progress_container_js", 60, "Create ksrates code")
                            Sys.sleep(1)

                            ksratesexpert <- paste0(ksratesDir, "/ksrates_expert_parameter.txt")
                            create_ksrates_expert_parameter_file(ksratesexpert)

                            incProgress(amount=.8, message="Create ksrates Running Script ...")
                            updateProgress("ksrates_progress_container_js", 80, "Create ksrates code")
                            Sys.sleep(1)

                            ksrates_cmd_sh_file <- paste0(ksratesDir, "/run_ksrates.sh")
                            ksrates_cmd <- create_ksrates_cmd_from_table(data_table, "ksrates_conf.txt", ksrates_cmd_sh_file, input$select_focal_species)

                            system(
                                paste(
                                    "cp",
                                    paste0(getwd()[1], "/tools/run_paralog_ks_rest_species.sh"),
                                    ksratesDir
                                )
                            )

                            incProgress(amount=1)
                            updateProgress("ksrates_progress_container_js", 100, "Create ksrates code")
                            Sys.sleep(1)
                        })
                        shinyjs::runjs('$("#progress_modal").modal("hide");');
                    }
                }
            }else{
                shinyalert(
                    "Oops",
                    "No cds or annotation file found. Please upload the data first",
                    type="error",
                )
            }
        }
    }

    ksratesconf <- paste0(working_wd, "/ksrates_wd/ksrates_conf.txt")
    ksratescommad <- paste0(working_wd, "/ksrates_wd/run_ksrates.sh")
    if( file.exists(ksratescommad) && file.exists(ksratesconf)){
        output$ksratesConfigureFileTxt <- renderText({
            rawText <- readChar(ksratesconf, file.info(ksratesconf)$size)
        })
        output$ksratesCommandTxt <- renderText({
            CommandText <- readChar(ksratescommad, file.info(ksratescommad)$size)
        })
        output$ksratesParameterPanel <- renderUI({
            fluidRow(
                div(
                    style="padding-bottom: 10px;
                           padding-left: 20px;
                           padding-right: 20px;
                           max-width: 100%;
                           overflow-x: auto;",
                    fluidRow(
                        column(
                            12,
                            h5(HTML(paste0("The configure file for ", "<span style=\"color:green\"><b><i>ksrates</i></b></span>", ":"))),
                            verbatimTextOutput(
                                "ksratesConfigureFileTxt",
                                placeholder=TRUE
                            )
                        )
                    ),
                    fluidRow(
                        column(
                            12,
                            h5(HTML(paste0("The command line for <font color='green'><b><i>ksrates</i></b></font>:"))),
                            verbatimTextOutput(
                                "ksratesCommandTxt",
                                placeholder=TRUE
                            )
                        )
                    )
                )
            )
        })
    }
})

observeEvent(input$iadhore_go, {
    working_wd <- data_preparation_dir_Val()
    original_data_wd <- original_data_wd_Val()
    species_info <- paste0(working_wd, "/Species.info.xls")
    if( is.null(input$upload_data_file) & is.null(input[[paste0("gff_", 1)]]) ){
        shinyalert(
            "Oops!",
            "Please upload the annotation file (gff) for at least one species to trigger the i-ADHoRe pipeline, then switch this on ...",
            type="error"
        )
    }
    else if ( !file.exists(species_info) ){
        shinyalert(
            "Oops",
            "Please click the Create-ksrates-Codes button first, then switch this on ...",
            type="error"
        )
    }
    else{
        shinyjs::runjs('$("#progress_modal").modal("show");')
        progress_data <- list(
            "actionbutton"="iadhore_go",
            "container"="iadhore_progress_container_js"
        )
        session$sendCustomMessage(
            "Progress_Bar_Complete",
            progress_data
        )
        withProgress(message='Creating in progress', value=0, {
            incProgress(amount=.2, message="Preparing i-ADHoRe configure file...")
            updateProgress("iadhore_progress_container_js", 20, "Creat i-ADHoRe code")
            Sys.sleep(1)

            syn_dir <- paste0(working_wd, "/i-ADHoRe_wd")
            if( !file.exists(syn_dir) ){
                dir.create(syn_dir)
                # system(paste("chmod -R 777", syn_dir))
            }
            cmd_file <- paste0(syn_dir, "/run_diamond_iadhore.sh")
            system(
                paste(
                    "cp",
                    paste0(getwd()[1], "/tools/running_diamond.shell"),
                    syn_dir
                )
            )
            system(
                paste(
                    "cp",
                    paste0(getwd()[1], "/tools/computing_anchorpoint_ks.MultiThreads.shell"),
                    syn_dir
                )
            )

            incProgress(amount=.3, message="Dealing with gff files ...")
            updateProgress("iadhore_progress_container_js", 30, "Creat i-ADHoRe code")
            Sys.sleep(1)

            system(
                paste(
                    "sh tools/preparing_iadhore_inputs.shell",
                    species_info,
                    syn_dir
                )
            )
            incProgress(amount=.7, message="Generating the codes for diamond and i-ADHoRe")
            updateProgress("iadhore_progress_container_js", 70, "Creat i-ADHoRe code")
            Sys.sleep(1)

            if( isTruthy(input$multiple_iadhore) ){
                system(
                    paste(
                        "sh tools/generating_iadhore_codes.local.shell",
                        species_info,
                        syn_dir,
                        cmd_file,
                        getwd()[1],
                        "running_diamond.shell",
                        4,
                        "mode"
                    )
                )
            }else{
                system(
                    paste(
                        "sh tools/generating_iadhore_codes.local.shell",
                        species_info,
                        syn_dir,
                        cmd_file,
                        getwd()[1],
                        "running_diamond.shell",
                        4
                    )
                )
            }
            incProgress(amount=1, message="Done")
            updateProgress("iadhore_progress_container_js", 100, "Creat i-ADHoRe code")
            Sys.sleep(1)
        })
        shinyjs::runjs('$("#progress_modal").modal("hide");');
    }

    iadhorecommandFile <- paste0(working_wd, "/i-ADHoRe_wd/run_diamond_iadhore.sh")
    if( file.exists(iadhorecommandFile) ){
        output$IadhoreCommandTxt <- renderText({
            command_info <- readChar(
                iadhorecommandFile,
                file.info(iadhorecommandFile)$size
            )
        })
        output$iadhoreParameterPanel <- renderUI({
            fluidRow(
                div(
                    style="padding-bottom: 10px;
                           padding-left: 20px;
                           padding-right: 20px;
                           max-width: 100%;
                           overflow-x: auto;",
                    fluidRow(
                        column(
                            12,
                            h5(HTML(paste0("The command line for <font color='green'><b><i>i-ADHoRe</i></b></font>:"))),
                            verbatimTextOutput(
                                "IadhoreCommandTxt",
                                placeholder=TRUE
                            )
                        )
                    )
                )
            )
        })
    }
})

observeEvent(input$orthofinder_go, {
    working_wd <- data_preparation_dir_Val()
    original_data_wd <- original_data_wd_Val()
    species_info <- paste0(working_wd, "/Species.info.xls")
    if( file.exists(species_info) ){
        shinyjs::runjs('$("#progress_modal").modal("show");')
        progress_data <- list(
            "actionbutton"="orthofinder_go",
            "container"="orthofinder_progress_container_js"
        )
        session$sendCustomMessage(
            "Progress_Bar_Complete",
            progress_data
        )
        withProgress(message='Creating in progress', value=0, {
            incProgress(amount=.1, message="Preparing OrthoFinder input file ...")
            updateProgress("orthofinder_progress_container_js", 10, "Create OrthoFinder code")
            Sys.sleep(1)

            orthofinder_dir <- paste0(working_wd, "/OrthoFinder_wd")
            if( !dir.exists(orthofinder_dir) ){
                dir.create(orthofinder_dir)
                # system(paste("chmod -R 777", orthofinder_dir))
            }
            ds_tree_dir <- paste0(orthofinder_dir, "/ds_tree_wd")
            if( !dir.exists(ds_tree_dir) ){
                dir.create(ds_tree_dir)
                # system(paste("chmod -R 777", ds_tree_dir))
            }
            cmd_file <- paste0(orthofinder_dir, "/run_orthofinder.sh")
            incProgress(amount=.1, message="Create inputs file for OrthoFinder ...")
            system(
                paste(
                    "cp",
                    paste0(getwd()[1], "/tools/computing_Ks_tree_of_SingleCopyOrthologues.shell"),
                    ds_tree_dir
                )
            )
            incProgress(amount=.3, message="Translate CDS into proteins ...")
            updateProgress("orthofinder_progress_container_js", 30, "Create OrthoFinder code")
            Sys.sleep(1)

            system(
                paste(
                    "Rscript tools/prepare_orthofinder.R",
                    "-i", species_info,
                    "-o", orthofinder_dir,
                    "-s", gsub(" ", "_", input$select_focal_species),
                    "-c", cmd_file
                )
            )

            incProgress(amount=1, message="Done")
            updateProgress("orthofinder_progress_container_js", 100, "Create OrthoFinder code")
            Sys.sleep(1)
        })
        shinyjs::runjs('$("#progress_modal").modal("hide");');
    }
    else{
        shinyalert(
            "Oops",
            "Please click the Create-Ksrate-Codes button first, then switch this on ...",
            type="error"
        )
    }

    orthofinderCommandFile <- paste0(working_wd, "/OrthoFinder_wd/run_orthofinder.sh")
    if( file.exists(orthofinderCommandFile) ){
        output$OrthoFinderCommandTxt <- renderText({
            command_info <- readChar(
                orthofinderCommandFile,
                file.info(orthofinderCommandFile)$size
            )
        })
        output$orthofinderParameterPanel <- renderUI({
            fluidRow(
                div(
                    style="padding-bottom: 10px;
                           padding-left: 20px;
                           padding-right: 20px;
                           max-width: 100%;
                           overflow-x: auto;",
                    fluidRow(
                        h5(HTML(paste0("The command line for <font color='#5A5AAD'><i><b>OrthoFinder</b></i></font>:"))),
                        column(
                            12,
                            verbatimTextOutput(
                                "OrthoFinderCommandTxt",
                                placeholder=TRUE
                            )
                        )
                    )
                )
            )
        })
    }
})

observeEvent(input$go_codes_wgd, {
    working_wd <- data_preparation_dir_Val()
    wgdcommmandFile <- paste0(working_wd, "/wgd_wd/run_wgd.sh")
    if( file.exists(wgdcommmandFile) ){
        showModal(
            modalDialog(
                title="",
                size="xl",
                uiOutput("wgdParameterPanel")
            )
        )
    }else{
        shinyalert(
            "Oops",
            "Please click the Create-wgd-Codes button first, then switch this on ...",
            type="error"
        )
    }
})

observeEvent(input$go_codes_ksrates, {
    working_wd <- data_preparation_dir_Val()
    ksratescommadFile <- paste0(working_wd, "/ksrates_wd/run_ksrates.sh")
    if( !file.exists(ksratescommadFile) ){
        shinyalert(
            "Oops",
            "Please click the Create-ksrates-Codes button first, then switch this on ...",
            type="error"
        )
    }
    else{
        showModal(
            modalDialog(
                title="",
                size="xl",
                uiOutput("ksratesParameterPanel")
            )
        )
    }
})

observeEvent(input$go_codes_iadhore, {
    working_wd <- data_preparation_dir_Val()
    iadhorecommandFile <- paste0(working_wd, "/i-ADHoRe_wd/run_diamond_iadhore.sh")
    if( !file.exists(iadhorecommandFile) ){
        shinyalert(
            "Oops",
            "Please click the Create-i-ADHoRe-Codes button first, then switch this on ...",
            type="error"
        )
    }
    else{
        showModal(
            modalDialog(
                title="",
                size="xl",
                uiOutput("iadhoreParameterPanel")
            )
        )
    }
})

observeEvent(input$go_codes_orthofinder, {
    working_wd <- data_preparation_dir_Val()
    orthofinderCommandFile <- paste0(working_wd, "/OrthoFinder_wd/run_orthofinder.sh")
    if( !file.exists(orthofinderCommandFile) ){
        shinyalert(
            "Oops",
            "Please click the Create-OrthoFinder-Codes button first, then switch this on ...",
            type="error"
        )
    }
    else{
        showModal(
            modalDialog(
                title="",
                size="xl",
                uiOutput("orthofinderParameterPanel")
            )
        )
    }
})

observeEvent(input$job_run_server, {
    if( input$number_of_study_species > 5 ){
        shinyalert(
            "Oops!",
            "To control resource usage, the PSB computing cluster restricts studies to a maximum of five species. Please execute the generated scripts locally.",
            type="error"
        )
    }else{
        working_wd <- data_preparation_dir_Val()
        unique_id <- gsub("Analysis_", "", basename(working_wd))

        output$job_unique_id <- renderUI({
            column(
                12,
                "Job id:",
                verbatimTextOutput("job_id_text"),
                HTML("Make sure to take note of this <b>ID</b> as you will need it for downloading the results from our server.")
            )
        })
        output$job_id_text <- renderText({
            unique_id
        })

        # submit the job to psb computing cluster
        sh_files <- list.files(working_wd, pattern="\\.sh$", full.names=TRUE, recursive=TRUE)

        ksrates_sh_file <- sh_files[grepl("run_ksrates.sh", sh_files)]

        ksrates_rest_sh_file <- sh_files[grepl("run_paralog_ks_rest_species.sh", sh_files)]

        iadhore_sh_file <- sh_files[grepl("run_diamond_iadhore.sh", sh_files)]

        sh_files <- sh_files[!(sh_files %in% c(ksrates_sh_file[1], ksrates_rest_sh_file[1]))]

        system(
            paste(
                "cat",
                ksrates_sh_file[1],
                ksrates_rest_sh_file[1],
                "| grep -v '#SBATCH'",
                "| sed 's/--n-threads 1/--n-threads 2/'",
                ">",
                paste0(dirname(ksrates_sh_file[1]), "/ksrates_qsub.sh")
            )
        )

        system(
            paste(
                "echo 'cd ..'",
                ">>",
                paste0(dirname(ksrates_sh_file[1]), "/ksrates_qsub.sh")
            )
        )

        system(
            paste(
                "head -n 2",
                paste0(getwd()[1], "/tools/archive_compress_files_for_visualization.shell"),
                ">>",
                paste0(dirname(ksrates_sh_file[1]), "/ksrates_qsub.sh")
            )
        )

        system(
            paste(
                "echo 'cd ..'",
                ">>",
                iadhore_sh_file[1]
            )
        )

        system(
            paste(
                "sed -n '4,13p'",
                paste0(getwd()[1], "/tools/archive_compress_files_for_visualization.shell"),
                ">>",
                iadhore_sh_file[1]
            )
        )

        sh_files <- c(
            paste0(dirname(ksrates_sh_file[1]), "/ksrates_qsub.sh"),
            sh_files
        )

        shinyjs::runjs('$("#progress_modal").modal("show");')
        shiny::withProgress(message='Submit job in progress', value=0, {

            total_jobs <- length(sh_files)
            progress_increment <- 0.8 / total_jobs

            submit_job <- function(script_file) {
                shiny::incProgress(
                    amount=progress_increment,
                    message=paste0("Dealing with ", basename(script_file), " ...")
                )

                qsub_wd <- dirname(script_file)
                setwd(qsub_wd)

                if (length(grepl("ksrates_qsub.sh", script_file)) > 0) {
                    # system(paste("qsub -cwd -l h_vmem=8g", script_file))
                    Sys.sleep(10)
                } else if (length(grepl("run_diamond_iadhore.sh", script_file)) > 0) {
                    # system(paste("qsub -cwd -l h_vmem=8g", script_file))
                    Sys.sleep(10)
                } else if (length(grepl("run_wgd.sh", script_file)) > 0) {
                    # system(paste("qsub -cwd -l h_vmem=8g", script_file))
                    Sys.sleep(10)
                } else if (length(grepl("run_orthofinder.sh", script_file)) > 0) {
                    # system(paste("qsub -cwd -l h_vmem=8g -pe serial 4", script_file))
                    Sys.sleep(10)
                }
            }
            lapply(sh_files, submit_job)

            incProgress(amount=1, message="Done")
        })
        setwd(working_wd)
        shinyjs::runjs('$("#progress_modal").modal("hide");');
    }
})

observeEvent(input$comfirm_email, {
    working_wd <- data_preparation_dir_Val()
    if( is.null(working_wd) ){
        shinyalert(
            "Oops",
            "Please upload the data first, then switch this on ...",
            type="error"
        )
    }else{
        if( isTruthy(input$users_email_address) ){
            email_file_path <- paste0(working_wd, "/user_email.txt")

            writeLines(input$users_email_address, email_file_path)
            shinyalert(
                "Your email",
                input$users_email_address,
                type="info"
            )
        }else{
            shinyalert(
                "Oops",
                "Please input the email address first, then switch this on ...",
                type="error"
            )
        }
    }
})

output$search_download_page <- renderUI({
    div(class="boxLike",
        style="background-color: #FFFFF4;",
        h4(icon("download"), "Downlaod Result for the PSB server"),
        hr(class="setting"),
        fluidRow(
            column(
                12,
                HTML("Please enter the <b>identifier</b> generated by <b>shinyWGD</b> after submitting your job to the <b>PSB computing cluster</b> below."),
            ),
            column(
                12,
                fluidRow(
                    column(
                        4,
                        textInput(
                            inputId="job_search_identifier",
                            label="",
                            value="",
                            width="100%",
                            placeholder="identifier"
                        )
                    ),
                    column(
                        1,
                        actionButton(
                            inputId="comfirm_job_search",
                            "",
                            width="40px",
                            icon=icon("search"),
                            status="secondary",
                            class="my-start-button-class",
                            style="color: #fff;
                                   background-color: #8080C0;
                                   border-color: #fff;
                                   margin: 22px 0px 0px 0px;"
                        )
                    )
                ),
                fluidRow(
                    column(
                        12,
                        div(
                            style="padding-bottom: 10px;
                                   padding-top:30px;",
                            id="job_search_progress_container_js"
                        )
                    )
                )
            ),
            column(
                12,
                div(
                    style="background-color: #F8F8FF;
                           padding: 10px 10px 10px 10px;
                           border-radius: 10px;
                           width: 450px;",
                    uiOutput("unfinished_job_panel")
                )
            ),
            column(
                12,
                uiOutput("download_output_panel")
            )
        )
    )
})

updateProgressDownload <- function(container, width, type) {
    session$sendCustomMessage(
        "UpdateProgressBarDownload",
        list(container=container, width=width, type=type)
    )
}

observeEvent(input$comfirm_job_search, {
    # data_wd <- "/www/bioinformatics01_rw/ShinyWGD"
    data_wd <- "/var/folders/dm/nv89839s3dngv7d76s59n0rr0000gn/T/Rtmpg0qnEU/"
    dirs_list <- list.dirs(path=data_wd, full.names=TRUE, recursive=FALSE)
    dirs_list <- dirs_list[grep("Analysis_", dirs_list)]
    if( isTruthy(input$job_search_identifier) ){
        if( paste0("Analysis_", input$job_search_identifier) %in% basename(dirs_list) ){
            progress_data <- list(
                "actionbutton"="job_search_identifier",
                "container"="job_search_progress_container_js"
            )
            session$sendCustomMessage(
                "Progress_Bar_Complete",
                progress_data
            )
            updateProgressDownload("job_search_progress_container_js", 10, "Searching database")

            job_wd <- dirs_list[grep(input$job_search_identifier, dirs_list)]
            Sys.sleep(2)
            updateProgressDownload("job_search_progress_container_js", 20, "Checking job status")
            compressed_job_files_list <- list.files(
                path=job_wd,
                pattern="\\.tar\\.gz$",
                full.names=TRUE
            )

            if( file.exists(paste0(job_wd, "/OrthoFinder_wd/OrthoFinderOutput_for_Whale.tar.gz")) ){
                compressed_job_files_list <- c(
                    compressed_job_files_list,
                    paste0(job_wd, "/OrthoFinder_wd/OrthoFinderOutput_for_Whale.tar.gz")
                )
            }

            if( file.exists(paste0(job_wd, "/OrthoFinder_wd/ds_tree_wd.tar.gz")) ){
                compressed_job_files_list <- c(
                    compressed_job_files_list,
                    paste0(job_wd, "/OrthoFinder_wd/ds_tree_wd.tar.gz")
                )
            }

            output_compressed_files_name <- c(
                "Ks_Data_for_Visualization.tar.gz",
                "Collinear_Data_for_Visualization.tar.gz",
                "OrthoFinderOutput_for_Whale.tar.gz",
                "ds_tree_wd.tar.gz"
            )

            not_existed_names_list <- setdiff(output_compressed_files_name, basename(compressed_job_files_list))

            job_status_list <- c()
            if( length(not_existed_names_list) > 0 ){
                for( i in 1:length(not_existed_names_list) ){
                    if( not_existed_names_list[i] == "Ks_Data_for_Visualization.tar.gz" ){
                        job_status_list <- c(
                            job_status_list,
                            "<b>ksrates</b> is still running. Please wait ..."
                        )
                    }else if( not_existed_names_list[i] == "Collinear_Data_for_Visualization.tar.gz" ){
                        job_status_list <- c(
                            job_status_list,
                            "<b>i-ADHoRe</b> is still running. Please wait ..."
                        )
                    }else if( not_existed_names_list[i] == "ds_tree_wd.tar.gz" ){
                        job_status_list <- c(
                            job_status_list,
                            "<b><i>K</i><sub>s</sub> unit tree</b> is still computing. Please wait ..."
                        )
                    }else if( not_existed_names_list[i] == "OrthoFinderOutput_for_Whale.tar.gz" ){
                        job_status_list <- c(
                            job_status_list,
                            "<b>OrthoFinder</b> is still running. Please wait ..."
                        )
                    }

                    updateProgressDownload(
                        "job_search_progress_container_js",
                        20 + 10 * i,
                        "Checking job status"
                    )
                }
                output$download_output_panel <- renderUI({
                    ""
                })
            }else{
                updateProgressDownload(
                    "job_search_progress_container_js",
                    60,
                    "Checking job status"
                )

                job_status_list <- c(
                    job_status_list,
                    "<b>All jobs</b> are done.<br>You can click the download button to download results now ..."
                )

                output$download_output_panel <- renderUI({
                    column(
                        12,
                        hr(class="setting"),
                        downloadButton(
                            outputId="wgd_ksrates_ouput_download",
                            label="Download Output Data",
                            icon=icon("download"),
                            title="Click to download the output",
                            status="secondary",
                            class="my-download-button-class",
                            style="color: #fff;
                               background-color: #6B8E23;
                               border-color: #fff;
                               padding: 5px 14px 5px 14px;
                               margin: 10px 5px 5px 5px;"
                        )
                    )
                })

                output$wgd_ksrates_ouput_download <- downloadHandler(
                    filename=function(){
                        paste0(input$job_search_identifier, ".output.tgz")
                    },
                    content=function(file){
                        withProgress(message='Downloading in progress', value=0, {
                            setwd(job_wd)
                            incProgress(amount=.1, message="Compressing files...")

                            shinyalert(
                                "Note",
                                "Pleae wait for compressing the files. Do not close the page.",
                                type="info"
                            )

                            files_to_compress <- gsub("^.*?Analysis_2023_12_16_01_55_31/", "", compress_job_files_list)

                            system(
                                paste(
                                    "tar czf",
                                    file,
                                    paste(shQuote(files_to_compress), collapse=' ')
                                )
                            )

                            incProgress(amount=.9, message="Downloading file ...")
                            incProgress(amount=1)
                            Sys.sleep(.1)
                        })
                    }
                )
            }


            output$unfinished_job_panel <- renderUI({
                HTML(paste0(job_status_list, collapse="<br>"))
            })

            Sys.sleep(2)
            updateProgressDownload("job_search_progress_container_js", 100, "Done")
        }else{
            shinyalert(
                "Oops",
                "The job identifier you provided is not found in the database. Please enter the correct one.",
                type="error"
            )
        }
    }
    else{
        shinyalert(
            "Oops",
            "Please enter the job identifier first, then switch this on ...",
            type="error"
        )
    }
})

# Create analysis data to download
output$wgd_ksrates_data_download <- downloadHandler(
    filename=function(){
        working_wd <- data_preparation_dir_Val()
        paste0(basename(working_wd), ".tgz")
    },
    content=function(file){
        working_wd <- data_preparation_dir_Val()
        original_data_wd <- original_data_wd_Val()
        if( !is.null(original_data_wd) ){
            unlink(original_data_wd)
        }
        ksratescommadFile <- paste0(working_wd, "/ksrates_wd/run_ksrates.sh")
        wgdcommmandFile <- paste0(working_wd, "/wgd_wd/run_wgd.sh")
        if( !file.exists(wgdcommmandFile) & !file.exists(ksratescommadFile) ){
            shinyalert(
                "Oops",
                "Please click the Create-wgd-Codes or Create-ksrates-Codes button first, then switch this on ...",
                type="error"
            )
            return(NULL)
        }
        else{
            withProgress(message='Downloading in progress', value=0, {
                species_info <- paste0(working_wd, "/Species.info.xls")
                species_data <- read.delim(
                    species_info,
                    header=FALSE,
                    sep="\t",
                    stringsAsFactors=FALSE
                )

                species_data$V1 <- gsub("_", " ", species_data$V1)
                species_data$V2 <- gsub("^.*/", "", species_data$V2)
                species_data$V3 <- gsub("^.*/", "", species_data$V3)

                write.table(
                    species_data,
                    file=species_info,
                    sep="\t",
                    quote=FALSE,
                    col.names=FALSE,
                    row.names=FALSE
                )

                system(
                    paste("rm -rf", original_data_wd)
                )

                incProgress(amount=.1, message="Compressing files...")
                shinyalert(
                    "Note",
                    "Pleae wait for compressing the files. Do not close the page.",
                    type="info"
                )
                run_dir <- getwd()
                setwd(dirname(working_wd))
                system(
                    paste(
                        "tar czf",
                        file,
                        basename(working_wd)
                    )
                )

                incProgress(amount=.9, message="Downloading file ...")
                incProgress(amount=1)
                Sys.sleep(.1)
                setwd(run_dir)
            })
        }
    }
)

# remove the folder created two weeks ago
# find_folders_two_weeks_ago <- function(base_dir) {
#     folders <- list.files(base_dir, full.names=TRUE)
#
#     pattern <- "\\d{4}_\\d{2}_\\d{2}_\\d{2}_\\d{2}_\\d{2}"
#     matching_folders <- grep(pattern, folders, value=TRUE)
#
#     folder_dates <- as.Date(
#         sub(".*_(\\d{4}_\\d{2}_\\d{2}_\\d{2}_\\d{2}_\\d{2}).*", "\\1", matching_folders),
#         format="%Y_%m_%d_%H_%M_%S"
#     )
#
#     two_weeks_ago <- Sys.Date() - 14
#
#     two_weeks_ago_folders <- matching_folders[folder_dates <= two_weeks_ago]
#
#     sapply(two_weeks_ago_folders, function(folder) system(paste("rm -rf", shQuote(folder))))
#
#     return(two_weeks_ago_folders)
# }
#
# timer <- reactiveTimer(60 * 1000)
#
# observe({
#     timer()
#
#     data_dir <- "/var/folders/dm/nv89839s3dngv7d76s59n0rr0000gn/T/RtmpmsMCLT"
#     removed_projects_log <- paste0(data_dir, "/removed_projects.xls")
#     if( format(Sys.time(), "%H:%M") == "17:22"){
#         folders_needed_to_be_removed <- find_folders_two_weeks_ago(data_dir)
#         if( file.exists(removed_projects_log)) {
#             write.table(
#                 folders_needed_to_be_removed,
#                 removed_projects_log,
#                 append=TRUE,
#                 col.names=FALSE,
#                 row.names=FALSE,
#                 quote=FALSE
#             )
#         }else{
#             write.table(
#                 folders_needed_to_be_removed,
#                 removed_projects_log,
#                 col.names=FALSE,
#                 row.names=FALSE,
#                 quote=FALSE
#             )
#         }
#     }
# })

#' obtain_chromosome_length
#'
#' Process species information file and extract chromosome lengths and mRNA counts from GFF files.
#'
#' @param species_info_file A character string specifying the path to the species information file.
#'
#' @importFrom vroom vroom
#' @importFrom dplyr group_by
#' @importFrom dplyr summarise
#' @importFrom dplyr filter
#'
#' @return A list containing two data frames: len_df for chromosome lengths and num_df for mRNA counts.
#'
obtain_chromosome_length <- function(species_info_file){
    actual_path <- dirname(species_info_file)

    df <- read.table(
        species_info_file,
        sep="\t",
        header=FALSE,
        fill=T,
        na.strings="",
		col.names=c("sp", "cds", "gff")
    )

    cds_files <- gsub(".*/", "", df$cds)
    gff_files <- gsub(".*/", "", df$gff)

    new_cds_files <- paste0(actual_path, "/", cds_files)
    new_gff_files <- paste0(actual_path, "/", gff_files)

    df$cds <- new_cds_files
    df$gff <- new_gff_files

    len_df <- data.frame(sp=character(),
                         chr=character(),
                         len=integer())
    # num_df <- data.frame(sp=character(),
    #                      chr=character(),
    #                      num=integer())

    for( i in 1:nrow(df) ){
        each_row <- df[i, ]
        gff_df <- suppressMessages(vroom(each_row$gff,
                        delim="\t",
                        comment="#",
                        col_names=c("seqchr", "source", "type",
                                    "start", "end", "score",
                                    "strand", "phase", "attributes")))

        seqchr <- end <- type <- NULL

        gff_grouped <- group_by(gff_df, seqchr)
        max_pos <- summarise(gff_grouped, len=max(end))
        max_pos$sp <- gsub(" ", "_", each_row$sp)
        len_df <- rbind(len_df, max_pos)
        # mRNA_counts <- gff_grouped %>%
        #     filter(type=="mRNA") %>%
        #     group_by(seqchr) %>%
        #     summarise(count=dplyr::n())
        # mRNA_counts$sp <- gsub(" ", "_", each_row$sp)
        # num_df <- rbind(num_df, mRNA_counts)
    }
    #colnames(num_df) <- c("seqchr", "num", "sp")
    #return(list(len_df=len_df, num_df=num_df))
    return(len_df)
}

#' obtain_chromosome_length_filter
#'
#' Process a data frame containing species information and extract chromosome lengths and mRNA counts from GFF files.
#'
#' @param species_info_df A data frame containing species information with columns "sp," "cds," and "gff."
#'
#' @importFrom vroom vroom
#' @importFrom dplyr group_by
#' @importFrom dplyr summarise
#' @importFrom dplyr filter
#'
#' @return A list containing two data frames: len_df for chromosome lengths and num_df for mRNA counts.
#'
obtain_chromosome_length_filter <- function(species_info_df){

    df <- species_info_df
    colnames(df) <- c("sp", "cds", "gff")

    len_df <- data.frame(sp=character(),
                         chr=character(),
                         len=integer())
    num_df <- data.frame(sp=character(),
                         chr=character(),
                         num=integer())

    seqchr <- end <- type <- NULL
    for( i in 1:nrow(df) ){
        each_row <- df[i, ]
        gff_df <- suppressMessages(
            vroom(
                each_row$gff,
                delim="\t",
                comment="#",
                col_names=c("seqchr", "source", "type",
                            "start", "end", "score",
                            "strand", "phase", "attributes")
            )
        )

        gff_grouped <- group_by(gff_df, seqchr)
        max_pos <- summarise(gff_grouped, len=max(end))
        max_pos$sp <- gsub(" ", "_", each_row$sp)
        len_df <- rbind(len_df, max_pos)
        mRNA_counts <- gff_grouped %>%
            filter(type=="mRNA") %>%
            group_by(seqchr) %>%
            summarise(count=dplyr::n())
        mRNA_counts$sp <- gsub(" ", "_", each_row$sp)
        num_df <- rbind(num_df, mRNA_counts)
    }
    colnames(num_df) <- c("seqchr", "num", "sp")
    return(list(len_df=len_df, num_df=num_df))
}

#' Obtain coordinates for segments in a comparison
#'
#' This function retrieves the coordinates for segments in a comparison based on the provided parameters.
#'
#' @param seg_file The file containing segment data.
#' @param sp1 The species name for the first genome.
#' @param gff_file1 The GFF file for the first genome.
#' @param out_file The output file to store the merged position data.
#' @param sp2 The species name for the second genome (optional).
#' @param gff_file2 The GFF file for the second genome (optional).
#'
#' @importFrom vroom vroom
#' @importFrom dplyr filter
#' @importFrom dplyr mutate
#' @importFrom dplyr select
#'
#' @return NULL (the results are saved in the output file).
#'
obtain_coordiantes_for_segments <- function(seg_file, sp1, gff_file1, out_file, sp2=NULL, gff_file2=NULL){
    # library(vroom)
    # library(dplyr)

    gff_df <- suppressMessages(
        vroom(gff_file1,
              delim="\t",
              comment="#",
              col_names=FALSE)
    )
    X1 <- X4 <- X5 <- X7 <- X9 <- NULL
    position_df <- gff_df %>%
        filter(gff_df$X3=="mRNA") %>%
        select(X1, X9, X4, X5, X7) %>%
        mutate(X9=gsub("ID=([^;]+).*", "\\1", X9))

    colnames(position_df) <- c("list", "gene", "start", "end", "strand")

    if( !is.null(gff_file2) ){
        gff_df2 <- suppressMessages(
            vroom(gff_file2,
                  delim="\t",
                  comment="#",
                  col_names=FALSE)
        )
        position_df2 <- gff_df2 %>%
            filter(gff_df2$X3=="mRNA") %>%
            select(X1, X9, X4, X5, X7) %>%
            mutate(X9=gsub("ID=([^;]+).*", "\\1", X9))

        colnames(position_df2) <- c("list", "gene", "start", "end", "strand")
        position_df <- rbind(position_df, position_df2)
    }
    segs <- suppressMessages(
        vroom(seg_file,
              delim="\t",
              col_names=TRUE)
    )

    gene <- start <- end <- multiplicon <- genome <- list <- first <- last <- NULL
    genome.1 <- list.1 <- first.1 <- last.1 <- start.1 <- end.1 <- order.1 <- NULL
    genomeX <- genomeY <- NULL

    start_subset <- select(position_df, gene, start)
    merged_data <- left_join(segs, start_subset, by = c("first"="gene"))
    end_subset <- select(position_df, gene, end)
    merged_data <- left_join(merged_data, end_subset, by=c("last"="gene"))

    multiplicon_list <- unique(merged_data$multiplicon)
    final_df <- data.frame()
    for (i in multiplicon_list) {
        df_subset <- merged_data[merged_data$multiplicon==i, ]
        df_row1 <- df_subset[1, ]
        for( y in 2:nrow(df_subset) ){
            df_row2 <- df_subset[y, ]
            new_row <- c(df_row1, df_row2)
            final_df <- rbind(final_df, new_row)
        }
    }
    final_output <- select(final_df, multiplicon, genome, list, first, last, start, end, order,
                           genome.1, list.1, first.1, last.1, start.1, end.1, order.1)
    colnames(final_output) <- c("multiplicon", "genomeX", "listX", "firstX", "lastX", "startX", "endX", "orderX",
                                "genomeY", "listY", "firstY", "lastY", "startY", "endY", "orderY")
    rownames(final_output) <- NULL
    final_output1 <- subset(final_output, genomeX != genomeY)
    sp_list <- unique(c(final_output1$genomeX, final_output1$genomeY))

    if( length(sp_list) > 1 ){
        final_table <- data.frame(matrix(ncol = ncol(final_output1), nrow = nrow(final_output1)))
        for( i in 1:nrow(final_output1) ){
            each_row <- final_output1[i, ]
            if( each_row$genomeX != sp1 ){
                tmp <- each_row[2:8]
                each_row[2:8] <- each_row[9:15]
                each_row[9:15] <- tmp
            }
            final_table[i,] <- each_row
        }
        colnames(final_table) <- c("multiplicon", "genomeX", "listX", "firstX", "lastX", "startX", "endX", "orderX",
                                    "genomeY", "listY", "firstY", "lastY", "startY", "endY", "orderY")
        write.table(final_table,
                    file=out_file,
                    sep="\t",
                    row.names=FALSE,
                    quote=FALSE)
    }
    else{
        final_table <- final_output
        for (i in 1:nrow(final_output)) {
            each_row <- final_output[i, ]
            tmp <- each_row[2:8]
            each_row[2:8] <- each_row[9:15]
            each_row[9:15] <- tmp
            final_table <- rbind(final_table, each_row)
        }
        colnames(final_table) <- c("multiplicon", "genomeX", "listX", "firstX", "lastX", "startX", "endX", "orderX",
                                   "genomeY", "listY", "firstY", "lastY", "startY", "endY", "orderY")
        write.table(final_table,
                    file=out_file,
                    sep="\t",
                    row.names=FALSE,
                    quote=FALSE)
    }
}

output$introductionPanel <- renderUI({
    div(
        style="max-width: 1200px;
               padding-left: 300px;
               padding-bottom: 50px;
               padding-top: 50px;
               color: black;",
        column(
            12,
            HTML(includeMarkdown("www/content/introduction.md")),
            tags$script("
              function switchToDataPreparationTab() {
                Shiny.setInputValue('switchTab', 'data_preparation', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToWhalePreparationTab() {
                Shiny.setInputValue('switchTab', 'whale_preparation', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToTreeExtractionTab() {
                Shiny.setInputValue('switchTab', 'extracting_tree', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToKsAnalysisTab() {
                Shiny.setInputValue('switchTab', 'ks_analysis', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToSyntenyAnalysisTab() {
                Shiny.setInputValue('switchTab', 'synteny_analysis', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToTreeBuildingTab() {
                Shiny.setInputValue('switchTab', 'jointTree', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToTreeReconTab() {
                Shiny.setInputValue('switchTab', 'tree_reconciliation', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToGalleryTab() {
                Shiny.setInputValue('switchTab', 'gallery_page', {priority: 'event'});
              }
            "),
            tags$script("
              function switchToHelpTab() {
                Shiny.setInputValue('switchTab', 'help', {priority: 'event'});
              }
            "),
        )
    )
})

observe({
    updateTabsetPanel(session, "shinywgd", selected=input$switchTab)
})

Help_ui <- tabPanel(
    HTML("Help"),
    value="help",
    id="help",
    fluidRow(
        column(
            12,
			HTML('<iframe src="https://li081766.github.io/shinyWGD_Demo_Data/intro_to_shinywgd.html" style="width:100%; height:800px; border:none;"></iframe>')
        ),
    ),
    icon=icon("question")
)

#' Compute the Depth of Anchored Points
#'
#' This function calculates the depth of anchored points based on the provided parameters.
#'
#' @param anchorpoint_ks_file The file containing anchorpoint and Ks data.
#' @param multiplicon_id The ID of the multiplicon to consider.
#' @param selected_query_chr A list of selected query chromosomes.
#' @param selected_subject_chr A list of selected subject chromosomes (optional).
#'
#' @return A list containing depth data frames, including "query_depth" and "subject_depth" if subject chromosomes are specified, or "depth" if not.
#'
computing_depth <- function(anchorpoint_ks_file,
                            multiplicon_id,
                            selected_query_chr,
                            selected_subject_chr=NULL){
    if( !is.null(selected_subject_chr) ){
        multiplicon <- listX <- listY <- coordX <- coordY <- NULL
        anchorpoint_df <- read.table(
            anchorpoint_ks_file,
            sep="\t",
            header=TRUE,
            fill=TRUE,
            na.strings=""
        ) %>%
            dplyr::filter(multiplicon %in% multiplicon_id) %>%
            dplyr::filter(listX %in% selected_query_chr) %>%
            dplyr::filter(listY %in% selected_subject_chr)

        suppressMessages(
            query_depth <- anchorpoint_df %>%
                dplyr::group_by(listX, coordX) %>%
                dplyr::summarise(count=dplyr::n())
        )

        suppressMessages(
            subject_depth <- anchorpoint_df %>%
                dplyr::group_by(listY, coordY) %>%
                dplyr::summarise(count=dplyr::n())
        )

        return(list(query_depth=query_depth,
                    subject_depth=subject_depth))
    }else{
        listX <- coordX <- NULL
        anchorpoint_df <- read.table(
            anchorpoint_ks_file,
            sep="\t",
            header=TRUE,
            fill=TRUE,
            na.strings=""
        ) %>%
            dplyr::filter(listX %in% selected_query_chr)

        suppressMessages(
            query_depth <- anchorpoint_df %>%
                dplyr::group_by(listX, coordX) %>%
                dplyr::summarise(count=dplyr::n())
        )

        return(list(depth=query_depth))
    }
}

#' Compute the Depth of Anchored Points in a Paranome Comparison
#'
#' This function computes the depth of anchored points in a paranome comparison based on the provided parameters.
#'
#' @param anchorpoint_ks_file The file containing anchor point and Ks value data.
#' @param multiplicon_id The IDs of the multiplicons to consider.
#' @param selected_query_chr The list of selected query chromosomes.
#'
#' @return A list containing the depth dataframe.
computing_depth_paranome <- function(
        anchorpoint_ks_file,
        multiplicon_id,
        selected_query_chr
    ){
    anchorpoint_df <- read.table(
        anchorpoint_ks_file,
        sep="\t",
        header=TRUE,
        fill=TRUE,
        na.strings=""
    )

    tmp_df <- anchorpoint_df

    data_names <- names(tmp_df)
    data_names_swapped <- gsub("X$", "TEMP", data_names)
    data_names_swapped <- gsub("Y$", "X", data_names_swapped)
    data_names_swapped <- gsub("TEMP$", "Y", data_names_swapped)

    colnames(tmp_df) <- data_names_swapped

    multiplicon <- listX <- coordX <- NULL
    anchorpoint_df <- rbind(anchorpoint_df, tmp_df) %>%
        dplyr::filter(multiplicon %in% multiplicon_id) %>%
        dplyr::filter(listX %in% selected_query_chr)

    suppressMessages(
        query_depth <- anchorpoint_df %>%
            dplyr::group_by(listX, coordX) %>%
            dplyr::summarise(count=dplyr::n())
    )
    return(list(depth=query_depth))
}

Introduction_ui <- tabPanel(
    "Home",
    value='introduction_page',
    fluidRow(
        column(
            12,
            uiOutput("introductionPanel")
        ),
    ),
    icon=icon("house", verify_fa=FALSE)
)

output$whaleDataUploadPanel <- renderUI({
    fluidRow(
        div(class="boxLike",
            style="background-color: #FAF9F6;
                   margin: 5px 5px 5px 5px;
                   padding: 5px 10px 10px 10px;",
            column(
                12,
                h4(icon("upload"), "Uploading"),
                hr(class="setting"),
                HTML("Uploading the Analysis Directory:</br>"),
                h5(HTML("<font color='green'><b><i>shinyWGD</i> OrthoFinder Analysis</b></font> Data")),
                fluidRow(
                    class="justify-content-left",
                    style="padding-bottom: 15px;
                           padding-top: 5px",
                    column(
                        12,
                        div(
                            style="padding-left: 10px;
                                   position: relative;",
                            fileInput(
                                'orthofinder_data_zip_file',
                                label=h6(icon("file-zipper"), HTML("Upload the <b>Zipped</b> File")),
                                multiple=FALSE,
                                accept=c(
                                    ".zip",
                                    ".gz"
                                ),
                                width="80%"
                            ),
                            actionButton(
                                inputId="orthofinder_data_example",
                                "",
                                icon=icon("question"),
                                status="secondary",
                                class="my-start-button-class",
                                title="Click to use the example data to demo run the Whale Preparation",
                                style="color: #fff;
                                       background-color: #87CEEB;
                                       border-color: #fff;
                                       position: absolute;
                                       top: 63%;
                                       left: 90%;
                                       margin-top: -15px;
                                       margin-left: -15px;
                                       padding: 5px 14px 5px 10px;
                                       width: 30px; height: 30px; border-radius: 50%;"
                            )
                        )
                    ),
                    column(
                        12,
                        uiOutput("selectedOrthoFinderDirName")
                    )
                )
            ),
            column(
                12,
                hr(class="setting"),
                fluidRow(
                    column(
                        12,
                        div(
                            style="padding-left: 10px;
                                   position: relative;",
                            fileInput(
                                inputId="uploadSpeciesTimeTree",
                                label=HTML("<font color='green'><b>Species Time Tree</b></font> in <font color='red'><b><i>Newick</b></i></font>:"),
                                width="80%"
                            ),
                            actionButton(
                                inputId="newick_time_file_example",
                                "",
                                icon=icon("question"),
                                status="secondary",
                                class="my-start-button-class",
                                title="Click to see the example of the Newick Time Tree File",
                                style="color: #fff;
                                       background-color: #87CEEB;
                                       border-color: #fff;
                                       position: absolute;
                                       top: 53%;
                                       left: 90%;
                                       margin-top: -15px;
                                       margin-left: -15px;
                                       padding: 5px 14px 5px 10px;
                                       width: 30px; height: 30px; border-radius: 50%;"
                            )
                        )
                    )
                )
            ),
            column(
                12,
                hr(class="setting"),
                h6(
                    HTML(
                        "If you don’t ensure the evolutionary relationships among the studied species, please use the ",
                    )
                ),
                actionLink(
                    "go_extracting_tree",
                    HTML(
                        paste0(
                            "<i class='fas fa-tree' style='color: #5151A2;'>&nbsp;</i>",
                            "<font color='#5151A2'>",
                            "<i><b>TimeTreeFetcher</b></i></font>"
                        )
                    )
                ),
                h6(
                    HTML(
                        "module of shinyWGD to obtain a tree from <a href=\"http://www.timetree.org/\">TimeTree.org</a>."
                    )
                )
            )
        )
    )
})

observeEvent(input$orthofinder_data_example, {
    showModal(
        modalDialog(
            title=HTML("The description of the demo data used to prepare the <b>Whale inputs</b>"),
            size="xl",
            uiOutput("orthofinder_data_example_panel")
        )
    )

    output$orthofinder_data_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    HTML(
                        paste0(
                            "<p>In the demo data, we selected four species: <i>Apostasia shenzhenica</i>, <i>Phalaenopsis equestris</i>, <i>Asparagus officinalis</i>, and <i>Dendrobium catenatum</i>, as also used in <b><i>K</i><sub>s</sub>Dist</b> module and <b>Collinearity</b> module, to generate the data.</p>",
                            "<p>First, we followed the preparation steps in the Data Preparation Page of the <b>shinyWGD</b> server to create the script for the corresponding package, <b>OrthoFinder</b>. ",
                            "We then submitted the job to the PSB computing server to obtain the output.</p>",
                            "<p>After obtaining the output, the data used for the preparation will be archived and compressed to a file, named by <b><i>OrthoFinderOutput_for_Whale.tar.gz</i></b>. ",
                            "<p>Users can upload the compressed file and start the preparation process.</p>",
                            "<p>After uploading the time tree of the studied species, users have the capability to specify the clade of interest, enabling them to selectively focus on particular gene families (",
                            "see more <a href='https://github.com/arzwa/Whale.jl/blob/master/scripts/orthofilter.py' target='_blank'>click here</a>). ",
                            "Additionally, users can identify and insert potential Whole Genome Duplication (WGD) events for subsequent analysis. ",
                            "The flexibility extends to choosing the appropriate model within the <b>Whale</b> and configuring the Markov Chain Monte Carlo (MCMC) chain.</p>",
                            "<p>To expedite the validation process, users can choose a subset of gene families for a rapid assessment of Whale's performance. ",
                            "This functionality allows for a quick check of the algorithm's run on a smaller, manageable dataset before executing a comprehensive analysis on the entire set of gene families.</p>",
                            "<p>To download the demo data, <a href='https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_OrthoFinderOutput_for_Whale.tar.gz' target='_blank'>click here</a>.</p>",
                            "<p><br></br></p>"
                        )
                    ),
                    h5(
                        HTML(
                            "<hr><p><b><font color='#BDB76B'>For true data</font></b>"
                        )
                    ),
                    HTML(
                        "<p>Users should upload the zipped-file, named as <b><i>OrthoFinderOutput_for_Whale.tar.gz</i></b> in the <b>Analysis-*</b> folder created by <b>shinyWGD</b>, to start the <b>Whale Preparation</b>.</p>"
                    )
                )
            )
        )
    })
})

example_data_dir <- file.path(getwd(), "demo_data")
whale_example_dir <- file.path(example_data_dir, "Example_Whale_Preparation")
og_check_file <- paste0(whale_example_dir, "/OrthoFinderOutputDir/Results_Nov18/MultipleSequenceAlignments/OG0002009.fa")

if( !dir.exists(whale_example_dir) & !file.exists(og_check_file) ){
    withProgress(message='Downloading Whale preparation demo data...', value=0, {
        if( !dir.exists(example_data_dir) ){
            dir.create(example_data_dir)
        }
        dir.create(whale_example_dir)

        Sys.sleep(.2)
        incProgress(amount=.3, message="Downloading in progress. Please wait...")

        downloadAndExtractData <- function() {
            download.file(
                "https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_OrthoFinderOutput_for_Whale.tar.gz",
                destfile=file.path(getwd(), "whale.data.zip"),
                mode="wb"
            )

            system(
                paste(
                    "tar xzf",
                    shQuote(file.path(getwd(), "whale.data.zip")),
                    "-C",
                    shQuote(whale_example_dir)
                )
            )

            file.remove(file.path(getwd(), "whale.data.zip"))
        }

        downloadAndExtractData()
        Sys.sleep(.2)
        incProgress(amount=1, message="Done")
    })
}else if( dir.exists(whale_example_dir) & !file.exists(og_check_file) ){
    withProgress(message='Downloading Whale preparation demo data...', value=0, {
        system(
            paste("rm -rf ", whale_example_dir)
        )
        dir.create(whale_example_dir)

        Sys.sleep(.2)
        incProgress(amount=.3, message="Downloading in progress. Please wait...")

        downloadAndExtractData <- function() {
            download.file(
                "https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_OrthoFinderOutput_for_Whale.tar.gz",
                destfile=file.path(getwd(), "whale.data.zip"),
                mode="wb"
            )

            system(
                paste(
                    "tar xzf",
                    shQuote(file.path(getwd(), "whale.data.zip")),
                    "-C",
                    shQuote(whale_example_dir)
                )
            )

            file.remove(file.path(getwd(), "whale.data.zip"))
        }

        downloadAndExtractData()
        Sys.sleep(.2)
        incProgress(amount=1, message="Done")
    })
}

buttonWhalePreparationClicked <- reactiveVal(NULL)
orthofinder_whale_analysis_dir_Val <- reactiveVal(whale_example_dir)

observeEvent(input$orthofinder_data_zip_file, {
    buttonWhalePreparationClicked("fileInput")

    base_dir <- tempdir()
    timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
    OrthoFinderWhaleAnalysisDir <- file.path(base_dir, paste0("Whale_Preparation_data_", gsub("[ :\\-]", "_", timestamp)))
    dir.create(OrthoFinderWhaleAnalysisDir)
    orthofinder_output_file <- "OrthoFinder_result.tar.gz"
    system(
        paste(
            "cp",
            input$orthofinder_data_zip_file$datapath,
            paste0(OrthoFinderWhaleAnalysisDir, "/", orthofinder_output_file)
        )
    )
    orthofinder_whale_analysis_dir_Val(OrthoFinderWhaleAnalysisDir)
})

observeEvent(input$orthofinder_data_example, {
    buttonWhalePreparationClicked("actionButton")
    orthofinder_whale_analysis_dir_Val(whale_example_dir)
})

observe({
    if( is.null(buttonWhalePreparationClicked()) ){
        OrthoFinderWhaleAnalysisDir <- whale_example_dir
        if( length(OrthoFinderWhaleAnalysisDir) > 0 ){
            output$selectedOrthoFinderDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("<b>Example:<br><font color='#EE82EE'>Whale Preparation</font></b>"))
                    )
                )
            })
        }
    }
    else if( buttonWhalePreparationClicked() == "fileInput" ){
        OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
        if( length(OrthoFinderWhaleAnalysisDir) > 0 ){
            dirName <- basename(OrthoFinderWhaleAnalysisDir)
            output$selectedOrthoFinderDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("Selected Directory:<br><b><font color='#EE82EE'>", dirName, "</font></b>"))
                    )
                )
            })
        }
    }
    else if( buttonWhalePreparationClicked() == "actionButton" ){
        OrthoFinderWhaleAnalysisDir <- whale_example_dir
        if( length(OrthoFinderWhaleAnalysisDir) > 0 ){
            output$selectedOrthoFinderDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("<b>Example:<br><font color='#EE82EE'>Whale Preparation</font></b>"))
                    )
                )
            })
        }
    }
})

observeEvent(input$newick_time_file_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the <font color='green'><b>Newick Time Tree</b></font> file"),
            size="xl",
            uiOutput("newick_time_file_example_panel")
        )
    )

    output$newick_time_file_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    verbatimTextOutput("newickTimeTreeExample"),
                    HTML(
                        paste0(
                            "<font color='green'><i aria-label='warning icon' class='fa fa-warning fa-fw' role='presentation'></i></font> The time scale on the tree is in units of <b>100 millions of years ago</b>.<br>",
                            "see more <a href='https://en.wikipedia.org/wiki/Newick_format#:~:text=In%20mathematics%2C%20Newick%20tree%20format,Maddison%2C%20Christopher%20Meacham%2C%20F.' target='_blank'>click here</a>."
                        )
                    )
                )
            )
        )
    })

    output$newickTimeTreeExample <- renderText({
        "(Asparagus_officinalis:117.61,(Apostasia_shenzhenica:84.55,(Dendrobium_catenatum:52.11,Phalaenopsis_equestris:52.11):32.44):33.06);"
    })
})

observeEvent(input$go_extracting_tree, {
    updateNavbarPage(inputId="shinywgd", selected="extracting_tree")
    shinyjs::runjs(
        'setTimeout(function () {
            document.querySelector("#ObtainTreeFromTimeTreeSettingDisplay").scrollIntoView({
                behavior: "smooth",
                block: "start",
            });
        }, 100);
    ')
})

speciesTimeTreeRv <- reactiveValues(data=NULL, clear=FALSE)

observeEvent(input$uploadSpeciesTimeTree, {
    speciesTimeTreeRv$clear <- FALSE
}, priority=1000)

widthSpacing <- reactiveValues(value=400)
heightSpacing <- reactiveValues(value=NULL)

observe({
    if( isTruthy(input$uploadSpeciesTimeTree) ){
        speciesTreeFile <- input$uploadSpeciesTimeTree$datapath
        speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
        closeAllConnections()
        sp_count <- str_count(speciesTree, ":")
        trunc_val <- as.numeric(sp_count) * 20
        heightSpacing$value <- trunc_val
    }
    else{
        speciesTree <- "(Asparagus_officinalis:117.61,(Apostasia_shenzhenica:84.55,(Dendrobium_catenatum:52.11,Phalaenopsis_equestris:52.11):32.44):33.06);"
        sp_count <- str_count(speciesTree, ":")
        trunc_val <- as.numeric(sp_count) * 20
        heightSpacing$value <- trunc_val
    }
})

observeEvent(input$svg_vertical_spacing_add_species, {
    heightSpacing$value <- heightSpacing$value + 30
})
observeEvent(input$svg_vertical_spacing_sub_species, {
    heightSpacing$value <- heightSpacing$value - 30
})
observeEvent(input$svg_horizontal_spacing_add_species, {
    widthSpacing$value <- widthSpacing$value + 30
})
observeEvent(input$svg_horizontal_spacing_sub_species, {
    widthSpacing$value <- widthSpacing$value - 30
})

# species_in_Tree_list <- reactiveVal(NULL)

observe({
    if( isTruthy(input$uploadSpeciesTimeTree) ){
        species_tree_data <- list(
            "width"=widthSpacing$value
        )
        speciesTreeFile <- input$uploadSpeciesTimeTree$datapath
        speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
        closeAllConnections()

        species_tree_data[["speciesTree"]] <- speciesTree[1]

        # species_in_Tree_list(regmatches(speciesTree[1], gregexpr("[A-Za-z_]+", speciesTree[1]))[[1]])

        species_tree_data[["height"]] <- heightSpacing$value[1]
        species_tree_data[["tree_plot_div"]] <- "speciesTree_plot"
        session$sendCustomMessage("speciesTreePlot", species_tree_data)
    }
    else{
        species_tree_data <- list(
            "width"=widthSpacing$value
        )
        dome_tree <- "(Asparagus_officinalis:117.61,(Apostasia_shenzhenica:84.55,(Dendrobium_catenatum:52.11,Phalaenopsis_equestris:52.11):32.44):33.06);"
        # species_in_Tree_list(regmatches(dome_tree, gregexpr("[A-Za-z_]+", dome_tree))[[1]])

        species_tree_data[["speciesTree"]] <- dome_tree
        species_tree_data[["height"]] <- heightSpacing$value[1]
        species_tree_data[["tree_plot_div"]] <- "speciesTree_plot"
        session$sendCustomMessage("speciesTreePlot", species_tree_data)
    }
})

wgdEventNote <- reactiveVal(NULL)

observe({
    OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
    if( length(OrthoFinderWhaleAnalysisDir) > 0 ){
        wgdNum <- toupper(as.english(length(input$wgdInput)))
        if( length(input$wgdInput) > 0 ){
            note <- HTML(paste0("<font color='#AD1F1F'><b>", wgdNum, "</b></font> WGD events will be examinated."))
        }else{
            note <- HTML(paste0("<font color='#C0C0C0'><b>", wgdNum, "</b></font> WGD event will be examinated."))
        }
        wgdEventNote(note)
    }
})

observe({
    OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
    if( length(OrthoFinderWhaleAnalysisDir) > 0 ){
        output$whaleCommandPanel <- renderUI({
            fluidRow(
                div(class="boxLike",
                    style="background-color: white;
                           margin: 5px 5px 5px 5px;
                           padding: 5px 10px 10px 10px;",
                    column(
                        id="WhaleSetting",
                        width=12,
                        h4(icon("cog"), HTML("<b><i>Whale</b></i> Setting")),
                        hr(class="setting"),
                        fluidRow(
                            column(
                                8,
                                h6(HTML("<font color='#AD1F1F'>Hypothetic WGDs</font> to test:")),
                                verbatimTextOutput(
                                    "wgdNeededTestedID",
                                    placeholder=TRUE
                                ),
                                wgdEventNote()
                            ),
                            column(
                                12,
                                fluidRow(
                                    column(
                                        12,
                                        hr(class="setting")
                                    ),
                                    # column(
                                    #     4,
                                    #     div(
                                    #         style="background-color: #F8F8FF;
                                    #                padding: 10px 10px 1px 10px;
                                    #                border-radius: 10px;",
                                    #         pickerInput(
                                    #             inputId="cladeSpecies",
                                    #             label=HTML("Please set a <b>clade species</b> to select the proper gene families. See more <a href='https://github.com/arzwa/Whale.jl/blob/master/scripts/orthofilter.py' target='_blank'>click here</a>."),
                                    #             options=list(
                                    #                 title='Please select species below'
                                    #             ),
                                    #             choices=species_in_Tree_list(),
                                    #             choicesOpt=list(
                                    #                 content=lapply(species_in_Tree_list(), function(choice) {
                                    #                     choice <- gsub("_", " ", choice)
                                    #                     paste0("<div style='color: steelblue; font-style: italic;'>", choice, "</div>")
                                    #                 })
                                    #             ),
                                    #             multiple=FALSE
                                    #         )
                                    #     )
                                    # ),
                                    column(
                                        4,
                                        div(
                                            style="background-color: #F8F8FF;
                                                   padding: 10px 10px 10px 10px;
                                                   border-radius: 10px;",
                                            selectInput(
                                                inputId="select_whale_model",
                                                label=HTML("<b>Base Model</b> for <b><i>Whale</b></i>:"),
                                                choices=c(
                                                    "Constant-rates model",
                                                    "Relaxed branch-specific DL+WGD model",
                                                    "Critical branch-specific DL+WGD model"
                                                ),
                                                width="100%",
                                                multiple=FALSE,
                                                selected="Constant-rates model"
                                            )
                                        )
                                    ),
                                    column(
                                        4,
                                        div(
                                            style="background-color: #F8F8FF;
                                                   padding: 10px 10px 1px 10px;
                                                   border-radius: 10px;",
                                            sliderInput(
                                                inputId="select_chain_num",
                                                label=HTML("Set the <b><font color='orange'>Chain</font></b> for <b><i>Whale</b></i>:"),
                                                min=100,
                                                max=1000,
                                                step=100,
                                                value=200
                                            )
                                        )
                                    ),
                                    column(
                                        4,
                                        div(
                                            style="background-color: #FAF0E6;
                                                   padding: 10px 10px 1px 10px;
                                                   border-radius: 10px;",
                                            sliderInput(
                                                inputId="input_gf_num",
                                                label=HTML("Choose a subset of gene families to quickly check <b><i>Whale</b></i> (Optional, recommended: 50):"),
                                                min=0,
                                                max=100,
                                                step=10,
                                                value=0
                                            )
                                        )
                                    )
                                )
                            ),
                            # column(
                            #     12,
                            #     fluidRow(
                            #         column(
                            #             12,
                            #             hr(class="setting")
                            #         ),
                            #         column(
                            #             7,
                            #             sliderInput(
                            #                 inputId="input_gf_num",
                            #                 label=HTML("Choose a small portion of gene families to quickly check the running of <b><i>Whale</b></i> (recommended: 50):"),
                            #                 min=0,
                            #                 max=200,
                            #                 step=10,
                            #                 value=0
                            #             )
                            #         )
                            #     )
                            # ),
                            column(
                                12,
                                hr(class="setting")
                            ),
                            column(
                                width=10,
                                div(
                                    class="row",
                                    div(
                                        class="col text-left",
                                        actionButton(
                                            inputId="whale_configure_go",
                                            HTML("Create <b><i>Whale</i></b> Codes"),
                                            icon=icon("play"),
                                            title="Start the configuration process",
                                            status="secondary",
                                            class="my-start-button-class",
                                            style="color: #fff;
                                                   background-color: #019858;
                                                   border-color: #fff;
                                                   padding: 5px 8px 5px 8px;"
                                        )
                                    ),
                                    div(
                                        class="col text-center",
                                        downloadButton(
                                            outputId="whale_prepared_data_download",
                                            label="Download Data to operate Whale",
                                            icon=icon("download"),
                                            title="Click to download the data",
                                            status="secondary",
                                            class="my-download-button-class",
                                            style="color: #fff;
                                                   background-color: #6B8E23;
                                                   border-color: #fff;
                                                   padding: 5px 8px 5px 8px;"
                                        )
                                    ),
                                    div(
                                        class="col text-right",
                                        style="padding: 5px 4px 5px 4px;",
                                        actionLink(
                                            "go_whale_codes",
                                            HTML(
                                                paste0(
                                                    "<font color='#5151A2'>",
                                                    icon("share"),
                                                    " Go to <i><b>Whale</b></i> Instructions</font>"
                                                )
                                            )
                                        )
                                    )
                                )
                            )
                        )
                    )
                )
            )
        })
    }
})

output$wgdNeededTestedID <- renderText({
    text <- input$wgdInput
    text
})

observeEvent(input$whale_configure_go, {
    OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
    whale_dir <- paste0(OrthoFinderWhaleAnalysisDir, "/Whale_wd")
    if( !dir.exists(whale_dir) ){
        dir.create(whale_dir)
    }
    # if( is.null(input$cladeSpecies) || input$cladeSpecies == "" ){
    #     shinyalert(
    #         "Opps",
    #         "Please add the clade species first...",
    #         type="error"
    #     )
    # }
    # else
    if( is.null(input$wgdInput) ){
        shinyalert(
            "Opps",
            "Please add the Hypothetical WGD events to the tree first...",
            type="error"
        )
    }
    else{
        archive_shell_file <- paste0(whale_dir, "/archive_compress_files_for_TreeRecon.shell")
        if( !file.exists(archive_shell_file) ){
            system(
                paste(
                    "cp",
                    paste0(getwd()[1], "/tools/archive_compress_files_for_TreeRecon.shell"),
                    whale_dir
                )
            )
        }

        wgdNodeFile <- paste0(whale_dir, "/wgdNodes.txt")
        writeLines(input$wgdInput, wgdNodeFile)

        uploaded_tree_file <- paste0(whale_dir, "/species_timetree.nwk")
        if( isTruthy(input$uploadSpeciesTimeTree) ){
            speciesTreeFile <- input$uploadSpeciesTimeTree$datapath
            if( !file.exists(uploaded_tree_file) ){
                system(
                    paste(
                        "cp",
                        speciesTreeFile,
                        uploaded_tree_file
                    )
                )
            }
        }else{
            demo_tree <- "(Asparagus_officinalis:117.61,(Apostasia_shenzhenica:84.55,(Dendrobium_catenatum:52.11,Phalaenopsis_equestris:52.11):32.44):33.06);"
            writeLines(demo_tree, uploaded_tree_file)
            speciesTreeFile <- uploaded_tree_file
        }

        speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
        contains_underscore <- any(sapply(speciesTree, function(line) grepl("_", line)))

        aleDirPath <- paste0(whale_dir, "/selected_tree_ALE_files")
        scriptPath <- paste0(whale_dir, "/script_bin")
        if( !dir.exists(scriptPath) ){
            dir.create(scriptPath)
        }
        if( !dir.exists(aleDirPath) ){
            withProgress(message='Creating the codes to prepare ALE files...', value=0, {
                if( !file.exists(paste0(scriptPath, "/preparing_Whale_inputs.shell")) ){
                    system(
                        paste(
                            "cp",
                            paste0(getwd()[1], "/tools/preparing_Whale_inputs.shell"),
                            scriptPath
                        )
                    )
                }
                # if( !file.exists(paste0(scriptPath, "/orthofilter.py")) ){
                #     system(
                #         paste(
                #             "cp",
                #             paste0(getwd()[1], "/tools/Whale.jl/scripts/orthofilter.py"),
                #             scriptPath
                #         )
                #     )
                # }
                if( !file.exists(paste0(scriptPath, "/ccddata.py")) ){
                    system(
                        paste(
                            "cp",
                            paste0(getwd()[1], "/tools/Whale.jl/scripts/ccddata.py"),
                            scriptPath
                        )
                    )
                }
                if( !file.exists(paste0(scriptPath, "/ccdfilter.py")) ){
                    system(
                        paste(
                            "cp",
                            paste0(getwd()[1], "/tools/Whale.jl/scripts/ccdfilter.py"),
                            scriptPath
                        )
                    )
                }

                whale_prepare_cmd_file <- paste0(whale_dir, "/run_ale_preparing.sh")
                cmd_con <- file(whale_prepare_cmd_file, open="w")
                writeLines(
                    c(
                        "#!/bin/bash",
                        "",
                        "#SBATCH -p all",
                        "#SBATCH -c 4",
                        "#SBATCH --mem 2G",
                        paste0("#SBATCH -o ", basename(whale_prepare_cmd_file), ".os%j"),
                        paste0("#SBATCH -e ", basename(whale_prepare_cmd_file), ".es%j"),
                        ""
                    ),
                    cmd_con
                )

                # writeLines(paste0("cd ", whale_dir), cmd_con)
                writeLines(
                    "tar xzf ../OrthoFinder_result.tar.gz -C ../",
                    cmd_con
                )
                writeLines("alignmentsDir=$(ls -d ../OrthoFinderOutputDir/Results_*)", cmd_con)
                # focal_species_w <- gsub(" ", "_", input$cladeSpecies)
                writeLines(
                    paste0(
                        "sh ",
                        "./script_bin/preparing_Whale_inputs.shell \\\n",
                        "\t../orthogroups.filtered.tsv \\\n",
                        "\t$alignmentsDir \\\n",
                        # "\t", focal_species_w, " \\\n",
                        "\t4"
                    ),
                    cmd_con
                )
                if( contains_underscore ){
                    aleUpdatedDirPath <- paste0(aleDirPath, ".updated")
                    if( !dir.exists(aleUpdatedDirPath) ){
                        incProgress(amount=.5, message="Change species names...")
                        Sys.sleep(.2)
                        if( !file.exists(paste0(scriptPath, "/rename_species.py")) ){
                            system(
                                paste(
                                    "cp",
                                    paste0(getwd()[1], "/tools/rename_species.py"),
                                    scriptPath
                                )
                            )
                        }
                        writeLines(
                            paste(
                                "python",
                                "./script_bin/rename_species.py",
                                "species_timetree.nwk",
                                "selected_tree_ALE_files",
                                "wgdNodes.txt"
                            ),
                            cmd_con
                        )
                    }
                    speciesTreeFile <- paste0(gsub(".nwk", "", uploaded_tree_file), ".updated.nwk")
                    aleDirPath <- paste0(aleDirPath, ".updated")
                    wgdNodeFile <- paste0(gsub(".txt", "", wgdNodeFile), ".updated.txt")
                }

                # writeLines("cd ..", cmd_con)
                # writeLines("tar czf orthofinderOutputDir.tar.gz orthofinderOutputDir && rm -r orthofinderOutputDir", cmd_con)
                close(cmd_con)
            })
        }

        withProgress(message='preparing in progress', value=0, {
            incProgress(amount=.1, message="Creating Whale command file...")
            Sys.sleep(.1)

            # Preparing Whale command lines
            whaleModel <- ""
            if( input$select_whale_model == "Constant-rates model" ){
                whaleModel <- "Constant_rates"
            }else if( input$select_whale_model == "Relaxed branch-specific DL+WGD model" ){
                whaleModel <- "Relaxed_branch"
            }else{
                whaleModel <- "Critical_branch"
            }

            running_dir <- paste0(whale_dir, "/run_", whaleModel, "_model_", input$select_chain_num)
            if( !file.exists(running_dir) ){
                dir.create(running_dir)
            }
            whaleCommandFile <- paste0(running_dir, "/whale.jl")
            whale_cmd_file <- paste0(running_dir, "/run_Whale_", whaleModel, ".sh")
            whale_cmd_con <- file(whale_cmd_file, open="w")

            writeLines(
                c(
                    "#!/bin/bash",
                    "",
                    "#SBATCH -p all",
                    "#SBATCH -c 4",
                    "#SBATCH --mem 8G",
                    paste0("#SBATCH -o ", basename(whale_cmd_file), ".so%j"),
                    paste0("#SBATCH -e ", basename(whale_cmd_file), ".se%j"),
                    ""
                ),
                whale_cmd_con
            )
            # writeLines(
            #     "module load julia",
            #     whale_cmd_con
            # )

            incProgress(amount=.5, message="Write whale command lines into file...")
            Sys.sleep(.4)

            if( !file.exists(paste0(scriptPath, "/prepare_Whale_command.v2.sh")) ){
                system(
                    paste(
                        "cp",
                        paste0(getwd()[1], "/tools/prepare_Whale_command.v2.sh"),
                        scriptPath
                    )
                )
            }

            if( input$input_gf_num != "" && input$input_gf_num > 0 ){
                writeLines(
                    paste(
                        "sh",
                        "../script_bin/prepare_Whale_command.v2.sh",
                        paste0("../", basename(speciesTreeFile)),
                        paste0("../", basename(aleDirPath)),
                        paste0("../", basename(wgdNodeFile)),
                        paste0("whale.", whaleModel, "_", input$select_chain_num, ".jl"),
                        whaleModel,
                        input$select_chain_num,
                        input$input_gf_num
                    ),
                    whale_cmd_con
                )
            }else{
                writeLines(
                    paste(
                        "sh",
                        "../script_bin/prepare_Whale_command.v2.sh",
                        paste0("../", basename(speciesTreeFile)),
                        paste0("../", basename(aleDirPath)),
                        paste0("../", basename(wgdNodeFile)),
                        paste0("whale.", whaleModel, "_", input$select_chain_num, ".jl"),
                        whaleModel,
                        input$select_chain_num
                    ),
                    whale_cmd_con
                )
            }

            writeLines(
                paste(
                    "julia",
                    paste0("whale.", whaleModel, "_", input$select_chain_num, ".jl")
                ),
                whale_cmd_con
            )

            close(whale_cmd_con)
            closeAllConnections()
        })
    }
})

observeEvent(input$go_whale_codes, {
    OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
    whale_dir <- paste0(OrthoFinderWhaleAnalysisDir, "/Whale_wd")
    alePreparingCommadFile <- paste0(whale_dir, "/run_ale_preparing.sh")

    whaleModel <- ""
    if( input$select_whale_model == "Constant-rates model" ){
        whaleModel <- "Constant_rates"
    }else if( input$select_whale_model == "Relaxed branch-specific DL+WGD model" ){
        whaleModel <- "Relaxed_branch"
    }else{
        whaleModel <- "Critical_branch"
    }

    running_dir <- paste0(whale_dir, "/run_", whaleModel, "_model_", input$select_chain_num)

    whaleCommandFile <- paste0(running_dir, "/run_Whale_", whaleModel, ".sh")

    if( !file.exists(alePreparingCommadFile) & !file.exists(whaleCommandFile) ){
        shinyalert(
            "Oops",
            "Please click the Create-Whale-Codes button first, then switch this on",
            type="error"
        )
    }
    else{
        output$alePreparingCommandTxt <- renderText({
            command_info <- readChar(
                alePreparingCommadFile,
                file.info(alePreparingCommadFile)$size
            )
        })

        output$whaleCommandTxt <- renderText({
            command_info <- readChar(
                whaleCommandFile,
                file.info(whaleCommandFile)$size
            )
        })

        showModal(
            modalDialog(
                title="",
                size="xl",
                uiOutput("whaleParameterPanel")
            )
        )

        output$whaleParameterPanel <- renderUI({
            fluidRow(
                div(
                    style="padding-bottom: 10px;
                           padding-left: 20px;
                           padding-right: 20px;
                           max-width: 100%;
                           overflow-x: auto;",
                    column(
                        12,
                        h5(HTML(paste0("Instructions to initiate the <font color='green'><b><i>Whale</i></b></font> analysis."))),
                        h6(
                            HTML(
                                paste0(
                                    "After decompressing the download file, please navigate to the folder: ",
                                    "<b><i><font color='#5F9EA0'>Whale_wd</b></i></font> ",
                                    "and execute <b><font color='#CD853F'>run_ale_preparing.sh</b></font> to generate input files for <font color='green'><b><i>whale</i></b></font>.",
                                    "<br><br><b><i>MrBayes</i></b> and <b><i>ALEobserve</i></b> are required for creating <b><i>ale</i></b> files. ",
                                    "Ensure these tools are installed.",
                                    "<br><br>After successfully generating <b><i>ale</i></b> files, proceed to the folder: <b><i><font color='#5F9EA0'>",
                                    basename(running_dir), "</b></i></font> ",
                                    "and run <b><font color='#CD853F'>run_Whale_", whaleModel, ".sh</b></font>.",
                                    "This script will create and execute the <font color='green'><b><i>whale</i></b></font> script.",
                                    "<br><br>Please ensure <b><i>Julia</i></b> is installed."
                                )
                            )
                        ),
                        hr(class="setting"),
                        h5(HTML(paste0("The command line to prepare the inputs for <font color='green'><b><i>whale</i></b></font>:"))),
                        verbatimTextOutput(
                            "alePreparingCommandTxt",
                            placeholder=TRUE
                        ),
                        h5(HTML(paste0("The command line for <font color='green'><b><i>whale</i></b></font>:"))),
                        verbatimTextOutput(
                            "whaleCommandTxt",
                            placeholder=TRUE
                        )
                    )
                )
            )
        })
    }
})

output$whale_prepared_data_download <- downloadHandler(
    filename=function(){
        OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
        paste0(basename(OrthoFinderWhaleAnalysisDir), ".tgz")
    },
    content=function(file){
        OrthoFinderWhaleAnalysisDir <- orthofinder_whale_analysis_dir_Val()
        whale_dir <- paste0(OrthoFinderWhaleAnalysisDir, "/Whale_wd")
        alePreparingCommadFile <- paste0(whale_dir, "/run_ale_preparing.sh")

        whaleModel <- ""
        if( input$select_whale_model == "Constant-rates model" ){
            whaleModel <- "Constant_rates"
        }else if( input$select_whale_model == "Relaxed branch-specific DL+WGD model" ){
            whaleModel <- "Relaxed_branch"
        }else{
            whaleModel <- "Critical_branch"
        }

        running_dir <- paste0(whale_dir, "/run_", whaleModel, "_model_", input$select_chain_num)

        whaleCommandFile <- paste0(running_dir, "/run_Whale_", whaleModel, ".sh")

        if( !file.exists(alePreparingCommadFile) & !file.exists(whaleCommandFile) ){
            shinyalert(
                "Oops",
                "Please click the Create-Whale-Codes button first, then switch this on",
                type="error"
            )
        }
        else{
            shinyjs::runjs('$("#progress_modal_whale").modal("show");')
            withProgress(message='Downloading in progress', value=0, {
                incProgress(amount=.1, message="Compressing files...")
                # shinyalert(
                #     "Note",
                #     "Please wait for compressing the files. Do not close the page.",
                #     type="info"
                # )
                run_dir <- getwd()
                setwd(dirname(OrthoFinderWhaleAnalysisDir))
                system(
                    paste(
                        "tar czf",
                        file,
                        basename(OrthoFinderWhaleAnalysisDir)
                    )
                )

                incProgress(amount=.9, message="Downloading file ...")
                incProgress(amount=1)
                Sys.sleep(.1)
                setwd(run_dir)
            })
            shinyjs::runjs('$("#progress_modal_whale").modal("hide");');
        }
    }
)

#' Count Ortholog Genes in a Species
#'
#' This function counts ortholog genes in a given species based on input data.
#'
#' @param atomic.df A data frame containing information about ortholog genes.
#'        It should have the following columns:
#'        - multiplicon: The multiplicon identifier.
#'        - geneX: The gene identifier in speciesX.
#'        - speciesX: The species name for geneX.
#'        - listX: The chromosome or list identifier for geneX.
#'        - coordX: The coordinate information for geneX.
#'        - geneY: The gene identifier in speciesY.
#'        - speciesY: The species name for geneY.
#'        - listY: The chromosome or list identifier for geneY.
#'        - coordY: The coordinate information for geneY.
#'        - level: The orthology level.
#'        - num_anchors: The number of anchors.
#'        - is_real: A flag indicating if the data is real.
#'        - Ks: The Ks value.
#' @param species The species for which ortholog gene counts should be computed.
#'
#' @return A data frame summarizing the counts of ortholog genes for each chromosome.
#'
#' @examples
#' # Example usage:
#' # Create a sample atomic.df data frame
#' atomic.df <- data.frame(
#'   multiplicon=c(2, 2, 2),
#'   geneX=c("GSVIVT01016360001", "GSVIVT01016362001", "GSVIVT01016362001"),
#'   speciesX=c("Vitis_vinifera", "Vitis_vinifera", "Vitis_vinifera"),
#'   listX=c("chr13:181-369", "chr13:181-369", "chr13:181-369"),
#'   coordX=c(188, 187, 1875),
#'   geneY=c("Os01t0854500-01", "Os01t0854000-01", "Os05t0449200-01"),
#'   speciesY=c("Oryza_sativa", "Oryza_sativa", "Oryza_sativa"),
#'   listY=c("chr01:3463-3614", "chr01:3463-3614", "chr05:1610-1714"),
#'   coordY=c(135, 133, 14),
#'   level=c(10000, 10000, 10000),
#'   num_anchors=c(10000, 10000, 10000),
#'   is_real=c(-1, -1, -1),
#'   Ks=c(81.3724, 77.5539, 76.6121)
#' )
#'
#' # Calculate ortholog gene counts for "Vitis_vinifera"
#' ortholog_counts <- CountOrthologs(atomic.df, species="Vitis_vinifera")
#' ortholog_counts
CountOrthologs <- function(atomic.df, species) {
  if (length(unique(atomic.df$speciesX)) > 1) {
    stop ("atomatic.df is not ordered")
  }

  if (length(unique(atomic.df$speciesY)) > 1) {
    stop ("atomatic.df is not ordered")
  }

  if(atomic.df$speciesX[1] == species){
    to.count.df <-atomic.df[,c("geneX", "listX", "listY")]  # here I added the chr of corresponding anchorpoints to further remove tandem duplicates
  } else if (atomic.df$speciesY[1] == species) {
    to.count.df <- atomic.df[,c("geneY", "listY", "listX")] # here I added the chr of corresponding anchorpoints to further remove tandem duplicates
  } else {
    stop ("atomatic.df do not have the speices")
  }

  names(to.count.df) <- c("gene", "chr", "anchor_chr")
  chrs <- unique(to.count.df$chr)
  summary <- list()
  for (ch in chrs) {
    genes <- to.count.df[to.count.df$chr == ch, c("gene", "anchor_chr")]
    summary[[ch]] <- table(table(genes$gene))
  }
  #return(summary)

  max.num <- max(unlist(lapply(summary, function(x) max(as.numeric(names(x))))))
  row.num <- length(chrs)
  summary.df <- as.data.frame(matrix(NA, nrow=row.num, ncol=max.num + 1))
  names(summary.df) <- c("chr_segs", as.character(rep(1:max.num)))

  for (i in 1:row.num) {
    summary.df[i,1] <- names(summary)[i]
    for (j in 1:max.num) {
      n <- summary[[i]][which(names(summary[[i]]) == j)]
      if (length(n) == 1) {
        summary.df[i,(j+1)] <- n
      } else if (length(n) == 0) {
        summary.df[i,(j+1)] <- 0
      } else {
        stop ("some counts are wrong!!!")
      }
    }
  }
  return(summary.df)
}

########## R function: SignifFeatureRegion ##########

# For determining the region of significant
# gradient for a particular bandwidth and
# significance level.

# Last changed: 18 JAN 2006

#' SignifFeatureRegion
#'
#' This function computes the significance of features based on gradient and curvature analysis.
#'
#' @param n The sample size.
#' @param d The dimensionality of the data.
#' @param gcounts A numeric vector representing data counts.
#' @param gridsize A numeric vector specifying the grid size.
#' @param dest A kernel density estimate.
#' @param bandwidth The bandwidth parameter.
#' @param signifLevel The significance level.
#' @param range.x The range of x values.
#' @param grad A logical value indicating whether to compute the gradient significance.
#' @param curv A logical value indicating whether to compute the curvature significance.
#' @param neg.curv.only A logical value indicating whether to consider negative curvature only.
#'
#' @importFrom stats pchisq
#'
#' @return A list containing the significance results for gradient and curvature.
#'
SignifFeatureRegion <- function(n, d, gcounts, gridsize, dest, bandwidth, signifLevel, range.x, grad=TRUE, curv=TRUE, neg.curv.only=TRUE)
{
  h <- bandwidth

  ESS <- n*dest$est*prod(h)*(sqrt(2*pi)^d)
  SigESS <- ESS >= 5

  Sig.scalar <- array(NA, dim=gridsize)
  Sig2.scalar <- array(NA, dim=gridsize)

  dest$est[dest$est<0] <- 0
  ## constant for variance of gradient estimate
  Sig.scalar <- 1/2*(2*sqrt(pi))^(-d)*n^(-1)*prod(h)^(-1)*dest$est

  ##  constants for variance of curvature estimate
  if (d==1)
    Sig2.scalar <- (8*sqrt(pi)*n*prod(h))^(-1)*dest$est
  else if (d==2)
    Sig2.scalar <- (16*pi*n*prod(h))^(-1)*dest$est
  else if (d==3)
    Sig2.scalar <- (32*pi^(3/2)*n*prod(h))^(-1)*dest$est
  else if (d==4)
    Sig2.scalar <- (64*pi^2*n*prod(h))^(-1)*dest$est

  ## Matrix square root - taken from Stephen Lake
  ## http://www5.biostat.wustl.edu/s-news/s-news-archive/200109/msg00067.html

  matrix.sqrt <- function(A)
  {
    sva <- svd(A)
    if (min(sva$d)>=0)
      Asqrt <- sva$u %*% diag(sqrt(sva$d)) %*% t(sva$v)
    else
      stop("Matrix square root is not defined")
    return(Asqrt)
  }


  if (d>1)
  {
    WaldGrad <- array(NA, dim=gridsize)
    WaldCurv <- array(NA, dim=gridsize)
    local.mode <- array(FALSE, dim=gridsize)
  }

  if (d==1)
  {
    if (grad)
    {
      obj1 <- drvkde(gcounts, drv=1, bandwidth=h, binned=TRUE, range.x=range.x, se=FALSE)
      fhat1 <- obj1$est

      Sig.inv12 <- 1/sqrt(Sig.scalar * h^(-2))
      WaldGrad <- (Sig.inv12 * fhat1)^2
    }

    if (curv)
    {
      obj2 <- drvkde(gcounts,drv=2,bandwidth=h,binned=TRUE,range.x=range.x, se=FALSE)
      fhat2 <- obj2$est

      Sig2.inv12 <- 1/sqrt(Sig2.scalar * 3*h^(-4))
      lambda1 <- Sig2.inv12 * fhat2
      WaldCurv <- lambda1^2
      local.mode <- (lambda1 < 0)
    }
  }

  if (d==2)
  {
    if (grad)
    {
      obj10 <- drvkde(gcounts,drv=c(1,0),bandwidth=h,binned=TRUE,
                      range.x=range.x,se=FALSE)
      obj01 <- drvkde(gcounts,drv=c(0,1),bandwidth=h,binned=TRUE,
                      range.x=range.x,se=FALSE)
      fhat10 <- obj10$est
      fhat01 <- obj01$est

      for (i1 in 1:gridsize[1])
        for (i2 in 1:gridsize[2])
          if (SigESS[i1,i2])
          {
            Sig.inv12 <- 1/sqrt(Sig.scalar[i1,i2] * h^(-2))
            WaldGrad[i1,i2] <- sum((Sig.inv12 * c(fhat10[i1,i2], fhat01[i1,i2]))^2)
          }
    }


    if (curv)
    {
      Sig2.mat <-
        matrix(c(3/h[1]^4, 0, 1/(h[1]^2*h[2]^2),
                 0, 1/(h[1]^2*h[2]^2), 0,
                 1/(h[1]^2*h[2]^2), 0, 3/h[2]^4),
               nrow=3, ncol=3)

      Sig2.mat.inv <- chol2inv(chol(Sig2.mat))

      obj20 <- drvkde(gcounts,drv=c(2,0),bandwidth=h,
                      binned=TRUE,range.x=range.x, se=FALSE)
      obj11 <- drvkde(gcounts,drv=c(1,1),bandwidth=h,
                      binned=TRUE,range.x=range.x, se=FALSE)
      obj02 <- drvkde(gcounts,drv=c(0,2),bandwidth=h,
                      binned=TRUE,range.x=range.x, se=FALSE)
      fhat20 <- obj20$est
      fhat11 <- obj11$est
      fhat02 <- obj02$est

      for (i1 in 1:gridsize[1])
        for (i2 in 1:gridsize[2])
          if (SigESS[i1,i2])
          {
            Sig2.inv12 <- sqrt(1/Sig2.scalar[i1,i2])*matrix.sqrt(Sig2.mat.inv)
            fhat.temp <- Sig2.inv12 %*%
              c(fhat20[i1,i2], fhat11[i1,i2], fhat02[i1,i2])

            WaldCurv[i1,i2] <- sum(fhat.temp^2)
          }

      lambda1 <- ((fhat20 + fhat02) - sqrt((fhat20-fhat02)^2 + 4*fhat11^2))/2
      lambda2 <- ((fhat20 + fhat02) + sqrt((fhat20-fhat02)^2 + 4*fhat11^2))/2
      local.mode <- (lambda1 < 0) & (lambda2 < 0)
    }
  }


  if (d==3)
  {
    if (grad)
    {
      obj100 <- drvkde(gcounts,drv=c(1,0,0),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj010 <- drvkde(gcounts,drv=c(0,1,0),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj001 <- drvkde(gcounts,drv=c(0,0,1),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)

      fhat100 <- obj100$est
      fhat010 <- obj010$est
      fhat001 <- obj001$est

      for (i1 in 1:gridsize[1])
        for (i2 in 1:gridsize[2])
          for (i3 in 1:gridsize[3])
            if (SigESS[i1,i2,i3])
            {
              Sig.inv12 <- 1/sqrt(Sig.scalar[i1,i2,i3] * h^(-2))
              WaldGrad[i1,i2,i3] <-
                sum((Sig.inv12 * c(fhat100[i1,i2,i3], fhat010[i1,i2,i3],
                                   fhat001[i1,i2,i3]))^2)
            }
    }


    if (curv)
    {
      obj200 <- drvkde(gcounts,drv=c(2,0,0),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj110 <- drvkde(gcounts,drv=c(1,1,0),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj101 <- drvkde(gcounts,drv=c(1,0,1),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj020 <- drvkde(gcounts,drv=c(0,2,0),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj011 <- drvkde(gcounts,drv=c(0,1,1),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      obj002 <- drvkde(gcounts,drv=c(0,0,2),bandwidth=h,
                       binned=TRUE,range.x=range.x, se=FALSE)
      fhat200 <- obj200$est
      fhat110 <- obj110$est
      fhat101 <- obj101$est
      fhat020 <- obj020$est
      fhat011 <- obj011$est
      fhat002 <- obj002$est

      Sig2.mat <-
        matrix(c(3/h[1]^4, 0, 0, 1/(h[1]*h[2])^2, 0, 1/(h[1]*h[3])^2,
                 0, 1/(h[1]*h[2])^2, 0, 0, 0, 0,
                 0, 0, 1/(h[1]*h[3])^2, 0, 0, 0,
                 1/(h[1]*h[2])^2, 0, 0, 3/h[2]^4, 0, 1/(h[2]*h[3])^2,
                 0, 0, 0, 0, 1/(h[2]*h[3])^2, 0,
                 1/(h[1]*h[3])^2, 0, 0, 1/(h[2]*h[3])^2, 0, 3/h[3]^4),
               nrow=6, ncol=6)

      Sig2.mat.inv <- chol2inv(chol(Sig2.mat))

      ## at each grid point, find eigenvalues of vech'ed curvature matrix
      for (i1 in 1:gridsize[1])
        for (i2 in 1:gridsize[2])
          for (i3 in 1:gridsize[3])
            if (SigESS[i1,i2,i3])
            {
              Sig2.inv12 <- sqrt(1/Sig2.scalar[i1,i2,i3]) *
                matrix.sqrt(Sig2.mat.inv)
              fhat.temp <- Sig2.inv12 %*%
                c(fhat200[i1,i2,i3], fhat110[i1,i2,i3], fhat101[i1,i2,i3],
                  fhat020[i1,i2,i3], fhat011[i1,i2,i3], fhat002[i1,i2,i3])

              D2.mat <-
                matrix(c(fhat200[i1,i2,i3], fhat110[i1,i2,i3], fhat101[i1,i2,i3],
                         fhat110[i1,i2,i3], fhat020[i1,i2,i3], fhat011[i1,i2,i3],
                         fhat101[i1,i2,i3], fhat011[i1,i2,i3], fhat002[i1,i2,i3]),
                       nrow=3)
              lambda <- eigen(D2.mat, symmetric=TRUE, only.values=TRUE)$values

              WaldCurv[i1,i2,i3] <- sum(fhat.temp^2)

              local.mode[i1,i2,i3] <- all(lambda < 0)
          }
    }

  }

  if (d==4)
  {
    if (grad)
    {
      obj1000 <- drvkde(gcounts,drv=c(1,0,0,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0100 <- drvkde(gcounts,drv=c(0,1,0,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0010 <- drvkde(gcounts,drv=c(0,0,1,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0001 <- drvkde(gcounts,drv=c(0,0,0,1),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)

      fhat1000 <- obj1000$est
      fhat0100 <- obj0100$est
      fhat0010 <- obj0010$est
      fhat0001 <- obj0001$est

      for (i1 in 1:gridsize[1])
        for (i2 in 1:gridsize[2])
          for (i3 in 1:gridsize[3])
            for (i4 in 1:gridsize[4])
              if (SigESS[i1,i2,i3,i4])
              {
                Sig.inv12 <- 1/sqrt(Sig.scalar[i1,i2,i3,i4] * h^(-2))
                WaldGrad[i1,i2,i3,i4] <-
                  sum((Sig.inv12*c(fhat1000[i1,i2,i3,i4],fhat0100[i1,i2,i3,i4],
                                   fhat0010[i1,i2,i3,i4],fhat0001[i1,i2,i3,i4]))^2)
              }
    }

    if (curv)
    {
      obj2000 <- drvkde(gcounts,drv=c(2,0,0,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj1100 <- drvkde(gcounts,drv=c(1,1,0,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj1010 <- drvkde(gcounts,drv=c(1,0,1,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj1001 <- drvkde(gcounts,drv=c(1,0,0,1),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0200 <- drvkde(gcounts,drv=c(0,2,0,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0110 <- drvkde(gcounts,drv=c(0,1,1,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0101 <- drvkde(gcounts,drv=c(0,1,0,1),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0020 <- drvkde(gcounts,drv=c(0,0,2,0),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0011 <- drvkde(gcounts,drv=c(0,0,1,1),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)
      obj0002 <- drvkde(gcounts,drv=c(0,0,0,2),bandwidth=h,
                        binned=TRUE,range.x=range.x, se=FALSE)

      fhat2000 <- obj2000$est
      fhat1100 <- obj1100$est
      fhat1010 <- obj1010$est
      fhat1001 <- obj1001$est
      fhat0200 <- obj0200$est
      fhat0110 <- obj0110$est
      fhat0101 <- obj0101$est
      fhat0020 <- obj0020$est
      fhat0011 <- obj0011$est
      fhat0002 <- obj0002$est

      Sig2.mat <-
        matrix(c(3/h[1]^4, 0, 0, 0, 1/(h[1]*h[2])^2, 0, 0, 1/(h[1]*h[3])^2, 0, 1/(h[1]*h[4])^2,
                 0, 1/(h[1]*h[2])^2, 0, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 1/(h[1]*h[3])^2, 0, 0, 0, 0, 0, 0, 0,
                 0, 0, 0, 1/(h[1]*h[4])^2, 0, 0, 0, 0, 0, 0,
                 1/(h[1]*h[2])^2, 0, 0, 0, 3/h[2]^4, 0, 0, 1/(h[2]*h[3])^2, 0, 1/(h[2]*h[4])^2,
                 0, 0, 0, 0, 0, 1/(h[2]*h[3])^2, 0, 0, 0, 0,
                 0, 0, 0, 0 ,0, 0, 1/(h[2]*h[4])^2, 0, 0, 0,
                 1/(h[1]*h[3])^2, 0, 0, 0, 1/(h[2]*h[3])^2, 0, 0, 3/h[3]^4, 0, 1/(h[3]*h[4])^2,
                 0, 0, 0, 0, 0, 0, 0, 0, 1/(h[3]*h[4])^2, 0,
                 1/(h[1]*h[4])^2, 0, 0, 0, 1/(h[2]*h[4])^2, 0, 0, 1/(h[3]*h[4])^2, 0, 3/h[4]^4),
               nrow=10, ncol=10)

      Sig2.mat.inv <- chol2inv(chol(Sig2.mat))

      for (i1 in 1:gridsize[1])
        for (i2 in 1:gridsize[2])
          for (i3 in 1:gridsize[3])
            for (i4 in 1:gridsize[4])
              if (SigESS[i1,i2,i3,i4])
              {
                Sig2.inv12 <- sqrt(1/Sig2.scalar[i1,i2,i3,i4]) *
                  matrix.sqrt(Sig2.mat.inv)
                fhat.temp <- Sig2.inv12 %*%
                  c(fhat2000[i1,i2,i3,i4], fhat1100[i1,i2,i3,i4],
                    fhat1010[i1,i2,i3,i4], fhat1001[i1,i2,i3,i4],
                    fhat0200[i1,i2,i3,i4], fhat0110[i1,i2,i3,i4],
                    fhat0101[i1,i2,i3,i4], fhat0020[i1,i2,i3,i4],
                    fhat0011[i1,i2,i3,i4], fhat0002[i1,i2,i3,i4])

                D2.mat <-
                  matrix(c(fhat2000[i1,i2,i3,i4], fhat1100[i1,i2,i3,i4], fhat1010[i1,i2,i3,i4], fhat1001[i1,i2,i3,i4],
                           fhat1100[i1,i2,i3,i4], fhat0200[i1,i2,i3,i4], fhat0110[i1,i2,i3,i4], fhat0101[i1,i2,i3,i4],
                           fhat1010[i1,i2,i3,i4], fhat0110[i1,i2,i3,i4], fhat0020[i1,i2,i3,i4], fhat0011[i1,i2,i3,i4],
                           fhat1001[i1,i2,i3,i4], fhat0101[i1,i2,i3,i4], fhat0011[i1,i2,i3,i4], fhat0002[i1,i2,i3,i4]),
                         nrow=4)
                WaldCurv[i1,i2,i3,i4] <- sum(fhat.temp^2)
                lambda <- eigen(D2.mat, symmetric=TRUE, only.values=TRUE)$values

                local.mode[i1,i2,i3,i4] <- all(lambda < 0)
              }
    }
  }


  ## multiple hypothesis testing - based on Hochberg's method
  ## - modified Bonferroni method using ordered p-values

  ## test statistic for gradient
  if (grad)
  {
    pval.Grad <- 1 - pchisq(WaldGrad, d)
    pval.Grad.ord <- pval.Grad[order(pval.Grad)]
    num.test <- sum(!is.na(pval.Grad.ord))

    if (num.test>=1)
      num.test.seq <- c(1:num.test, rep(NA, prod(gridsize) - num.test))
    else
      num.test.seq <- rep(NA, prod(gridsize))

    reject.nonzero <- ((pval.Grad.ord <= signifLevel/(num.test + 1 - num.test.seq)) &
                       (pval.Grad.ord > 0))
    reject.nonzero.ind <- which(reject.nonzero)

    ## p-value == 0 => reject null hypotheses automatically
    SignifGrad <- array(FALSE, dim=gridsize)
    SignifGrad[which(pval.Grad==0, arr.ind=TRUE)] <- TRUE

    ## p-value > 0 then reject null hypotheses indicated in reject.nonzero.ind
    for (i in reject.nonzero.ind)
      SignifGrad[which(pval.Grad==pval.Grad.ord[i], arr.ind=TRUE)] <- TRUE
  }

  ## test statistic for curvature
  if (curv)
  {
    pval.Curv <- 1 - pchisq(WaldCurv, d*(d+1)/2)
    pval.Curv.ord <- pval.Curv[order(pval.Curv)]
    num.test <- sum(!is.na(pval.Curv.ord))

    if (num.test>=1)
      num.test.seq <- c(1:num.test, rep(NA, prod(gridsize) - num.test))
    else
      num.test.seq <- rep(NA, prod(gridsize))
    reject.nonzero <- ((pval.Curv.ord <= signifLevel/(num.test + 1 - num.test.seq)) &(pval.Curv.ord > 0))
    reject.nonzero.ind <- which(reject.nonzero)

    SignifCurv <- array(FALSE, dim=gridsize)

    ## p-value == 0 => reject null hypotheses automatically
    SignifCurv[which(pval.Curv==0, arr.ind=TRUE)] <- TRUE

    ## p-value > 0 then reject null hypotheses indicated in reject.nonzero.ind
    for (i in reject.nonzero.ind)
      SignifCurv[which(pval.Curv==pval.Curv.ord[i], arr.ind=TRUE)] <- TRUE

    if (neg.curv.only) SignifCurv <- SignifCurv & local.mode
  }

  if (grad & !curv)
    return(list(grad=SignifGrad))
  else if (!grad & curv)
    return(list(curv=SignifCurv))
  else if (grad & curv)
    return(list(grad=SignifGrad, curv=SignifCurv))
}


########## End of SignifFeatureRegion ##########

# These are from package feature. They not exprted in the package.
######### R-function dfltBWrange  #########

# Obtain default set of grid counts from a
# multivariate point cloud 'x'.

# Last changed: 22 JUL 2005


#' dfltBWrange
#'
#' This function computes the default bandwidth range for kernel density estimation.
#'
#' @param x The input data, which can be a numeric vector or matrix.
#' @param tau A parameter used in bandwidth calculation.
#'
#' @return A list of bandwidth ranges for each dimension of the input data.
#'
dfltBWrange <- function(x,tau) {
  d <- ncol(x)
  if (d==1) x <- as.matrix(x)

  r <- 2
  cmb.fac.upp <- (4/((d+2*r+2)*nrow(x)))^(1/(d+2*r+4))
  r <- 0
  cmb.fac.low <- (4/((d+2*r+2)*nrow(x)))^(1/(d+2*r+4))

  sd <- IQR <- NULL
  ## Compute the scale in each direction
  st.devs <- apply(x,2,sd)
  IQR.vals <- apply(x, 2, IQR)/(stats::qnorm(3/4) - stats::qnorm(1/4))
  sig.hats <- apply(cbind(st.devs,IQR.vals),1,min)
  ##range.vals <- apply(x,2,max) - apply(x,2,min)

  range.h <- list()
  for (id in 1:d)
  {
    h.upp <- cmb.fac.upp*sig.hats[id]
    h.low <- 0.1*cmb.fac.low*sig.hats[id] ##3*(range.vals[id] + 2*tau*h.upp)/((gridsize[id]-1)*tau)
    range.h[[id]] <- c(h.low,h.upp)
  }

  return(range.h)
}

######## End of dfltBWrange ########
######### R-function:dfltCounts  #########
# Obtain default set of grid counts from a
# multivariate point cloud 'x'.
# Last changed: 18 JUL 2005

#' dfltCounts
#'
#' This function bins the input data into a regular grid.
#'
#' @param x The input data, which should be a numeric matrix.
#' @param gridsize A vector specifying the number of bins along each dimension.
#' @param h A vector specifying the bandwidth (smoothing parameter) along each dimension.
#' @param supp A parameter for determining the range of the bins.
#' @param range.x A list specifying the range of values for each dimension.
#' @param w A vector of weights for the data points.
#'
#' @return A list containing the binned counts and the range of values for each dimension.
#'
dfltCounts <- function(x,gridsize=rep(64,NCOL(x)),h=rep(0,NCOL(x)), supp=3.7, range.x, w)
{
  x <- as.matrix(x)
  d <- ncol(x)
  n <- nrow(x)
  if (missing(w)) w <- rep(1,n)

  if (missing(range.x))
  {
    range.x <- list()
    for (id in 1:d)
      range.x[[id]] <- c(min(x[,id])-supp*h[id],max(x[,id])+supp*h[id])
  }

  a <- unlist(lapply(range.x,min))
  b <- unlist(lapply(range.x,max))

  gpoints <- list()
  for (id in 1:d)
    gpoints[[id]] <- seq(a[id],b[id],length=gridsize[id])

  if ((d!=1)&(d!=2)&(d!=3)&(d!=4)) stop("binning implemented only for d=1,2,3,4")

  if (d==1) gcounts <- ks::binning(x=x, bgridsize=gridsize, h=h, xmin=a, xmax=b, w=w)$counts
  else if (d>1) gcounts <- ks::binning(x=x, bgridsize=gridsize, H=diag(h^2), xmin=a, xmax=b, w=w)$counts

  ##if (d==1) gcounts <- linbin.ks(x,gpoints[[1]], w=w)
  ##if (d==2) gcounts <- linbin2D.ks(x,gpoints[[1]],gpoints[[2]], w=w)
  ##if (d==3) gcounts <- linbin3D.ks(x,gpoints[[1]],gpoints[[2]],gpoints[[3]], w=w)
  ##if (d==4) gcounts <- linbin4D.ks(x,gpoints[[1]],gpoints[[2]],gpoints[[3]],gpoints[[4]], w=w)

  return(list(counts=gcounts,range.x=range.x))
}

######## End of dfltCounts ########

#####################################################################
## Matt Wand's version of binned kernel density derivative estimation
##
## Computes the mth derivative of a binned
## d-variate kernel density estimate based
## on grid counts.
#############################################################

#' drvkde
#'
#' Compute the mth derivative of a binned d-variate kernel density estimate based on grid counts.
#'
#' @param x The input data.
#' @param drv The order of the derivative to compute.
#' @param bandwidth The bandwidth (smoothing parameter) along each dimension.
#' @param gridsize The size of the grid.
#' @param range.x A list specifying the range of values for each dimension.
#' @param binned A logical indicating whether the input data is already binned.
#' @param se A logical indicating whether to compute standard errors.
#' @param w A vector of weights for the data points.
#'
#' @return A list containing the estimated density or derivative, and optionally, standard errors.
#'
drvkde <- function(x,drv,bandwidth,gridsize,range.x,binned=FALSE,se=TRUE, w)
{
  d <- length(drv)
  if (d==1) x <- as.matrix(x)

  ## Rename common variables
  h <- bandwidth
  tau <- 4 + max(drv)
  if (length(h)==1) h <- rep(h,d)

  if (missing(gridsize))
    if (!binned)   ## changes 16/02/2009
    {
      if (d==1) gridsize <- 401
      else if (d==2) gridsize <- rep(151,d)
      else if (d==3) gridsize <- rep(51, d)
      else if (d==4) gridsize <- rep(21, d)
    }
  else
  {
    if (d==1) gridsize <- dim(x)[1]
    else gridsize <- dim(x)
  }

  if(missing(w)) w <- rep(1,nrow(x))
  ## Bin the data if not already binned

  if (missing(range.x))
  {
    range.x <- list()
    for (id in 1:d)
      range.x[[id]] <- c(min(x[,id])-tau*h[id],max(x[,id])+tau*h[id])
  }

  a <- unlist(lapply(range.x,min))
  b <- unlist(lapply(range.x,max))

  M <- gridsize
  gpoints <- list()

  for (id in 1:d)
    gpoints[[id]] <- seq(a[id],b[id],length=M[id])

  if (binned==FALSE)
  {
    if (d==1) gcounts <- ks::binning(x=x, bgridsize=gridsize, h=h, xmin=a, xmax=b, w=w)$counts
    else if (d>1) gcounts <- ks::binning(x=x, bgridsize=gridsize, H=diag(h^2), xmin=a, xmax=b, w=w)$counts
  }
  else
    gcounts <- x

  n <- sum(gcounts)

  kapmid <- list()
  for (id in (1:d))
  {
    ## changes to Lid 13/02/2009
    Lid <- max(min(floor(tau*h[id]*(M[id]-1)/(b[id]-a[id])),M[id]),d)
    lvecid <- (0:Lid)
    facid  <- (b[id]-a[id])/(h[id]*(M[id]-1))
    argid <- lvecid*facid
    kapmid[[id]] <- stats::dnorm(argid)/(h[id]^(drv[id]+1))
    hmold0 <- 1
    hmold1 <- argid
    if (drv[id]==0) hmnew <- 1
    if (drv[id]==1) hmnew <- argid
    if (drv[id] >= 2)
      for (ihm in (2:drv[id]))
      {
        hmnew <- argid*hmold1 - (ihm-1)*hmold0
        hmold0 <- hmold1   # Compute drv[id] degree Hermite polynomial
        hmold1 <- hmnew    # by recurrence.
      }
    kapmid[[id]] <- hmnew*kapmid[[id]]*(-1)^drv[id]
  }

  if (d==1) kappam <- kapmid[[1]]/n
  if (d==2) kappam <- outer(kapmid[[1]],kapmid[[2]])/n
  if (d==3) kappam <- outer(kapmid[[1]],outer(kapmid[[2]],kapmid[[3]]))/n
  if (d==4) kappam <- outer(kapmid[[1]],outer(kapmid[[2]],outer(kapmid[[3]],kapmid[[4]])))/n

  if (!any(c(d==1,d==2,d==3,d==4))) stop("only for d=1,2,3,4")

  if (d==1)
  {
    est <- symconv.ks(kappam,gcounts,skewflag=(-1)^drv)
    if (se) est.var <- ((symconv.ks((n*kappam)^2,gcounts)/n) - est^2)/(n-1)
  }

  if (d==2)
  {
    ##est <- ks:::symconv.nd(kappam,gcounts,d=d)
    ##if (se) est.var <- ((ks:::symconv.nd((n*kappam)^2,gcounts,d=d)/n) - est^2)/(n-1)
    est <- symconv2D.ks(kappam,gcounts,skewflag=(-1)^drv)
    if (se) est.var <- ((symconv2D.ks((n*kappam)^2,gcounts)/n) - est^2)/(n-1)
  }

  if (d==3)
  {
    est <- symconv3D.ks(kappam,gcounts,skewflag=(-1)^drv)
    if (se) est.var <- ((symconv3D.ks((n*kappam)^2,gcounts)/n) - est^2)/(n-1)
  }

  if (d==4)
  {
    est <- symconv4D.ks(kappam,gcounts,skewflag=(-1)^drv)
    if (se) est.var <- ((symconv4D.ks((n*kappam)^2,gcounts)/n) - est^2)/(n-1)
  }

  if (se)
  {
    est.var[est.var<0] <- 0
    return(list(x.grid=gpoints,est=est,se=sqrt(est.var)))
  }
  else if (!se)
    return(list(x.grid=gpoints,est=est))
}


########################################################################
## Discrete convolution
########################################################################


## Computes the discrete convolution of
## a symmetric or skew-symmetric response
## vector r and a data vector s.
## If r is symmetric then "skewflag"=1.
## If r is skew-symmetric then "skewflag"=-1.


#' symconv.ks
#'
#' Perform symmetric convolution using FFT.
#'
#' @param rr The first input vector.
#' @param ss The second input vector.
#' @param skewflag A scalar value to apply skew correction.
#'
#' @importFrom stats fft
#'
#' @return A vector representing the result of the symmetric convolution.
#'
symconv.ks <- function (rr,ss,skewflag)
{
  L <- length(rr) - 1
  M <- length(ss)
  P <- 2^(ceiling(log(M + L)/log(2)))
  rp <- rep(0,P)
  rp[1:(L+1)] <- rr
  if (L>0) rp[(P-L+1):P] <- skewflag*rr[(L+1):2]
  sp <- rep(0,P)
  sp[1:M] <- ss
  R <- fft(rp)
  S <- fft(sp)
  t <- fft(R * S, TRUE)
  return((Re(t)/P)[1:M])
}


#' symconv2D.ks
#'
#' Perform symmetric 2D convolution using FFT.
#'
#' @param rr The first input matrix.
#' @param ss The second input matrix.
#' @param skewflag A vector of two scalar values for skew correction along each dimension.
#'
#' @importFrom stats fft
#'
#' @return A matrix representing the result of the symmetric 2D convolution.
#'
symconv2D.ks <- function(rr, ss, skewflag=rep(1,2))
{
  L <- dim(rr)-1
  M <- dim(ss)
  L1 <- L[1]
  L2 <- L[2]               # find dimensions of r,s
  M1 <- M[1]
  M2 <- M[2]
  P1 <- 2^(ceiling(log(M1+L1)/log(2))) # smallest power of 2 >= M1+L1
  P2 <- 2^(ceiling(log(M2+L2)/log(2))) # smallest power of 2 >= M2+L2

  rp <- matrix(0,P1,P2)
  rp[1:(L1+1),1:(L2+1)] <- rr
  if (L1>0)
    rp[(P1-L1+1):P1,1:(L2+1)] <- skewflag[1]*rr[(L1+1):2,]
  if (L2>0)
    rp[1:(L1+1),(P2-L2+1):P2] <- skewflag[2]*rr[,(L2+1):2]
  if (L1 > 0 & L2 > 0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2] <- prod(skewflag)*rr[(L1+1):2,(L2+1):2]
  # wrap around version of rr
  sp <- matrix(0,P1,P2)
  sp[1:M1,1:M2] <- ss                 # zero-padded version of ss

  RR <- fft(rp)        # Obtain FFT's of rr and ss
  SS <- fft(sp)
  tt <- fft(RR*SS,TRUE)               # invert element-wise product of FFT's
  return((Re(tt)/(P1*P2))[1:M1,1:M2]) # return normalized truncated tt
}

#' symconv3D.ks
#'
#' Perform symmetric 3D convolution using FFT.
#'
#' @param rr The first input 3D array.
#' @param ss The second input 3D array.
#' @param skewflag A vector of three scalar values for skew correction along each dimension.
#'
#' @importFrom stats fft
#'
#' @return A 3D array representing the result of the symmetric 3D convolution.
#'
symconv3D.ks <- function(rr, ss, skewflag=rep(1,3))
{
  L <- dim(rr) - 1
  M <- dim(ss)
  P <- 2^(ceiling(log(M+L)/log(2))) # smallest powers of 2 >= M+L
  L1 <- L[1] ; L2 <- L[2] ; L3 <- L[3]
  M1 <- M[1] ; M2 <- M[2] ; M3 <- M[3]
  P1 <- P[1] ; P2 <- P[2] ; P3 <- P[3]
  sf <- skewflag

  rp <- array(0,P)
  rp[1:(L1+1),1:(L2+1),1:(L3+1)] <- rr
  if (L1>0)
    rp[(P1-L1+1):P1,1:(L2+1),1:(L3+1)] <- sf[1]*rr[(L1+1):2,1:(L2+1),1:(L3+1)]
  if (L2>0)
    rp[1:(L1+1),(P2-L2+1):P2,1:(L3+1)] <- sf[2]*rr[1:(L1+1),(L2+1):2,1:(L3+1)]
  if (L3>0)
    rp[1:(L1+1),1:(L2+1),(P3-L3+1):P3] <- sf[3]*rr[1:(L1+1),1:(L2+1),(L3+1):2]
  if (L1>0 & L2>0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2,1:(L3+1)] <- sf[1]*sf[2]*rr[(L1+1):2,(L2+1):2,1:(L3+1)]
  if (L2>0 & L3>0)
    rp[1:(L1+1),(P2-L2+1):P2,(P3-L3+1):P3] <- sf[2]*sf[3]*rr[1:(L1+1),(L2+1):2,(L3+1):2]
  if (L1>0 & L3>0)
    rp[(P1-L1+1):P1,1:(L2+1),(P3-L3+1):P3] <- sf[1]*sf[3]*rr[(L1+1):2,1:(L2+1),(L3+1):2]
  if (L1>0 & L2>0 & L3>0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2,(P3-L3+1):P3] <- sf[1]*sf[2]*sf[3]*rr[(L1+1):2,(L2+1):2,(L3+1):2]

  sp <- array(0,P)
  sp[1:M1,1:M2,1:M3] <- ss            # zero-padded version of ss

  RR <- fft(rp)                       # Obtain FFT's of rr and ss
  SS <- fft(sp)
  tt <- fft(RR*SS,TRUE)               # invert element-wise product of FFT's
  return((Re(tt)/(P1*P2*P3))[1:M1,1:M2,1:M3]) # return normalized truncated tt
}

#' symconv4D.ks
#'
#' Perform symmetric 4D convolution using FFT.
#'
#' @param rr The first input 4D array.
#' @param ss The second input 4D array.
#' @param skewflag A vector of four scalar values for skew correction along each dimension.
#' @param fftflag A vector of two Boolean values for FFT flag.
#'
#' @importFrom stats fft
#'
#' @return A 4D array representing the result of the symmetric 4D convolution.
#'
symconv4D.ks <- function(rr, ss, skewflag=rep(1,4) , fftflag=rep(TRUE,2))
{
  L <- dim(rr) - 1
  M <- dim(ss)
  P <- 2^(ceiling(log(M+L)/log(2))) # smallest powers of 2 >= M+L
  L1 <- L[1] ; L2 <- L[2] ; L3 <- L[3] ; L4 <- L[4]
  M1 <- M[1] ; M2 <- M[2] ; M3 <- M[3] ; M4 <- M[4]
  P1 <- P[1] ; P2 <- P[2] ; P3 <- P[3] ; P4 <- P[4]
  sf <- skewflag

  rp <- array(0,P)
  rp[1:(L1+1),1:(L2+1),1:(L3+1),1:(L4+1)] <- rr

  if (L1>0)
    rp[(P1-L1+1):P1,1:(L2+1),1:(L3+1),1:(L4+1)] <- sf[1]*rr[(L1+1):2,1:(L2+1),1:(L3+1),1:(L4+1)]
  if (L2>0)
    rp[1:(L1+1),(P2-L2+1):P2,1:(L3+1),1:(L4+1)] <- sf[2]*rr[1:(L1+1),(L2+1):2,1:(L3+1),1:(L4+1)]
  if (L3>0)
    rp[1:(L1+1),1:(L2+1),(P3-L3+1):P3,1:(L4+1)] <- sf[3]*rr[1:(L1+1),1:(L2+1),(L3+1):2,1:(L4+1)]
  if (L4>0)
    rp[1:(L1+1),1:(L2+1),1:(L3+1),(P4-L4+1):P4] <- sf[4]*rr[1:(L1+1),1:(L2+1),1:(L3+1),(L4+1):2]

  if (L1>0 & L2 >0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2,1:(L3+1),1:(L4+1)] <- sf[1]*sf[2]*rr[(L1+1):2,(L2+1):2,1:(L3+1),1:(L4+1)]
  if (L2>0 & L3>0)
    rp[1:(L1+1),(P2-L2+1):P2,(P3-L3+1):P3,1:(L4+1)] <- sf[2]*sf[3]*rr[1:(L1+1),(L2+1):2,(L3+1):2,1:(L4+1)]
  if (L3>0 & L4>0)
    rp[1:(L1+1),1:(L2+1),(P3-L3+1):P3,(P4-L4+1):P4] <- sf[3]*sf[4]*rr[1:(L1+1),1:(L2+1),(L3+1):2,(L4+1):2]
  if (L1>0 & L3>0)
    rp[(P1-L1+1):P1,1:(L2+1),(P3-L3+1):P3,1:(L4+1)] <- sf[1]*sf[3]*rr[(L1+1):2,1:(L2+1),(L3+1):2,1:(L4+1)]
  if (L2>0 & L4>0)
    rp[1:(L1+1),(P2-L2+1):P2,1:(L3+1),(P4-L4+1):P4] <- sf[2]*sf[4]*rr[1:(L1+1),(L2+1):2,1:(L3+1),(L4+1):2]
  if (L1>0 & L4>0)
    rp[(P1-L1+1):P1,1:(L2+1),1:(L3+1),(P4-L4+1):P4] <- sf[1]*sf[4]*rr[(L1+1):2,1:(L2+1),1:(L3+1),(L4+1):2]

  if (L1>0 & L2>0 & L3>0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2,(P3-L3+1):P3,1:(L4+1)] <- sf[1]*sf[2]*sf[3]*rr[(L1+1):2,(L2+1):2,(L3+1):2,1:(L4+1)]
  if (L1>0 & L2>0 & L4>0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2,1:(L3+1),(P4-L4+1):P4] <- sf[1]*sf[2]*sf[4]*rr[(L1+1):2,(L2+1):2,1:(L3+1),(L4+1):2]
  if (L2>0 & L3>0 & L4>0)
    rp[1:(L1+1),(P2-L2+1):P2,(P3-L3+1):P3,(P4-L4+1):P4] <- sf[2]*sf[3]*sf[4]*rr[1:(L1+1),(L2+1):2,(L3+1):2,(L4+1):2]

  if (L1>0 & L2>0 & L3>0 & L4>0)
    rp[(P1-L1+1):P1,(P2-L2+1):P2,(P3-L3+1):P3,(P4-L4+1):P4] <- sf[1]*sf[2]*sf[3]*sf[4]*rr[(L1+1):2,(L2+1):2,(L3+1):2,(L4+1):2]

  sp <- array(0,P)
  sp[1:M1,1:M2,1:M3,1:M4] <- ss            # zero-padded version of ss

  RR <- fft(rp)                       # Obtain FFT's of rr and ss
  SS <- fft(sp)
  tt <- fft(RR*SS,TRUE)               # invert element-wise product of FFT's
  return((Re(tt)/(P1*P2*P3*P4))[1:M1,1:M2,1:M3,1:M4]) # return normalized truncated tt
}

#' Read the output file of wgd ksd
#'
#' @param file The output file of `wgd ksd`
#' @param include_outliers Include outliers or not, default FALSE.
#' @param min_ks Minimum Ks value, default 0.
#' @param min_aln_len Minimum alignment length, default 0.
#' @param min_idn Minimum alignment identity, default 0.
#' @param min_cov Minimum alignment coverage, default 0.
#'
#' @return A `ksv` object, which is a list including:
#'   * `ks_df`: the data frame that used for following analysis
#'   * `ks_dist`: a list including a vector of Ks values in the distribution
#'   * `raw_df`: raw data
#'   * `filters`: filters that applied to the raw data
#'
#'
read.wgd_ksd <- function(file, include_outliers=FALSE,
                         min_ks=0, min_aln_len=0,
                         min_idn=0, min_cov=0) {

  df <- read.table(file, sep="\t", header=TRUE)

  Ks <- AlignmentCoverage <- AlignmentLength <- AlignmentIdentity <- NULL
  WeightOutliersIncluded <- WeightOutliersExcluded <- NULL

  filtered_df <- df %>%
    dplyr::filter(Ks >= min_ks &
                    AlignmentCoverage >= min_cov &
                    AlignmentLength >= min_aln_len &
                    AlignmentIdentity >= min_idn)

  if (include_outliers == TRUE) {
    filtered_df <- filtered_df %>%
      dplyr::mutate(Weight=WeightOutliersIncluded)
  } else {
    filtered_df <- filtered_df %>%
      dplyr::mutate(Weight=WeightOutliersExcluded)
  }

  ksv_list <- list(ks_df=filtered_df,
                   ks_dist=generate_ksd(filtered_df),
                   raw_df=df,
                   filters=list(include_outliers=include_outliers,
                                  min_ks=min_ks,
                                  min_aln_len=min_aln_len,
                                  min_idn=min_idn,
                                  min_cov=min_cov,
                                  redundant=TRUE))

  class(ksv_list) <- "ksv"

  ksv_list
}

#' Check if an object is of class "ksv"
#'
#' This function checks if the provided object is of class "ksv."
#'
#' @param x The object to be checked.
#'
#' @return
#' Returns TRUE if the object is of class "ksv"; otherwise, returns FALSE.
#'
is.ksv <- function(x) {
  # if (class(x) == "ksv") {
  if (inherits(x, "ksv")) {
    TRUE
  } else {
    FALSE
  }
}

#' Generate Kernel Density Estimates (KDE) for Ks Distribution
#'
#' This function generates Kernel Density Estimates (KDE) for the Ks (synonymous substitution rates) distribution.
#'
#' @param ks_df A data frame containing Ks values.
#' @param bin_width The width of each bin for KDE calculation.
#' @param maxK The maximum Ks value for the distribution.
#'
#' @return A list containing the following components:
#'   - `Ks`: A numeric vector representing the KDE values.
#'   - `bin_width`: The width of each bin used for KDE calculation.
#'   - `maxK`: The maximum Ks value for the distribution.
#'
generate_ksd <- function(ks_df, bin_width=0.01, maxK=5) {

    full_df <- ks_df

    valuesPerBin <- bin_width / 0.01
    full_df$Ks.bin <- cut(full_df$Ks, seq(0, maxK, bin_width),
                          right=F, include.lowest=T)
    maxBin <- length(levels(full_df$Ks.bin))

    ks.aggregate <- stats::aggregate(full_df$Weight,
                                     by=list(ks.bin=full_df$Ks.bin),
                                     FUN=sum)

    ks.bin.df <- data.frame(
        ks.bin=cut(seq(0, maxK, bin_width)[1:maxBin] + bin_width / 2,
                     seq(0, 5, bin_width),right=F, include.lowest=T),
        ks=seq(0, maxK, bin_width)[1:maxBin] + bin_width / 2)

    ks.dist <- merge(ks.aggregate, ks.bin.df, by="ks.bin", all=TRUE)

    ksDist <- ks.dist$x
    ksValues <- NULL
    maxCount <- maxK * 100
    for (i in seq(1, maxCount)) {
        ks <- seq(bin_width, maxK, bin_width)
        if (is.na(ksDist[i])) {
            next
        } else {
            ksValues <- c(ksValues, rep(ks[i], ksDist[i]))
        }
    }

    ksd_list <- list(Ks=ksValues,
                     bin_width=bin_width,
                     maxK=maxK)
    ksd_list
}

library(httr)
library(jsonlite)
library(phangorn)

parser <- argparse::ArgumentParser()
parser$add_argument("-i", "--input_file", help="Path to the input file", required=TRUE)
parser$add_argument("-p", "--prefix", help="Prefix name of output file", required=TRUE)

args <- parser$parse_args()


base_url <- "https://timetree.org/api"

taxon_names <- readLines(args$input_file)

get_taxon_id <- function(name_t){
    name <- gsub(" ", "+", name_t)
    endpoint <- paste0(base_url, "/taxon/", name)
    response <- GET(endpoint)
    response_content <- content(response, as="text")
    json <- fromJSON(response_content)
    taxon_id <- json$taxon_id
    # if( is.null(taxon_id) ){
    #     # only try genus if the scientific name is not available
    #     name <- strsplit(name_t, " ")[[1]][1]
    #     endpoint <- paste0(base_url, "/taxon/", name)
    #     response <- GET(endpoint)
    #     response_content <- content(response, as="text")
    #     json <- fromJSON(response_content)
    #     taxon_id <- json$taxon_id
    # }
    return(taxon_id)
}

dist_matrix_as_timetree <- matrix(
    0,
    nrow=length(taxon_names),
    ncol=length(taxon_names)
)
rownames(dist_matrix_as_timetree) <- taxon_names
colnames(dist_matrix_as_timetree) <- taxon_names

dist_matrix_as_median <- matrix(
    0,
    nrow=length(taxon_names),
    ncol=length(taxon_names)
)
rownames(dist_matrix_as_median) <- taxon_names
colnames(dist_matrix_as_median) <- taxon_names

for( i in 1:length(taxon_names) ){
    for( j in i:length(taxon_names) ){
        if( i == j ){
            dist_matrix_as_timetree[i, j] <- 0
            dist_matrix_as_median[i, j] <- 0
        }
        else{
            taxon_id1 <- get_taxon_id(taxon_names[i])
            taxon_id2 <- get_taxon_id(taxon_names[j])

            if( is.null(taxon_id1) | is.null(taxon_id2) ){
                if( is.null(taxon_id1) ){
                    stop(paste(taxon_names[i], "is not in the Timetree Database"))
                }
                if( is.null(taxon_id2) ){
                    stop(paste(taxon_names[j], "is not in the Timetree Database"))
                }
                #stop()
            }

            endpoint <- paste0(base_url, "/pairwise/", taxon_id1, "/", taxon_id2)

            response <- GET(endpoint)

            response_content <- content(response, as="text")
            json <- fromJSON(response_content)

            if( as.numeric(json$studies$adjusted_age) != 0 ){
                dist_matrix_as_timetree[i, j] <- round(as.numeric(json$studies$adjusted_age), 2)
                dist_matrix_as_timetree[j, i] <- round(as.numeric(json$studies$adjusted_age), 2)
            }else{
                dist_matrix_as_timetree[i, j] <- round(as.numeric(json$sum_median_time), 2)
                dist_matrix_as_timetree[j, i] <- round(as.numeric(json$sum_median_time), 2)
            }

            dist_matrix_as_median[i, j] <- round(as.numeric(json$sum_median_time), 2)
            dist_matrix_as_median[j, i] <- round(as.numeric(json$sum_median_time), 2)

        }
    }
}

tree_as_timetree <- as.phylo(
    hclust(as.dist(dist_matrix_as_timetree)*2, method="average")
)
tree_as_median <- as.phylo(
    hclust(as.dist(dist_matrix_as_median)*2, method="average")
)

write.tree(tree_as_timetree, file=paste0(args$prefix, ".as_timetree.nwk"))
write.nexus(
    tree_as_timetree,
    file=paste0(args$prefix, ".as_timetree.nexus"),
    translate=FALSE
)
write.tree(tree_as_median, file=paste0(args$prefix, ".sum_median_time.nwk"))
write.nexus(
    tree_as_median,
    file=paste0(args$prefix, ".sum_median_time.nexus"),
    translate=FALSE
)

#' Extracts a timetree from TimeTree.org based on species names.
#'
#' This function takes a file with species names as input and a prefix to define the output.
#'
#' @param input_file A character string specifying the path to the file containing species names.
#' @param prefix A character string providing the prefix for the output file.
#'
#' @return A timetree object representing the estimated divergence times between species.
#'
#' @import httr jsonlite
#'
#' @export
#'
#' @seealso \code{\link{httr}}, \code{\link{jsonlite}}
TimeTreeFecher <- function(input_file, prefix){
    suppressPackageStartupMessages({
        library(httr)
        library(jsonlite)
    })
    base_url <- "https://timetree.org/api"

    taxon_names <- readLines(input_file)
    taxon_names <- str_trim(taxon_names, side="right")
    print(taxon_names)

    get_taxon_id <- function(name_t){
        name <- gsub(" ", "+", name_t)
        endpoint <- paste0(base_url, "/taxon/", name)
        response <- GET(endpoint)
        response_content <- content(response, as="text")
        json <- fromJSON(response_content)
        taxon_id <- json$taxon_id
        miss_name <- 0
        if( is.null(taxon_id) ){
            # only try genus if the scientific name is not available
            name <- strsplit(name_t, " ")[[1]][1]
            endpoint <- paste0(base_url, "/taxon/", name)
            response <- GET(endpoint)
            response_content <- content(response, as="text")
            json <- fromJSON(response_content)
            taxon_id <- json$taxon_id
            miss_name <- 1
        }
        return(c(taxon_id, miss_name))
    }

    dist_matrix_as_timetree <- matrix(
        0,
        nrow=length(taxon_names),
        ncol=length(taxon_names)
    )
    rownames(dist_matrix_as_timetree) <- taxon_names
    colnames(dist_matrix_as_timetree) <- taxon_names

    dist_matrix_as_median <- matrix(
        0,
        nrow=length(taxon_names),
        ncol=length(taxon_names)
    )
    rownames(dist_matrix_as_median) <- taxon_names
    colnames(dist_matrix_as_median) <- taxon_names

    # species_df <- data.frame(
    #     stringsAsFactors=FALSE
    # )
    x <- 0
    total_iterations <- length(taxon_names) * (length(taxon_names) + 1) / 2
    shiny::withProgress(message='Searching in progress', value=0, {
        for( i in 1:length(taxon_names) ){
            for( j in i:length(taxon_names) ){
                x <- x + 1
                incProgress(
                    amount=1/total_iterations,
                    message=paste(
                        x,
                        "/",
                        total_iterations,
                        "..."
                    )
                )

                if( i == j ){
                    dist_matrix_as_timetree[i, j] <- 0
                    dist_matrix_as_median[i, j] <- 0
                }
                else{
                    taxon_id_i <- get_taxon_id(taxon_names[i])
                    taxon_id1 <- taxon_id_i[1]

                    taxon_id_j <- get_taxon_id(taxon_names[j])
                    taxon_id2 <- taxon_id_j[1]

                    # if( taxon_id_i[2] > 0 ){
                    #     temp_df1 <- data.frame(
                    #         species=taxon_names[i],
                    #         miss_type=taxon_id_i[2],
                    #         genus=strsplit(taxon_names[i], " ")[[1]][1],
                    #         stringsAsFactors=FALSE
                    #     )
                    #     species_df <- rbind(species_df, temp_df1)
                    # }
                    #
                    # if( taxon_id_j[2] > 0 ){
                    #     temp_df2 <- data.frame(
                    #         species=taxon_names[j],
                    #         miss_type=taxon_id_j[2],
                    #         genus=strsplit(taxon_names[j], " ")[[1]][1],
                    #         stringsAsFactors=FALSE
                    #     )
                    #     species_df <- rbind(species_df, temp_df2)
                    # }

                    if( is.null(taxon_id1) | is.null(taxon_id2) ){
                        if( is.null(taxon_id1) ){
                            #return(species_df)
                            stop(
                                paste(
                                    "Both",
                                    taxon_names[i],
                                    "and",
                                    strsplit(taxon_names[i], " ")[[1]][1],
                                    "are not in the Timetree Database"
                                )
                            )
                        }
                        if( is.null(taxon_id2) ){
                            #return(species_df)
                            stop(
                                paste(
                                    "Both",
                                    taxon_names[j],
                                    "and",
                                    strsplit(taxon_names[j], " ")[[1]][1],
                                    "are not in the Timetree Database"
                                )
                            )
                        }
                    }

                    endpoint <- paste0(base_url, "/pairwise/", taxon_id1, "/", taxon_id2)

                    response <- GET(endpoint)


                    response_content <- content(response, as="text")
                    json <- fromJSON(response_content)

                    if( !is.null(json$studies$adjusted_age) && !is.null(json$sum_median_time) ){
                        if( as.numeric(json$studies$adjusted_age) != 0 ){
                            dist_matrix_as_timetree[i, j] <- round(as.numeric(json$studies$adjusted_age), 2)
                            dist_matrix_as_timetree[j, i] <- round(as.numeric(json$studies$adjusted_age), 2)
                        }else{
                            dist_matrix_as_timetree[i, j] <- round(as.numeric(json$sum_median_time), 2)
                            dist_matrix_as_timetree[j, i] <- round(as.numeric(json$sum_median_time), 2)
                        }

                        dist_matrix_as_median[i, j] <- round(as.numeric(json$sum_median_time), 2)
                        dist_matrix_as_median[j, i] <- round(as.numeric(json$sum_median_time), 2)
                    }
                }
            }
        }
    })

    tree_as_timetree <- as.phylo(
        hclust(as.dist(dist_matrix_as_timetree)*2, method="average")
    )
    tree_as_median <- as.phylo(
        hclust(as.dist(dist_matrix_as_median)*2, method="average")
    )

    write.tree(tree_as_timetree, file=paste0(prefix, ".as_timetree.nwk"))
    write.nexus(
        tree_as_timetree,
        file=paste0(prefix, ".as_timetree.nexus"),
        translate=FALSE
    )
    write.tree(tree_as_median, file=paste0(prefix, ".sum_median_time.nwk"))
    write.nexus(
        tree_as_median,
        file=paste0(prefix, ".sum_median_time.nexus"),
        translate=FALSE
    )

    duplicated_rows <- duplicated(dist_matrix_as_timetree) | duplicated(dist_matrix_as_timetree, fromLast=TRUE)
    unique_rows_with_same_values <- rownames(dist_matrix_as_timetree)[duplicated_rows]

    row_groups <- list()
    for( unique_row in unique_rows_with_same_values ){
        group <- rownames(dist_matrix_as_timetree)[dist_matrix_as_timetree[unique_row, ] == dist_matrix_as_timetree[unique_row, unique_row]]
        row_groups[[unique_row]] <- group
    }

    species_result_df <- data.frame(
        group_id=integer(),
        species_list=character(),
        stringsAsFactors=FALSE
    )

    group_id_counter <- 1

    for( group_name in names(row_groups) ){
        species_list <- paste(row_groups[[group_name]], collapse=", ")
        existing_row <- species_result_df[species_result_df$species_list == species_list, ]

        if (nrow(existing_row) == 0) {
            species_result_df <- rbind(
                species_result_df,
                data.frame(
                    group_id=group_id_counter,
                    species_list=species_list
                )
            )
            group_id_counter <- group_id_counter + 1
        } else {
            species_result_df[species_result_df$species_list == species_list, "group_id"] <- existing_row$group_id
        }
    }

    return(species_result_df)
}

#' The main code to run shinyWGD
#'
#'
#' @export
runshinyWGD <- function() {
    appDir <- system.file("shinyWGD", package="shinyWGD")
    if (appDir == "") {
        stop("Could not find shinyWGD Try re-installing `shinyWGD`.", call.=FALSE)
    }

    shiny::runApp(appDir)
}

observeEvent(input$whale_TreeRecon_example, {
    showModal(
        modalDialog(
            title=HTML("The example of the TreeRecon"),
            size="xl",
            uiOutput("whale_TreeRecon_example_panel")
        )
    )

    output$whale_TreeRecon_example_panel <- renderUI({
        fluidRow(
            div(
                style="padding-bottom: 10px;
                       padding-left: 20px;
                       padding-right: 20px;
                       max-width: 100%;
                       overflow-x: auto;",
                column(
                    12,
                    HTML(
                        paste0(
                            "After successfully operating the code of <font color='green'><i>Whale</i></font> created by <b>shinyWGD</b>, ",
                            "users need to compress the Whale result into a compressed file and the upload this file to the <b>TreeRecon</b> module of <b>shinyWGD</b>. ",
                            "<p></p><p>How to compress the result, please follow the code of <font color='green'><i><b>archive_compress_files_for_TreeRecon.shell</b></i></font> in the <b>Whale_wd</b> folder. ",
                            "<p>User can upload this compressed file, this mudule will analyze the potential WGD events among the studied species",
                            "<p><br></br></p>",
                            "<p>To download the demo data, <a href='https://github.com/li081766/shinyWGD_Demo_Data/blob/main/4sp_Whale_TreeRecon.tar.gz' target='_blank'>click here</a>.</p>",
                            "<p><br></br></p>"
                        )
                    )
                )
            )
        )
    })
})

# example_data_dir <- "/www/bioinformatics01_rw/ShinyWGD/Example_4Sp/"
example_data_dir <- file.path(getwd(), "demo_data")
whale_TreeRecon_example_dir <- file.path(example_data_dir, "Example_Whale_TreeRecon")
whale_check_file <- paste0(whale_TreeRecon_example_dir, "/run_Critical_branch_model_200/output/chaincritical.csv")

if( !dir.exists(whale_TreeRecon_example_dir) & !file.exists(whale_check_file) ){
    withProgress(message='Downloading tree reconciliation demo data...', value=0, {
        if( !dir.exists(example_data_dir) ){
            dir.create(example_data_dir)
        }
        dir.create(whale_TreeRecon_example_dir)

        Sys.sleep(.2)
        incProgress(amount=.3, message="Downloading in progress. Please wait...")

        downloadAndExtractData <- function() {
            download.file(
                "https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_Example_Whale_TreeRecon.tar.gz",
                destfile=file.path(getwd(), "data.zip"),
                mode="wb"
            )

            system(
                paste(
                    "tar xzf",
                    shQuote(file.path(getwd(), "data.zip")),
                    "-C",
                    shQuote(whale_TreeRecon_example_dir)
                )
            )

            file.remove(file.path(getwd(), "data.zip"))
        }

        downloadAndExtractData()

        Sys.sleep(.2)
        incProgress(amount=1, message="Done")
    })
}else if( dir.exists(whale_TreeRecon_example_dir) & !file.exists(whale_check_file) ){
    withProgress(message='Downloading tree reconciliation demo data...', value=0, {
        system(
            paste("rm -rf ", whale_TreeRecon_example_dir)
        )
        dir.create(whale_TreeRecon_example_dir)

        Sys.sleep(.2)
        incProgress(amount=.3, message="Downloading in progress. Please wait...")

        downloadAndExtractData <- function() {
            download.file(
                "https://github.com/li081766/shinyWGD_Demo_Data/raw/main/4sp_Example_Whale_TreeRecon.tar.gz",
                destfile=file.path(getwd(), "data.zip"),
                mode="wb"
            )

            system(
                paste(
                    "tar xzf",
                    shQuote(file.path(getwd(), "data.zip")),
                    "-C",
                    shQuote(whale_TreeRecon_example_dir)
                )
            )

            file.remove(file.path(getwd(), "data.zip"))
        }

        downloadAndExtractData()

        Sys.sleep(.2)
        incProgress(amount=1, message="Done")
    })
}

buttonTreeReconClicked <- reactiveVal(NULL)
whale_analysis_dir_Val <- reactiveVal(whale_TreeRecon_example_dir)

observeEvent(input$whale_data_zip_file, {
    buttonTreeReconClicked("fileInput")

    base_dir <- tempdir()
    timestamp <- format(Sys.time(), "%Y_%m_%d_%H_%M_%S")
    whaleAnalysisDir <- file.path(base_dir, paste0("Whale_output_", gsub("[ :\\-]", "_", timestamp)))
    dir.create(whaleAnalysisDir)
    system(
        paste(
            "tar xzf",
            input$whale_data_zip_file$datapath,
            "-C",
            whaleAnalysisDir
        )
    )
    whale_analysis_dir_Val(whaleAnalysisDir)
})

observe({
    if( is.null(buttonTreeReconClicked()) ){
        whaleAnalysisDir <- whale_TreeRecon_example_dir
        if( length(whaleAnalysisDir) > 0 ){
            output$selectedTreeReconDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("<b>Example:<br><font color='#EE82EE'>TreeRecon Analysis</font></b>"))
                    )
                )
            })
        }
    }
    else if( buttonTreeReconClicked() == "fileInput" ){
        whaleAnalysisDir <- whale_analysis_dir_Val()
        if( length(whaleAnalysisDir) > 0 ){
            dirName <- basename(whaleAnalysisDir)
            output$selectedTreeReconDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(
                            paste(
                                "Selected Directory:<br><b><font color='#EE82EE'>",
                                gsub("Ks_data", "Whale_data_", dirName),
                                "</font></b>"
                            )
                        )
                    )
                )
            })
        }
    }
    else if( buttonTreeReconClicked() == "actionButton" ){
        whaleAnalysisDir <- whale_TreeRecon_example_dir
        if( length(whaleAnalysisDir) > 0 ){
            output$selectedTreeReconDirName <- renderUI({
                column(
                    12,
                    div(
                        style="background-color: #FAF0E6;
                               margin-top: 5px;
                               padding: 10px 10px 1px 10px;
                               border-radius: 10px;
                               text-align: center;",
                        HTML(paste("<b>Example:<br><font color='#EE82EE'>TreeRecon Analysis</font></b>"))
                    )
                )
            })
        }
    }
})

output$selectedSubAnalysisDir <- renderUI({
    whaleAnalysisDir <- whale_analysis_dir_Val()
    all_subdirs <- list.dirs(
        whaleAnalysisDir,
        full.names=TRUE,
        recursive=FALSE
    )
    all_whale_study_list <- basename(all_subdirs[grep("^run_", basename(all_subdirs))])

    fluidRow(
        style="padding-top: 20px;",
        column(
            12,
            pickerInput(
                inputId="select_sub_study",
                label=HTML("Please select a <b>sub-study</b> to check the <b>Whale</b> result."),
                options=list(
                    title='Please select sub-study below'
                ),
                choices=all_whale_study_list,
                choicesOpt=list(
                    content=lapply(all_whale_study_list, function(each) {
                        element <- strsplit(each, "_")[[1]]
                        HTML(
                            paste0(
                                "<b>",
                                element[2], " ", element[3],
                                "</b>. Chain: <b>",
                                element[5],
                                "</b>"
                            )
                        )
                    })
                ),
                #selected=all_whale_study_list[1],
                width="400px"
            )
        )
    )
})

observeEvent(input$whale_TreeRecon_example, {
    buttonTreeReconClicked("actionButton")
    whale_analysis_dir_Val(whale_TreeRecon_example_dir)
})

widthTreeReconSpacing <- reactiveValues(value=500)
heightTreeReconSpacing <- reactiveValues(value=NULL)

observe({
    whaleAnalysisDir <- whale_analysis_dir_Val()
    speciesTreeFile <- paste0(whaleAnalysisDir, "/species_timetree.nwk")
    speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
    closeAllConnections()
    sp_count <- str_count(speciesTree, ":")
    trunc_val <- as.numeric(sp_count) * 20
    heightTreeReconSpacing$value <- trunc_val
})

observeEvent(input$TreeRecon_svg_vertical_spacing_add_species, {
    heightTreeReconSpacing$value <- heightTreeReconSpacing$value + 50
})
observeEvent(input$TreeRecon_svg_vertical_spacing_sub_species, {
    heightTreeReconSpacing$value <- heightTreeReconSpacing$value - 50
})
observeEvent(input$TreeRecon_svg_horizontal_spacing_add_species, {
    widthTreeReconSpacing$value <- widthTreeReconSpacing$value + 50
})
observeEvent(input$TreeRecon_svg_horizontal_spacing_sub_species, {
    widthTreeReconSpacing$value <- widthTreeReconSpacing$value - 50
})

observe({
    whaleAnalysisDir <- whale_analysis_dir_Val()
    species_tree_data <- list(
        "width"=widthTreeReconSpacing$value[1]
    )
    speciesTreeFile <- paste0(whaleAnalysisDir, "/species_timetree.nwk")
    speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))

    wgdNoteFile <- paste0(whaleAnalysisDir, "/wgdNodes.txt")
    wgdNoteFile_content <- readLines(wgdNoteFile)
    wgdNoteFile_content <- wgdNoteFile_content[wgdNoteFile_content != ""]
    wgdNoteInfo <- data.frame(
        wgd=gsub(":", " ", gsub("(wgd[0-9]+): (.+?) - (.+)", "\\1", wgdNoteFile_content)),
        comp=gsub("wgd[0-9]+: (.+?) - (.+)", "\\1:\\2", wgdNoteFile_content)
    )

    closeAllConnections()

    species_tree_data[["speciesTree"]] <- speciesTree[1]
    species_tree_data[["wgdNote"]] <- wgdNoteInfo
    species_tree_data[["height"]] <- heightTreeReconSpacing$value[1]
    species_tree_data[["tree_plot_div"]] <- "speciesWhaleTreeRecon_plot"
    session$sendCustomMessage("speciesTreeUpdatedPlot", species_tree_data)
    output$whaleReconTreeDesPanel <- renderUI({
        fluidRow(
            column(
                10,
                HTML("In the above plot, the WGD with the <b><font color='green'>solid green bars</font></b> are supported with <b>retention rates (q)</b> significantly different from zero, while the <b><font color='#6F6B0A'>hollow WGD bars</font></b> are the ones with retention rates not different from zero."),
            )
        )
    })
})

output$whaleOutputPanel <- renderUI({
    fluidRow(
        column(
            6,
            h5(HTML("<b>Hypothetic WGDs</b>:")),
            verbatimTextOutput(
                "wgdNoteSummaryTxt",
                placeholder=TRUE
            )
        ),
        column(
            6,
            uiOutput("selectedSubAnalysisDir")
        )
    )
})

widthPosteriorSpacing <- reactiveValues(value=900)
heightPosteriorSpacing <- reactiveValues(value=NULL)
observeEvent(input$posterior_svg_vertical_spacing_add_species, {
    heightPosteriorSpacing$value <- heightPosteriorSpacing$value + 50
})
observeEvent(input$posterior_svg_vertical_spacing_sub_species, {
    heightPosteriorSpacing$value <- heightPosteriorSpacing$value - 50
})
observeEvent(input$posterior_svg_horizontal_spacing_add_species, {
    widthPosteriorSpacing$value <- widthPosteriorSpacing$value + 50
})
observeEvent(input$posterior_svg_horizontal_spacing_sub_species, {
    widthPosteriorSpacing$value <- widthPosteriorSpacing$value - 50
})

observe({
    whaleAnalysisDir <- whale_analysis_dir_Val()

    wgdNoteFile <- paste0(whaleAnalysisDir, "/wgdNodes.txt")
    if( file.exists(wgdNoteFile) ){
        wgdNoteFile_content <- readLines(wgdNoteFile)
        wgdNoteFile_content <- wgdNoteFile_content[wgdNoteFile_content != ""]
        heightPosteriorSpacing$value <- ceiling(length(wgdNoteFile_content) / 4) * 300
    }
})

observe({
    if( isTruthy(input$select_sub_study) && input$select_sub_study != "" ){
        whaleAnalysisDir <- whale_analysis_dir_Val()

        subStudyAnalysisDir <- paste0(whaleAnalysisDir, "/", input$select_sub_study)

        whaleModelFile <- paste0(subStudyAnalysisDir, "/output/model.txt")
        whaleBranchModelFile <- paste0(subStudyAnalysisDir, "/output/bmodel.txt")
        whaleOutputFile <- paste0(
            subStudyAnalysisDir,
            "/output/",
            "posterior_mean_of_duplicate_retention_rate_Bayes_factor.txt"
        )

        if( !file.exists(whaleOutputFile) ){
            shinyalert(
                "Opps",
                "No correct result of Whale is detected. Please make sure the operation of Whale without errors...",
                type="error"
            )
        }
        else{
            chain_files <- list.files(
                path=subStudyAnalysisDir,
                pattern="chain.*\\.csv$",
                full.names=TRUE,
                recursive=TRUE
            )

            suppressMessages(
                chain_info_df <- vroom(
                    chain_files[1],
                    delim=",",
                    col_names=TRUE
                )
            )

            posterior_dist_df <- chain_info_df[, grepl("^q", names(chain_info_df))]
            colnames(posterior_dist_df) <- gsub("^q", "wgd", colnames(posterior_dist_df))

            posterior_dist_data <- list(
                "width"=widthPosteriorSpacing$value[1]
            )
            posterior_dist_data[["posterior_dist_df"]] <- posterior_dist_df
            posterior_dist_data[["height"]] <- heightPosteriorSpacing$value[1]
            posterior_dist_data[["posterior_plot_div"]] <- "posterior_Dist_plot_div"
            session$sendCustomMessage("posteriorDistPlot", posterior_dist_data)
        }
    }
})

observe({
    whaleAnalysisDir <- whale_analysis_dir_Val()

    wgdNoteFile <- paste0(whaleAnalysisDir, "/wgdNodes.txt")
    if( file.exists(wgdNoteFile) ){
        wgdNoteFile_content <- readLines(wgdNoteFile)
        wgdNoteFile_content <- wgdNoteFile_content[wgdNoteFile_content != ""]

        output$wgdNoteSummaryTxt <- renderText({
            formatted_text <- paste(wgdNoteFile_content, collapse="\n")
            return(formatted_text)
        })
    }

    if( isTruthy(input$select_sub_study) && input$select_sub_study != "" ){
        subStudyAnalysisDir <- paste0(whaleAnalysisDir, "/", input$select_sub_study)

        whaleModelFile <- paste0(subStudyAnalysisDir, "/output/model.txt")
        whaleBranchModelFile <- paste0(subStudyAnalysisDir, "/output/bmodel.txt")
        whaleOutputFile <- paste0(
            subStudyAnalysisDir,
            "/output/",
            "posterior_mean_of_duplicate_retention_rate_Bayes_factor.txt"
        )

        if( !file.exists(whaleOutputFile) ){
            shinyalert(
                "Opps",
                "No correct result of Whale is detected. Please make sure the operation of Whale without errors...",
                type="error"
            )
        }
        else{
            # update plot
            species_tree_updated_data <- list(
                "width"=widthTreeReconSpacing$value
            )
            speciesTreeFile <- paste0(whaleAnalysisDir, "/species_timetree.nwk")
            speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
            closeAllConnections()

            species_tree_updated_data[["speciesTree"]] <- speciesTree
            species_tree_updated_data[["height"]] <- heightTreeReconSpacing$value

            # read whale output and determine which wgd events are retained by whale
            lines <- readLines(whaleOutputFile, warn=FALSE)
            lines <- lines[-(1:4)]

            dataTmp <- strsplit(lines, "\\s+\\|\\s+|\\s+")
            data_cleaned <- lapply(dataTmp, function(x) x[x != ""])
            data_matrix <- do.call(rbind, data_cleaned)

            whaleOutTmp <- data.frame(data_matrix, stringsAsFactors=FALSE)
            col_names <- c("id", "cut", "wgdId", "q", "K")
            colnames(whaleOutTmp) <- col_names
            whaleOut <- whaleOutTmp[c("wgdId", "q", "K")]
            whaleOut$K <- apply(whaleOut, 1, function(row) gsub("[><]", "", row["K"]))
            species_tree_updated_data[["wgdInfo"]] <- whaleOut
            session$sendCustomMessage("speciesTreeUpdatedPlotOLD", species_tree_updated_data)

            whaleModel <- gsub("run_", "", input$select_sub_study)
            whaleModel <- gsub("_model_\\d+", "", whaleModel)
            panelTitle <- ""
            if( whaleModel == "Constant_rates" ){
                panelTitle <- h4(icon("poll"), HTML("Whale Output in <font color='#FA9B21'><b><i>constant-rates DL + WGD model</i></b></font>"))
                posteriorPanelTitleTexT <- h5(HTML("The <font color='#FA9B21'><b>posterior distributions</b></font> of the WGD <b>retention rates (q)</b> for the hypothetic WGDs in <font color='#FA9B21'><b><i>constant-rates DL + WGD model</i></b></font>"))
            }else if( whaleModel == "Relaxed_branch" ){
                panelTitle <- h4(icon("poll"), HTML("Whale Output in <font color='#FA9B21'><b><i>relaxed branch-specific DL + WGD model</i></b></font>"))
                posteriorPanelTitleTexT <- h5(HTML("The <font color='#FA9B21'><b>posterior distributions</b></font> of the WGD <b>retention rates (q)</b> for the hypothetic WGDs in <font color='#FA9B21'><b><i>relaxed branch-specific DL + WGD model</i></b></font>"))
            }else{
                panelTitle <- h4(icon("poll"), HTML("Whale Output in <font color='#FA9B21'><b><i>critical branch-specific DL + WGD model</i></b></font>"))
                posteriorPanelTitleTexT <- h5(HTML("The <font color='#FA9B21'><b>posterior distributions</b></font> of the WGD <b>retention rates (q)</b> for the hypothetic WGDs in <font color='#FA9B21'><b><i>critical branch-specific DL + WGD model</i></b></font>"))
            }

            output$posteriorPanelTitle <- renderUI({
                column(
                    12,
                    posteriorPanelTitleTexT
                )
            })

            output$whaleModelTxt <- renderText({
                whaleModelFile <- paste0(subStudyAnalysisDir, "/output/model.txt")
                whaleBranchModelFile <- paste0(subStudyAnalysisDir, "/output/bmodel.txt")
                if( file.exists(whaleBranchModelFile) ){
                    model_info <- readChar(
                        whaleBranchModelFile,
                        file.info(whaleBranchModelFile)$size
                    )
                }else{
                    model_info <- readChar(
                        whaleModelFile,
                        file.info(whaleModelFile)$size
                    )
                }
            })

            output$mcmcChainSummaryTxt <- renderText({
                mcmcChainSummaryFile <- paste0(subStudyAnalysisDir, "/output/MCMCchain.s")
                if( file.exists(mcmcChainSummaryFile) ){
                    summary_info <- readChar(
                        mcmcChainSummaryFile,
                        file.info(mcmcChainSummaryFile)$size
                    )
                }
            })

            output$posteriorMeanBayesFactorTxT <- renderText({
                posteriorMeanBayesFactorFile <- paste0(subStudyAnalysisDir, "/output/posterior_mean_of_duplicate_retention_rate_Bayes_factor.txt")
                if( file.exists(posteriorMeanBayesFactorFile) ){
                    mean_factor_info <- readChar(
                        posteriorMeanBayesFactorFile,
                        file.info(posteriorMeanBayesFactorFile)$size
                    )
                }
                else{
                    shinyalert(
                        "Error",
                        "The output of Whale is uncorrect. Please make sure that you have the proper input setting...",
                        type="error"
                    )
                }
            })

            output$whaleTextOutputPanel <- renderUI({
                fluidRow(
                    column(
                        id="whaleConfigure",
                        width=12,
                        hr(class="splitting"),
                        panelTitle,
                        hr(class="setting"),
                        fluidRow(
                            column(
                                12,
                                h5(HTML("Hypothetic WGDs: the <font color='orange'>posterior mean of duplicate retention rate (q)</font> and the <font color='orange'>Bayes factor (K)</font>")),
                                verbatimTextOutput(
                                    "posteriorMeanBayesFactorTxT",
                                    placeholder=TRUE
                                ),
                                h6(HTML("This is the log10 Bayes factor in favor of the <i>q</i> = 0 model. A Bayes factor <font color='red'><b>smaller than -2</b></font> could be considered as evidence in favor of the <i>q</i> &#x2260; 0 model compared to the <i>q</i> = 0 mode."))
                            )
                        ),
                        hr(class="setting"),
                        fluidRow(
                            column(
                                12,
                                h5("Please download the analysis data through the left Download button to see more details about the julia script, the model, and MCMC chains.")
                            )
                        ),
                        column(
                            12,
                            h5(HTML("The Chains MCMC chain summary info")),
                            verbatimTextOutput(
                                "mcmcChainSummaryTxt",
                                placeholder=TRUE
                            ),
                            h6(HTML("<font color='red'>Note:</font> if the <code>ESS</code> is less than 100, please increase the chain and restart <font color='#a23400'><b><i>Whale</b></i></font>"))
                        ),
                        hr(class="setting"),
                        column(
                            12,
                            h5(HTML("The model used in <font color='#a23400'><i><b>whale</b></i></font>:")),
                            verbatimTextOutput(
                                "whaleModelTxt",
                                placeholder=TRUE
                            )
                        )
                    )
                )
            })
        }
    }
})

#'
#' output$wgdOutput <- renderText({
#'     text <- input$wgdInput
#'     text
#' })
#'
#' observe({
#'     aleDirPath <- parseDirPath(roots=c(computer="/"), input$aleDir)
#'     aleFileCount <- length(list.files(aleDirPath))
#'     output$numberAleFiles <- renderUI({
#'         column(
#'             12,
#'             if( aleFileCount > 1 ){
#'                 h6(HTML(paste0("<font color='green'><b>", aleFileCount, "</font></b> ALE files will be used." )))
#'             }else{
#'                 h6(HTML(paste0("<font color='green'><b>", aleFileCount, "</font></b> ALE file will be used." )))
#'             }
#'         )
#'     })
#'     wgdNum <- toupper(as.english(length(input$wgdInput)))
#'     if( length(input$wgdInput) > 1 ){
#'         note <- HTML(paste0("<font color='#F75000'><b>", wgdNum, "</b></font> WGD events will be examinated."))
#'     }else{
#'         note <- HTML(paste0("<font color='#F75000'><b>", wgdNum, "</b></font> WGD event will be examinated."))
#'     }
#'     # output$wgdCommnadPanel <- renderUI({
#'     #     fluidRow(
#'     #         column(
#'     #             id="wgdSetting",
#'     #             width=12,
#'     #             div(class="boxLike",
#'     #                 style="background-color: #FFFFF9;",
#'     #                 h4(icon("cog"), "Hypothetic WGDs"),
#'     #                 hr(class="setting"),
#'     #                 h5(HTML("<font color='#AD1F1F'>Hypothetic WGDs</font> to test:")),
#'     #                 fluidRow(
#'     #                     column(
#'     #                         12,
#'     #                         verbatimTextOutput(
#'     #                             "wgdOutput",
#'     #                             placeholder=TRUE)
#'     #                     )
#'     #                 ),
#'     #                 note
#'     #             )
#'     #         )
#'     #     )
#'     # })
#'     output$whaleCommandPanel <- renderUI({
#'         fluidRow(
#'             column(
#'                 id="WhaleSetting",
#'                 width=12,
#'                 h4(icon("cog"), HTML("<b><i>Whale</b></i> Setting")),
#'                 hr(class="setting"),
#'                 fluidRow(
#'                     column(
#'                         12,
#'                         selectInput(
#'                             inputId="select_whale_model",
#'                             label=HTML("<b>Base Model</b> for <b><i>Whale</b></i>:"),
#'                             choices=c(
#'                                 "Constant-rates model",
#'                                 "Relaxed branch-specific DL+WGD model",
#'                                 "Critical branch-specific DL+WGD model"),
#'                             width="100%",
#'                             multiple=FALSE,
#'                             selected="Constant-rates model"
#'                         ),
#'                         sliderInput(
#'                             inputId="select_chain_num",
#'                             label=HTML("Set the <b><font color='orange'>Chain</font></b> for <b><i>Whale</b></i>:"),
#'                             min=100,
#'                             max=500,
#'                             step=50,
#'                             value=200
#'                         ),
#'                         hr(class="setting"),
#'                         h5(HTML("<font color='#AD1F1F'>Hypothetic WGDs</font> to test:")),
#'                         fluidRow(
#'                             column(
#'                                 12,
#'                                 verbatimTextOutput(
#'                                     "wgdOutput",
#'                                     placeholder=TRUE
#'                                 )
#'                             )
#'                         ),
#'                         note,
#'                         hr(class="setting")
#'                     ),
#'                     column(
#'                         12,
#'                         tags$head(
#'                             tags$style(HTML(
#'                                 "@keyframes glowing {
#'                                          0% { background-color: #548C00; box-shadow: 0 0 5px #0795ab; }
#'                                          50% { background-color: #73BF00; box-shadow: 0 0 20px #43b0d1; }
#'                                          100% { background-color: #548C00; box-shadow: 0 0 5px #0795ab; }
#'                                          }
#'                                         @keyframes glowingD {
#'                                          0% { background-color: #5B5B00; box-shadow: 0 0 5px #0795ab; }
#'                                          50% { background-color: #8C8C00; box-shadow: 0 0 20px #43b0d1; }
#'                                          100% { background-color: #5B5B00; box-shadow: 0 0 5px #0795ab; }
#'                                          }"
#'                                 )
#'                             )
#'                         ),
#'                         div(class="float-left",
#'                             actionButton(
#'                                 inputId="whale_configure_go",
#'                                 HTML("Start <b><i>Whale</b></i>"),
#'                                 icon=icon("play"),
#'                                 status="secondary",
#'                                 style="color: #fff;
#'                                        background-color: #019858;
#'                                        border-color: #fff;
#'                                        padding: 5px 14px 5px 14px;
#'                                        margin: 5px 5px 5px 5px;
#'                                        animation: glowing 5300ms infinite;"
#'                             )
#'                         ),
#'                         div(class="float-right",
#'                             downloadButton(
#'                                 outputId="whale_data_download",
#'                                 label="Download",
#'                                 #width="215px",
#'                                 icon=icon("download"),
#'                                 status="secondary",
#'                                 style="background-color: #5151A2;
#'                                padding: 5px 10px 5px 10px;
#'                                margin: 5px 5px 5px 5px;
#'                                animation: glowingD 5000ms infinite; "
#'                             )
#'                         )
#'                     )
#'                 )
#'             )
#'         )
#'     })
#' })
#'
#' whale_dir <- paste0(paste0(tempdir(), "/Analysis_", Sys.Date(), "/Whale_wd"))
#' if( !file.exists(whale_dir) ){
#'     dir.create(whale_dir)
#' }
#' observeEvent(input$whale_configure_go, {
#'     wgdNodeFile <- paste0(whale_dir, "/wgdNodes.txt")
#'
#'     withProgress(message='Run in progress', value=0, {
#'         incProgress(amount=.1, message="Preparing whale command file...")
#'         Sys.sleep(.1)
#'
#'         # Output the wgd Nodes to a file
#'         writeLines(input$wgdInput, wgdNodeFile)
#'
#'         if( is.null(input$wgdInput) ){
#'             shinyalert(
#'                 "Opps",
#'                 "Please add the Hypothetic WGD events to the tree first...",
#'                 type="error"
#'             )
#'         }
#'
#'         aleDirPath <- parseDirPath(roots=c(computer="/"), input$aleDir)
#'
#'         # Run Whale
#'         whaleModel <- ""
#'         if( input$select_whale_model == "Constant-rates model" ){
#'             whaleModel <- "Constant_rates"
#'         }else if( input$select_whale_model == "Relaxed branch-specific DL+WGD model" ){
#'             whaleModel <- "Relaxed_branch"
#'         }else{
#'             whaleModel <- "Critical_branch"
#'         }
#'
#'         subStudyAnalysisDir <- paste0(whale_dir, "/run_", whaleModel, "_model_", input$select_chain_num)
#'         if( !file.exists(subStudyAnalysisDir) ){
#'             dir.create(subStudyAnalysisDir)
#'         }
#'
#'         whaleCommandFile <- paste0(subStudyAnalysisDir, "/run_whale.jl")
#'
#'         speciesTreeFile <- input$uploadSpeciesTree$datapath
#'         speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
#'         closeAllConnections()
#'
#'         contains_underscore <- any(sapply(speciesTree, function(line) grepl("_", line)))
#'
#'         if( contains_underscore ){
#'             incProgress(amount=.1, message="Change species names...")
#'             Sys.sleep(.2)
#'             system(
#'                 paste(
#'                     "python",
#'                     "tools/rename_species.py",
#'                     input$uploadSpeciesTree$datapath,
#'                     aleDirPath,
#'                     wgdNodeFile
#'                 )
#'             )
#'             new_nwk <- paste0(input$uploadSpeciesTree$datapath, ".updated.nwk")
#'             new_ale <- paste0(aleDirPath, ".updated")
#'             new_wgd <- paste0(wgdNodeFile, ".updated.txt")
#'
#'             incProgress(amount=.3, message="Run whale command...")
#'             Sys.sleep(.4)
#'
#'             system(
#'                 paste(
#'                     "sh",
#'                     "tools/prepare_Whale_command.v2.sh",
#'                     new_nwk,
#'                     new_ale,
#'                     new_wgd,
#'                     whaleCommandFile,
#'                     whaleModel,
#'                     input$select_chain_num
#'                 )
#'             )
#'         }else{
#'             incProgress(amount=.5, message="Run whale command...")
#'             Sys.sleep(.4)
#'
#'             system(
#'                 paste(
#'                     "sh",
#'                     "tools/prepare_Whale_command.v2.sh",
#'                     input$uploadSpeciesTree$datapath,
#'                     aleDirPath,
#'                     wgdNodeFile,
#'                     whaleCommandFile,
#'                     whaleModel,
#'                     input$select_chain_num
#'                 )
#'             )
#'         }
#'
#'         julia_path <- Sys.which("julia")
#'
#'         if( julia_path == "" ){
#'             shinyalert(
#'                 "Error",
#'                 "This step will use Julia to run the code. Please make sure that you have already installed Julia in the system's PATH variable first...",
#'                 type="error"
#'             )
#'         }
#'         else {
#'             system(paste(
#'                 "julia",
#'                 whaleCommandFile
#'             ))
#'         }
#'
#'     })
#'     # output$whaleModelTxt <- renderText({
#'     #     whaleModelFile <- paste0(subStudyAnalysisDir, "/output/model.txt")
#'     #     whaleBranchModelFile <- paste0(subStudyAnalysisDir, "/output/bmodel.txt")
#'     #     if( file.exists(whaleBranchModelFile) ){
#'     #         model_info <- readChar(
#'     #             whaleBranchModelFile,
#'     #             file.info(whaleBranchModelFile)$size
#'     #         )
#'     #     }else{
#'     #         model_info <- readChar(
#'     #             whaleModelFile,
#'     #             file.info(whaleModelFile)$size
#'     #         )
#'     #     }
#'     # })
#'     # output$mcmcChainSummaryTxt <- renderText({
#'     #     mcmcChainSummaryFile <- paste0(subStudyAnalysisDir, "/output/MCMCchain.s")
#'     #     if( file.exists(mcmcChainSummaryFile) ){
#'     #         summary_info <- readChar(
#'     #             mcmcChainSummaryFile,
#'     #             file.info(mcmcChainSummaryFile)$size
#'     #         )
#'     #     }
#'     # })
#'     output$posteriorMeanBayesFactorTxT <- renderText({
#'         posteriorMeanBayesFactorFile <- paste0(subStudyAnalysisDir, "/output/posterior_mean_of_duplicate_retention_rate_Bayes_factor.txt")
#'         if( file.exists(posteriorMeanBayesFactorFile) ){
#'             mean_factor_info <- readChar(
#'                 posteriorMeanBayesFactorFile,
#'                 file.info(posteriorMeanBayesFactorFile)$size
#'             )
#'         }
#'         else{
#'             shinyalert(
#'                 "Error",
#'                 "The output of Whale is uncorrect. Please make sure that you have the proper input setting...",
#'                 type="error"
#'             )
#'         }
#'     })
#'     panelTitle <- ""
#'     if( whaleModel == "Constant_rates" ){
#'         panelTitle <- h4(icon("poll"), HTML("Whale Output in <font color='#FA9B21'><b><i>constant-rates model</i></b></font>"))
#'     }else if( whaleModel == "Relaxed_branch" ){
#'         panelTitle <- h4(icon("poll"), HTML("Whale Output in <font color='#FA9B21'><b><i>relaxed branch-specific DLWGD model</i></b></font>"))
#'     }else{
#'         panelTitle <- h4(icon("poll"), HTML("Whale Output in <font color='#FA9B21'><b><i>critical branch-specific DLWGD model</i></b></font>"))
#'     }
#'     output$whaleConfigurePanel <- renderUI({
#'         fluidRow(
#'             column(
#'                 id="whaleConfigure",
#'                 width=12,
#'                 div(class="boxLike",
#'                     style="background-color: #F2FFE4 ;",
#'                     panelTitle,
#'                     hr(class="setting"),
#'                     fluidRow(
#'                         column(
#'                             12,
#'                             h5(HTML("Hypothetic WGDs: the <font color='orange'>posterior mean of duplicate retention rate (q)</font> and the <font color='orange'>Bayes factor (K)</font>")),
#'                             verbatimTextOutput(
#'                                 "posteriorMeanBayesFactorTxT",
#'                                 placeholder=TRUE
#'                             ),
#'                             #h6(HTML("<font color='red'>Note:</font> if K < 1, <i>H</i><sub>1</sub> supported, not worth more than a bare mention; K > 1, <i>H</i><sub>0</sub> supported.<br>&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbsp&nbspIf K < 0.1, strong evidence against <i>H</i><sub>0</sub>")),
#'                             h6(HTML("This is the log10 Bayes factor in favor of the <i>q</i> = 0 model. A Bayes factor <font color='red'><b>smaller than -2</b></font> could be considered as evidence in favor of the <i>q</i> ≠ 0 model compared to the <i>q</i> = 0 mode."))
#'                         )
#'                     ),
#'                     hr(class="setting"),
#'                     fluidRow(
#'                         column(
#'                             12,
#'                             h5("Please download the analysis data through the left Download button to see more details about the julia script, the model, and MCMC chains.")
#'                         )
#'                     )
#'                         # column(
#'                         #     12,
#'                         #     h5(HTML("The Chains MCMC chain summary info")),
#'                         #     verbatimTextOutput(
#'                         #         "mcmcChainSummaryTxt",
#'                         #         placeholder=TRUE
#'                         #     ),
#'                         #     h6(HTML("<font color='red'>Note:</font> if the <code>ESS</code> is less than 100, please increase the chain and restart <font color='#a23400'><b><i>Whale</b></i></font>"))
#'                         # ),
#'                         # hr(class="setting"),
#'                         # column(
#'                         #     12,
#'                         #     h5(HTML("The model used in <font color='#a23400'><i><b>whale</b></i></font>:")),
#'                         #     verbatimTextOutput(
#'                         #         "whaleModelTxt",
#'                         #         placeholder=TRUE
#'                         #     )
#'                         # )
#'                 #    )
#'                 )
#'             )
#'         )
#'     })
#' })
#'
#' observeEvent(input$update_output, {
#'     whale_dir <- paste0(paste0(tempdir(), "/Analysis_", Sys.Date(), "/Whale_wd"))
#'     whaleModel <- ""
#'     if( input$select_whale_model == "Constant-rates model" ){
#'         whaleModel <- "Constant_rates"
#'     }else if( input$select_whale_model == "Relaxed branch-specific DL+WGD model" ){
#'         whaleModel <- "Relaxed_branch"
#'     }else{
#'         whaleModel <- "Critical_branch"
#'     }
#'
#'     subStudyAnalysisDir <- paste0(whale_dir, "/run_", whaleModel, "_model_", input$select_chain_num)
#'     whaleOutputFile <- paste0(
#'         subStudyAnalysisDir,
#'         "/output/",
#'         "posterior_mean_of_duplicate_retention_rate_Bayes_factor.txt"
#'     )
#'     if( !file.exists(whaleOutputFile) ){
#'         shinyalert(
#'             "Warning!",
#'             "Please run Whale first. Then switch this button ...",
#'             type="warning",
#'         )
#'     }else{
#'         species_tree_updated_data <- list(
#'             "width"=widthTreeReconSpacing$value
#'         )
#'         speciesTreeFile <- input$uploadSpeciesTree$datapath
#'         speciesTree <- readLines(textConnection(readChar(speciesTreeFile, file.info(speciesTreeFile)$size)))
#'         closeAllConnections()
#'
#'         species_tree_updated_data[["speciesTree"]] <- speciesTree
#'         species_tree_updated_data[["height"]] <- heightTreeReconSpacing$value
#'
#'         # read whale output and determine which wgd events are retained by whale
#'         lines <- readLines(whaleOutputFile)
#'         lines <- lines[-(1:4)]
#'
#'         dataTmp <- strsplit(lines, "\\s+\\|\\s+|\\s+")
#'         data_cleaned <- lapply(dataTmp, function(x) x[x != ""])
#'         data_matrix <- do.call(rbind, data_cleaned)
#'
#'         whaleOutTmp <- data.frame(data_matrix, stringsAsFactors=FALSE)
#'         col_names <- c("id", "cut", "wgdId", "q", "K")
#'         colnames(whaleOutTmp) <- col_names
#'         whaleOut <- whaleOutTmp[c("wgdId", "q", "K")]
#'         whaleOut$K <- apply(whaleOut, 1, function(row) gsub("[><]", "", row["K"]))
#'         species_tree_updated_data[["wgdInfo"]] <- whaleOut
#'         session$sendCustomMessage("speciesTreeUpdatedPlot", species_tree_updated_data)
#'     }
#' })
#'
#' output$whale_data_download <- downloadHandler(
#'     filename=function(){
#'         paste0("Whale_output.", Sys.Date(), ".tgz")
#'     },
#'     content=function(file){
#'         withProgress(message='Downloading in progress', value=0, {
#'             incProgress(amount=.1, message="Compressing files...")
#'             shinyalert(
#'                 "Note",
#'                 "Pleae wait for compressing the files. Do not close the window",
#'                 type="info"
#'             )
#'             run_dir <- getwd()
#'             setwd(paste0(paste0(tempdir(), "/Analysis_", Sys.Date())))
#'             system(
#'                 paste0(
#'                     "tar czf ", file,
#'                     " --dereference ",
#'                     "Whale_wd"
#'                 )
#'             )
#'             incProgress(amount=.9, message="Downloading file...")
#'             incProgress(amount=1)
#'             Sys.sleep(.1)
#'             setwd(run_dir)
#'         })
#'     }
#' )

#' ksv class
#'
#' @examples
#' x <- c(0.1, 0.3, 0.8, 1,1, 0.8, 0.9)
#' is.ksv(x)
is.ksv <- function(x) {
    # if (class(x) == "ksv") {
    if (inherits(x, "ksv")) {
        TRUE
    } else {
        FALSE
    }
}

#' modeFinder
#'
#' Find the mode (peak) of a univariate distribution.
#'
#' @param x A numeric vector or a kernel density estimate (KDE).
#' @param bw Bandwidth for the KDE. Default is 0.1.
#' @param from Starting point for mode search. Default is 0.
#' @param to Ending point for mode search. Default is 5.
#'
#' @return The mode (peak) of the distribution.
#'
#' @examples
#' x <- c(1.1, 1.1, 1.2, 1.4, 1.8, 0.9, 0.8, 0.7)
#' modeFinder(x)
modeFinder <- function(x, bw = 0.1, from = 0, to = 5) {
    if (is.ksv(x)) {
        k <- x$ks.value
    } else {
        k <- x
    }
    d <- stats::density(k, bw = bw, from = from, to = to)
    peak <- d$x[which.max(d$y)]
    return(peak)
}

#' relativeRate
#'
#' Compute relative rates using input data files and statistical computations.
#'
#' @param ksv2out_1_file A character string specifying the path to the first input data file.
#' @param ksv2out_2_file A character string specifying the path to the second input data file.
#' @param ksv_between_file A character string specifying the path to the third input data file.
#' @param KsMax A numeric value representing a maximum threshold for Ks values.
#' @param low A numeric value specifying the lower quantile for bootstrapping. Default is 0.025.
#' @param up A numeric value specifying the upper quantile for bootstrapping. Default is 0.975.
#' @param bs An integer specifying the number of bootstrap iterations. Default is 1000.
#'
#' @importFrom stats quantile
#'
#' @return A list containing computed relative rates and their confidence intervals.
#'
relativeRate <- function(ksv2out_1_file, ksv2out_2_file, ksv_between_file, KsMax, low = 0.025, up = 0.975, bs = 1000) {
    ksv2out_1 <- read.table(ksv2out_1_file, sep="\t", header=T)
    ksv2out_1 <- ksv2out_1[ksv2out_1$Ks <= KsMax, ]
    ksv2out_2 <- read.table(ksv2out_2_file, sep="\t", header=T)
    ksv2out_2 <- ksv2out_2[ksv2out_2$Ks <= KsMax, ]
    ksv_between <- read.table(ksv_between_file, sep="\t", header=T)
    ksv_between <- ksv_between[ksv_between$Ks <= KsMax, ]

    outgroup.d1 <- ksv2out_1$Ks
    outgroup.d2 <- ksv2out_2$Ks
    ingroup.d <- ksv_between$Ks
    moderel <- list()
    a.dist <- vector()
    b.dist <- vector()
    c.dist <- vector()
    for (i in seq(1:bs)) {
        a <- sample(outgroup.d1, size = length(outgroup.d1), replace = T)
        b <- sample(outgroup.d2, size = length(outgroup.d2), replace = T)
        c <- sample(ingroup.d, size = length(ingroup.d), replace = T)
        a_mode <- .5 * (modeFinder(c) + modeFinder(a) - modeFinder(b))
        b_mode <- .5 * (modeFinder(c) + modeFinder(b) - modeFinder(a))
        c_mode <- .5 * (modeFinder(a) + modeFinder(b) - modeFinder(c))
        a.dist <- c(a.dist, a_mode)
        b.dist <- c(b.dist, b_mode)
        c.dist <- c(c.dist, c_mode)
    }

    moderel$a_mode <- modeFinder(a.dist)
    moderel$b_mode <- modeFinder(b.dist)
    moderel$c_mode <- modeFinder(c.dist)
    moderel$a_low_bound <- as.vector(quantile(a.dist, low))
    moderel$a_up_bound <- as.vector(quantile(a.dist, up))
    moderel$b_low_bound <- as.vector(quantile(b.dist, low))
    moderel$b_up_bound <- as.vector(quantile(b.dist, up))
    moderel$c_low_bound <- as.vector(quantile(c.dist, low))
    moderel$c_up_bound <- as.vector(quantile(c.dist, up))

    return(moderel)
}

#' Get Segmented Data from Anchorpoints and Ks Values
#'
#' This function extracts segmented data from anchorpoints and Ks (synonymous substitution rate) values,
#' based on specified criteria, and writes the results to output files.
#
#' @param genes_file A character string specifying the file path for genes information created by i-ADHoRe.
#' @param anchors_ks_file A character string specifying the file path for anchorpoints Ks values data.
#' @param multiplicons_file A character string specifying the file path for multiplicons information created by i-ADHoRe.
#' @param segmented_file A character string specifying the output file path for segmented data.
#' @param segmented_anchorpoints_file A character string specifying the output file path for segmented anchorpoints.
#' @param num_anchors An integer specifying the minimum number of anchorpoints required.
#'
#' @importFrom utils read.table
#' @importFrom utils write.table
#' @importFrom dplyr select
#' @importFrom dplyr arrange
#' @importFrom dplyr mutate
#' @importFrom dplyr row_number
#' @importFrom dplyr lag
#'
#' @return NULL (output files are generated with the specified information).
#'
get_segments <- function(
        genes_file,
        anchors_ks_file,
        multiplicons_file,
        segmented_file,
        segmented_anchorpoints_file,
        num_anchors=10){

    # requireNamespace("IRanges", quietly=TRUE)
    # requireNamespace("dplyr", quietly=TRUE)
    # library(IRanges)
    # library(dplyr)

    remapped_coordinate <- multiplicon <- geneX <- listX <- coordX <- geneY <- listY <- coordY <- Ks <- id <- level <- number_of_anchorpoints <-  NULL
    scaf_df <- read.table(
        genes_file,
        header=TRUE,
        sep="\t"
    ) %>%
        dplyr::group_by(genome, list) %>%
        dplyr::summarise(num_gene_remapped=max(remapped_coordinate), .groups="drop")

    anchors_ks_df <- read.table(
        anchors_ks_file,
        header=TRUE,
        sep="\t"
    ) %>%
        select(multiplicon,
               geneX, speciesX, listX, coordX,
               geneY, speciesY, listY, coordY, Ks)

    speciesX <- unique(anchors_ks_df$speciesX)[1]
    speciesY <- unique(anchors_ks_df$speciesY)[1]

    multiplicons_df <- read.table(
        multiplicons_file,
        header=TRUE,
        sep="\t"
    ) %>%
        select(id, level, number_of_anchorpoints)

    anchors_df <- merge(anchors_ks_df, multiplicons_df, by.x="multiplicon", by.y="id") %>%
        dplyr::filter(number_of_anchorpoints >= num_anchors)

    df_A <- anchors_df[, c(1:5)]
    df_B <- anchors_df[, c(1, 6:9)]
    row.names(df_A) <- NULL
    colnames(df_A) <- c("multiplicon_id", "gene", "genome", "chr", "pos")
    row.names(df_B) <- NULL
    colnames(df_B) <- c("multiplicon_id", "gene", "genome", "chr", "pos")

    anchors <- rbind(df_A, df_B)
    anchors.arranged <- dplyr::arrange(anchors)

    multiplicon_id <- NULL
    ap <- group_by(anchors.arranged, multiplicon_id, genome, chr)

    segs <- summarise(ap, pos_min=min(pos), pos_max=max(pos), .groups="drop")

    segments <- data.frame()
    for( chr in unique(segs$chr) ){
        seg.chr <- segs[segs$chr==chr, ]
        seg.chr <- seg.chr %>%
            mutate(group=row_number()) %>%
            arrange(pos_min) %>%
            mutate(group=cumsum(lag(pos_max, default=0) < pos_min))
        class(seg.chr)<-"data.frame"
        pos_min <- group <- pos_max <- is_real <- NULL
        df <- summarise(group_by(arrange(seg.chr, pos_min), genome, chr, group),
                        pos_min=min(pos_min), pos_max=max(pos_max), .groups="drop")
        class(df) <- "data.frame"
        segments <- rbind(segments, df)
    }

    segments$min <- segments$pos_min
    segments$max <- segments$pos_max

    segments$name <- paste0(segments$chr, ":", segments$pos_min, "-", segments$pos_max)

    anchors.arranged$atomic_id <- vector("character", nrow(anchors.arranged))
    anchors.arranged$atomic_pos <- vector("integer", nrow(anchors.arranged))

    for( i in 1:nrow(anchors.arranged) ){
        genome <- anchors.arranged$genome[i]
        chr <- anchors.arranged$chr[i]
        pos <- anchors.arranged$pos[i]
        pos.ranges <- segments[segments$genome==genome & segments$chr==chr,
                               c("name","pos_min", "pos_max")]
        for( j in 1:nrow(pos.ranges) ){
            if( pos %in% seq(pos.ranges$pos_min[j], pos.ranges$pos_max[j]) ){
                anchors.arranged$atomic_id[i] <- pos.ranges$name[j]
                anchors.arranged$atomic_pos[i] <- pos - pos.ranges$pos_min[j]
                break
            }
        }
    }

    atomic.df <- data.frame()
    for( mid in unique(anchors.arranged$multiplicon_id) ){
        df_x <- anchors.arranged[anchors.arranged$multiplicon_id==mid & anchors.arranged$genome==speciesX,
                                 c("multiplicon_id", "gene", "genome","atomic_id", "atomic_pos")]
        df_y <- anchors.arranged[anchors.arranged$multiplicon_id==mid & anchors.arranged$genome==speciesY,
                                 c("gene", "genome","atomic_id", "atomic_pos")]
        double_check_len <- nrow(anchors_df[anchors_df$multiplicon==mid, ])
        if( nrow(df_x) == double_check_len & nrow(df_y) == double_check_len ){
            atomic.df <- rbind(atomic.df, cbind(df_x, df_y))
        }
    }


    atomic.df$level <- 10000
    atomic.df$num_anchors <- 10000
    atomic.df$is_real <- -1
    colnames(atomic.df) <- c("multiplicon",
                             "geneX", "speciesX", "listX", "coordX",
                             "geneY", "speciesY", "listY", "coordY",
                             "level", "num_anchors", "is_real")

    atomic.df <- atomic.df[!duplicated(atomic.df[, -1]), ]

    file_tmp <- gsub("txt", "no_ks.txt", segmented_anchorpoints_file)
    write.table(
        atomic.df,
        file=file_tmp,
        sep="\t",
        col.names=TRUE,
        row.names=FALSE,
        quote=FALSE
    )

    ks_df <- anchors_ks_df %>%
        select(multiplicon, geneX, geneY, Ks)

    final_table_ks_df <- merge(
        atomic.df,
        ks_df,
        by.x=c("multiplicon", "geneX", "geneY"),
        by.y=c("multiplicon", "geneX", "geneY"),
        all.x=TRUE
    ) %>%
        select(multiplicon,
               geneX, speciesX, listX, coordX,
               geneY, speciesY, listY, coordY,
               level, num_anchors, is_real,
               Ks) %>%
        arrange(multiplicon)

    atomic.df <- final_table_ks_df

    segs.df <- cbind(segments$genome, segments$name)
    segs.df <- cbind(
        segs.df,
        data.frame(num_gene_remapped=segments$pos_max - segments$pos_min + 1)
    )
    names(segs.df) <- names(scaf_df)

    write.table(
        segs.df,
        file=segmented_file,
        sep="\t",
        col.names=TRUE,
        row.names=FALSE,
        quote=FALSE
    )

    write.table(
        atomic.df,
        file=segmented_anchorpoints_file,
        sep="\t",
        col.names=TRUE,
        row.names=FALSE,
        quote=FALSE
    )
}

#' Cluster Synteny Data and Generate Trees
#'
#' This function clusters synteny data based on calculated p-values and generates trees
#' for both column-based and row-based clustering. It then saves the cluster information and
#' trees to output files.
#'
#' @param segmented_file A character string specifying the file path for segmented data.
#' @param segmented_anchorpoints_file A character string specifying the file path for segmented anchorpoints.
#' @param genes_file A character string specifying the file path for genes information created by i-ADHoRe.
#' @param out_file A character string specifying the output file path for saving cluster information.
#'
#' @importFrom utils read.table
#' @importFrom stats cutree
#' @importFrom dplyr select
#' @importFrom stats cor
#' @importFrom stats hclust
#' @importFrom stats as.dist
#' @importFrom ape as.phylo
#' @importFrom ape write.tree
#'
#' @return NULL (output files are generated with the specified information).
#'
cluster_synteny <- function(
        segmented_file,
        segmented_anchorpoints_file,
        genes_file,
        out_file){

    segs.df <- read.table(
        segmented_file,
        header=TRUE,
        sep="\t"
    )
    atomic.df <- read.table(
        segmented_anchorpoints_file,
        header=TRUE,
        sep="\t"
    )

    id <- genome <- list <- remapped_coordinate <- NULL

    genes_df <- read.table(
        genes_file,
        header=TRUE,
        sep="\t"
    ) %>%
        select(id, genome, list, remapped_coordinate)

    colnames(genes_df) <- c("gene", "genome", "list", "coord")

    SpeciesX <- unique(atomic.df$speciesX)
    SpeciesY <- unique(atomic.df$speciesY)
    p_value_matrix <- matrix(
        ncol=nrow(segs.df[segs.df$genome==SpeciesX, ]),
        nrow=nrow(segs.df[segs.df$genome==SpeciesY, ]),
        dimnames=list(segs.df[segs.df$genome==SpeciesY, "list"],
                      segs.df[segs.df$genome==SpeciesX, "list"])
    )


    p_value_matrix.bycol <- p_value_matrix
    p_value_matrix.byrow <- p_value_matrix

    m <- nrow(atomic.df)
    n <- nrow(genes_df[genes_df$genome==SpeciesX, ]) * nrow(genes_df[genes_df$genome==SpeciesY, ])

    for( i in 1:nrow(p_value_matrix) ){
        row.chr <- row.names(p_value_matrix)[i]
        for( j in 1:ncol(p_value_matrix) ){
            col.chr <- colnames(p_value_matrix)[j]
            q <- nrow(atomic.df[atomic.df$listX==col.chr & atomic.df$listY==row.chr,])
            col.k <- segs.df[segs.df$list==col.chr, "num_gene_remapped"]
            row.k <- segs.df[segs.df$list==row.chr, "num_gene_remapped"]
            k <- col.k * row.k
            ## using poisson distribution to get values
            p_value_matrix.bycol[i, j] <- CalHomoConcentration(m, n, q, k)
            p_value_matrix.byrow[i, j] <- CalHomoConcentration(m, n, q, k)
        }
    }


    d.bycol <- cor(p_value_matrix.bycol)
    d.byrow <- cor(t(p_value_matrix.byrow))
    # print(p_value_matrix.bycol)

    d.bycol[is.na(d.bycol)] <- 0
    d.byrow[is.na(d.byrow)] <- 0

    upgma.bycol <- hclust(as.dist(1 - d.bycol), method="average")
    upgma.byrow <- hclust(as.dist(1 - d.byrow), method="average")

    cluster_info <- list()
    upgma.bycol$height <- round(upgma.bycol$height, 10)
    upgma.byrow$height <- round(upgma.byrow$height, 10)
    cluster_info[["bycol"]] <- upgma.bycol
    cluster_info[["byrow"]] <- upgma.byrow

    newick_bycol_file <- gsub(".RData", ".bycol.newick", out_file)
    newick_byrow_file <- gsub(".RData", ".byrow.newick", out_file)

    # suppressMessages(
    #     requireNamespace("ape", quietly=TRUE)
    #     #library(ape)
    # )
    newick_tree_bycol <- ape::as.phylo(cluster_info[['bycol']])
    write.tree(newick_tree_bycol, file=newick_bycol_file)

    newick_tree_byrow <- ape::as.phylo(cluster_info[['byrow']])
    write.tree(newick_tree_byrow, file=newick_byrow_file)

    save(cluster_info, file=out_file)
}

#' Compute the -log10 of Poisson Distribution
#'
#' This function calculates the -log10 of the p-value of a Poisson distribution given the parameters.
#'
#' @param m The total number of trials.
#' @param n The total number of possible outcomes.
#' @param q The observed number of successful outcomes.
#' @param k The expected number of successful outcomes.
#'
#' @importFrom stats ppois
#'
#' @return The -log10 of the p-value.
#'
#'
#' @examples
#' # Example usage:
#' p_value <- CalHomoConcentration(m=100, n=1000, q=10, k=1)
CalHomoConcentration <- function(m, n, q, k) {
    p <- m / n
    mean <- k * p
    return(-log10(ppois(q, mean, lower.tail=F)))
}

#' Perform synteny analysis for identified clusters
#'
#' This function performs synteny analysis for clusters identified by hierarchical clustering.
# It identifies clusters within a given height threshold and calculates p-values for each cluster.
# The results are saved in a list containing cluster information and p-values.
#
#' @param segmented_file The path to the segmented chromosome file.
#' @param segmented_anchorpoints_file The path to the segmented anchorpoints file.
#' @param genes_file genes.txt created by i-ADHoRe.
#' @param cluster_info_file The path to the clustering information file.
#' @param identified_cluster_file The path to the output file for identified clusters.
#' @param hcheight The cutoff height for cluster identification (default: 0.3).
#'
#' @importFrom utils read.table
#' @importFrom dplyr %>%
#' @importFrom dplyr select
#' @importFrom stats p.adjust
#'
#' @return A list containing information about identified clusters and their p-values.
#'
analysisEachCluster <- function(
        segmented_file,
        segmented_anchorpoints_file,
        genes_file,
        cluster_info_file,
        identified_cluster_file,
        hcheight=0.3){
    # requireNamespace("grid", quietly=TRUE)
    # requireNamespace("dplyr", quietly=TRUE)
    # requireNamespace("gridBase", quietly=TRUE)
    # requireNamespace("gridExtra", quietly=TRUE)
    # library(grid)
    # library(dplyr)
    # library(gridBase)
    # library(gridExtra)

    segs.df <- read.table(
        segmented_file,
        header=TRUE,
        sep="\t"
    )

    atomic.df <- read.table(
        segmented_anchorpoints_file,
        header=TRUE,
        sep="\t"
    )
    SpeciesX <- unique(atomic.df$speciesX)
    SpeciesY <- unique(atomic.df$speciesY)

    id <- genome <- list <- remapped_coordinate <- NULL
    genes.df <- read.table(
        genes_file,
        header=TRUE,
        sep="\t"
    ) %>%
        select(id, genome, list, remapped_coordinate)

    cluster_info <- NULL
    load(cluster_info_file)
    hc <- cluster_info
    hc.bycol <- hc$bycol
    hc.byrow <- hc$byrow

    cl.bycol <- cutree(hc$bycol, h=hcheight)
    cl.byrow <- cutree(hc$byrow, h=hcheight)

    cl.bycol.num <- max(cl.bycol)
    cl.byrow.num <- max(cl.byrow)

    identified_cluster_list <- list()
    par_num <- 0
    for( i in 1:cl.bycol.num  ){
        scaf.bycol <- names(cl.bycol[which(cl.bycol == i)])
        for( j in 1:cl.byrow.num ){
            scaf.byrow <- names(cl.byrow[which(cl.byrow == j)])
            clust <- extractCluster(
                segs.df,
                atomic.df,
                scaf.bycol,
                scaf.byrow
            )
            if( length(clust) > 0 ){
                m <- nrow(atomic.df)
                n <- nrow(genes.df[genes.df$genome==SpeciesX, ]) * nrow(genes.df[genes.df$genome==SpeciesY, ])
                col.k <- sum(clust[[1]][clust[[1]]$genome==SpeciesX, "num_gene_remapped"])
                row.k <- sum(clust[[1]][clust[[1]]$genome==SpeciesY, "num_gene_remapped"])
                k <- col.k * row.k
                q <- nrow(clust[[2]])
                p.value <- p.adjust(CalPvalue(m, n, q, k), method="bonferroni",
                                    n=cl.bycol.num * cl.byrow.num)

                if( p.value >= 1E-10 ){ #1E-100
                    next
                }

                p.value <- prettyNum(p.value, digits=4)

                par_num <- par_num + 1
                iteration_results <- list(
                    cluster_chr=clust[[1]],
                    cluster_anchorpoints=clust[[2]],
                    p_value=p.value,
                    par_id=par_num
                )
                identified_cluster_list <- c(identified_cluster_list, list(iteration_results))
            } else {
                next
            }
        }
    }

    save(identified_cluster_list, file=identified_cluster_file)
}

#' Compute the P-value of a Cluster using the Poisson Distribution
#'
#' This function computes the P-value of a cluster using the Poisson distribution.
# It calculates the statistical significance of observing a given number of anchorpoints
# in a cluster, assuming a Poisson distribution with the expected mean.
#
#' @param m The total number of all anchored points.
#' @param n The product of the remapped gene number of the query species and subject species.
#' @param q The number of anchored points in the cluster.
#' @param k The product of the remapped gene number of the segmented chromosomes
#'          of the query species and subject species.
#'
#' @importFrom stats ppois
#'
#' @return The computed P-value.
#'
#' @examples
#' # Example usage:
#' p_value <- CalPvalue(m=100, n=10000, q=5, k=250)
#' cat("P-value:", p_value, "\n")
CalPvalue <- function(m, n, q, k) {
    p <- m / n
    mean <- p * k
    return(ppois(q, mean, lower.tail=F))
}

#' Extract clusters based on specified scaffolds
#'
#' This function extracts clusters based on the specified scaffolds for both query and subject species.
#' It filters the data frames containing segment information and atomic anchorpoints to retain only the relevant clusters.
#'
#' @param segs.df A data frame containing segment information.
#' @param atomic.df A data frame containing atomic anchorpoints.
#' @param scaf.bycol A character vector specifying scaffolds for the query species.
#' @param scaf.byrow A character vector specifying scaffolds for the subject species.
#'
#' @return A list containing two data frames: "segs" for segment information and "atomic" for atomic anchorpoints.
#'
extractCluster <- function(
        segs.df,
        atomic.df,
        scaf.bycol,
        scaf.byrow){

    atomic.df1 <- atomic.df[atomic.df$listX %in% scaf.bycol, ]
    atomic.df2 <- atomic.df1[atomic.df1$listY %in% scaf.byrow, ]

    SpeciesX <- unique(atomic.df$speciesX)
    SpeciesY <- unique(atomic.df$speciesY)

    if( nrow(atomic.df2) == 0 ){
        return(NULL)
    }else{
        segs.df1 <- segs.df[segs.df$genome == SpeciesX & segs.df$list %in% scaf.bycol, ]
        segs.df2 <- segs.df[segs.df$genome == SpeciesY & segs.df$list %in% scaf.byrow, ]
        cluster <- list()
        cluster[["segs"]] <- rbind(segs.df1, segs.df2)
        cluster[["atomtic"]] <- atomic.df2
        return(cluster)
    }
}

Extracting_Tree_ui <- tabPanel(
    "TimeTreeFetcher",
    value='extracting_tree',
    fluidRow(
        column(
            3,
            uiOutput("ObtainTreeFromTimeTreeSettingDisplay")
        ),
        column(
            9,
            uiOutput("timeTreeOrgPlot")
        )
    ),
    icon=icon("tree", verify_fa=FALSE)
)

#' Remove directories older than a specified day
#'
#' This function removes directories in the specified base directory that are
#' older than a specified maximum age in days. It logs the removed directories
#' and any errors encountered during removal.
#'
#' @param base_dir The base directory to search for old directories.
#' @param max_age_in_days The maximum age (in days) for directories to be considered old.
#' @param log_file The name of the log file to store information about removed directories and errors.
#'
#' @return The function does not return anything. It logs information about removed directories and errors.
#'
#' @examples
#' \dontrun{
#' # Remove directories older than 3 days in the specified directory
#' remove_old_dirs("~/path/to/base/directory", max_age_in_days=3)
#' }
#'
remove_old_dirs <- function(base_dir, max_age_in_days=3, log_file="remove_old_dirs.log") {
    current_time <- Sys.time()

    dirs <- list.dirs(base_dir, full.names=TRUE, recursive=FALSE)
    dirs <- dirs[grep("Analysis_", dirs)]

    con <- file(log_file, open="a")

    for( dir_path in dirs ){
        dir_creation_time <- file.info(dir_path)$ctime

        time_difference_days <- as.numeric(
            difftime(current_time, dir_creation_time, units="days")
        )

        if( time_difference_days >= max_age_in_days ){
            tryCatch({
                print(dir_path)
                unlink(dir_path, recursive=TRUE)
                cat(paste("Removed:", dir_path, "\n"), file=log_file, append=TRUE)
            }, error=function(e) {
                cat(paste("Error removing:", dir_path, "\n", "Error message:", e$message, "\n"), file=log_file, append=TRUE)
            })
        }
    }
    close(con)
}

Whale_Preparation_ui <- tabPanel(
    "Whale Preparation",
    value="whale_preparation",
    fluidRow(
        tags$head(
            tags$link(rel="stylesheet", type="text/css", href="sand_clock_loader.css"),
        ),
        tags$head(
            tags$style(
                HTML(
                    "
                    #progress_modal_whale {
                        display: none;
                        position: fixed;
                        width: 100%;
                        height: 100%;
                        background-color: none;
                        left: 50%;
                        top: 50%;
                        transform: translate(-50%, -50%);
                        align-items: center;
                        justify-content: center;
                        z-index: 2000;
                    }

                    #progress_container_parent_whale {
                        background-color: rgba(128, 128, 128, 0.1);
                        width: 100%;
                        height: 100%;
                        border: none;
                        align-items: center;
                        justify-content: center;
                        z-index: 2001;
                    }
                    "
                )
            )
        ),
        fluidRow(
            div(
                id="progress_modal_whale",
                div(
                    id="progress_container_parent_whale",
                    class="modal-content",
                    includeHTML("www/sand_clock_loader.html"),
                )
            )
        )
    ),
    fluidRow(
        column(
            id="whalePreparationTitle",
            width=12,
            h4(icon(
                name=NULL,
                style="
                    background: url('images/treeReconciliationIcon.svg');
                    background-size: contain;
                    background-position: center;
                    background-repeat: no-repeat;
                    height: 50px;
                    width: 50px;
                    display: inline-block;
                    vertical-align: middle;
                "
                ),
                "Whale Preparation for Gene Tree – Species Tree Reconciliation Analysis"
            ),
            style="padding-bottom: 5px;
                   padding-top: 5px;
                   padding-left: 30px;",
        ),
        column(
            id="whalePreparationSettingPanel",
            width=3,
            column(
                12,
                uiOutput("whaleDataUploadPanel")
            )
        ),
        column(
            id="speciesTreeOutputPanel",
            width=9,
            div(class="boxLike",
                style="background-color: white;
                       margin: 5px 5px 5px 5px;
                       padding: 5px 10px 10px 20px;",
                fluidRow(
                    column(
                        12,
                        h4(icon("cog"), HTML("<b><i>Hypothetic WGD events</b></i> Inserting")),
                        hr(class="setting"),
                        HTML("Please insert the <b>Hypothetic WGD events</b> to test in the <b>below tree panel</b>. Click the branch and then follow the rule to insert.<br>"),
                        tags$style(
                            HTML(".rotate-135 {
                                    transform: rotate(135deg);
                                }"),
                            HTML(
                                ".rotate+45 {
                                    transform: rotate(-45deg);
                                }"
                            ),
                            HTML(".rotate-45{
                                    transform: rotate(45deg);
                                }")
                        ),
                        actionButton(
                            "svg_vertical_spacing_add_species",
                            "",
                            icon("arrows-alt-v"),
                            title="Expand vertical spacing",
                            style="border-color: #fff;"
                        ),
                        actionButton(
                            "svg_vertical_spacing_sub_species",
                            "",
                            icon(
                                "down-left-and-up-right-to-center",
                                verify_fa=FALSE,
                                class="rotate-135"
                            ),
                            title="Compress vertical spacing",
                            style="border-color: #fff;"
                        ),
                        actionButton(
                            "svg_horizontal_spacing_add_species",
                            "",
                            icon("arrows-alt-h"),
                            title="Expand horizontal spacing",
                            style="border-color: #fff;"
                        ),
                        actionButton(
                            "svg_horizontal_spacing_sub_species",
                            "",
                            icon(
                                "down-left-and-up-right-to-center",
                                verify_fa=FALSE,
                                class="rotate-45"
                            ),
                            title="Compress horizontal spacing",
                            style="border-color: #fff; "
                        ),
                        # actionButton(
                        #     "update_output",
                        #     "",
                        #     icon("sync"),
                        #     title="Update plot",
                        #     style="color: #fff;
                        #            background-color: #019858;
                        #            border-color: #fff;
                        #            padding: 5px 14px 5px 14px;
                        #            margin: 5px 5px 5px 200px;
                        #            animation: glowing 5000ms infinite;"
                        # ),
                        downloadButton_custom(
                            "speciesTreePlotDownload",
                            status="secondary",
                            icon=icon("download"),
                            label=HTML(""),
                            title="Download svg figure",
                            style="color: #fff;
                                   background-color: #019858;
                                   border-color: #fff;
                                   padding: 5px 14px 5px 14px;
                                   margin: 5px 5px 5px 5px;
                                   animation: glowingD 5000ms infinite;"
                        )
                    ),
                    column(
                        12,
                        div(
                            id="speciesTree_plot",
                        )
                    )
                )
            ),
            column(
                12,
                uiOutput("whaleConfigurePanel")
            ),
            column(
                12,
                uiOutput("whaleCommandPanel")
            )
        )
    ),
    icon=icon(
        name=NULL,
        style="
            background: url('images/treeReconciliationIcon.svg');
            background-size: contain;
            background-position: center;
            background-repeat: no-repeat;
            height: 21px;
            width: 21px;
            display: inline-block;
            vertical-align: middle;
        "
    )
)

#' Obtain coordinates for anchorpoints from GFF files
#'
#' This function takes a file containing anchorpoints, GFF files for two species, and species names,
#' and retrieves the coordinates of anchorpoints and associated genes from the GFF files.
#'
#' @param anchorpoints A file containing anchorpoints information with columns like gene_x, gene_y, and other relevant data.
#' @param species1 The name of the first species.
#' @param gff_file1 The path to the GFF file for the first species.
#' @param out_file The output file where the results will be saved.
#' @param species2 (Optional) The name of the second species. Specify this parameter and gff_file2 if working with two species.
#' @param gff_file2 (Optional) The path to the GFF file for the second species.
#'
#' @importFrom vroom vroom
#' @importFrom dplyr filter
#' @importFrom dplyr select
#' @importFrom dplyr mutate
#' @importFrom dplyr left_join
#'
#'
#' @return None. The function saves the results to the specified out_file.
#'
obtain_coordiantes_for_anchorpoints <- function(anchorpoints, species1, gff_file1, out_file, species2=NULL, gff_file2=NULL){
    gff_df <- suppressMessages(
        vroom(
            gff_file1,
            delim="\t",
            comment="#",
            col_names=FALSE
        )
    )
    # library(vroom)
    # library(dplyr)

    X1 <- X4 <- X5 <- X7 <- X9 <- NULL
    position_df <- gff_df %>%
        filter(gff_df$X3=="mRNA") %>%
        select(X1, X9, X4, X5, X7) %>%
        mutate(X9=gsub("ID=([^;]+).*", "\\1", X9))

    colnames(position_df) <- c("list", "gene", "start", "end", "strand")

    position_df$sp <- species1
    if( !is.null(gff_file2) ){
        gff_df2 <- suppressMessages(
            vroom(
                gff_file2,
                delim="\t",
                comment="#",
                col_names=FALSE
            )
        )
        position_df2 <- gff_df2 %>%
            filter(gff_df2$X3=="mRNA") %>%
            select(X1, X9, X4, X5, X7) %>%
            mutate(X9=gsub("ID=([^;]+).*", "\\1", X9))

        colnames(position_df2) <- c("list", "gene", "start", "end", "strand")
        position_df2$sp <- species2

        position_df <- rbind(position_df, position_df2)
    }
    anchors_df <- suppressMessages(
        vroom(
            anchorpoints,
            delim="\t",
            col_names=TRUE
        )
    )

    final_df <- data.frame()
    merged_x <- left_join(anchors_df, position_df, by=c("gene_x"="gene"))
    merged_y <- left_join(merged_x, position_df, by=c("gene_y"="gene"))
    colnames(merged_y) <- c("id", "multiplicon", "basecluster",
                            "geneX", "geneY", "coordX", "coordY",
                            "is_real_anchorpoint",
                            "listX", "startX", "endX", "strandX", "speciesX",
                            "listY", "startY", "endY", "strandY", "speciesY")
    final_df <- merged_y

    speciesX <- speciesY <- NULL
    # if( !is.null(gff_file2) ){
    #     final_df <- final_df %>% filter(speciesX != speciesY)
    #     final_table <- data.frame(matrix(ncol=ncol(final_df), nrow=nrow(final_df)))
    #     for( i in 1:nrow(final_df) ){
    #         each_row <- final_df[i, ]
    #         if( each_row$speciesX != species1 ){
    #             tmp <- each_row[9:13]
    #             each_row[9:13] <- each_row[14:18]
    #             each_row[14:18] <- tmp
    #
    #             tmp2 <- each_row$geneX
    #             each_row$geneX <- each_row$geneY
    #             each_row$geneY <- tmp2
    #
    #             tmp3 <- each_row$coordX
    #             each_row$coordX <- each_row$coordY
    #             each_row$coordY <- tmp3
    #         }
    #         final_table[i,] <- each_row
    #     }
    #     colnames(final_table) <- c("id", "multiplicon", "basecluster",
    #                                "geneX",  "geneY",  "coordX", "coordY",
    #                                "is_real_anchorpoint",
    #                                "listX", "startX", "endX", "strandX", "speciesX",
    #                                "listY", "startY", "endY", "strandY", "speciesY")
    #     write.table(final_table,
    #                 file=out_file,
    #                 sep="\t",
    #                 row.names=FALSE,
    #                 quote=FALSE)
    # }else{
    #     write.table(
    #         final_df,
    #         file=out_file,
    #         sep="\t",
    #         row.names=FALSE,
    #         quote=FALSE
    #     )
    # }
    write.table(
        final_df,
        file=out_file,
        sep="\t",
        row.names=FALSE,
        quote=FALSE
    )
}

#' Calculate the Ks Distribution for Multiple Speices
#'
#' This function takes a list of data files, calculates the Ks distribution, and returns the results.
#'
#' @param files_list A list of file paths containing Ks data.
#' @param binWidth The width of Ks bins for the distribution.
#' @param maxK The maximum Ks value to consider.
#' @param plot.mode The mode for plotting ("weighted", "average", "min", or "pairwise").
#' @param include.outliers Whether to include outliers in the calculation.
#' @param minK The minimum Ks value to include in the distribution.
#' @param minAlnLen The minimum alignment length to include in the distribution.
#' @param minIdn The minimum alignment identity to include in the distribution.
#' @param minCov The minimum alignment coverage to include in the distribution.
#'
#' @importFrom stats aggregate
#' @importFrom stats complete.cases
#' @importFrom dplyr bind_rows
#'
#' @return A list containing two data frames: "bar" for Ks distribution and "density" for density data.
#'
calculateKsDistribution4wgd_multiple <- function(
        files_list, binWidth=0.1, maxK=5,
        plot.mode="weighted",
        include.outliers=F, minK=0, minAlnLen=0, minIdn=0, minCov=0){
    # library(dplyr)

    full.data <- data.frame()
    full.data.density <- data.frame()
    full_df <- data.frame()
    cols_to_select <- c("Ks", "title")

    for( i in files_list ){
        title <- basename(i)
        title <- gsub(".tsv", "", title)
        plottitle <- gsub("wgd_", "", title)
        df <- read.table(i, sep="\t", header=T)
        df <- df[df$Ks>=minK
                 & df$AlignmentCoverage>=minCov
                 & df$AlignmentLength>=minAlnLen
                 & df$AlignmentIdentity>=minIdn, ]

        if (include.outliers == F) {
            df <- df[df$WeightOutliersExcluded>0,]
        }

        if (plot.mode == "weighted") {
            if (include.outliers == T) {
                df$Weight <- df$WeightOutliersIncluded
            } else {
                df$Weight <- df$WeightOutliersExcluded
            }
        }
        else if (plot.mode == "average") {
            temp.df <- aggregate(df$Ks, by=list(df$Family, df$Node), FUN=mean)
            colnames(temp.df) <- c("Family", "Node", "Ks")
            temp.df$Weight <- 1
            df <- temp.df
        }
        else if (plot.mode == "min") {
            temp.df <- aggregate(df$Ks, by=list(df$Family, df$Node), FUN=min)
            colnames(temp.df) <- c("Family", "Node", "Ks")
            temp.df$Weight <- 1
            df <- temp.df
        }
        else if (plot.mode == "pairwise") {
            df$Weight <- 1
        } else {
            stop("plot.mode should be \"weighted\", \"average\", \"min\", or \"pairwise\"")
        }
        df <- df[complete.cases(df$Ks), ]

        valuesPerBin <- binWidth / 0.01
        df$Ks.bin <- cut(df$Ks, seq(0, maxK, binWidth), right=F, include.lowest=T)
        maxBin <- length(levels(df$Ks.bin))

        df$title <- plottitle
        selected_cols <- df[cols_to_select]
        full_df <- bind_rows(full_df, selected_cols)

        ks.aggregate <- aggregate(df$Weight, by=list(ks.bin=df$Ks.bin), FUN=sum)
        ks.bin.df <- data.frame(ks.bin=cut(seq(minK, maxK, binWidth)[1:maxBin] + binWidth / 2,
                                           seq(minK, maxK, binWidth),
                                           right=F, include.lowest=T),
                                ks=seq(minK, maxK, binWidth)[1:maxBin] + binWidth / 2)
        ks.dist <- merge(ks.aggregate, ks.bin.df, by="ks.bin")

        ks.dist$title <- plottitle
        full.data <- bind_rows(full.data, ks.dist)

        density_df <- data.frame(
            ks=df$Ks,
            title=plottitle,
            binWidth=binWidth
        )
        full.data.density <- rbind(full.data.density, density_df)

    }
    return(list(bar=full.data,
                density=full.data.density))
}

library(data.table)
library(argparse)

parser <- argparse::ArgumentParser()
parser$add_argument("-i", "--input_file", help="Path to the input file", required=TRUE)
parser$add_argument("-o", "--output_dir", help="Path to the output directory", required=TRUE)
parser$add_argument("-s", "--select_clade", help="Species to select gene families for Whale", required=TRUE)
parser$add_argument("-c", "--command_file", help="Path to the command file for i-ADHoRe", required=TRUE)

args <- parser$parse_args()

input_data <- fread(args$input_file,
                    header=FALSE,
                    col.names=c("species", "fasta_file", "gff_file"),
                    fill=TRUE,
                    sep="\t",
                    na.strings=c("", "NA"))
for( i in seq_len(nrow(input_data)) ){
    species <- input_data$species[i]
    fasta_file <- file.path(input_data$fasta_file[i])
    pep_dir <- paste0(args$output_dir, "/pepDir")
    if( !file.exists(pep_dir) ){
        dir.create(pep_dir)
    }
    pep_file <- paste0(pep_dir, "/", gsub(" ", "_", species), ".pep")
    system(
        paste(
            "Rscript tools/cds2aa.R",
            "-i", fasta_file,
            "-o", pep_file
        )
    )
}

cmd_file <- args$command_file
if( file.exists(cmd_file) ){
    file.remove(cmd_file)
}

cmd_con <- file(cmd_file, open="w")

#outDir <- paste0(args$output_dir, "/OrthoFinderOutputDir")
writeLines(
    c(
        "#!/bin/bash",
        "",
        "#SBATCH -p all",
        "#SBATCH -c 4",
        "#SBATCH --mem 8G",
        paste0("#SBATCH -o ", basename(cmd_file), ".o%j"),
        paste0("#SBATCH -e ", basename(cmd_file), ".e%j"),
        ""
    ),
    cmd_con
)

writeLines(
    "module load OrthoFinder",
    cmd_con
)
writeLines(
    paste("orthofinder",
          "-f pepDir",
          "-S diamond -t 4 -a 4 -I 3 -M msa -ot",
          "-o OrthoFinderOutputDir"
    ),
    cmd_con
)
writeLines("folder=$(find ./OrthoFinderOutputDir -maxdepth 1 -type d -name \"Results_*\" -printf '%f')", cmd_con)
writeLines("cd ds_tree_wd", cmd_con)
writeLines(
    paste0(
        "sh ",
        "./computing_Ks_tree_of_SingleCopyOrthologues.shell \\\n",
        "\t-i ../OrthoFinderOutputDir/$folder/Orthogroups/Orthogroups_SingleCopyOrthologues.txt \\\n",
        "\t-o ../OrthoFinderOutputDir/$folder/Orthogroups/Orthogroups.tsv \\\n",
        "\t-d ../OrthoFinderOutputDir/$folder/Orthogroup_Sequences/ \\\n",
        "\t-s SingleCopyOrthologues.tsv \\\n",
        "\t-c ../../Species.info.xls \\\n",
        "\t-p singleCopyAlign.phylip \\\n",
        "\t-t ../../tree.newick \\\n",
        "\t-n 4"
    ),
    cmd_con
)
writeLines("cd ..", cmd_con)
writeLines("# Prepare data for Whale", cmd_con)
filter_script <- paste0(getwd()[1], "/tools/Whale.jl/scripts/orthofilter.py")
system(
    paste(
        "cp",
        filter_script,
        dirname(cmd_file)
    )
)
writeLines(
    paste(
        "python",
        "./orthofilter.py",
        "OrthoFinderOutputDir/$folder/Orthogroups/Orthogroups.tsv",
        args$select_clade,
        "orthogroups.filtered.tsv",
        "1"
    ),
    cmd_con
)
writeLines(
    paste0(
        "tar -czf OrthoFinderOutput_for_Whale.tar.gz orthogroups.filtered.tsv ",
        "--files-from=<(awk -F'\\t' -v folder=\"$folder\" ",
        "'$1 ~ /^OG/ {print \"OrthoFinderOutputDir/\" folder \"/MultipleSequenceAlignments/\"$1\".fa\"}' ",
        "orthogroups.filtered.tsv)"
    ),
    cmd_con
)
writeLines("tar czf OrthoFinderOutputDir/$folder/MultipleSequenceAlignments.tar.gz OrthoFinderOutputDir/$folder/MultipleSequenceAlignments/ && rm -r OrthoFinderOutputDir/$folder/MultipleSequenceAlignments/", cmd_con)
writeLines("tar czf OrthoFinderOutputDir/$folder/Orthogroup_Sequences.tar.gz OrthoFinderOutputDir/$folder/Orthogroup_Sequences/ && rm -r OrthoFinderOutputDir/$folder/Orthogroup_Sequences/", cmd_con)
writeLines("tar czf OrthoFinderOutputDir/$folder/Single_Copy_Orthologue_Sequences.tar.gz OrthoFinderOutputDir/$folder/Single_Copy_Orthologue_Sequences/ && rm -r OrthoFinderOutputDir/$folder/Single_Copy_Orthologue_Sequences/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Comparative_Genomics_Statistics/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Gene_Duplication_Events/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Gene_Trees/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Orthologues/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Phylogenetically_Misplaced_Genes/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Phylogenetic_Hierarchical_Orthogroups/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Putative_Xenologs/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Resolved_Gene_Trees/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/Species_Tree/", cmd_con)
writeLines("rm -r OrthoFinderOutputDir/$folder/WorkingDirectory/", cmd_con)
close(cmd_con)

Tree_Reconciliation_ui <- tabPanel(
    "TreeRecon",
    value="tree_reconciliation",
    fluidRow(
        column(
            id="treeReconciliationTitle",
            width=12,
            h4(icon(
                name=NULL,
                style="
                    background: url('images/treeReconciliationIcon.svg');
                    background-size: contain;
                    background-position: center;
                    background-repeat: no-repeat;
                    height: 50px;
                    width: 50px;
                    display: inline-block;
                    vertical-align: middle;
                "
                ),
                HTML("TreeRecon: Gene Tree – Species Tree Reconciliation Analysis")
            ),
            style="padding-bottom: 5px;
                   padding-top: 5px;
                   padding-left: 30px;",
        ),
        column(
            id="treeReconciliationSettingPanel",
            width=3,
            div(class="boxLike",
                style="background-color: #FAF9F6;
                       padding-left: 10px;",
                h4(icon("upload"), "Uploading"),
                hr(class="setting"),
                h5(HTML("<font color='green'><b><i>shinyWGD</i> Whale Analysis</b></font> Data")),
                style="background-color: #FAF9F6;
                       margin: 5px 5px 5px 5px;
                       padding: 5px 10px 10px 10px;",
                fluidRow(
                    class="justify-content-left",
                    style="padding-bottom: 15px;
                           padding-top: 5px",
                    column(
                        12,
                        div(
                            style="padding-left: 10px;
                                   position: relative;",
                            fileInput(
                                'whale_data_zip_file',
                                label=h6(icon("file-zipper"), "Upload the Zipped File"),
                                multiple=FALSE,
                                accept=c(
                                    ".zip",
                                    ".gz"
                                ),
                                width="80%"
                            ),
                            actionButton(
                                inputId="whale_TreeRecon_example",
                                "",
                                icon=icon("question"),
                                status="secondary",
                                class="my-start-button-class",
                                title="Click to use the example data to demo run the TreeRecon Analysis",
                                style="color: #fff;
                                       background-color: #87CEEB;
                                       border-color: #fff;
                                       position: absolute;
                                       top: 63%;
                                       left: 90%;
                                       margin-top: -15px;
                                       margin-left: -15px;
                                       padding: 5px 14px 5px 10px;
                                       width: 30px; height: 30px; border-radius: 50%;"
                            )
                        )
                    ),
                    column(
                        12,
                        uiOutput("selectedTreeReconDirName")
                    )
                )
                # fluidRow(
                #     column(
                #         12,
                #         hr(class="setting")
                #     ),
                #     column(
                #         12,
                #         uiOutput("selectedSubAnalysisDir")
                #     )
                # )
            ),
            # div(class="boxLike",
            #     style="background-color: #FAF9F6;
            #            padding-bottom: 10px;
            #            padding-top: 10px",
            #     h4(icon("cog"), "Setting"),
            #     fluidRow(
            #         column(
            #             12,
            #             textInput(
            #                 inputId="wgdNodes",
            #                 label=HTML("Add <font color='#977C00'><b>WGD Nodes</b></font> to the Tree:"),
            #                 value="",
            #                 width="100%",
            #                 placeholder="wgd_1: PPAT, PPAT; wgd_2: ATHA, ATRI"
            #             )
            #         ),
            #         h6(HTML("Use <font color='orange'><b>\";\"</b></font> to separate."))
            #     )
            # )
            # column(
            #     12,
            #     uiOutput("wgdCommnadPanel")
            # ),
            # div(class="boxLike",
            #     style="background-color: #FBFEEC;
            #            margin: 5px 5px 5px 5px;
            #            padding: 5px 10px 10px 10px;",
            #     column(
            #         12,
            #         uiOutput("whaleOutputPanel")
            #     )
            # )
        ),
        column(
            # id="speciesTreeOutputPanel",
            width=9,
            div(class="boxLike",
                style="background-color: white;",
                column(
                    12,
                    uiOutput("whaleOutputPanel")
                ),
                column(
                    12,
                    hr(class="setting")
                ),
                column(
                    12,
                    h5(HTML("The species cladograms with the putative WGD events in the <b>Whale</b> analysis"))
                ),
                # column(
                #     12,
                #     hr(class="setting")
                # ),
                column(
                    12,
                    fluidRow(
                        column(
                            9,
                            tags$style(
                                HTML(".rotate-135 {
                                    transform: rotate(135deg);
                                }"),
                                HTML(".rotate-45{
                                    transform: rotate(45deg);
                                }")
                            ),
                            actionButton(
                                "TreeRecon_svg_vertical_spacing_add_species",
                                "",
                                icon("arrows-alt-v"),
                                title="Expand vertical spacing",
                                style="border-color: #fff;"
                            ),
                            actionButton(
                                "TreeRecon_svg_vertical_spacing_sub_species",
                                "",
                                icon(
                                    "down-left-and-up-right-to-center",
                                    verify_fa=FALSE,
                                    class="rotate-135"
                                ),
                                title="Compress vertical spacing",
                                style="border-color: #fff;"
                            ),
                            actionButton(
                                "TreeRecon_svg_horizontal_spacing_add_species",
                                "",
                                icon("arrows-alt-h"),
                                title="Expand horizontal spacing",
                                style="border-color: #fff;"
                            ),
                            actionButton(
                                "TreeRecon_svg_horizontal_spacing_sub_species",
                                "",
                                icon(
                                    "down-left-and-up-right-to-center",
                                    verify_fa=FALSE,
                                    class="rotate-45"
                                ),
                                title="Compress horizontal spacing",
                                style="border-color: #fff; "
                            ),
                            downloadButton_custom(
                                "speciesWhaleTreeReconPlotDownload",
                                status="secondary",
                                icon=icon("download"),
                                label=HTML(""),
                                title="Download svg figure",
                                class="my-download-button-class",
                                style="color: #fff;
                                   background-color: #6B8E23;
                                   border-color: #fff;
                                   padding: 5px 14px 5px 14px;
                                   margin: 5px 5px 5px 5px;"
                            )
                        ),
                        column(
                            9,
                            div(
                                id="speciesWhaleTreeRecon_plot",
                            ),
                            uiOutput("whaleReconTreeDesPanel")
                        )
                    )
                ),
                column(
                    12,
                    hr(class="setting")
                ),
                column(
                    12,
                    uiOutput("posteriorPanelTitle"),
                    fluidRow(
                        column(
                            9,
                            # tags$style(
                            #     HTML(".rotate-135 {
                            #     transform: rotate(135deg);
                            # }"),
                            #     HTML(".rotate-45{
                            #     transform: rotate(45deg);
                            # }")
                            # ),
                            # actionButton(
                            #     "posterior_svg_vertical_spacing_add_species",
                            #     "",
                            #     icon("arrows-alt-v"),
                            #     title="Expand vertical spacing",
                            #     style="border-color: #fff;"
                            # ),
                            # actionButton(
                            #     "posterior_svg_vertical_spacing_sub_species",
                            #     "",
                            #     icon(
                            #         "down-left-and-up-right-to-center",
                            #         verify_fa=FALSE,
                            #         class="rotate-135"
                            #     ),
                            #     title="Compress vertical spacing",
                            #     style="border-color: #fff;"
                            # ),
                            # actionButton(
                            #     "posterior_svg_horizontal_spacing_add_species",
                            #     "",
                            #     icon("arrows-alt-h"),
                            #     title="Expand horizontal spacing",
                            #     style="border-color: #fff;"
                            # ),
                            # actionButton(
                            #     "posterior_svg_horizontal_spacing_sub_species",
                            #     "",
                            #     icon(
                            #         "down-left-and-up-right-to-center",
                            #         verify_fa=FALSE,
                            #         class="rotate-45"
                            #     ),
                            #     title="Compress horizontal spacing",
                            #     style="border-color: #fff; "
                            # ),
                            downloadButton_custom(
                                "posteriorDistPlotDownload",
                                status="secondary",
                                icon=icon("download"),
                                label=HTML(""),
                                title="Download svg figure",
                                class="my-download-button-class",
                                style="color: #fff;
                                       background-color: #6B8E23;
                                       border-color: #fff;
                                       padding: 5px 14px 5px 14px;
                                       margin: 5px 5px 5px 5px;"
                            )
                        ),
                        column(
                            8,
                            div(
                                id="posterior_Dist_plot_div",
                            ),
                            # uiOutput("whaleReconTreeDesPanel")
                        )
                    )
                ),
                column(
                    12,
                    uiOutput("whaleTextOutputPanel")
                )
            )
        )
    ),
    #icon=icon("chart-line")
    icon=icon(
        name=NULL,
        style="
            background: url('images/treeReconciliationIcon.svg');
            background-size: contain;
            background-position: center;
            background-repeat: no-repeat;
            height: 21px;
            width: 21px;
            display: inline-block;
            vertical-align: middle;
        "
    )
)

#!/bin/bash

# Check if the correct number of arguments is provided
if [ $# -ne 6 ] && [ $# -ne 7 ]; then
    echo "Usage: $0 <tree.newick> <aleDir> <wgdNodes> <outShellFile> <model> <chain> <number>"
    exit 1
fi

whale_configure() {
	local tree=$1
	local aleDir=$2
	local wgdNodes=$3
	local outFile=$4
	local whaleModel=$5
	local whaleChain=$6

	whaleDir=$(dirname "$outFile")

	cat >$outFile <<EOF
using Pkg; Pkg.activate(@__DIR__)
using Whale, NewickTree, Turing, DataFrames, CSV, JSON, Serialization, LinearAlgebra, Optim, Distributions, Logging, KernelDensity


function compute_average_q(df)
	qcols = filter(x -> occursin("q", x), names(df))
	avg_q = [mean(df[!, col]) for col in qcols]
    return avg_q
end


out = mkpath("${whaleDir}/output")
log_file = joinpath(out, "run.log")
logger = SimpleLogger(open(log_file, "w"), LogLevel(Info))
global_logger(logger)

tree = readnw(readline("${tree}"))
@info "Tree read from file" tree
for n in postwalk(tree)
	n.data.distance / 100
end

nn = length(postwalk(tree))
@info "Number of internal nodes: " nn
EOF

	row_count=0
	random_list=()
	while IFS= read -r line || [[ -n "$line" ]]; do
		if [[ -z "$line" ]]; then
			continue
		fi

		name=$(echo "$line" | sed 's/ //g' | awk -F '[:-]' '{print $1}')
    	value2=$(echo "$line" | sed 's/ //g' | awk -F '[:-]' '{print $2}')
    	value3=$(echo "$line" | sed 's/ //g' | awk -F '[:-]' '{print $3}')

	   	if [[ "$value2" == "$value3" ]]; then
		   	parts="insertnode!(getlca(tree, \"$value2\"), name=\"$name\")"
		else
			parts="insertnode!(getlca(tree, \"$value2\", \"$value3\"), name=\"$name\")"
		fi

		#parts=$(echo "$line" | sed 's/ //g' |awk -F '[:-]' '{print "insertnode!(getlca(tree, \""$2"\", \""$3"\"), name=\""$1"\")"}')
		echo ${parts} >> ${outFile}
		((row_count++))
		random_value=$(awk -v seed="${row_count}" 'BEGIN{srand(seed); r=rand(); printf "%.3f\n", r}')
		random_list+=($random_value)

	done < $wgdNodes

	qList="["
	#pList="["
	for ((i=0; i<${#random_list[@]}; i++)); do
		if ((i != 0)); then
			#pList+=", "
			qList+=", "
		fi
		qList+="0.1"
		#pList+="${random_list[i]}"
	done
	#pList+="]"
	qList+="]"

	if [ "$whaleModel" == "Constant_rates" ]; then
		echo "param = ConstantDLWGD(λ=1., μ=1., η=0.9, q=${qList})" >>$outFile
		echo "@info \"param\" param" >>$outFile
		echo "model = WhaleModel(param, tree, .1, condition=Whale.RootCondition(), minn=10, maxn=20)" >>$outFile
		echo "@info \"model\" model" >>$outFile
		echo "write(joinpath(out, \"model.txt\"), repr(model))" >>$outFile
		echo "data = read_ale(\"${aleDir}\", model)" >>$outFile
		echo "@info \"ccd data\" data" >>$outFile

		echo "@model constantrates(model, data) = begin" >>$outFile
		echo -e "\tλ  ~ Exponential()" >>$outFile
		echo -e "\tμ  ~ Exponential()" >>$outFile
		echo -e "\tη  ~ Beta(3,1)" >>$outFile

		qq="["
		for ((i=0; i<${#random_list[@]}; i++)); do
			if ((i != 0)); then
				qq+=", "
			fi
			j=$((i+1))
			echo -e "\tq$j ~ Beta()" >>$outFile
			qq+="q$j"
		done
		qq+="]"

		echo -e "\tdata ~ model((λ=λ, μ=μ, η=η, q=${qq}))" >>$outFile
		echo "end" >>$outFile

		echo "chain0 = sample(constantrates(model, data), NUTS(), ${whaleChain})" >>$outFile
		echo "summary_file = joinpath(out, \"MCMCchain.s\")" >>$outFile
		echo "summary_stream = open(summary_file, \"w\")" >>$outFile
		echo "redirect_stdout(summary_stream)" >>$outFile
		echo "show(summary_stream, \"text/plain\", chain0)" >>$outFile
		echo "close(summary_stream)" >>$outFile
		echo "CSV.write(joinpath(out, \"chainConstantRates.csv\"), chain0)" >>$outFile
		echo "serialize(joinpath(out, \"chainConstantRates.jls\"), chain0)" >>$outFile
		echo "df = DataFrame(CSV.File(joinpath(out, \"chainConstantRates.csv\")))" >>$outFile
		echo "qcols = [col for col in names(df) if startswith(string(col), \"q\")]" >>$outFile

		qqK="["
		for ((i=0; i<${#random_list[@]}; i++)); do
			if ((i != 0)); then
				qqK+=", "
			fi
			j=$((i+1))
			qqK+=":q$j"
		done
		qqK+="]"

		echo "Q = compute_average_q(df)" >>$outFile
		echo "Kdf = summarize(chain0[${qqK}], Whale.bayesfactor)" >>$outFile
		echo "K = DataFrame(Kdf)" >>$outFile
		echo "dfQK = DataFrame(q̅ = Q, K = K.bayesfactor)" >>$outFile

	elif [ "$whaleModel" == "Critical_branch" ]; then
		echo "param = DLWGD(λ=zeros(nn), μ=zeros(nn), η=0.9, q=${qList})" >>$outFile
		echo "@info \"param\" param" >>$outFile
		echo "model = WhaleModel(param, tree, .1)" >>$outFile
		echo "@info \"model\" model" >>$outFile
		echo "write(joinpath(out, \"model.txt\"), repr(model))" >>$outFile
		echo "data = read_ale(\"${aleDir}\", model)" >>$outFile
		echo "@info \"ccd data\" data" >>$outFile

		echo "@model critical(model, X, nn) = begin" >>$outFile
		echo -e "\tη ~ Beta(3,1)" >>$outFile
		echo -e "\tr ~ Turing.Flat()" >>$outFile
		echo -e "\tσ ~ Exponential(0.1)" >>$outFile
		echo -e "\tλ ~ MvNormal(repeat([r], nn-1), σ)" >>$outFile
		echo -e "\tl = [λ; r]" >>$outFile

		qq="["
		for ((i=0; i<${#random_list[@]}; i++)); do
			if ((i != 0)); then
				qq+=", "
			fi
			j=$((i+1))
			echo -e "\tq$j ~ Beta()" >>$outFile
			qq+="q$j"
		done
		qq+="]"

		echo -e "\tX ~ model((λ=l, μ=l, η=η, q=${qq}))" >>$outFile
		echo "end" >>$outFile

		echo "cmodel = critical(model, data, nn)" >>$outFile
		echo "@info \"critical branch model\" cmodel" >>$outFile
		echo "write(joinpath(out, \"cmodel.txt\"), repr(cmodel))" >>$outFile
		echo "chaincritical = sample(cmodel, NUTS(0.65), ${whaleChain})" >>$outFile
		echo "summary_file = joinpath(out, \"MCMCchain.s\")" >>$outFile
		echo "summary_stream = open(summary_file, \"w\")" >>$outFile
		echo "redirect_stdout(summary_stream)" >>$outFile
		echo "show(summary_stream, \"text/plain\", chaincritical)" >>$outFile
		echo "close(summary_stream)" >>$outFile
		echo "CSV.write(joinpath(out, \"chaincritical.csv\"), chaincritical)" >>$outFile
		echo "serialize(joinpath(out, \"chaincritical.jls\"), chaincritical)" >>$outFile
		echo "df = DataFrame(CSV.File(joinpath(out, \"chaincritical.csv\")))" >>$outFile

		qqK="["
		for ((i=0; i<${#random_list[@]}; i++)); do
			if ((i != 0)); then
				qqK+=", "
			fi
			j=$((i+1))
			qqK+=":q$j"
		done
		qqK+="]"

		echo "Q = compute_average_q(df)" >>$outFile
		echo "Kdf = summarize(chaincritical[${qqK}], Whale.bayesfactor)" >>$outFile
		echo "K = DataFrame(Kdf)" >>$outFile
		echo "dfQK = DataFrame(q̅ = Q, K = K.bayesfactor)" >>$outFile

    else
		echo "param = DLWGD(λ=zeros(nn), μ=zeros(nn), η=0.9, q=${qList})" >>$outFile
		echo "@info \"param\" param" >>$outFile
		echo "model = WhaleModel(param, tree, .1)" >>$outFile
		echo "@info \"model\" model" >>$outFile
		echo "write(joinpath(out, \"model.txt\"), repr(model))" >>$outFile
		echo "data = read_ale(\"${aleDir}\", model)" >>$outFile
		echo "@info \"ccd data\" data" >>$outFile

		echo "@model branchrates(model, X, n, τmean=1.) = begin" >>$outFile
		echo -e "\tη ~ Beta(3,1)" >>$outFile
		echo -e "\tρ ~ Uniform(-1, 1.)" >>$outFile
		echo -e "\tτ ~ Exponential(τmean)" >>$outFile
		echo -e "\tT = typeof(ρ)" >>$outFile
		echo -e "\tS = [τ 0. ; 0. τ]" >>$outFile
		echo -e "\tR = [1.  ρ; ρ 1.]" >>$outFile
		echo -e "\tΣ = S*R*S" >>$outFile
		echo -e "\t!isposdef(Σ) && return -Inf" >>$outFile
		echo -e "\tr = Matrix{T}(undef, 2, n)" >>$outFile
		echo -e "\to = id(getroot(model))" >>$outFile
		echo -e "\tr[:,o] ~ MvNormal(zeros(2), ones(2))" >>$outFile
		echo -e "\tfor i=1:n" >>$outFile
		echo -e "\t\ti == o && continue" >>$outFile
		echo -e "\t\tr[:,i] ~ MvNormal(r[:,o], Σ)" >>$outFile
		echo -e "\tend" >>$outFile

		qq="["
		for ((i=0; i<${#random_list[@]}; i++)); do
			if ((i != 0)); then
				qq+=", "
			fi
			j=$((i+1))
			echo -e "\tq$j ~ Beta()" >>$outFile
			qq+="q$j"
		done
		qq+="]"

		echo -e "\tX ~ model((λ=r[1,:], μ=r[2,:], η=η, q=${qq}))" >>$outFile
		echo "end" >>$outFile

		echo "bmodel = branchrates(model, data, nn)" >>$outFile
		echo "@info \"relaxed branch model\" bmodel" >>$outFile
		echo "write(joinpath(out, \"bmodel.txt\"), repr(bmodel))" >>$outFile
		echo "chainrelaxed = sample(bmodel, NUTS(0.65), ${whaleChain})" >>$outFile
		echo "summary_file = joinpath(out, \"MCMCchain.s\")" >>$outFile
		echo "summary_stream = open(summary_file, \"w\")" >>$outFile
		echo "redirect_stdout(summary_stream)" >>$outFile
		echo "show(summary_stream, \"text/plain\", chainrelaxed)" >>$outFile
		echo "close(summary_stream)" >>$outFile
		echo "CSV.write(joinpath(out, \"chainrelaxed.csv\"), chainrelaxed)" >>$outFile
		echo "serialize(joinpath(out, \"chainrelaxed.jls\"), chainrelaxed)" >>$outFile
		echo "df = DataFrame(CSV.File(joinpath(out, \"chainrelaxed.csv\")))" >>$outFile

		#echo "chain = deserialize("chainrelaxed.jls")" >>$outFile

		qqK="["
		for ((i=0; i<${#random_list[@]}; i++)); do
			if ((i != 0)); then
				qqK+=", "
			fi
			j=$((i+1))
			qqK+=":q$j"
		done
		qqK+="]"

		echo "Q = compute_average_q(df)" >>$outFile
		echo "Kdf = summarize(chainrelaxed[${qqK}], Whale.bayesfactor)" >>$outFile
		echo "K = DataFrame(Kdf)" >>$outFile
		echo "dfQK = DataFrame(q̅ = Q, K = K.bayesfactor)" >>$outFile
    fi

	echo "wgd_data = readlines(joinpath(out, \"../..\", \"wgdNodes.txt\"))" >>$outFile
	echo "wgd_names = String[]" >>$outFile
	echo "for line in wgd_data" >>$outFile
	echo -e "\tline = strip(line)" >>$outFile
	echo -e "\tisempty(line) && continue" >>$outFile
	echo -e "\tparts = split(line, \":\")" >>$outFile
    echo -e "\tname = strip(parts[1])" >>$outFile
	echo "push!(wgd_names, name)" >>$outFile
	echo "end" >>$outFile
	echo "dfQK_modified = hcat(DataFrame(Hypotheses = wgd_names), dfQK)" >>$outFile
	echo "dfQK_modified.q̅ = round.(dfQK_modified.q̅; digits = 5)" >>$outFile
	#echo "dfQK_modified.K = round.(dfQK_modified.K; digits = 3)" >>$outFile
	echo "@info dfQK_modified" >>$outFile
	echo "write(joinpath(out, \"posterior_mean_of_duplicate_retention_rate_Bayes_factor.txt\"), repr(dfQK_modified))" >>$outFile

}

randomly_selecting_gf() {
    local dirpath="$1"
    local num_files="$2"
    local random_seed="$3"

    if [ -n "$random_seed" ]; then
        RANDOM="$random_seed"
    fi

    local selected_dir="${dirpath}.selected"

    if [ -d "$selected_dir" ]; then
        rm -r "$selected_dir"
    fi

    mkdir "$selected_dir"

    shopt -s nullglob
    files=("$dirpath"/*)
    shopt -u nullglob

    for ((i = 0; i < num_files; i++)); do
        random_index=$((RANDOM % ${#files[@]}))
        selected_file="${files[random_index]}"

        cp "$selected_file" "$selected_dir"

        unset "files[random_index]"
        files=("${files[@]}")
    done
}

if [ $# -eq 6 ]; then
    whale_configure $1 $2 $3 $4 $5 $6
elif [ $# -eq 7 ]; then
    randomly_selecting_gf $2 $7 42
    newAleDir="$2.selected"
    whale_configure $1 $newAleDir $3 $4 $5 $6
else
    echo "Usage: $0 <tree.newick> <aleDir> <wgdNodes> <outShellFile> <model> <chain> <number>"
    exit 1
fi

#!/bin/bash

#SBATCH -p all
#SBATCH -c 4
#SBATCH --mem 10G
#SBATCH -o run_paralog_ks_rest_species.sh.o%j
#SBATCH -e run_paralog_ks_rest_species.sh.e%j

export OPENBLAS_NUM_THREADS=4
export GOTO_NUM_THREADS=4
export OMP_NUM_THREADS=4

module load ksrates/x86_64/1.1.3


species_names=$(ls ../*fa | cut -d '/' -f 2 | sed 's/\.fa//')
focal_species=$(awk -F'=' '/focal_species/ {gsub(/^[ \t]+|[ \t]+$/, "", $2); print $2}' ksrates_conf.txt)

readarray -t species_array <<< "$species_names"

species_array=("${species_array[@]/$focal_species}")

for species in "${species_array[@]}"; do
	if [[ "$species" == "" ]]; then
		continue
	fi

	gff_file="../$species.gff"

	if [[ -f "$gff_file" ]]; then
		focal_species_renew="$species"
		awk -v focal_species="$focal_species_renew" -v gff_file="$gff_file" '/focal_species/ {$0 = "focal_species=" focal_species} /gff_filename/ {$0 = "gff_filename=" gff_file} 1' ksrates_conf.txt > ksrates_conf_$species.txt
	else
		focal_species_renew="$species"
		awk -v focal_species="$focal_species_renew" '/focal_species/ {$0 = "focal_species=" focal_species} 1' ksrates_conf.txt > ksrates_conf_$species.txt
		sed -i '/gff_filename/d' ksrates_conf_$species.txt
		sed -i 's/collinearity=yes/collinearity=no/' ksrates_conf_$species.txt
	fi

	ksrates init ksrates_conf_$species.txt
	ksrates paralogs-ks ksrates_conf_$species.txt --n-threads 4
	gzip paralog_distributions/wgd_$species/$species.blast.tsv
	rm ksrates_conf_$species.txt
	rm wgd_runs_$species.txt
done


echo Done at: `date`

<div style="float: right; width: 60%;">

### <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>

</div>

<!-- Content with the floating layout -->

<style>
  .sidebar {
    position: fixed;
    left: 0;
    bottom: 0;
    height: 100%;
    top: 10%;
    color: #4C4C4C;
    padding-left: 50px;
    padding-top: 50px;
  }
</style>

<div class="sidebar">
<!-- div style="position: fixed; top: 40%; left: 10px; bottom: 0; transform: translate(0, -50%); background-color: #F9F9F9; color: #4C4C4C; padding: 5px 5px;" -->


##### Index

- [Introduction](#introduction)
- [Usage](#usage)
  - [<font color="orange"><i aria-label="microscope icon" class="fa fa-microscope fa-fw fa-fade" role="presentation"></i> Data Preparation</font>](#data-preparation)
    - [Preparing the input data](#preparing)
      - [*Option One*](#option-one)
      - [*Option Two*](#option-two)
    - [Excuting `wgd`, `krates`, and `orthofinder`](#excuting)
    - [Extracting the tree from Timetree.org](#extracting)
  - [<font color="orange"><i class="fas fa-chart-line" role="presentation"></i> <i>K</i><sub>s</sub> Age Distribution Analysis</font>](#ks-inference)
    - [Paralog <i>K</i><sub>s</sub> age distribution](#paralog-ks)
    - [Ortholog <i>K</i><sub>s</sub> age distribution](#ortholog-ks)
    - [Substitution rate correctiong](#rate-correction)
  - [<font color="orange"><i aria-label="water icon" class="fa-solid fa-water" role="presentation"></i> Synteny Analysis</font>](#synteny-analysis)
    - [Intra- or Inter-species alignment](#intra-inter-comparing)
    - [Multiplicon-level synteny](#multiplicon-level-synteny)
    - [Multiple species alignment](#multiple-species-alignment)
    - [Clustering analysis](#clustering-analysis)
  - [<font color="orange"><img src="../images/ksTreeIcon.svg" alt="Icon" width="20" height="20"> Tree Building</font>](#tree-building)
    - [<i>K</i><sub>s</sub> unit tree](#ks-tree)
    - [TimeTree](#time-tree)
    - [<i>K</i><sub>s</sub> unit tree and TimeTree joint tree](#joint-tree)
  - [<font color="orange"><img src="../images/treeReconciliationIcon.svg" alt="Icon" width="20" height="20"> Gene Tree – Species Tree Reconciliation Analysis</font>](#reconciliation-analysis)
</div>

<div style="float: right; width: 60%; text-align: justify; padding-top: 50px;">

  ---

  <a name="introduction"></a>
  ### Introduction

  <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>
   can prepare the input and command lines for [`wgd`](https://github.com/arzwa/wgd), [`ksrates`](https://github.com/VIB-PSB/ksrates), [`i-ADHoRe`](https://www.vandepeerlab.org/?q=tools/i-adhore30), and [`OrthoFinder`](https://github.com/davidemms/OrthoFinder)
  (<font color="orange"><i aria-label="microscope icon" class="fa fa-microscope fa-fw fa-fade" role="presentation"></i> Data Preparation</font>). 

  <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> can also assist users to use [`Whale`](https://github.com/arzwa/whaleprep/tree/master) to infer reconciled gene trees and parameters of a model of gene family evolution given a known species tree.

  After directly uploading the output of `wgd`, `ksrates`, `i-ADHoRe`, or `OrthoFinder`, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> can study the whole genome duplication events (WGDs).


  This document will provide an example to demonstrate the usage of
  <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>.

  ---

  <a name="usage"></a>

  ### Usage

  ---

  <a name="data-preparation"></a>

  #### <font color="orange"><i aria-label="microscope icon" class="fa fa-microscope fa-fw fa-fade" role="presentation"></i> Data Preparation</font>


  <a name="preparing"></a>

  ##### <font color="#23B395">Preparing the input for `wgd`, `ksrates`, `i-ADHoRe` or `OrthoFinder`</font>

  Please select the proper number of species you want to study in <font color="9f5000">Select Number of Species to Analyze:</font> first.

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="green"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> If only one species is provided, the core program will be set to <b>wgd</b>, otherwise, <b>ksrates</b> will be chosen.</div></blockquote>

  <br><span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>
   provides two options to upload the data in <font color="orange"><i aria-label="microscope icon" class="fa fa-microscope fa-fw" role="presentation"></i> Data Preparation</font> Page.

  <a name="option-one"></a>

  ###### *Option One*

  <br>Users can prepare a <font color="green">**Tab-Separated**</font> file in which each row includes the data of a species. 

  This option is suitable to study multiple species. 

  The detailed format of this file is below: </br>


  + **`Tab-Separated file`**
    <table>
      <tr>
        <th><i>latin name</i></th>
        <th style="padding-left: 50px;">cds path<sup>a</sup></th>
        <th style="padding-left: 40px;">gff path<sup>b</sup> (optional)</th>
      </tr>
      <tr>
        <td><i>Elaeis guineensis</i></td>
        <td style="padding-left: 50px;">PATH/elaeis.fasta</td>
        <td style="padding-left: 40px;">PATH/elaeis.gff</td>
      </tr>
      <tr>
        <td><i>Oryza sativa</i></td>
        <td style="padding-left: 50px;">PATH/aryza.fasta</td>
        <td style="padding-left: 40px;">PATH/oryza.gff3</td>
      </tr>
      <tr>
        <td><i>Asparagus officinalis</i></td>
        <td style="padding-left: 50px;">PATH/asparagus.fasta</td>
        <td style="padding-left: 40px;"></td>
      </tr>
    </table>

    - **<sup>a</sup>** `cds` is the coding region of a gene or the portion of a gene's DNA or RNA that codes for protein. This file can be `fasta` or `gzipped-fasta`. <br>This column is **mandatory**.
    - <font color="green"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> Do not contain the [alternative isoforms](https://en.wikipedia.org/wiki/Alternative_splicing) of each gene.
    - **<sup>b</sup>** `gff` is the format consists of one line per feature, each containing nine columns of data, plus optional track definition lines, see more [click here](https://www.ensembl.org/info/website/upload/gff.html). <br>The file can be `gff`, `gff3`, `gzipped-gff`, or `gzipped-gff3`. <br>This column is **mandatory** for **`focal species`** and is **optional** for **`other species`**.</br>

    - **`focal species`** is the parameter in `ksrates`, which will be set in the [`configure file`](https://ksrates.readthedocs.io/en/latest/configuration.html) of `ksrates`.

  After successfully uploading the tab-separated file, the number of species in the file will be calculated. 
  + If only one species needed to be studied, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>
   will generate the command line for `wgd` <br>by clicking the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="code icon" class="fa fa-code fa-fw" role="presentation"></i> Create WGD Codes</font> or <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="code icon" class="fa fa-code fa-fw" role="presentation"></i> Create i-ADHoRe Codes</font> button in the right **`Setting`** panel. 
    + Users can check the codes by clicking the <font color="#ad5a5a"><i aria-label="share icon" class="fa fa-share" role="presentation"></i> Check WGD Codes</font> or <font color="#ad5a5a"><i aria-label="share icon" class="fa fa-share" role="presentation"></i> Check i-ADHoRe Codes</font> button.
    + Then, users can click the <font style="background-color: #508ab2; color: white; padding: 5px; border-radius: 2px;"><i arial-label="download icon" class="fa fa-download" role="presentation"></i> Download Analysis Data</font> button to download the prepared files and script for `wgd` program. 
    <br>
  + If multiple species needed to be study, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>
   will generate the command line for `ksrates`. 
    + Two more parameters should be added.
      + [`focal species`](https://ksrates.readthedocs.io/en/latest/configuration.html) set the focal species for `ksrates`.
      + `newick tree` upload a tree file in `newick` format. The tree should contain all the species.
    + Then, users can click the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="code icon" class="fa fa-code fa-fw" role="presentation"></i> Create Ksrates Codes</font> button to generate the `configure file` and command line for `ksrates`.
    + After that, users can click the <font color="#ad5a5a"><i aria-label="share icon" class="fa fa-share" role="presentation"></i> Check Ksrates Codes</font> button to review the `configure file` and command line for `ksrates`.
    + Then, users can click the rest buttons to generate the codes for `i-ADHoRe` and `OrthoFinder`.
    + Finally, users can click the <font style="background-color: #508ab2; color: white; padding: 5px; border-radius: 2px;"><i arial-label="download icon" class="fa fa-download" role="presentation"></i> Download Analysis Data</font> button to download the prepared files and script for `ksrates`, `i-ADHoRe`, and `OrthoFinder` program. 

    <!--<br><img src="../images/data_preparation.gif" alt="" class="docImage" width="90%"></br>--->
    <br></br> 


  <a name="option-two"></a>

  ###### *Option Two*

  <br>This option provide users a more straightforward way to upload data.</br>

  + After selecting the number of studied species, the corresponding uploading panels will be created. 
  + Three items (`latin name`, `cds fasta`, and annotation `gff`) of each species should be set or uploaded by users. 
  + Once more than one species needed to be analyzed, the `focal species` and `newick tree` should be defined by users.
  + Then users can click the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="code icon" class="fa fa-code fa-fw" role="presentation"></i> Create WGD Codes</font> or <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="code icon" class="fa fa-code fa-fw" role="presentation"></i> Create Ksrates Codes</font> button to create the codes for `wgd` or `ksrates` according to the species number.
  + After reviewing and downloading the necessary files, users can execute `wgd`, `ksrates`, `i-ADHoRe` or `OrthoFinder` in their local environment. 
  <br>

  <a name="excuting"></a>

  ##### <font color="#23B395">Executing `wgd`, `krates`, `i-ADHoRe` or `OrthoFinder`</font>
  After decompressing the downloaded compressed file (**`Analysis_Date.tgz`**), users will find the `bash` and `configure` files (`run_wgd.sh` / `run_ksrates.sh`, `run_iadhore.sh`, `run_orthofinder.sh`). <br>Users can execute them in their local environment.  
    <!-- + downloading files for `wgd`
    <br><img src="../images/wgd_download.png" alt="" class="docImage" width="90%"></br>
    + downloading files for `ksrates` and `i-ADHoRe`
    <br><img src="../images/ksrates_iadhore_download.png" alt="" class="docImage" width="90%"></br> --->
    - How to install `wgd` to your local environment, please read the [`wgd` document](https://github.com/arzwa/wgd).
    - How to install `ksrate` to your local environment, please read the [`ksrates` document](https://ksrates.readthedocs.io/en/latest/installation.html).
    - How to install `i-ADHoRe` to your local environment, please read the [`i-ADHoRe` document](https://www.vandepeerlab.org/?q=tools/i-adhore30).
    - How to install `OrthoFinder` to your local environment, please read the [`OrthoFinder` document](https://github.com/davidemms/OrthoFinder).
    <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="green"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> When creating the Code and files for <font color="green">i-ADHoRe</font>, the alignment file will be produced. The alignment process will use the <font color="green">diamond</font> program. So please install <font color="green">diamond</font> program first.</blockquote>

  <br>

  <a name="extracting"></a>

  ##### <font color="#23B395">Extracting the tree from TimeTree.org</font>

  If users don’t ensure the evolutionary relationships among the studied species, <br><span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>
   provides an alternative way to generate the newick tree through interacting with [TimeTree.org](http://www.timetree.org/) database.

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="red"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> TimeTree database does not include all the species.
  <br>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;If the species are not included in the TimeTree database, you should ensure the relationship in other ways.</div></blockquote>

  Please click the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="tree icon" class="fa fa-tree fa-fw" role="presentation"></i> Extract Tree</font> button to extract a tree from TimTree.org database.<br>
  After successfully obtaining the tree, users can review the tree in the box below <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i aria-label="tree icon" class="fa fa-tree fa-fw" role="presentation"></i> Extract Tree</font> button.<br>
  <br><img src="../images/timetree.png" alt="" class="docImage" width="70%"></br>
  Users need to save this tree to a local file and then upload the file to `newick tree` panel in the <b>`Setting`</b> block.

  ---

  <a name="ks-inference"></a>

  #### <font color="orange"><img src="../images/ksIcon.svg" alt="Icon" width="20" height="20"><i>K</i><sub>s</sub> Age Distribution Analysis</font>

  After successfully running the corresponding shell files locally, the necessary outputs will be saved in the **`Analysis_Date`** folder. Then users can choose this folder in the `Uploading` panel in <font color="orange"><i class="fas fa-chart-line" role="presentation"></i> <i>K</i><sub>s</sub> Age Distribution Analysis</font> page.

  <!--<br><img src="../images/Ks_age_distribution.gif" alt="" class="docImage" width="90%"></br>--->

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="red"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> Users should choose the folder which is originated from the folder created by <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> program to avoid the failure of displaying the correct file names.</div></blockquote>

  <br></br>
  <a name="paralog-ks"></a>
  ##### <font color="#23B395">Paralog <i>K</i><sub>s</sub> age distribution</font>
  Once selecting one / multiple species from the drop-down menu of <font style="background-color: #17a2b8; color: #EBD1FC; padding: 5px; border-radius: 2px;"> <i class="fas fa-list" role="presentation"></i> Paralog <i>K</i><sub>s</sub> &#x25BC; </font>, users can click the <font style="background-color: #548C00; color: white; padding: 2px; border-radius: 2px;"><i class="fa-solid fa-gear"></i> Configure <b><i>K</i><sub>s</sub></b> Analysis</font> to create the output panel in the right side of the page. 

  After clicking the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i class="fa-solid fa-play"></i> Start <b><i>K</i><sub>s</sub></b> Analysis</font>, the <i>K</i><sub>s</sub> age distribution of the selected species will be generated. 

  <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> employs two methods to model the potential peaks of the distribution. 

  First, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> fits normal mixture models to the <b><i>K</i><sub>s</sub></b> data using maximum likelihood with EMMIX. <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> will test one to ten components with 500 random starts. Fitted models are selected by the Bayesian Information Criterion (BIC) within EMMIX. Finally, the fitted models are drawn above the bar plot of each species.

  Second, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> uses SiZer to identify significant peaks based on the significant zero crossings of derivatives from kernel density estimation. Then, a subplot to show the output of SiZer will be added under the bar plot.

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  Please refer to the papers below to see details about EMMIX and SiZer.

  McLachlan G, Peel D. 1999. The EMMIX algorithm for the fitting of normal and t-components. <i>J Stat Softw Artic</i>. 4:1–14.

  Barker MS, et al. . 2008. Multiple paleopolyploidizations during the evolution of the Compositae reveal parallel patterns of duplicate gene retention after millions of years. <i>Mol Biol Evol</i>. 25(11):2445–2455.

  Chaudhuri P, Marron JS. 1999. SiZer for exploration of structures in curves. <i>J Am Stat Assoc</i>. 94(447):807–823.
  </div></blockquote>

  The example output is below.

  <img src="../images/Paralogous_Multiple_Species_Ks-01.jpg" width="90%">

  <br></br>
  <a name="ortholog-ks"></a>
  ##### <font color="#23B395">Ortholog <i>K</i><sub>s</sub> age distribution</font>
  Users need to select one species from the <font color="#38B0E4">Group A</font> of the <font style="background-color: #17a2b8; color: #FF9191; padding: 5px; border-radius: 2px;"><i class="fas fa-list" role="presentation"></i> Ortholog <i>K</i><sub>s</sub> &#x25BC;</font> as the focal species. Then users can choose multiple species from the <font color="#B97D4B">Group B</font> to compute the density of <i>K</i><sub>s</sub> age distribution between the focal species and the species in the <font color="#B97D4B">Group B</font>. 

  To obtain the potential peak of the plot, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> will fit the peak by using the Epanechnikov kernel funciton. Then 95% confidence interval (CI) is estimated by 1000 iterations.

  The example output is below.

  <img src="../images/Wgd_plot_ortholog.svg" width="90%">

  In the above figure, the dash line is the peak of a combination of focal species and another species. The colored rectangle is the 95% CI. Users can retrieve the detail information about the density, the peaks, and the 95% CIs by hovering the mouse above the lines or rectangles.  

  <a name="rate-correction"></a>
  ##### <font color="#23B395">Substitution rate correction</font>
  When comparing the one-to-one ortholog <i>K</i><sub>s</sub> distributions between studied species and the reference species, the different <i>K</i><sub>s</sub> peaks for the same speciation event may exist, suggesting different synonymous substitution rates among the studied species, resulting in an over- or under-estimate for the divergence between the studied species and the reference species. 

  To quantify the difference in substitution rates, <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> employs the relative rate tests. First, the Newick tree, which contains the phylogenetic relationship, will be used to ensure enough species. Then, users need to define a <b> reference species</b> and an <b>outgroup species</b> to determine a tree structure among the species which will be used in the relative rate test. Then, the <i>K</i><sub>s</sub> distance between two species is estimated by the mode of their ortholog KS distribution. Then, using the <i>K</i><sub>s</sub> distance between the reference species and the studied species, the <i>K</i><sub>s</sub> distance between the outgroup species and the reference species, the <i>K</i><sub>s</sub> distance between the outgroup species and the studied species, the distances to the studied species and the reference species after their divergence are computed. Finally, ortholog <i>K</i><sub>s</sub> between the studied species and the reference species is corrected by the double of the <i>K</i><sub>s</sub> distance to the reference species, assuming that studied species has the same substitution rate as the reference species. 

  The example output is below.

  <img src="../images/Rate_correction_with_paralogous_species.svg" width="90%">

  In the above figure, Kernel-density estimates (KDEs) of <i>K</i><sub>s</sub> distribution for one-to-one orthologues between <i>V. vinifera</i> and other four species, namely <i>A. officinalis</i>, <i>E. guineesis</i>, <i>A. comosus</i>, and <i>Z. marina</i>. The bar and line plots are the paralog<i>K</i><sub>s</sub> distributions of <i>E. guineensis</i>. As the modes (peaks) of the KDE all represent the distances between <i>V. vinifera</i> and the compared species, the differences observed among the <i>K</i><sub>s</sub> values of the modes indicate substitution rate variations among the studied species. The segments with arrows above the KDE denote, from bottom to top, the <i>K</i><sub>s</sub> distances computed for <i>A. comosus</i> (dark red), <i>E. guineensis</i> (yellow), and <i>A. officinalis</i> (blue) using <i>Z. marina</i> (purple) as the reference and <i>V. vinifera</i> as the outgroup. The dotted lines align with the observed modes in the KDE of the ortholog <i>K</i><sub>s</sub> distributions. The black dots on the segments represent the divergence between <i>Z. marina</i> with the other three species; hence, the lengths of segments pointed to the right show the <i>K</i><sub>s</sub> accumulated in <i>A. comosus</i>, <i>E. guineensis</i> and <i>A. officinalis</i>, whereas the grey segments pointed to the left show the <i>K</i><sub>s</sub>  distance between <i>V. vinifera</i> and the divergence of the other species. The grey rectangles show the 95% confidence intervals of the <i>K</i><sub>s</sub> inferred from 200 bootstraps.

  ---

  <a name="synteny-analysis"></a>
  #### <font color="orange"><img src="../images/syntenyIcon.svg" alt="Icon" width="20" height="20">Synteny Analysis</font>

  <a name="intra-inter-comparing"></a>
  ##### <font color="#23B395">Intra- or Inter-species alignment</font>

  After uploading the folder of `i-ADHoRe` in this page, users can study the *intra-*, *inter-*, and *multiple-* species synteny, as well as the *clustering analysis*.

  After selecting the studied species, users can click the <font style="background-color: #548C00; color: white; padding: 2px; border-radius: 2px;"><i class="fa-solid fa-gear"></i> Configure Analysis</font> to create the output panel in the right side of the page. 

  <!--<br><img src="../images/synteny.gif" alt="" class="docImage" width="90%"></br>--->

  Users can set the threshold for the number of anchor pairs in each multiplicon to filter out the small multiplicons which contain less anchor pairs. 

  The dot plot and parallel link plot will be generated.

  <img src="../images/Elaeis_guineensis.self.dot_plot-01.jpg" width="60%">
  <img src="../images/Elaeis_guineensis.Parallel.svg" width="80%">

  <a name="multiplicon-level-synteny"></a>
  ##### <font color="#23B395">Multiplicon-level synteny</font>

  Users can also search a certain gene to check the location of the gene in the multiplicon in the <font color="green">Multiplicon-level Synteny</font> panel and then plot the searched multiplicon.

  <img src="../images/microSyn_one_species.svg" width="80%">

  <a name="multiple-species-alignment"></a>
  ##### <font color="#23B395">Multiple species alignment</font>

  In the multiple species alignment panel, users can define the order of species in the plot. Users can change the color of species by clicking the text of the species name. When the mouse hovers a link, the related links with this link among species can be highlighted using different colors.

  The example output is below.

  <img src="../images/Multiple_Species_Alignment.Parallel-01.jpg" width="80%">

  <a name="clustering-analysis"></a>
  ##### <font color="#bb5e00">Clustering analysis</font>

  <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> employs the Hierarchical clustering method for constructing putative ancestral regions (PARs). The segments are identified from the outputs of `i-ADHoRe`, and the **homolog concentration scores** are calculated using `-log(p)`, where `p` is the probability of the observed number of homolog pairs as modeled by a Poisson distribution.

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="green"><i aria-label="fa-info icon" class="fa fa-info fa-fw" role="presentation"></i></font> To read more about <b><i>homolog concentration scores</i></b>, please refer to <i>Putnam NH, et al. (2008) The amphioxus genome and the evolution of the chordate karyotype. Nature 453:1064–1071</i>.</div></blockquote> 
  
  For each segment, the array of scores against all segments in the other genome forms a unique profile. The segments are then clustered based on the similarity of these profiles (determined by **Pearson correlation coefficient r**) using the average linkage method. The default cutoff of r is 0.3. Users can change this cutoff through the sliding block. The faint yellow rectangle will highlight the significantly enriched regions (`P < 1E-10`). Then, users can zoom in each PAR through the <font color="green"><b>Putative Ancestral Regions</b></font> block.

  The example output is below.

  <img src="../images/Two_species.cluster-01.jpg" width="80%">

  In the example, we used 0.5 as the cutoff for **r** and 10 to filter out the segment with fewer anchor points. In total, we identified 61 PARs. 

  A single PAR zooms in is below.

  <img src="../images/PAR.cluster.svg" width="50%">

  ---

  <a name="tree-building"></a>
  #### <font color="orange"><img src="../images/ksTreeIcon.svg" alt="Icon" width="20" height="20"> Tree Building</font>

  In this page, users can generate the <i>K</i><sub>s</sub> unit tree, time tree, and a joint tree with <i>K</i><sub>s</sub> unit tree and time tree.

  <!--<br><img src="../images/Tree_building.gif" alt="" class="docImage" width="90%"></br>--->

  <a name="ks-tree"></a>
  ##### <i>K</i><sub>s</sub> unit tree

  <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span> uses the single-copy gene families built by *OrthoFinder* to calculate the <i>K</i><sub>s</sub> unit tree. Users can check the tree in the *Orthofinder_wd* folder, named *singleCopyGene.ds_tree.newick*. Once uploading the tree file in the <font color="green"><i>K</i><sub>s</sub> Tree</font> File panel, the <i>K</i><sub>s</sub> unit tree will displayed in the right side of the page. Users can click the text of the species name to change the color or add a symbol to the species. Users can click the branch to add a <i>K</i><sub>s</sub> peak into the branch.

  Users can upload a file which contains the <i>K</i><sub>s</sub> peaks of each studied species to place these peaks into the <i>K</i><sub>s</sub> unit tree.

  <!--
  + **<i>K</i><sub>s</sub> Peaks file** separated by Tab
  |<i>latin name</i>      |      <i>K</i><sub>s</sub> peak |        95% CI   | color    |
  |--------|:--------|:--------|:--------|
  |Elaeis_guineensis      |      0.25        |0.19-0.26  |  #79FF79|
  |Oryza_sativa           |      1.1         |1.02-1.3   |  #5CADAD|
  |Oryza_sativa           |      0.3         |0.22-0.4   |  #5CADAD|
  -->

  + **<i>K</i><sub>s</sub> Peaks file** separated by Tab

    <table>
      <tr>
        <th><i>latin name</i></th>
        <th style="padding-left: 30px;"><i>K</i><sub>s</sub> peak</th>
        <th style="padding-left: 40px;">95% CI</th>
        <th style="padding-left: 30px;">color</th>
      </tr>
      <tr>
        <td>Elaeis_guineensis</td>
        <td style="padding-left: 30px;">0.25</td>
        <td style="padding-left: 40px;">0.19-0.26</td>
        <td style="padding-left: 30px;">#79FF79</td>
      </tr>
      <tr>
        <td>Oryza_sativa</td>
        <td style="padding-left: 30px;">1.1</td>
        <td style="padding-left: 40px;">1.02-1.3</td>
        <td style="padding-left: 30px;">#5CADAD</td>
      </tr>
      <tr>
        <td>Oryza_sativa</td>
        <td style="padding-left: 30px;">0.3</td>
        <td style="padding-left: 40px;">0.22-0.4</td>
        <td style="padding-left: 30px;">#5CADAD</td>
      </tr>
    </table>

  <br>

  The example output is below.

  <img src="../images/ksTree.Plot.svg" width="95%">

  <a name="time-tree"></a>
  ##### Time tree

  Users can upload a nexus tree withe the divergence time information. The format of the tree is the output file of *MCMCTree*, named *FigTree.tre*. Be careful with the time unit, please use the 100 million yeas as the time scale. Users can click the text of the species name to change the color or add a symbol to the species. Users can click the branch to add a WGD into the branch.

  Users can upload a file which contains the WGD events of each studied species to place these WGDs into the tree.

  + **WGD Events file** separated by Tab

    <table>
      <tr>
        <th><i>latin name</i></th>
        <th style="padding-left: 50px;">WGD events<sup>a</sup></th>
        <th style="padding-left: 30px;">color</th>
      </tr>
      <tr>
        <td>Elaeis_guineensis</td>
        <td style="padding-left: 50px;">0.85,0.1-0.2</td>
        <td style="padding-left: 30px;">#79FF79</td>
      </tr>
      <tr>
        <td>Oryza_sativa</td>
        <td style="padding-left: 50px;">0.32,0.1-0.2,1.26-1.3</td>
        <td style="padding-left: 30px;">5CADAD</td>
    </table>

  - **<sup>a</sup>** WGD events can be a single time point or a time interval. Please use "**,**" to separate multiple WGD events within a species. 
  <br>

  The example output is below.

  <img src="../images/timeTree.Plot.svg" width="95%">

  <a name="joint-tree"></a>
  ##### <i>K</i><sub>s</sub> unit tree and TimeTree joint tree
  If users upload both <i>K</i><sub>s</sub> unit tree and the time tree, a joint <i>K</i><sub>s</sub> unit tree and time tree will be generated.

  The example output is below.

  <img src="../images/ksTree_timeTree.jointPlot.svg" width="95%">

  In the above figure, the left subplot is the <i>K</i><sub>s</sub> unit tree. The right subplot is the time tree. Users can also add the <i>K</i><sub>s</sub> peaks or WGD events into the tree when clicking the branch.

  ---

  <a name="reconciliation-analysis"></a>
  #### <font color="orange"><img src="../images/treeReconciliationIcon.svg" alt="Icon" width="20" height="20"> Gene Tree – Species Tree Reconciliation Analysis</font>

  In this page, users can use `Whale` to operate the gene tree - species tree reconciliation analysis.

  Users need to upload two inputs to operate `Whale.jl`. One is the species time tree in `Newick` format. Another is the folder which includes all ALE files. If users use `OrthoFinder` in <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>, users can find the folder in `Orthofinder_wd`, named *all_tree_ALE_files*. Users can also follow the [instruction](https://github.com/arzwa/whaleprep/tree/master) of `Whale.jl` to prepare the ALE files by using the gene families built by other software.

  <!--<br><img src="../images/Whale_analysis.gif" alt="" class="docImage" width="90%"></br>-->

  Once the tree file uploaded, users can check the topology of the tree in the right panel. Then users can add the **hypothetical WGD event** to a certain branch by clicking the branch.

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="red"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> Users should use <font color="green"><b>wgd</b></font> as the prefix for the name of <b>hypothetical WGD event</b> to satisfy the requirements of <i>Whale.jl</i>.</div></blockquote>


  <img src="../images/speciesTree.whale.svg" width="500px">

  In the above tree figure, four hypothetical WGD events are tested, named as *wgd1*, *wgd2*, *wgd3* and *wgd4*, respectively.

  Users can review the added **hypothetical WGD event** in the **<font color='#AD1F1F'>Hypothetical WGDs</font> to test** panel in the left side.

  Users can define the **Base Model** in the **Base Model** for **Whale** panel and set the **Chain** rounds in the **Set the <b><font color='orange'>Chain</font></b> for <b><i>Whale</b></i>** panel.

  After setting all the parameters, users can start `Whale` by clicking the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i class="fa-solid fa-play"></i> Start <b><i>Whale</i></b></font> button.


  After `Whale` is done, users can check the output in the right side.

  The example output is below.

  <img src="../images/Whale_output.jpg" width="600px">

  <br></br>
  We can judge whether the **hypothetical WGD event** is true or not based on the Bayes factor (*K*) to compare the likelihood of q = 0(H<sub>0</sub>) to the likelihood of q > 0(H<sub>1</sub>) using the Savage–Dickey density ratio.

  <blockquote><div style="padding: 10px; background-color: #f0f0f0;">
  <font color="red"><i aria-label="warning icon" class="fa fa-warning fa-fw" role="presentation"></i></font> Please Note: the log10 Bayes factor is calculated. Thus, a Bayes factor <font color='red'><b>smaller than -2</b></font> could be considered as evidence in favor of the <i>q</i> ≠ 0 model compared to the <i>q</i> = 0 mode.</div></blockquote>

  Besides, users should also check the **ESS** values. Generally, ESS values exceeding at least 100 is better for a good model, although short chains may be good for exploring and testing different models. If the ESS values below 100, please increase the **Chain Rounds** in the left setting panel.

  Then, users can click the <font style="background-color: #548C00; color: white; padding: 5px; border-radius: 2px;"><i class="fa-solid fa-sync"></i></font> button to update the species tree to display the output of Whale.

  The example output is below.

  <img src="../images/speciesTree.updated.whale.svg" width="800px">

  In the above plot, the WGD with the solid green bar is supported with retention rates significantly different from zero, while the hollow WGD bars are the ones with retention rates not different from zero.

  Then users can download the plot by clicking the <font style="background-color: #5B5B00; color: white; padding: 5px; border-radius: 2px;"><i arial-label="download icon" class="fa fa-download" role="presentation"></i></font> button.


</div>

<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0-beta3/css/all.min.css">

<div style="float: right; width: 60%;">
 
#### This page gives users the figure examples generated by <span style="color:red; background:linear-gradient(to right, red, yellow, green, cyan, yellow, red); -webkit-background-clip: text; -webkit-text-fill-color: transparent;">ShinyWGD</span>

</div>

<style>
  .sidebar {
    position: fixed;
    left: 0;
    bottom: 0;
    height: 100%;
    color: #4C4C4C;
    padding: 50px;
    padding-top: 50px;
  }
</style>

<div class="sidebar">

<!-- Content with the floating layout     border-right: 1px solid #A9A9A9;     background-color: #F9F9F9;
 -->
<!-- div style="position: fixed; top: 50%; left: 120px; transform: translate(0, -50%); background-color: #F9F9F9; color: #4C4C4C; padding: 10px 20px; border-radius: 8px; cursor: pointer; border: 2px solid #A9A9A9;" -->


##### Index:
- [<font color="orange"><i>K</i><sub>s</sub> Age Distribution</font>](#ks-plot)
  - [Paralog <i>K</i><sub>s</sub> age distribution](#paralog-ks)
    - [*Without GMM modeling*](#paralog-ks-without-gmm-modeling)
    - [*With GMM modeling*](#paralog-ks-with-gmm-modeling)
  - [Ortholog <i>K</i><sub>s</sub> age distribution](#ortholog-ks)
  - [Substitution rate correctiong](#rate-correction)
    - [*With paralogous species*](#rate-with-paralog)
    - [*Without paralogous species*](#rate-without-paralog)
- [<font color="orange">Synteny Analysis</font>](#synteny-analysis)
  - [Intra-comparing alignment](#intra-comparing)
    - [*Dot plot*](#intra-comparing-dot)
    - [*Parallel-line plot*](#intra-comparing-parallel)
    - [*Micro-synteny plot*](#intra-comparing-micro)
  - [Inter-comparing alignment](#inter-comparing)
    - [*Dot plot*](#inter-comparing-dot)
    - [*Parallel-line plot*](#inter-comparing-parallel)
    - [*Micro-synteny plot*](#inter-comparing-micro)
  - [Multiple-species alignment](#multiple-species-alignment)
  - [Clustering analysis](#clustering-analysis)
    - [*Clustering plot*](#clustering-plot)
    - [*PAR zoom-in plot*](#PAR-zoom-in)
- [<font color="orange">Tree Building</font>](#tree-building)
  - [<i>K</i><sub>s</sub> unit tree plot](#ks-tree)
  - [TimeTree plot](#time-tree)
  - [<i>K</i><sub>s</sub> unit tree and TimeTree joint tree plot](#joint-tree)
- [<font color="orange">Gene Tree – Species Tree Reconciliation</font>](#tree-reconciliation)


</div>

<div style="float: right; width: 60%;">

  <a name="ks-plot"></a>
  #### <font color="orange"><i>K</i><sub>s</sub> Age Distribution</font>
  ---
  <a name="paralog-ks"></a>
  ##### Paralog <i>K</i><sub>s</sub> age distribution

  <a name="paralog-ks-without-gmm-modeling"></a>
  ###### *Without GMM modeling*
  <img src="../images/Paralogous_Ks.Elaeis2.ks.png">

  <a name="paralog-ks-with-gmm-modeling"></a>
  ###### *With GMM modeling*
  <img src="../images/Paralogous_Ks.Elaeis2.ks_with_GMM.png">

  ---
  <a name="ortholog-ks"></a>
  ##### Ortholog <i>K</i><sub>s</sub> age distribution
  <img src="../images/Wgd_plot_ortholog.svg">

  ---
  <a name="rate-correction"></a>
  ##### Raletive rate correctiong

  <a name="rate-with-paralog"></a>
  ###### *With paralogous species*
  <img src="../images/Rate_correction_with_paralogous_species.svg">

  <a name="rate-without-paralog"></a>
  ###### *Without paralogous species*
  <img src="../images/Rate_correction.svg">

  ---
  <a name="synteny-analysis"></a>
  #### <font color="orange">Synteny Aanlysis</font>
  ---
  <a name="intra-comparing"></a>
  ##### Intra-comparing alignment
  <a name="intra-comparing-dot"></a>
  ###### *Dot plot*
  <img src="../images/Elaeis_guineensis.self.dot_plot-01.jpg">

  <a name="intra-comparing-parallel"></a>
  ###### *Parallel-line plot*
  <img src="../images/Elaeis_guineensis.Parallel.svg">

  <a name="intra-comparing-micro"></a>
  ###### *Micro-synteny plot*
  
  *In true chromosomal length scale*
  <img src="../images/one_species.microSyn.Multiplicon_5.true_length.svg">

  *In gene number scale*
  <img src="../images/one_species.Multiplicon_5.gene_number.svg">
  ---
  <a name="inter-comparing"></a>
  ##### Inter-comparing alignment

  <a name="inter-comparing-dot"></a>
  ###### *Dot plot*
  <img src="../images/Elaeis_guineensis_vs_Asparagus_officinalis.dot_plot-01.jpg">

  <a name="inter-comparing-parallel"></a>
  ###### *Parallel-line plot*
  <img src="../images/Elaeis_guineensis_vs_Asparagus_officinalis.Parallel-01.jpg">

  <a name="inter-comparing-micro"></a>
  ###### *Micro-synteny plot*
  <img src="../images/Elaeis_guineensis_vs_Asparagus_officinalis.microSyn.svg">

  ---
  <a name="multiple-species-alignment"></a>
  ##### Multiple-species alignment
  <img src="../images/Multiple_Species_Alignment.Parallel.svg">
  
  ---
  <a name="clustering-analysis"></a>
  ##### Clustering analysis

  <a name="clustering-plot"></a>
  ###### *Clustering plot*
  <img src="../images/Two_species.cluster-01.jpg">

  <a name="PAR-zoom-in"></a>
  ###### *PAR zoom-in plot*
  <img src="../images/PAR.cluster.svg" width="50%">
  <img src="../images/Vitis_vinifera_vs_Oryza_sativa.PAR_1.cluster.link.svg" width="80%">

  ---
  <a name="tree-building"></a>
  #### <font color="orange">Tree Building</font>

  <a name="ks-tree"></a>
  ##### <i>K</i><sub>s</sub> unit tree plot
  <img src="../images/ksTree.Plot.svg">

  <a name="time-tree"></a>
  ##### TimeTree plot
  <img src="../images/timeTree.Plot.svg">

  <a name="joint-tree"></a>
  ##### <i>K</i><sub>s</sub> unit tree and TimeTree joint tree plot
  <img src="../images/ksTree_timeTree.jointPlot.svg">

  ---
  <a name="tree-reconciliation"></a>
  #### <font color="orange">Gene Tree – Species Tree Reconciliation</font>
  <img src="../images/speciesTree.updated.whale.svg">
</div>

<link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0-beta3/css/all.min.css">

### **`shinyWGD`**

---

Welcome to **`shinyWGD`** Server – **`a freely accessible and inclusive platform`**, providing an open space for users to explore and analyze Whole Genome Duplication (WGD) events.

Designed with a user-friendly interface, **`shinyWGD`** empowers users with an intuitive, interactive platform that eliminates the need for programming skills. Whether you're a experienced researcher or new to WGD analysis, **`shinyWGD`** offers a comprehensive solution.

Our server comprises two key components. First, we've streamlined the process of preparing essential inputs for existing packages, providing an interactive experience for users. Second, **`shinyWGD`** offers a meticulously crafted environment for exploring and visualizing WGD events, ensuring a smooth and insightful analysis.

Dive into the world of WGD analysis with **`shinyWGD`** Server – freely distributed and tailored to meet your genomic exploration needs.

---

#### The structure of **`shinyWGD`**

- ##### <font color="#6650C9"><i class="fa-solid fa-home"></i> Home</font>
- ##### <font color="#6650C9"><i aria-label="terminal icon" class="fa fa-terminal fa-fw fa-fade" role="presentation"></i> Scripts</font>
  - <a href="javascript:void(0);" onclick="switchToDataPreparationTab()"><font color="orange"><i aria-label="microscope icon" class="fa fa-microscope fa-fw fa-fade" role="presentation"></i> Data Preparation</font></a>
    - This module effortlessly readies inputs for a selection of integrated packages, including [`wgd`](https://github.com/arzwa/wgd), [`ksrates`](https://github.com/VIB-PSB/ksrates), [`i-ADHoRe`](https://www.vandepeerlab.org/?q=tools/i-adhore30) and [`OrthoFinder`](https://github.com/davidemms/OrthoFinder). With a fixed number of studied species, this module automatically generates the necessary inputs for each package, simplifying your workflow and enhancing the efficiency of your genomic analyses.
  - <a href="javascript:void(0);" onclick="switchToWhalePreparationTab()"><font color="orange"><img src="images/treeReconciliationIcon.svg" alt="Icon" width="20" height="20"> Whale Preparation</font></a>
    - This module seamlessly integrates the powerful capabilities of the [`Whale`](https://github.com/arzwa/Whale.jl/tree/master) package, enabling the inference of reconciled gene trees and model parameters for gene family evolution. This module leverages outputs from [`OrthoFinder`](https://github.com/davidemms/OrthoFinder), selecting optimal gene families to create the [`ALE`](https://github.com/ssolo/ALE) file for each gene family. Users can conveniently upload a time divergence tree, interactively specify WGD events for testing directly on the tree, and effortlessly generate [`Whale`](https://github.com/arzwa/Whale.jl/tree/master) code. This streamlined process eliminates the need for users to delve into Julia scripting, ensuring a user-friendly experience.
  - <a href="javascript:void(0);" onclick="switchToTreeExtractionTab()"><font color="orange"><i aria-label="tree icon" class="fa fa-tree fa-fw fa-fade" role="presentation"></i> TimeTreeFetcher</font></a>
    - This module simplifies the process of obtaining a species tree from [TimeTree.org](http://www.timetree.org/), especially when users are uncertain about the evolutionary relationships among studied species. By simply uploading a file containing species names, this module seamlessly communicates with [TimeTree.org](http://www.timetree.org/), providing users with two types of trees. The first type mirrors the sample tree structure found on [TimeTree.org](http://www.timetree.org/), while the second represents a tree constructed based on the median divergence time among studied species. This module offers a straightforward solution for users seeking clarity in their species relationships.
- ##### <font color="#6650C9"><i class="fa-solid fa-pencil" role="presentation"></i> Analysis</font>
  - <a href="javascript:void(0);" onclick="switchToKsAnalysisTab()"><font color="orange"><img src="images/ksIcon.svg" alt="Icon" width="20" height="20"> <i>K</i><sub>s</sub>Dist</font></a>
    - This module simplifies the identification of WGD events using <i>K</i><sub>s</sub> age distribution. Leveraging Data Preparation outputs, users customize selections in the configuration settings for tailored results. The module corrects paralog <i>K</i><sub>s</sub> distribution peaks with two methods: normal mixture model fitting and SiZer. It also utilizes relative rate tests to quantify substitution rate differences. Ideal for users exploring WGD events through <i>K</i><sub>s</sub> age distribution analysis.
  - <a href="javascript:void(0);" onclick="switchToSyntenyAnalysisTab()"><font color="orange"><img src="images/syntenyIcon.svg" alt="Icon" width="20" height="20"> Collinearity</font></a>
    - This module broadens the investigation of collinear relationships across all studied species, surpassing the capabilities of [`ksrates`](https://github.com/VIB-PSB/ksrates). It provides visualizations of collinear blocks. Additionally, users can delve into the exploration of putative ancestral regions (PARs) through clustering analysis, offering a more detailed and interactive exploration of the genomic architecture.
  - <a href="javascript:void(0);" onclick="switchToTreeReconTab()"><font color="orange"><img src="images/treeReconciliationIcon.svg" alt="Icon" width="20" height="20"> TreeRecon</font></a>
    - This module visualizes [`Whale`](https://github.com/arzwa/Whale.jl/tree/master) output and posterior distributions of WGD retention rates (q) for potential WGDs under selected models.
  - <a href="javascript:void(0);" onclick="switchToTreeBuildingTab()"><font color="orange"><img src="images/ksTreeIcon.svg" alt="Icon" width="20" height="20"> VizWGD </font></a>
    - This module lets users customize the MCMCtree's time tree, or a ultrametic tree. Upload a tree file, click branches to add WGD events, and adjust colors and symbols. Highlight geological time regions with ease.
- ##### <a href="javascript:void(0);" onclick="switchToGalleryTab()"><font color="#6650C9"><i class="fa-solid fa-image"></i> Gallery</font></a>
- ##### <a href="javascript:void(0);" onclick="switchToHelpTab()"><font color="#6650C9"><i class="fa-solid fa-question"></i> Help</font></a>
